
#include "StdAfx.h"
#include "BitItem.h"
#include "GdiplusWrapper.h"

CBitItem::CBitItem(UINT nResID, int cx, int cy) : m_nItemNums(0), m_nItemWidth(cx), m_nItemHeight(cy), m_nDir(kBitHorizon), m_Offset(0, 0)
{
    //
    // 1.0 Check the input value is validate...
    BOOL bRet = FALSE;
    ASSERT(nResID > 0);
    //
    // 2.0 Create the Read Memory DC...
    bRet = m_ReadBit.LoadBitmap(nResID);
    ASSERT(bRet);
    BITMAP bit =
    {
        0
    };
    m_ReadBit.GetBitmap(&bit);

    bRet = m_ReadDC.CreateCompatibleDC(NULL);
    ASSERT(bRet);
    m_ReadDC.SelectObject(&m_ReadBit);
    //
    // 4.0 Calculate the Sub-Item Direction, Number, Width, Height...
    memset(&m_Bitmap, 0, sizeof(BITMAP));
    m_ReadBit.GetBitmap(&m_Bitmap);
    ASSERT(m_Bitmap.bmWidth > 0 && m_Bitmap.bmHeight > 0);

    m_nDir = (m_Bitmap.bmWidth >= m_Bitmap.bmHeight) ? kBitHorizon : kBitVertical;
    m_nItemWidth = (m_nItemWidth <= 0) ? m_Bitmap.bmWidth : m_nItemWidth;
    m_nItemHeight = (m_nItemHeight <= 0) ? m_Bitmap.bmHeight : m_nItemHeight;
    m_nItemNums = (m_nDir == kBitHorizon) ? m_Bitmap.bmWidth / m_nItemWidth : m_Bitmap.bmHeight / m_nItemHeight;

    ASSERT(m_nItemHeight > 0);
    ASSERT(m_nItemWidth > 0);
    ASSERT(m_nItemNums > 0);
}

CBitItem::~CBitItem()
{
    (m_ReadDC.m_hDC != NULL) ? m_ReadDC.DeleteDC() : NULL;
    (m_ReadBit.m_hObject != NULL) ? m_ReadBit.DeleteObject() : NULL;
}
//
// Copy hole DC by input value...
BOOL CBitItem::CopyHoleDC(CDC* lpDesDC, int x, int y, int cx, int cy, int xSrc/* = 0*/, int ySrc/* = 0*/)
{
    CDC* lpSrcDC = &m_ReadDC;
    if (lpDesDC == NULL || lpSrcDC == NULL)
        return FALSE;
    ASSERT(m_nItemWidth > 0);
    ASSERT(m_nItemHeight > 0);

    int nWidth = (cx == 0) ? m_Bitmap.bmWidth : cx;
    int nHeight = (cy == 0) ? m_Bitmap.bmHeight : cy;

    if (gdiplusWrapper.IsInitialized())
    {
        HDC hDC2 = CreateCompatibleDC(lpDesDC->m_hDC);
        HBITMAP bm2 = CreateCompatibleBitmap(lpDesDC->m_hDC, nWidth, nHeight);
        SelectObject(hDC2, bm2);

        BitBlt(hDC2, 0, 0, nWidth, nHeight, lpSrcDC->m_hDC, xSrc, ySrc, SRCPAINT);

        Graphics* myGraphics = new Graphics(lpDesDC->m_hDC);
        Bitmap* pBmp = new Bitmap(bm2, NULL);
        myGraphics->DrawImage(pBmp, x, y, nWidth, nHeight);
        delete pBmp;
        delete myGraphics;
        DeleteDC(hDC2);
        DeleteObject(bm2);
        return TRUE;
    }
    else
        return lpDesDC->BitBlt(x, y, nWidth, nHeight, lpSrcDC, xSrc, ySrc, SRCCOPY);
}

//
// Copy hole DC by input RECT object...
BOOL CBitItem::CopyHoleDC(CDC* lpDesDC, LPRECT lpRect, int xSrc/* = 0*/, int ySrc/* = 0*/)
{
    if (lpRect == NULL)
        return FALSE;
    ASSERT(lpRect != NULL);

    return CopyHoleDC(lpDesDC, lpRect->left, lpRect->top, lpRect->right - lpRect->left, lpRect->bottom - lpRect->top, xSrc, ySrc);
}
//
// Get Item rectangle by input item index...
BOOL CBitItem::GetRectByIndex(LPRECT lpRect, int nIndex, BOOL bSrc/* = TRUE*/, POINT* lpOff/* = NULL*/)
{
    //
    // 1.0 Check the input value is validate...
    if (lpRect == NULL || nIndex >= m_nItemNums)
        return FALSE;
    ASSERT(lpRect != NULL);
    ASSERT(nIndex >= 0);
    //
    // 2.0 Calculate the rectangle...
    lpRect->left = (m_nDir == kBitHorizon) ? nIndex * m_nItemWidth : 0;
    lpRect->top = (m_nDir == kBitVertical) ? nIndex * m_nItemHeight : 0;

    lpRect->left += (bSrc ? 0 : ((lpOff != NULL) ? lpOff->x : m_Offset.x));
    lpRect->top += (bSrc ? 0 : ((lpOff != NULL) ? lpOff->y : m_Offset.y));

    lpRect->right = lpRect->left + m_nItemWidth;
    lpRect->bottom = lpRect->top + m_nItemHeight;
    return TRUE;
}
//
// Check the input point is in first bititem's rectangle...
BOOL CBitItem::IsInBitItem(CPoint& pt, POINT* lpOff/* = NULL*/)
{
    CRect rcRect;
    GetRectByIndex(rcRect, 0, FALSE, lpOff);
    return rcRect.PtInRect(pt);
}
BOOL CBitItem::CopyItemIndex(CDC* lpDesDC, int nIndex, BOOL bSrc/* = TRUE*/, POINT* lpOff /* = NULL*/)
{
    ASSERT(nIndex < m_nItemNums);
    ASSERT(lpDesDC != NULL);
    if (lpDesDC == NULL || nIndex >= m_nItemNums)
        return FALSE;
    RECT rcSrc =
    {
        0, 0, 0, 0
    };
    if (!GetRectByIndex(&rcSrc, nIndex))
        return FALSE;

    if (gdiplusWrapper.IsInitialized())
    {
        HDC hDC2 = CreateCompatibleDC(lpDesDC->m_hDC);
        HBITMAP bm2 = CreateCompatibleBitmap(lpDesDC->m_hDC, rcSrc.right - rcSrc.left, rcSrc.bottom - rcSrc.top);
        SelectObject(hDC2, bm2);

        BitBlt(hDC2, 0, 0, rcSrc.right - rcSrc.left, rcSrc.bottom - rcSrc.top, m_ReadDC, rcSrc.left, rcSrc.top, SRCPAINT);

        Graphics* myGraphics = new Graphics(lpDesDC->m_hDC);
        Bitmap* pBmp = new Bitmap(bm2, NULL);
        myGraphics->DrawImage(pBmp, (bSrc ? rcSrc.left : 0) + ((lpOff != NULL) ? lpOff->x : m_Offset.x), (bSrc ? rcSrc.top : 0) + ((lpOff != NULL) ? lpOff->y : m_Offset.y), rcSrc.right - rcSrc.left, rcSrc.bottom - rcSrc.top);
        delete pBmp;
        delete myGraphics;
        DeleteDC(hDC2);
        DeleteObject(bm2);
        return TRUE;
    }
    else
        return lpDesDC->BitBlt((bSrc ? rcSrc.left : 0) + ((lpOff != NULL) ? lpOff->x : m_Offset.x), (bSrc ? rcSrc.top : 0) + ((lpOff != NULL) ? lpOff->y : m_Offset.y), rcSrc.right - rcSrc.left, rcSrc.bottom - rcSrc.top, &m_ReadDC, rcSrc.left, rcSrc.top, SRCCOPY);
}

void CBitItem::GetRectPos(LPRECT lpRect, POINT* lpOff/* = NULL*/)
{
    ASSERT(lpRect != NULL);
    lpRect->left = (lpOff != NULL) ? lpOff->x : m_Offset.x;
    lpRect->top = (lpOff != NULL) ? lpOff->y : m_Offset.y;
    lpRect->right = ((lpOff != NULL) ? lpOff->x : m_Offset.x) + m_Bitmap.bmWidth;
    lpRect->bottom = ((lpOff != NULL) ? lpOff->y : m_Offset.y) + m_Bitmap.bmHeight;
}
