///////////////////////////////////////////////////////////////////////////////
//
// ComboBoxXP.cpp : implementation file
//
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ComboBoxXP.h"
#include "Draw.h"
#include "aeroglss.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


////////////////////////////////////////////////////////////////////////////////
CComboBoxXP::CComboBoxXP()
{
}

///////////////////////////////////////////////////////////////////////////////
BOOL CComboBoxXP::Create(DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID)
{
    if (!CComboBox::Create(dwStyle, rect, pParentWnd, nID))
        return false;
    if (!CMenuXP::GetXPLookNFeel(GetParent()))
        return true;
    SetFont(CFont::FromHandle((HFONT)::GetStockObject(DEFAULT_GUI_FONT)));
    m_MouseMgr.Init(m_hWnd);
    return true;
}

////////////////////////////////////////////////////////////////////////////////
void CComboBoxXP::PreSubclassWindow()
{
    m_MouseMgr.Init(m_hWnd);
}

///////////////////////////////////////////////////////////////////////////////
BEGIN_MESSAGE_MAP(CComboBoxXP, CComboBox)
  ON_WM_GETDLGCODE()
  ON_WM_PAINT()
  ON_WM_SETFOCUS()
  ON_WM_KILLFOCUS()
  ON_WM_MOUSEMOVE()
  ON_WM_MOUSEOUT()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////////////////
BOOL CComboBoxXP::PreTranslateMessage(MSG* pMsg)
{
    if (CMenuXP::GetXPLookNFeel(GetParent()))
    {
        if ((GetParent()->IsKindOf(RUNTIME_CLASS(CControlBar)) && ((CControlBar *)GetParent())->m_pDockBar != NULL) && pMsg->message == WM_KEYDOWN)
            switch (pMsg->wParam)
            {
                case VK_RETURN:
                    GetParent()->SendMessage(WM_COMMAND, GetDlgCtrlID());
                    break;

                case VK_ESCAPE:
                    if (GetDroppedState())
                        ShowDropDown(false);
                    else
                        GetParentFrame()->GetActiveFrame()->SetFocus();
                    break;
            }
    }
    return CComboBox::PreTranslateMessage(pMsg);
}

///////////////////////////////////////////////////////////////////////////////
UINT CComboBoxXP::OnGetDlgCode()
{
    if (CMenuXP::GetXPLookNFeel(GetParent()))
    {
        if (KEYDOWN(VK_RETURN))
            return DLGC_WANTALLKEYS | CComboBox::OnGetDlgCode();
    }
    return CComboBox::OnGetDlgCode();
}

///////////////////////////////////////////////////////////////////////////////
void CComboBoxXP::OnPaint()
{
    CDwmApiImpl dwmApi;
    if ((dwmApi.Initialize() && dwmApi.IsDwmCompositionEnabled()) || !CMenuXP::GetXPLookNFeel(GetParent()))
    {
        CComboBox::OnPaint();
        return;
    }
    CRect rc;
    GetClientRect(&rc);
    CRect rcBtn(rc.right - ::GetSystemMetrics(SM_CXHTHUMB), rc.top, rc.right, rc.bottom);

    ValidateRect(rcBtn);
    Default();

    CClientDC cDC(this);
    DWORD wStyle = DB_FLAT | DB_DOWN | (IsWindowEnabled() ? DB_ENABLED : DB_DISABLED);
    bool bInToolbar = GetParent()->IsKindOf(RUNTIME_CLASS(CControlBar)) && ((CControlBar*)GetParent())->m_pDockBar != NULL;
    bool bDropped = DefWindowProc(CB_GETDROPPEDSTATE, 0, 0L) != 0;
    bool bFocus = (::GetFocus() == m_hWnd || ::GetParent(::GetFocus()) == m_hWnd) && bInToolbar;

    if (!bInToolbar)
        wStyle |= DB_BORDER;
    if (bFocus || m_MouseMgr.MouseOver())
        wStyle |= DB_OVER;
    if (bDropped)
        wStyle |= DB_OVER | DB_PRESSED;
    CDrawButton(wStyle, rcBtn).Draw(&cDC);
    CPenDC pen(cDC, ::GetSysColor((bFocus || bDropped || m_MouseMgr.MouseOver()) ? COLOR_HIGHLIGHT : (bInToolbar ? COLOR_WINDOW : COLOR_3DSHADOW)));

    cDC.SelectObject(::GetStockObject(NULL_BRUSH));
    cDC.Rectangle(rc);
    rc.right -= ::GetSystemMetrics(SM_CXHTHUMB) - 1;
    cDC.Rectangle(rc);

    pen.Color(::GetSysColor(IsWindowEnabled() ? COLOR_WINDOW : COLOR_3DFACE));
    rc.DeflateRect(1, 1);
    cDC.Rectangle(rc);
}

///////////////////////////////////////////////////////////////////////////////
void CComboBoxXP::OnSetFocus(CWnd* pOldWnd)
{
    CComboBox::OnSetFocus(pOldWnd);

    if (!CMenuXP::GetXPLookNFeel(GetParent()))
        return;

    if (GetParent()->IsKindOf(RUNTIME_CLASS(CControlBar)) && ((CControlBar *)GetParent())->m_pDockBar != NULL)
        Invalidate();
}

///////////////////////////////////////////////////////////////////////////////
void CComboBoxXP::OnKillFocus(CWnd* pNewWnd)
{
    CComboBox::OnKillFocus(pNewWnd);

    if (!CMenuXP::GetXPLookNFeel(GetParent()))
        return;

    if (GetParent()->IsKindOf(RUNTIME_CLASS(CControlBar)) && ((CControlBar *)GetParent())->m_pDockBar != NULL)
        Invalidate();
}

///////////////////////////////////////////////////////////////////////////////
void CComboBoxXP::OnMouseMove(UINT nFlags, CPoint point)
{
    Default();
    m_MouseMgr.OnMouseMove();
}

///////////////////////////////////////////////////////////////////////////////
void CComboBoxXP::OnMouseOut()
{
    Default();
    m_MouseMgr.OnMouseOut();
}
