//- -----------------------------------------------------------------
//- (C) Copyright 2000 Blake V. Miller
//- All Rights Reserved.
//- -----------------------------------------------------------------
//- File    :   CGFILTYP.CPP
//- Author  :   Blake Miller
//- Version :   June 16, 2000
//- Purpose :   File Type Access
//- -----------------------------------------------------------------

#include "stdafx.h"
#include "winerror.h"

#ifndef __CGFILTYP_H__
#include "CGFILTYP.H"   //  CGCFileTypeAccess
#endif

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNAMIC(CGCFileTypeAccess, CObject)

CGCFileTypeAccess::CGCFileTypeAccess()
{
}

// virtual 
CGCFileTypeAccess::~CGCFileTypeAccess()
{
}

//- *****************************************************************
//- Functions Affecting Registry
//- *****************************************************************

// virtual
BOOL CGCFileTypeAccess::RegSetAllExtensionInfo()
{
    RegSetExtension();
    RegSetDocumentType();

    return TRUE;
}

// HKEY_CLASSES_ROOT\.<Extension>
// virtual
BOOL CGCFileTypeAccess::RegSetExtension()
{
    if (m_csExtension.IsEmpty())
    {
        return TRUE;
    }

    CString csKey = _T(".") + m_csExtension;

    SetRegistryValue(HKEY_CLASSES_ROOT, csKey, _T(""), m_csDocumentClassName);

    if (!m_csShellOpenCommand.IsEmpty())
    {
        csKey += _T("\\shell\\openstp\\command");
        SetRegistryValue(HKEY_CLASSES_ROOT, csKey, _T(""), m_csShellOpenCommand);
    }

    csKey = _T(".") + m_csExtension;
    if (!m_csShellOpenDescription.IsEmpty())
    {
        csKey += _T("\\shell\\openstp");
        SetRegistryValue(HKEY_CLASSES_ROOT, csKey, _T(""), m_csShellOpenDescription);
    }
    csKey = _T(".") + m_csExtension;

    if (!m_csShellAddCommand.IsEmpty())
    {
        csKey += _T("\\shell\\addstp\\command");
        SetRegistryValue(HKEY_CLASSES_ROOT, csKey, _T(""), m_csShellAddCommand);
    }

    csKey = _T(".") + m_csExtension;

    if (!m_csShellAddDescription.IsEmpty())
    {
        csKey += _T("\\shell\\addstp");
        SetRegistryValue(HKEY_CLASSES_ROOT, csKey, _T(""), m_csShellAddDescription);
    }

    return TRUE;
}

// HKEY_CLASSES_ROOT\<Directory>
// virtual
BOOL CGCFileTypeAccess::RegSetDirectory()
{
    if (m_csExtension.IsEmpty())
    {
        return TRUE;
    }

    CString csKey = m_csExtension;

    csKey = m_csExtension;

    if (!m_csShellAddCommand.IsEmpty())
    {
        csKey += "\\shell\\addstp\\command";
        SetRegistryValue(HKEY_CLASSES_ROOT, csKey, _T(""), m_csShellAddCommand);
    }

    csKey = m_csExtension;

    if (!m_csShellAddDescription.IsEmpty())
    {
        csKey += "\\shell\\addstp";
        SetRegistryValue(HKEY_CLASSES_ROOT, csKey, _T(""), m_csShellAddDescription);
    }

    csKey = m_csExtension;

    if (!m_csShellOpenCommand.IsEmpty())
    {
        csKey += "\\shell\\playstp\\command";
        SetRegistryValue(HKEY_CLASSES_ROOT, csKey, _T(""), m_csShellOpenCommand);
    }

    csKey = m_csExtension;

    if (!m_csShellOpenDescription.IsEmpty())
    {
        csKey += "\\shell\\playstp";
        SetRegistryValue(HKEY_CLASSES_ROOT, csKey, _T(""), m_csShellOpenDescription);
    }

    return TRUE;
}

// HKEY_CLASSES_ROOT\.<Document Type>
// virtual
BOOL CGCFileTypeAccess::RegSetDocumentType()
{
    if (m_csDocumentClassName.IsEmpty())
    {
        return TRUE;
    }

    CString csKey = m_csDocumentClassName;

    SetRegistryValue(HKEY_CLASSES_ROOT, csKey, _T(""), m_csDocumentDescription);

    // Default
    if (!m_csDocumentDefault.IsEmpty())
    {
        csKey = m_csDocumentClassName;
        csKey += "\\shell";
        SetRegistryValue(HKEY_CLASSES_ROOT, csKey, _T(""), m_csDocumentDefault);
    }

    // DefaultIcon
    if (!m_csDocumentDefaultIcon.IsEmpty())
    {
        csKey = m_csDocumentClassName;
        csKey += "\\DefaultIcon";
        SetRegistryValue(HKEY_CLASSES_ROOT, csKey, _T(""), m_csDocumentDefaultIcon);
    }

    // shell\openstp\command
    if (!m_csShellOpenCommand.IsEmpty())
    {
        csKey = m_csDocumentClassName;
        csKey += "\\shell\\openstp\\command";
        SetRegistryValue(HKEY_CLASSES_ROOT, csKey, _T(""), m_csShellOpenCommand);
    }

    // shell\openstp
    if (!m_csShellOpenDescription.IsEmpty())
    {
        csKey = m_csDocumentClassName;
        csKey += "\\shell\\openstp";
        SetRegistryValue(HKEY_CLASSES_ROOT, csKey, _T(""), m_csShellOpenDescription);
    }

    // shell\addstp\command
    if (!m_csShellAddCommand.IsEmpty())
    {
        csKey = m_csDocumentClassName;
        csKey += "\\shell\\addstp\\command";
        SetRegistryValue(HKEY_CLASSES_ROOT, csKey, _T(""), m_csShellAddCommand);
    }
    // shell\addstp
    if (!m_csShellAddDescription.IsEmpty())
    {
        csKey = m_csDocumentClassName;
        csKey += "\\shell\\addstp";
        SetRegistryValue(HKEY_CLASSES_ROOT, csKey, _T(""), m_csShellAddDescription);
    }

    return TRUE;
}

//- *****************************************************************
//- Member Variable Get/Set Methods
//- *****************************************************************

void CGCFileTypeAccess::SetExtension(LPCTSTR szExtension)
{
    m_csExtension = szExtension;
}
void CGCFileTypeAccess::GetExtension(CString& csExtension) const
{
    csExtension = m_csExtension;
}

void CGCFileTypeAccess::SetContentType(LPCTSTR szContentType)
{
    m_csContentType = szContentType;
}
void CGCFileTypeAccess::GetContentType(CString& csContentType) const
{
    csContentType = m_csContentType;
}

void CGCFileTypeAccess::SetShellOpenCommand(LPCTSTR szShellOpenCommand)
{
    m_csShellOpenCommand = szShellOpenCommand;
}
void CGCFileTypeAccess::GetShellOpenCommand(CString& csShellOpenCommand) const
{
    csShellOpenCommand = m_csShellOpenCommand;
}

void CGCFileTypeAccess::SetShellOpenDescription(LPCTSTR szShellOpenDescription)
{
    m_csShellOpenDescription = szShellOpenDescription;
}
void CGCFileTypeAccess::GetShellOpenDescription(CString& csShellOpenDescription) const
{
    csShellOpenDescription = m_csShellOpenDescription;
}

void CGCFileTypeAccess::SetShellAddCommand(LPCTSTR szShellAddCommand)
{
    m_csShellAddCommand = szShellAddCommand;
}
void CGCFileTypeAccess::GetShellAddCommand(CString& csShellAddCommand) const
{
    csShellAddCommand = m_csShellAddCommand;
}

void CGCFileTypeAccess::SetShellAddDescription(LPCTSTR szShellAddDescription)
{
    m_csShellAddDescription = szShellAddDescription;
}
void CGCFileTypeAccess::GetShellAddDescription(CString& csShellAddDescription) const
{
    csShellAddDescription = m_csShellAddDescription;
}

void CGCFileTypeAccess::SetDocumentClassName(LPCTSTR szDocumentClassName)
{
    m_csDocumentClassName = szDocumentClassName;
}
void CGCFileTypeAccess::GetDocumentClassName(CString& csDocumentClassName) const
{
    csDocumentClassName = m_csDocumentClassName;
}

void CGCFileTypeAccess::SetDocumentDescription(LPCTSTR szDocumentDescription)
{
    m_csDocumentDescription = szDocumentDescription;
}
void CGCFileTypeAccess::GetDocumentDescription(CString& csDocumentDescription) const
{
    csDocumentDescription = m_csDocumentDescription;
}

void CGCFileTypeAccess::SetDocumentDefault(LPCTSTR szDocumentDefault)
{
    m_csDocumentDefault = szDocumentDefault;
}
void CGCFileTypeAccess::GetDocumentDefault(CString& csDocumentDefault) const
{
    csDocumentDefault = m_csDocumentDefault;
}

void CGCFileTypeAccess::SetDocumentCurrentVersion(LPCTSTR szDocumentCurrentVersion)
{
    m_csDocumentCurrentVersion = szDocumentCurrentVersion;
}
void CGCFileTypeAccess::GetDocumentCurrentVersion(CString& csDocumentCurrentVersion) const
{
    csDocumentCurrentVersion = m_csDocumentCurrentVersion;
}

void CGCFileTypeAccess::SetDocumentDefaultIcon(LPCTSTR szDocumentDefaultIcon)
{
    m_csDocumentDefaultIcon = szDocumentDefaultIcon;
}
void CGCFileTypeAccess::GetDocumentDefaultIcon(CString& csDocumentDefaultIcon) const
{
    csDocumentDefaultIcon = m_csDocumentDefaultIcon;
}

void CGCFileTypeAccess::SetDocumentShellOpenCommand(LPCTSTR szDocumentShellOpenCommand)
{
    m_csDocumentShellOpenCommand = szDocumentShellOpenCommand;
}
void CGCFileTypeAccess::GetDocumentShellOpenCommand(CString& csDocumentShellOpenCommand) const
{
    csDocumentShellOpenCommand = m_csDocumentShellOpenCommand;
}

void CGCFileTypeAccess::SetDocumentShellOpenDescription(LPCTSTR szDocumentShellOpenDescription)
{
    m_csDocumentShellOpenDescription = szDocumentShellOpenDescription;
}
void CGCFileTypeAccess::GetDocumentShellOpenDescription(CString& csDocumentShellOpenDescription) const
{
    csDocumentShellOpenDescription = m_csDocumentShellOpenDescription;
}

void CGCFileTypeAccess::SetDocumentShellAddCommand(LPCTSTR szDocumentShellAddCommand)
{
    m_csDocumentShellAddCommand = szDocumentShellAddCommand;
}
void CGCFileTypeAccess::GetDocumentShellAddCommand(CString& csDocumentShellAddCommand) const
{
    csDocumentShellAddCommand = m_csDocumentShellAddCommand;
}

void CGCFileTypeAccess::SetDocumentShellAddDescription(LPCTSTR szDocumentShellAddDescription)
{
    m_csDocumentShellAddDescription = szDocumentShellAddDescription;
}
void CGCFileTypeAccess::GetDocumentShellAddDescription(CString& csDocumentShellAddDescription) const
{
    csDocumentShellAddDescription = m_csDocumentShellAddDescription;
}

// virtual 
void CGCFileTypeAccess::ClearAllData()
{
    m_csExtension.Empty();
    m_csContentType.Empty();
    m_csShellOpenCommand.Empty();
    m_csShellOpenDescription.Empty();
    m_csShellAddCommand.Empty();
    m_csShellAddDescription.Empty();

    m_csDocumentClassName.Empty();
    m_csDocumentDescription.Empty();
    m_csDocumentDefault.Empty();
    m_csDocumentCurrentVersion.Empty();
    m_csDocumentDefaultIcon.Empty();
    m_csDocumentShellOpenCommand.Empty();
    m_csDocumentShellOpenDescription.Empty();
    m_csDocumentShellAddCommand.Empty();
    m_csDocumentShellAddDescription.Empty();
}

//- -----------------------------------------------------------------
//- set string value into registry

BOOL CGCFileTypeAccess::SetRegistryValue(HKEY   hOpenKey, LPCTSTR szKey, LPCTSTR szValue, LPCTSTR szData)
{
    // validate input
    if (!hOpenKey || !szKey || !szKey[0] || !szValue || !szData)
    {
        ::SetLastError(E_INVALIDARG);
        return FALSE;
    }

    BOOL bRetVal = FALSE;
    DWORD dwDisposition;
    DWORD dwReserved = 0;
    HKEY hTempKey = (HKEY)0;

    // length specifier is in bytes, and some TCHAR 
    // are more than 1 byte each
    DWORD dwBufferLength = lstrlen(szData) * sizeof(TCHAR);

    // Open key of interest
    // Assume all access is okay and that all keys will be stored to file
    // Utilize the default security attributes
    if (ERROR_SUCCESS == ::RegCreateKeyEx(hOpenKey, szKey, dwReserved, (LPTSTR)0, REG_OPTION_NON_VOLATILE, KEY_SET_VALUE, 0, &hTempKey, &dwDisposition))
    {
        // dwBufferLength must include size of terminating nul 
        // character when using REG_SZ with RegSetValueEx function
        dwBufferLength += sizeof(TCHAR);

        if (ERROR_SUCCESS == ::RegSetValueEx(hTempKey, (LPTSTR)szValue, dwReserved, REG_SZ, (LPBYTE)szData, dwBufferLength))
        {
            bRetVal = TRUE;
        }
    }

    // close opened key
    if (hTempKey)
    {
        ::RegCloseKey(hTempKey);
    }

    return bRetVal;
}

//- -----------------------------------------------------------------
//- END CGFILTYP.CPP Source File
//- -----------------------------------------------------------------
