// ColourPicker.cpp : implementation file
//
// ColourPicker is a drop-in colour picker control. Check out the 
// header file or the accompanying HTML doc file for details.
//
// Written by Chris Maunder (chrismaunder@codeguru.com)
// Extended by Alexander Bischofberger (bischofb@informatik.tu-muenchen.de)
// Copyright (c) 1998.
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed unmodified by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name is included. If 
// the source code in  this file is used in any commercial application 
// then a simple email would be nice.
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability if it causes any damage to your
// computer, causes your pet cat to fall ill, increases baldness or
// makes you car start emitting strange noises when you start it up.
//
// Expect bugs.
// 
// Please use and enjoy. Please let me know of any bugs/mods/improvements 
// that you have found/implemented and I will fix/incorporate them into this
// file. 
//
// Updated 16 May 1998
//         31 May 1998 - added Default text (CJM)
//          9 Jan 1999 - minor vis update

#include "stdafx.h"
#include "aeroglss.h"
#include "aerosubc.h"
#include "ColourPopup.h"
#include "ColourPicker.h"
#include "mainHelper.h"
#include "resource.h"
#include "draw.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

void AFXAPI DDX_ColourPicker(CDataExchange* pDX, int nIDC, COLORREF& crColour)
{
    HWND hWndCtrl = pDX->PrepareCtrl(nIDC);
    ASSERT(hWndCtrl != NULL);                

    CColourPicker* pColourPicker = (CColourPicker*)CWnd::FromHandle(hWndCtrl);
    if (pDX->m_bSaveAndValidate)
    {
        crColour = pColourPicker->GetColour();
    }
    else // initializing
    {
        pColourPicker->SetColour(crColour);
    }
}

/////////////////////////////////////////////////////////////////////////////
// CColourPicker

CColourPicker::CColourPicker()
{
    SetBkColour(GetSysColor(COLOR_3DFACE));
    SetTextColour(GetSysColor(COLOR_BTNTEXT));

    m_bTrackSelection = FALSE;
    m_nSelectionMode = CP_MODE_BK;
    m_bActive = FALSE;

    m_strDefaultText = m_Helper.MyLoadString(IDS_AUTOMATIC);
    m_strCustomText = m_Helper.MyLoadString(IDS_MORECOLOR);
}

CColourPicker::~CColourPicker()
{
}

IMPLEMENT_DYNCREATE(CColourPicker, CButton)

BEGIN_MESSAGE_MAP(CColourPicker, CButtonXP)
    ON_CONTROL_REFLECT_EX(BN_CLICKED, OnClicked)
    ON_WM_CREATE()
    ON_MESSAGE(CPN_SELENDOK,     OnSelEndOK)
    ON_MESSAGE(CPN_SELENDCANCEL, OnSelEndCancel)
    ON_MESSAGE(CPN_SELCHANGE,    OnSelChange)
    ON_WM_PAINT()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CColourPicker message handlers

LONG CColourPicker::OnSelEndOK(UINT lParam, LONG /*wParam*/)
{
    COLORREF crNewColour = (COLORREF)lParam;
    m_bActive = FALSE;
    SetColour(crNewColour);

    CWnd* pParent = GetParent();
    if (pParent)
    {
        pParent->SendMessage(CPN_CLOSEUP, lParam, (WPARAM)GetDlgCtrlID());
        pParent->SendMessage(CPN_SELENDOK, lParam, (WPARAM)GetDlgCtrlID());
    }

    if (crNewColour != GetColour())
        if (pParent)
            pParent->SendMessage(CPN_SELCHANGE, lParam, (WPARAM)GetDlgCtrlID());

    return TRUE;
}

LONG CColourPicker::OnSelEndCancel(UINT lParam, LONG /*wParam*/)
{
    m_bActive = FALSE;
    SetColour((COLORREF)lParam);

    CWnd* pParent = GetParent();
    if (pParent)
    {
        pParent->SendMessage(CPN_CLOSEUP, lParam, (WPARAM)GetDlgCtrlID());
        pParent->SendMessage(CPN_SELENDCANCEL, lParam, (WPARAM)GetDlgCtrlID());
    }

    return TRUE;
}

LONG CColourPicker::OnSelChange(UINT lParam, LONG /*wParam*/)
{
    if (m_bTrackSelection)
        SetColour((COLORREF)lParam);

    CWnd* pParent = GetParent();
    if (pParent)
        pParent->SendMessage(CPN_SELCHANGE, lParam, (WPARAM)GetDlgCtrlID());

    return TRUE;
}

int CColourPicker::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (CButtonXP::OnCreate(lpCreateStruct) == -1)
        return -1;

    SetWindowSize();    // resize appropriately
    return 0;
}

// On mouse click, create and show a CColourPopup window for colour selection
BOOL CColourPicker::OnClicked()
{
    m_bActive = TRUE;
    CRect rect;
    GetWindowRect(rect);
    new CColourPopup(CPoint(rect.left, rect.bottom),    // Point to display popup
                     GetColour(),                       // Selected colour
                     this,                              // parent
                     m_strDefaultText,                  // "Default" text area
                     m_strCustomText);                  // Custom Text

    CWnd* pParent = GetParent();
    if (pParent)
        pParent->SendMessage(CPN_DROPDOWN, (LPARAM)GetColour(), (WPARAM)GetDlgCtrlID());

    // Docs say I should return FALSE to stop the parent also getting the message.
    // HA! What a joke.

    return TRUE;
}

void CColourPicker::OnPaint()
{
    CDC* pDC = CDC::FromHandle(::GetDC(m_hWnd));
    CRect rcItem, rect;
    GetClientRect(rcItem);

    rect = rcItem;

    UINT state = GetState();
    CString m_strText;

    if (m_bActive || m_MouseMgr.MouseOver())
        state |= 0x0004;

    CSize Margins(::GetSystemMetrics(SM_CXEDGE), ::GetSystemMetrics(SM_CYEDGE));

    CDwmApiImpl dwmApi;
    if (dwmApi.Initialize() && dwmApi.IsDwmCompositionEnabled())
    {
        CUxThemeAeroImpl g_ThemeLib;
        if (g_ThemeLib.Initialize() && g_ThemeLib.IsThemeActive())
        {
            HTHEME hTheme = g_ThemeLib.OpenThemeData(m_hWnd, L"combobox");
            if (hTheme)
            {
                HDC hdcPaint = NULL;
                BP_PAINTPARAMS params =
                {
                    sizeof(BP_PAINTPARAMS)
                };
                params.dwFlags = BPPF_ERASE;
                HPAINTBUFFER hBufferedPaint = g_ThemeLib.BeginBufferedPaint(pDC->m_hDC, &rcItem, BPBF_TOPDOWNDIB, &params, &hdcPaint);
                if (hdcPaint)
                {
                    VERIFY(S_OK == g_ThemeLib.BufferedPaintSetAlpha(hBufferedPaint, &rcItem, 0x00));
                    VERIFY(S_OK == g_ThemeLib.DrawThemeBackground(hTheme, hdcPaint, CP_DROPDOWNBUTTON, CBXS_NORMAL | ((state & 0x0004) ? CBXS_HOT : 0) | (!IsWindowEnabled() ? CBXS_DISABLED : 0), &rect, NULL));
                    rect.left = m_ArrowRect.left;
                    VERIFY(S_OK == g_ThemeLib.DrawThemeBackground(hTheme, hdcPaint, CP_DROPDOWNBUTTON, CBXS_NORMAL | ((state & 0x0004) ? CBXS_HOT : 0) | (!IsWindowEnabled() ? CBXS_DISABLED : 0), &rect, NULL));
                    rect = rcItem;
                    // Fill remaining area with colour
                    rect.left+=2;
                    rect.right -= m_ArrowRect.Width() + 2;
                    rect.top+=2;
                    rect.bottom--;

                    CBrush brush((!IsWindowEnabled() || m_crColourBk == CLR_DEFAULT) ? ::GetSysColor(COLOR_3DFACE) : m_crColourBk);
                    CBrush* pOldBrush = (CBrush*)CDC::FromHandle(hdcPaint)->SelectObject(&brush);
                    CDC::FromHandle(hdcPaint)->SelectStockObject(NULL_PEN);
                    CDC::FromHandle(hdcPaint)->Rectangle(rect);
                    CDC::FromHandle(hdcPaint)->SelectObject(pOldBrush);
                    if (state & 0x0008)
                    {
                        rect.right--;
                        rect.bottom--;
                        CDC::FromHandle(hdcPaint)->DrawFocusRect(rect);
                    }

                    //////////////////////////////
                    // Draw the window text (if any)
                    GetWindowText(m_strText);
                    if (m_strText.GetLength())
                    {
                        CDC::FromHandle(hdcPaint)->SetBkMode(TRANSPARENT);
                        if (!IsWindowEnabled())
                        {
                            rect.OffsetRect(1, 1);
                            CDC::FromHandle(hdcPaint)->SetTextColor(::GetSysColor(COLOR_3DHILIGHT));
                            CDC::FromHandle(hdcPaint)->DrawText(m_strText, rect, DT_CENTER | DT_SINGLELINE | DT_VCENTER);
                            rect.OffsetRect(-1, -1);
                            CDC::FromHandle(hdcPaint)->SetTextColor(::GetSysColor(COLOR_3DSHADOW));
                            CDC::FromHandle(hdcPaint)->DrawText(m_strText, rect, DT_CENTER | DT_SINGLELINE | DT_VCENTER);
                        }
                        else
                        {
                            CDC::FromHandle(hdcPaint)->SetTextColor((m_crColourText == CLR_DEFAULT) ? 0 : m_crColourText);
                            CDC::FromHandle(hdcPaint)->DrawText(m_strText, rect, DT_CENTER | DT_SINGLELINE | DT_VCENTER);
                        }
                    }

                    VERIFY(S_OK == g_ThemeLib.BufferedPaintMakeOpaque_(hBufferedPaint, &rcItem));
                    VERIFY(S_OK == g_ThemeLib.EndBufferedPaint(hBufferedPaint, TRUE));
                }
                VERIFY(S_OK == g_ThemeLib.CloseThemeData(hTheme));
                return;
            }
        }
    }

    if (!CMenuXP::GetXPLookNFeel(GetParent()))
    {
        // Draw arrow
        pDC->DrawFrameControl(&m_ArrowRect, DFC_SCROLL, DFCS_SCROLLDOWN | ((state & 0x0004) ? DFCS_PUSHED : 0) | (!IsWindowEnabled() ? DFCS_INACTIVE : 0));

        pDC->DrawEdge(rect, EDGE_SUNKEN, BF_RECT);

        // Must reduce the size of the "client" area of the button due to edge thickness.
        rect.DeflateRect(Margins.cx, Margins.cy);
        // Fill remaining area with colour
        rect.bottom++;
        rect.right -= m_ArrowRect.Width();
        CBrush brush((!IsWindowEnabled() || m_crColourBk == CLR_DEFAULT) ? ::GetSysColor(COLOR_3DFACE) : m_crColourBk);
        CBrush* pOldBrush = (CBrush*)pDC->SelectObject(&brush);
        pDC->SelectStockObject(NULL_PEN);
        pDC->Rectangle(rect);
        pDC->SelectObject(pOldBrush);
        rect.left--;
        rect.top--;
    }
    //////////////////////////////
    else
    {
        CRect rc;
        GetClientRect(&rc);
        CRect rcBtn(rc.right - ::GetSystemMetrics(SM_CXHTHUMB), rc.top, rc.right, rc.bottom);

        ValidateRect(rcBtn);
        Default();

        CClientDC cDC(this);
        DWORD wStyle = DB_FLAT | DB_DOWN | (IsWindowEnabled() ? DB_ENABLED : DB_DISABLED);
        bool bInToolbar = GetParent()->IsKindOf(RUNTIME_CLASS(CControlBar)) && ((CControlBar*)GetParent())->m_pDockBar != NULL;
        bool bDropped = DefWindowProc(CB_GETDROPPEDSTATE, 0, 0L) != 0;
        bool bFocus = (::GetFocus() == m_hWnd || ::GetParent(::GetFocus()) == m_hWnd) && bInToolbar;

        if (!bInToolbar)
            wStyle |= DB_BORDER;
        if (bFocus || m_MouseMgr.MouseOver())
            wStyle |= DB_OVER;
        if (bDropped)
            wStyle |= DB_OVER | DB_PRESSED;
        CDrawButton(wStyle, rcBtn).Draw(&cDC);

        // Fill remaining area with colour
        rect.right -= m_ArrowRect.Width() - 1;
        CBrush brush((!IsWindowEnabled() || m_crColourBk == CLR_DEFAULT) ? ::GetSysColor(COLOR_3DFACE) : m_crColourBk);
        CBrush* pOldBrush = (CBrush*)pDC->SelectObject(&brush);
        pDC->SelectStockObject(NULL_PEN);
        pDC->Rectangle(rect);
        pDC->SelectObject(pOldBrush);

        CPenDC pen(cDC, ::GetSysColor((bFocus || bDropped || m_MouseMgr.MouseOver()) ? COLOR_HIGHLIGHT : (bInToolbar ? COLOR_WINDOW : COLOR_3DSHADOW)));

        cDC.SelectObject(::GetStockObject(NULL_BRUSH));
        cDC.Rectangle(rc);
        cDC.Rectangle(rect);

        pen.Color(::GetSysColor(IsWindowEnabled() ? COLOR_WINDOW : COLOR_3DFACE));
        rect.DeflateRect(1, 1);
        cDC.Rectangle(rect);
    }
    //////////////////////////////
    // Draw the window text (if any)
    GetWindowText(m_strText);
    if (m_strText.GetLength())
    {
        pDC->SetBkMode(TRANSPARENT);
        if (!IsWindowEnabled())
        {
            rect.OffsetRect(1, 1);
            pDC->SetTextColor(::GetSysColor(COLOR_3DHILIGHT));
            pDC->DrawText(m_strText, rect, DT_CENTER | DT_SINGLELINE | DT_VCENTER);
            rect.OffsetRect(-1, -1);
            pDC->SetTextColor(::GetSysColor(COLOR_3DSHADOW));
            pDC->DrawText(m_strText, rect, DT_CENTER | DT_SINGLELINE | DT_VCENTER);
        }
        else
        {
            pDC->SetTextColor((m_crColourText == CLR_DEFAULT) ? 0 : m_crColourText);
            pDC->DrawText(m_strText, rect, DT_CENTER | DT_SINGLELINE | DT_VCENTER);
        }
    }

    // Draw focus rect
    if (state & 0x0008)
    {
        rect.DeflateRect(1, 1);
        pDC->DrawFocusRect(rect);
    }
}

/////////////////////////////////////////////////////////////////////////////
// CColourPicker overrides

void CColourPicker::PreSubclassWindow()
{
    CButtonXP::PreSubclassWindow();
    SetWindowSize();                     // resize appropriately
}

/////////////////////////////////////////////////////////////////////////////
// CColourPicker attributes

COLORREF CColourPicker::GetColour()
{
    return (m_nSelectionMode == CP_MODE_TEXT) ? GetTextColour() : GetBkColour();
}

void CColourPicker::SetColour(COLORREF crColour)
{
    (m_nSelectionMode == CP_MODE_TEXT) ? SetTextColour(crColour) : SetBkColour(crColour);
}

void CColourPicker::SetBkColour(COLORREF crColourBk)
{
    m_crColourBk = crColourBk;
    if (IsWindow(m_hWnd))
        RedrawWindow();
}

void CColourPicker::SetTextColour(COLORREF crColourText)
{
    m_crColourText = crColourText;
    if (IsWindow(m_hWnd))
        RedrawWindow();
}

void CColourPicker::SetDefaultText(LPCTSTR szDefaultText)
{
    m_strDefaultText = (szDefaultText) ? szDefaultText : _T("");
}

void CColourPicker::SetCustomText(LPCTSTR szCustomText)
{
    m_strCustomText = (szCustomText) ? szCustomText : _T("");
}

/////////////////////////////////////////////////////////////////////////////
// CColourPicker implementation

void CColourPicker::SetWindowSize()
{
    // Get size dimensions of edges
    CSize MarginSize(::GetSystemMetrics(SM_CXEDGE), ::GetSystemMetrics(SM_CYEDGE));

    // Get size of dropdown arrow
    int nArrowWidth = max(::GetSystemMetrics(SM_CXHTHUMB), 5 * MarginSize.cx);
    int nArrowHeight = max(::GetSystemMetrics(SM_CYVTHUMB), 5 * MarginSize.cy);
    CSize ArrowSize(max(nArrowWidth, nArrowHeight), max(nArrowWidth, nArrowHeight));

    // Get window size
    CRect rect;
    GetWindowRect(rect);

    CWnd* pParent = GetParent();
    if (pParent)
        pParent->ScreenToClient(rect);

    // Set window size at least as wide as 2 arrows, and as high as arrow + margins
    int nWidth = max(rect.Width(), 2 * ArrowSize.cx + 2 * MarginSize.cx);
    MoveWindow(rect.left, rect.top, nWidth, ArrowSize.cy + 2 * MarginSize.cy, TRUE);

    // Get the new coords of this window
    GetWindowRect(rect);
    ScreenToClient(rect);

    // Get the rect where the arrow goes, and convert to client coords.
    m_ArrowRect.SetRect(rect.right - ArrowSize.cx - MarginSize.cx, rect.top + MarginSize.cy, rect.right - MarginSize.cx, rect.bottom - MarginSize.cy);
}
