/********************************************************************
*
* Copyright (c) 2002 Sven Wiegand <forum@sven-wiegand.de>
*
* You can use this and modify this in any way you want,
* BUT LEAVE THIS HEADER INTACT.
*
* Redistribution is appreciated.
*
* $Workfile:$
* $Revision: 1.4 $
* $Modtime:$
* $Author: ytkaczyk $
*
* Revision History:
*   $History:$
*
*********************************************************************/

#include "stdafx.h"
#include "PropPageFrameDefault.h"
#include "aeroglss.h"
#include "GdiplusWrapper.h"
using namespace Gdiplus;

namespace TreePropSheet
{
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif

    //-------------------------------------------------------------------
    // class CPropPageFrameDefault
    //-------------------------------------------------------------------

    BEGIN_MESSAGE_MAP(CPropPageFrameDefault, CWnd)
        ON_WM_PAINT()
        ON_WM_ERASEBKGND()
    END_MESSAGE_MAP()


    CPropPageFrameDefault::CPropPageFrameDefault()
    {
    }


    CPropPageFrameDefault::~CPropPageFrameDefault()
    {
        if (m_Images.GetSafeHandle())
            m_Images.DeleteImageList();
    }


    /////////////////////////////////////////////////////////////////////
    // Overridings

    BOOL CPropPageFrameDefault::Create(DWORD dwWindowStyle, const RECT& rect, CWnd* pwndParent, UINT nID)
    {
        return CWnd::Create(AfxRegisterWndClass(CS_HREDRAW | CS_VREDRAW, AfxGetApp()->LoadStandardCursor(IDC_ARROW), GetSysColorBrush(COLOR_3DFACE)), _T("Page Frame"), dwWindowStyle, rect, pwndParent, nID);
    }


    CWnd* CPropPageFrameDefault::GetWnd()
    {
        return static_cast<CWnd*>(this);
    }


    void CPropPageFrameDefault::SetCaption(LPCTSTR lpszCaption, HICON hIcon /*= NULL*/)
    {
        CPropPageFrame::SetCaption(lpszCaption, hIcon);

        // build image list
        if (m_Images.GetSafeHandle())
            m_Images.DeleteImageList();
        if (hIcon)
        {
            ICONINFO ii;
            if (!GetIconInfo(hIcon, &ii))
                return;

            CBitmap bmMask;
            bmMask.Attach(ii.hbmMask);
            if (ii.hbmColor)
                DeleteObject(ii.hbmColor);

            BITMAP bm;
            bmMask.GetBitmap(&bm);

            if (!m_Images.Create(bm.bmWidth, bm.bmHeight, ILC_COLOR32 | ILC_MASK, 0, 1))
                return;

            if (m_Images.Add(hIcon) == -1)
                m_Images.DeleteImageList();
        }
    }


    CRect CPropPageFrameDefault::CalcMsgArea()
    {
        CRect rect;
        GetClientRect(rect);
        CUxThemeAeroImpl g_ThemeLib;
        if (g_ThemeLib.Initialize() && g_ThemeLib.IsThemeActive())
        {
            HTHEME hTheme = g_ThemeLib.OpenThemeData(m_hWnd, L"Tab");
            if (hTheme)
            {
                CRect rectContent;
                CDC* pDc = GetDC();
                g_ThemeLib.GetThemeBackgroundContentRect(hTheme, pDc->m_hDC, TABP_PANE, 0, rect, rectContent);
                ReleaseDC(pDc);
                g_ThemeLib.CloseThemeData(hTheme);

                if (GetShowCaption())
                    rectContent.top = rect.top + GetCaptionHeight() + 1;
                rect = rectContent;
            }
        }
        else if (GetShowCaption())
            rect.top += GetCaptionHeight() + 1;

        return rect;
    }


    CRect CPropPageFrameDefault::CalcCaptionArea()
    {
        CRect rect;
        GetClientRect(rect);
        CUxThemeAeroImpl g_ThemeLib;
        if (g_ThemeLib.Initialize() && g_ThemeLib.IsThemeActive())
        {
            HTHEME hTheme = g_ThemeLib.OpenThemeData(m_hWnd, L"Tab");
            if (hTheme)
            {
                CRect rectContent;
                CDC* pDc = GetDC();
                g_ThemeLib.GetThemeBackgroundContentRect(hTheme, pDc->m_hDC, TABP_PANE, 0, rect, rectContent);
                ReleaseDC(pDc);
                g_ThemeLib.CloseThemeData(hTheme);

                if (GetShowCaption())
                    rectContent.bottom = rect.top + GetCaptionHeight();
                else
                    rectContent.bottom = rectContent.top;

                rect = rectContent;
            }
        }
        else
        {
            if (GetShowCaption())
                rect.bottom = rect.top + GetCaptionHeight();
            else
                rect.bottom = rect.top;
        }

        return rect;
    }

    void CPropPageFrameDefault::DrawCaption(CDC* pDc, CRect rect, LPCTSTR lpszCaption, HICON hIcon)
    {
        COLORREF clrLeft = AfxGetApp()->GetProfileInt(_T("MenuXP"), _T("GradientStart"), RGB(0xB4, 0xC4, 0xE4));
        COLORREF clrRight = AfxGetApp()->GetProfileInt(_T("MenuXP"), _T("GradientEnd"), RGB(0xFF, 0xFF, 0xFF));

        FillGradientRectH(pDc, rect, clrLeft, clrRight);

        // draw icon
        if (hIcon && m_Images.GetSafeHandle() && m_Images.GetImageCount() == 1)
        {
            IMAGEINFO ii;
            m_Images.GetImageInfo(0, &ii);
            CPoint pt(3, rect.CenterPoint().y - (ii.rcImage.bottom - ii.rcImage.top) / 2);
            m_Images.Draw(pDc, 0, pt, ILD_TRANSPARENT);
            rect.left += (ii.rcImage.right - ii.rcImage.left) + 3;
        }

        // draw text
        rect.left += 2;

        COLORREF clrPrev = pDc->SetTextColor(GetSysColor(COLOR_CAPTIONTEXT));
        int nBkStyle = pDc->SetBkMode(TRANSPARENT);
        CFont* pFont = (CFont*)pDc->SelectStockObject(SYSTEM_FONT);

        if (gdiplusWrapper.IsInitialized())
        {
            Font font(_T("Arial"), 15);

            SolidBrush pBrush(GetSysColor(COLOR_CAPTIONTEXT));
            Pen myPen(Color(GetSysColor(COLOR_CAPTIONTEXT)), 1);

            HDC hdc1 = CreateCompatibleDC(pDc->m_hDC);
            HBITMAP bm1 = CreateCompatibleBitmap(pDc->m_hDC, rect.Width(), rect.Height());
            SelectObject(hdc1, bm1);
            BitBlt(hdc1, 0, 0, rect.Width(), rect.Height(), pDc->m_hDC, rect.left, rect.top, SRCCOPY);
            CDC::FromHandle(hdc1)->SetBkMode(TRANSPARENT);
            CDC::FromHandle(hdc1)->DrawText(lpszCaption, CRect(0, 0, rect.Width(), rect.Height()), DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_END_ELLIPSIS);

            Graphics* myGraphics = new Graphics(pDc->m_hDC);
            Bitmap* pBmp = new Bitmap(bm1, NULL);
            myGraphics->DrawImage(pBmp, rect.left, rect.top, rect.Width(), rect.Height());
            delete pBmp;
            delete myGraphics;
        }
        else
            pDc->DrawText(lpszCaption, rect, DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_END_ELLIPSIS);

        pDc->SetTextColor(clrPrev);
        pDc->SetBkMode(nBkStyle);
        pDc->SelectObject(pFont);
    }


    /////////////////////////////////////////////////////////////////////
    // Implementation helpers

    void CPropPageFrameDefault::FillGradientRectH(CDC* pDc, const CRect& rect, COLORREF clrLeft, COLORREF clrRight)
    {
        // pre calculation
        int nSteps = rect.Width();
        int nRRange = GetRValue(clrRight) - GetRValue(clrLeft);
        int nGRange = GetGValue(clrRight) - GetGValue(clrLeft);
        int nBRange = GetBValue(clrRight) - GetBValue(clrLeft);

        double dRStep = (double)nRRange / (double)nSteps;
        double dGStep = (double)nGRange / (double)nSteps;
        double dBStep = (double)nBRange / (double)nSteps;

        double dR = (double)GetRValue(clrLeft);
        double dG = (double)GetGValue(clrLeft);
        double dB = (double)GetBValue(clrLeft);

        HDC hDC = CreateCompatibleDC(pDc->m_hDC);
        HBITMAP bm = CreateCompatibleBitmap(pDc->m_hDC, rect.Width(), rect.Height());
        SelectObject(hDC, bm);

        CPen* pPrevPen = NULL;
        for (int x = 0; x <= rect.Width(); ++x)
        {
            CPen Pen(PS_SOLID, 1, RGB((BYTE)dR, (BYTE)dG, (BYTE)dB));
            pPrevPen = CDC::FromHandle(hDC)->SelectObject(&Pen);
            CDC::FromHandle(hDC)->MoveTo(x, 0);
            CDC::FromHandle(hDC)->LineTo(x, rect.Height());
            CDC::FromHandle(hDC)->SelectObject(pPrevPen);

            dR += dRStep;
            dG += dGStep;
            dB += dBStep;
        }

        if (gdiplusWrapper.IsInitialized())
        {
            HDC hDC2 = CreateCompatibleDC(pDc->m_hDC);
            HBITMAP bm2 = CreateCompatibleBitmap(pDc->m_hDC, rect.Width(), rect.Height());
            SelectObject(hDC2, bm2);

            BitBlt(hDC2, 0, 0, rect.Width(), rect.Height(), hDC, 0, 0, SRCPAINT);

            Graphics* myGraphics = new Graphics(pDc->m_hDC);
            Bitmap* pBmp = new Bitmap(bm2, NULL);
            myGraphics->DrawImage(pBmp, rect.left, rect.top, rect.Width(), rect.Height());
            delete pBmp;
            delete myGraphics;
            DeleteDC(hDC2);
            DeleteObject(bm2);
        }
        else
            pDc->BitBlt(rect.left, rect.top, rect.Width(), rect.Height(), CDC::FromHandle(hDC), 0, 0, SRCCOPY);
    }


    /////////////////////////////////////////////////////////////////////
    // message handlers

    void CPropPageFrameDefault::OnPaint()
    {
        CPaintDC dc(this);
        Draw(&dc);
    }


    BOOL CPropPageFrameDefault::OnEraseBkgnd(CDC* pDC)
    {
        CWnd::OnEraseBkgnd(pDC);
        CDwmApiImpl dwmApi;

        CRect rcClient;
        GetClientRect(rcClient);

        if (dwmApi.Initialize() && dwmApi.IsDwmCompositionEnabled())
        {
            CRect rect;
            GetClientRect(&rect);

            CUxThemeAeroImpl g_ThemeLib;
            if (g_ThemeLib.Initialize())
            {
                HTHEME hTheme = g_ThemeLib.OpenThemeData(m_hWnd, L"Tab");
                if (hTheme)
                {
                    HDC hdcPaint = NULL;
                    CRect rect;
                    GetClientRect(rect);

                    PAINTSTRUCT ps;
                    BeginPaint(&ps);

                    HPAINTBUFFER hBufferedPaint = g_ThemeLib.BeginBufferedPaint(pDC->m_hDC, &rect, BPBF_TOPDOWNDIB, NULL, &hdcPaint);
                    if (hdcPaint)
                    {
                        VERIFY(S_OK == g_ThemeLib.BufferedPaintSetAlpha(hBufferedPaint, &rect, 0x00));
                        VERIFY(PatBlt(hdcPaint, 0, 0, rect.Width(), rect.Height(), BLACKNESS));
                        COLORREF cr = RGB(0x00, 0x00, 0x00);
                        VERIFY(S_OK == g_ThemeLib.GetThemeColor(g_ThemeLib.OpenThemeData(m_hWnd, L"Edit"), EP_BACKGROUNDWITHBORDER, EBWBS_NORMAL, TMT_BORDERCOLOR, &cr));

                        ///
                        /// add the alpha value:
                        ///
                        cr |= 0xff000000;


                        Pen* myPen;
                        Graphics* myGraphics;
                        myPen = new Pen(cr, 1.0);
                        if (myPen)
                        {
                            myPen->SetDashStyle(DashStyleSolid);
                            myGraphics = new Graphics(hdcPaint);
                            if (myGraphics)
                            {
                                myGraphics->DrawRectangle(myPen, rect.left, rect.top, rect.right - 1 - rect.left, rect.bottom - 1 - rect.top);
                                delete myGraphics;
                            }
                            delete myPen;
                        }

                        VERIFY(S_OK == g_ThemeLib.EndBufferedPaint(hBufferedPaint, TRUE));
                    }

                    VERIFY(S_OK == g_ThemeLib.CloseThemeData(hTheme));

                    g_ThemeLib.CloseThemeData(hTheme);
                }
            }
        }
        else
        {
            CUxThemeAeroImpl g_ThemeLib;
            if (g_ThemeLib.Initialize() && g_ThemeLib.IsThemeActive())
            {
                HTHEME hTheme = g_ThemeLib.OpenThemeData(m_hWnd, L"Tab");
                if (hTheme)
                {
                    CRect rect;
                    GetClientRect(rect);
                    g_ThemeLib.DrawThemeBackground(hTheme, pDC->m_hDC, TABP_PANE, 0, rect, NULL);

                    g_ThemeLib.CloseThemeData(hTheme);
                }
            }
        }
        return TRUE;
    }
} //namespace TreePropSheet
