#ifndef NULLSOFT_API_PLAYLIST_H
#define NULLSOFT_API_PLAYLIST_H

#include <bfc/dispatch.h>
#include <bfc/platform/types.h>

enum
{
	PLAYLIST_SUCCESS = 0,
	PLAYLIST_UNIMPLEMENTED = 1,
};
class api_playlist : public Dispatchable
{
protected:
	api_playlist() {}
	~api_playlist() {}
	public:
	DISPATCH_CODES
	{
		API_PLAYLIST_CLEAR = 10,
		//API_PLAYLIST_APPENDWITHINFO = 20,
		//API_PLAYLIST_APPEND = 30,		
		API_PLAYLIST_GETNUMITEMS = 40,
		API_PLAYLIST_GETITEM = 50,
		API_PLAYLIST_GETITEMTITLE = 60,
		API_PLAYLIST_GETITEMLENGTHMILLISECONDS = 70,
		API_PLAYLIST_GETITEMEXTENDEDINFO = 80,
		API_PLAYLIST_REVERSE = 90,
		API_PLAYLIST_SWAP = 100,
		API_PLAYLIST_RANDOMIZE=110,
		API_PLAYLIST_REMOVE=120,
		API_PLAYLIST_SORTBYTITLE=130,
		API_PLAYLIST_SORTBYFILENAME=140,
	};
public:
	void Clear();
	//void AppendWithInfo(const wchar_t *filename, const char *title, int lengthInMS);
	//void Append(const wchar_t *filename);

	size_t GetNumItems();
	size_t GetItem(size_t item, wchar_t *filename, size_t filenameCch);
	size_t GetItemTitle(size_t item, wchar_t *title, size_t titleCch);
	int GetItemLengthMilliseconds(size_t item); // TODO: maybe microsecond for better resolution?
	size_t GetItemExtendedInfo(size_t item, const wchar_t *metadata, wchar_t *info, size_t infoCch);
	int Reverse(); // optional, return 1 to indicate that you did the reversal (otherwise, caller must perform manually)
	int Swap(size_t item1, size_t item2);
	int Randomize(int (*generator)()); // optional, return 1 to indicate that you did the randomization (otherwise, caller must perform manually)
	void Remove(size_t item);
	int SortByTitle(); // optional, return 1 to indicate that you did the sort (otherwise, caller must perform manually)
	int SortByFilename();// optional, return 1 to indicate that you did the sort (otherwise, caller must perform manually)
};

inline void api_playlist::Clear()
{
	_voidcall(API_PLAYLIST_CLEAR);
}
/*
inline void api_playlist::AppendWithInfo(const wchar_t *filename, const char *title, int lengthInMS)
{
	_voidcall(API_PLAYLIST_APPENDWITHINFO, filename, title, lengthInMS);
}
*/
/*
inline void api_playlist::Append(const wchar_t *filename)
{
	_voidcall(API_PLAYLIST_APPEND, filename);
}*/

inline size_t api_playlist::GetNumItems()
{
	return _call(API_PLAYLIST_GETNUMITEMS, (size_t)0);
}

	inline size_t api_playlist::GetItem(size_t item, wchar_t *filename, size_t filenameCch)
	{
		return _call(API_PLAYLIST_GETITEM, (size_t)0, item, filename, filenameCch);
	}
	inline size_t api_playlist::GetItemTitle(size_t item, wchar_t *title, size_t titleCch)
	{
		return _call(API_PLAYLIST_GETITEMTITLE, (size_t)0, item, title, titleCch);
	}
	inline int api_playlist::GetItemLengthMilliseconds(size_t item)
	{
		return _call(API_PLAYLIST_GETITEMLENGTHMILLISECONDS, (int)-1, item);
	}
	inline size_t api_playlist::GetItemExtendedInfo(size_t item, const wchar_t *metadata, wchar_t *info, size_t infoCch)
	{
		return _call(API_PLAYLIST_GETITEMEXTENDEDINFO, (size_t)0, item, metadata, info, infoCch);
	}

	inline int api_playlist::Reverse()
	{
		return _call(API_PLAYLIST_REVERSE, (int)PLAYLIST_UNIMPLEMENTED);
	}

	inline int api_playlist::Swap(size_t item1, size_t item2)
	{
		return _call(API_PLAYLIST_SWAP, (int)PLAYLIST_UNIMPLEMENTED, item1, item2);
	}

	inline int api_playlist::Randomize(int (*generator)())
	{
		return _call(API_PLAYLIST_RANDOMIZE, (int)PLAYLIST_UNIMPLEMENTED, generator);
	}
	inline void api_playlist::Remove(size_t item)
	{
		_voidcall(API_PLAYLIST_REMOVE, item);
	}
	inline int api_playlist::SortByTitle()
	{
		return _call(API_PLAYLIST_SORTBYTITLE, (int)0);
	}
	inline int api_playlist::SortByFilename()
	{
			return _call(API_PLAYLIST_SORTBYFILENAME, (int)0);
	}
#endif