#ifndef NULLSOFT_API_PLAYLISTHANDLER_H
#define NULLSOFT_API_PLAYLISTHANDLER_H


#include <bfc/dispatch.h>
#include "api_playlistloader.h"
enum
{
	
	API_PLAYLISTHANDLER_SUCCESS = 0,
	API_PLAYLISTHANDLER_FAILED = 1,
};
class api_playlisthandler : public Dispatchable
{
protected:
	api_playlisthandler() {}
	~api_playlisthandler() {}

public:
	const wchar_t *EnumerateExtensions(size_t n); // returns 0 when it's done
	const char *EnumerateMIMETypes(size_t n); // returns 0 when it's done, returns char * to match HTTP specs
	const wchar_t *GetName();  // returns a name suitable for display to user of this playlist form (e.g. PLS Playlist)
	int SupportedFilename(const wchar_t *filename); // returns SUCCESS and FAILED, so be careful ...
	int SupportedMIMEType(const char *filename); // returns SUCCESS and FAILED, so be careful ...
	api_playlistloader *CreateLoader(const wchar_t *filename);
	void ReleaseLoader(api_playlistloader *loader);
	//api_playlistwriter CreateWriter(const wchar_t *writer);
	//void ReleaseWriter(api_playlistwriter *writer);

	size_t SniffSizeRequired(); // return number of bytes required for detection on an unknown file
	bool IsOurs(const __int8 *data, size_t sizeBytes); 

	public:
DISPATCH_CODES
{
	API_PLAYLISTHANDLER_ENUMEXTENSIONS = 10,
		API_PLAYLISTHANDLER_ENUMMIMETYPES = 20,
		API_PLAYLISTHANDLER_SUPPORTFILENAME= 30,
		API_PLAYLISTHANDLER_SUPPORTMIME= 40,
		API_PLAYLISTHANDLER_CREATELOADER = 50,
		API_PLAYLISTHANDLER_RELEASELOADER = 60,
		API_PLAYLISTHANDLER_CREATEWRITER= 70,
		API_PLAYLISTHANDLER_RELEASEWRITER= 80,
		API_PLAYLISTHANDLER_SNIFFSIZE=90,
		API_PLAYLISTHANDLER_SNIFF=100,		
			API_PLAYLISTHANDLER_GETNAME=110,		
};

};

inline const wchar_t *api_playlisthandler::GetName()
{
	return _call(API_PLAYLISTHANDLER_GETNAME, (const wchar_t *)0);
}

inline const wchar_t *api_playlisthandler::EnumerateExtensions(size_t n)
{
	return _call(API_PLAYLISTHANDLER_ENUMEXTENSIONS, (const wchar_t *)0, n);
}; 

inline const char *api_playlisthandler::EnumerateMIMETypes(size_t n)
{
	return _call(API_PLAYLISTHANDLER_ENUMMIMETYPES, (const char *)0, n);
}

	inline int api_playlisthandler::SupportedFilename(const wchar_t *filename)
	{
		return _call(API_PLAYLISTHANDLER_SUPPORTFILENAME, (int)API_PLAYLISTHANDLER_FAILED, filename);
	}

	inline int api_playlisthandler::SupportedMIMEType(const char *filename)
	{
			return _call(API_PLAYLISTHANDLER_SUPPORTMIME, (int)API_PLAYLISTHANDLER_FAILED, filename);
	}

	inline api_playlistloader *api_playlisthandler::CreateLoader(const wchar_t *filename)
	{
		return _call(API_PLAYLISTHANDLER_CREATELOADER, (api_playlistloader *)0, filename);
	}

	inline void api_playlisthandler::ReleaseLoader(api_playlistloader *loader)
	{
		_voidcall(API_PLAYLISTHANDLER_RELEASELOADER, loader);
	}

	inline size_t api_playlisthandler::SniffSizeRequired()
	{
		return _call(API_PLAYLISTHANDLER_SNIFFSIZE, (size_t)0);
	}

	inline bool api_playlisthandler::IsOurs(const __int8 *data, size_t sizeBytes)
	{
			return _call(API_PLAYLISTHANDLER_SNIFF, (bool)false, data, sizeBytes);
	}

#endif