#ifndef MAINWND_H
#define MAINWND_H

#include "MenuXP.h"
#include "PPTooltip.h"

#define WM_ICON_NOTIFY WM_USER+1
#define IDM_RECENT_FILES 1200
#define IDM_TRACKS 1240
#define IDM_DRIVES 1210
#define ID_POPUPMENU_PIC 999

#define LBUTTON_CLICK                       2
#define RBUTTON_CLICK                       3   
#define RBUTTON_UP                          4
#define LBUTTON_UP                          5
#define REFRESH_WINDOW                      7
#define AUTOMATION_START                    8
#define DELAY_FROM_TOOLTIP                  9
#define DELAY_FROM_BEBORE_RESTART_TOOLTIP  10
#define WM_DELAY                           11

/////////////////////////////////////////////////////////////////////////////
// CMainWnd frame


struct OpenPlaylistStruct
{
    CString fname;
    BOOL randomFirst;
    bool add;
    bool forceStart;
};

typedef struct _SkinStruct
{
    HICON playIcon;
    HICON pauseIcon;
    HICON stopIcon;
    HICON prevIcon;
    HICON nextIcon;
    HBITMAP buttons;
    HBITMAP buttonsDn;
    CString playIconName;
    CString pauseIconName;
    CString stopIconName;
    CString prevIconName;
    CString nextIconName;
    CString buttonsName;
    CString buttonsDnName;
    COLORREF startColor;
    COLORREF endColor;
    COLORREF border_color;
    COLORREF end_bk;
    COLORREF mid_bk;
    COLORREF start_bk;
    CString toolTipText;
    int theme;
    int effect_bk;
    BYTE granularity;
    BOOL border;
    int custom_border;
    BYTE border_width;
    BYTE border_height;
    BYTE transparency;
    BOOL xpLookNFeel;
} SkinStruct;

template <class T>
struct ThreadData
{
public:
	typedef void (T::*TFunc)();
	HANDLE hEvent;
	T* pThreadObject;
	TFunc pThreadFunc;
	static DWORD _ThreadFunc(ThreadData<T>* pThis)
	{
		//copying data, because after SetEvent is called,
		//the caller thread could restart and delete the
		//local data
		ThreadData<T> td=*pThis;
		SetEvent(td.hEvent);
		((*(td.pThreadObject)).*(td.pThreadFunc))();
		return 0;
	}
};

template <class T>
void CreateMemberThread(T* p,void (T::*func)(), HANDLE* threadHandle)
{
    if(*threadHandle == NULL)
    {
	    ThreadData<T> td;
	    td.pThreadObject=p;
	    td.pThreadFunc=func;
	    td.hEvent=CreateEvent(NULL,0,0,NULL);
	    DWORD Dummy;   //To make win 9x Happy with the lpThreadId param
	    *threadHandle=CreateThread(NULL,NULL,(LPTHREAD_START_ROUTINE)ThreadData<T>::_ThreadFunc,&td,NULL,&Dummy);
	    WaitForSingleObject(td.hEvent,INFINITE);
	    ::CloseHandle(td.hEvent);
        SetThreadPriority(*threadHandle, THREAD_PRIORITY_LOWEST);
    }
}

class CMainWnd : public CFrameWnd
{

public:
    CMainWnd();
    ~CMainWnd();
    void AddRecent(CString fdn);
    void UseSkin(SkinStruct* useSkin);
    void ShowTracksMenu(int MenuType = 0, bool CurAlbOnly = false);
    void Start();
    void Start2();
    bool TryToEnd();
    void UnregHotKeys();
    void SaveThePlaylist();
    void ShowPopup(int MenuType = 0);
    void SendMSNMessage(bool enable, CString category, CString message);
    void RegHotKeys();
    void OpenPlaylist(CString fname, BOOL randomFirst = false, bool add = false, bool forceStart = false);
    void OpenPlaylistEngine(CString fname, BOOL randomFirst = false, bool add = false, bool forceStart = false, bool firstStart = false);
    void OpenFile(CString fname, bool add = false, bool forceStart = false);
    void OpenFolder(CString theDir, BOOL randomFirst = false, bool add = false, bool forceStart = false);
    void OpenFolderEngine(CString theDir, BOOL randomFirst = false, bool add = false, bool forceStart = false);
    void Finish();
    afx_msg void OnPopupMenuOpenUrl();
    UINT MyRandom(bool forw, bool init = false);
    void LoadSkin(CString fname, SkinStruct* loadSkin);
    bool FileExists(CString fname);
    void GetAlbumRange(int* b, int* e, bool force = false, int track = -1);
    void BuildMenu(CMenu* m);
    void FindMP3Files(CString path, bool nodups = false);
    void OpenUrl(CString m_URL, bool Add = false);
    void SaveSettings();
    void DisplayTooltip();
    void DrawGradient(CDC* pDC, COLORREF clrStart, COLORREF clrEnd, LPCRECT pRect);
    void ReloadMenu();

    void InternalOpenEqualizer();
    void InternalSettingsAdvanced();
    void InternalSeek();

    void OpenFolderThread();
    void OpenPlaylistThread();

    CString m_initDir;

    // Plugins
    BOOL m_activateWindowsMessenger;
    CString m_wmTitle;
    int m_wmIcon;
    int m_wmSpeed;
    int m_scrollMode;

    // W2F
    CString m_W2Ffile;
    CString m_W2Fplay;
    CString m_W2Fpause;
    CString m_W2Fstop;
    CString m_W2Fexit;
    BOOL m_W2Foem;
    BOOL m_W2Fdel;
    BOOL m_W2Fenabled;
    BOOL m_W2FFtp;
    CString m_W2FSite;
    CString m_W2FUser;
    CString m_W2FPassword;
    void PerformW2File(bool ex);

    // Automation
    BOOL m_autOnStart;
    BOOL m_autSaveset;
    BOOL m_autSort;
    BOOL m_autRemove;
    BOOL m_autRemovedup;
    BOOL m_autRescan;
    int m_autPeriod;
    BOOL m_autEvery;
    CStringArray m_autDirs;

    // Hotkey
    UINT m_NPKey;
    UINT m_NPMod;
    UINT m_forwKey;
    UINT m_forwMod;
    UINT m_backKey;
    UINT m_backMod;
    UINT m_exitKey;
    UINT m_exitMod;
    UINT m_playKey;
    UINT m_playMod;
    UINT m_stopKey;
    UINT m_stopMod;
    UINT m_prevKey;
    UINT m_prevMod;
    UINT m_nextKey;
    UINT m_nextMod;
    UINT m_vUpKey;
    UINT m_vUpMod;
    UINT m_vDownKey;
    UINT m_vDownMod;
    UINT m_openKey1; 
    UINT m_openMod1; 
    UINT m_openKey2; 
    UINT m_openMod2;
    UINT m_openKey3; 
    UINT m_openMod3;
    UINT m_minibarKey;
    UINT m_minibarMod;
    UINT m_MP3InfoKey;
    UINT m_MP3InfoMod;
    UINT m_playlistKey;
    UINT m_playlistMod;
    UINT m_equalizerKey;
    UINT m_equalizerMod;
    UINT m_repeatKey;
    UINT m_repeatMod;
    UINT m_shuffleKey;
    UINT m_shuffleMod;
    UINT m_albumModeKey;
    UINT m_albumModeMod;
    UINT m_trayIconKey;
    UINT m_trayIconMod;
    UINT m_menuKey;
    UINT m_menuMod;
    UINT m_tracksKey;
    UINT m_tracksMod;
    UINT m_albumKey;
    UINT m_albumMod;
    UINT m_automationKey;
    UINT m_automationMod;

    // Equalizer
    int m_eQOnTop;
    CRect m_equalizerRect;
    HICON m_stpIcon;
    HICON m_stpIconSm;

    BOOL m_autoPlay;
    int m_extPlaylistFormat;
    CRect m_playlistRect;
    BOOL m_playlistOnTop;
    CString m_rNTemplate;

    // PlaylistManager
    CString m_sortTemplate;
    BOOL m_randomFirst;
    BOOL m_shuffleMode;
    UINT m_firstInMyRandom;

    // Seekbar
    BOOL m_showTTT;
    int m_showRemainingTime;
    int m_trackDurI;
    CRect m_seekBarRect;
    BOOL m_dockSD;

    // Video
    CRect m_videoRect;
    BOOL m_fullScreen;

    // Tooltip
    UINT m_show_time;
    UINT m_hide_time;
    UINT m_fade_in;
    UINT m_fade_out;
    DWORD m_animation;

    //WAEmu
    BOOL m_repeat;
    BOOL m_repeatTrack;
    bool m_opening;
    int m_currentPanning; // TODO Remove??
    bool m_created;
    CMenu m_mainMenu;

    // Mouseclicks
    int m_clicks[8];
    BOOL m_noClicks;
    int m_pressAndHold;

    // Input
    int m_gapBtwTracks;

    // General
    CString m_language;
    BOOL m_autoUpdate;
    BOOL m_SMAlbum;
    BOOL m_SMTracks;
    BOOL m_SMPrevNext;
    BOOL m_wAEmu;
    BOOL m_graphicalButtons;
    int m_tpc;
    BOOL m_showGradientBar;

    // Apparence
    BOOL m_showGradient;
    BOOL m_showGradientSel;
    BOOL m_showToolTip;

    // Apparence skin
    BOOL m_randomSkin;
    BOOL m_changeSkinOnEachTrack;
    SkinStruct m_skin;
    CStringArray m_currentSkin;
    int m_currentSkinSel;
private:

    HHOOK hHook;
    CMenu* m_pPopup;
    BOOL m_displayConsole;
    BOOL m_displayConsoleAOT;

    NOTIFYICONDATA m_nid;
    BOOL m_albumStop;
    POINT m_curMousePos;
    bool m_lbd;
    bool m_rbd;
    bool m_iconShown;
    bool m_traySeeking;

    int m_showingMode;
    int m_rndlst[50000];
    int* m_rndpntr;
    bool m_shutdownAfterEnd;
    bool m_exitAfterEnd;
    CString m_recent[10];
    int m_lbs;
    int m_rbs;
    CRect m_consoleRect;
    bool m_startToolTip;
    bool m_firstPass;
    OpenPlaylistStruct m_playlistStruct;
    CString m_prevTitle;
    int m_incCounter;
    CPPToolTip* m_traytip;
    int m_currentVolume;
    CMenu m_consoleMenu;
    HANDLE m_openPlaylistThreadHan;
    HANDLE m_openFolderThreadHan;

protected:
    virtual BOOL PreTranslateMessage(MSG* pMsg);
    afx_msg void OnAutomationPerformNow();
    afx_msg void OnAutomationSetup();
    afx_msg void OnClose();
    afx_msg BOOL OnCopyData(CWnd* pWnd, COPYDATASTRUCT* pCopyDataStruct);
    afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
    afx_msg void OnDrivesMenu(UINT nID);
    afx_msg void OnEndSession(BOOL bEnding);
    afx_msg void OnPopupMenuAbout();
    afx_msg void OnPopupMenuArtistTitle();
    afx_msg void OnPopupMenuDockToActiveWindow();
    afx_msg void OnPopupMenuElapsedTime();
    afx_msg void OnPopupMenuEqualizer();
    afx_msg void OnPopupMenuExit();
    afx_msg void OnPopupMenuFileInfo();
    afx_msg void OnPopupMenuFilename();
    afx_msg void OnPopupMenuOpenFolder();
    afx_msg void OnPopupMenuAddFolder();
    afx_msg void OnPopupMenuOpenRecent(UINT nID);
    afx_msg void OnPopupMenuPlaylistEditor();
    afx_msg void OnPopupMenuRemainingTime();
    afx_msg void OnPopupMenuSettingsAdvanced();
    afx_msg void OnPopupMenuSettingsAfterEndOfThePlaybackExitStp();
    afx_msg void OnPopupMenuSettingsAfterEndOfThePlaybackTurnOffTheComputer();
    afx_msg void OnPopupMenuSettingsRandomlySelectFirstTrack();
    afx_msg void OnPopupMenuSettingsRepeat();
    afx_msg void OnPopupMenuSettingsRepeatTrack();
    afx_msg void OnPopupMenuSettingsShuffleMode();
    afx_msg void OnPopupMenuSettingsSortByName();
    afx_msg void OnPopupMenuSettingsSortByFilename();
    afx_msg void OnPopupMenuSettingsSortByTemplate();
    afx_msg void OnPopupMenuSettingsStopAfterEachAlbum();
    afx_msg void OnPopupMenuShowTotalTrackTime();
    afx_msg void OnPopupMenuSeek();
    afx_msg void OnPopupMenuVideo();
    afx_msg void OnPopupMenuConsole();
    afx_msg void OnPopupMenuTitle();
    afx_msg void OnTimer(UINT nIDEvent);
    afx_msg void OnTrackMenu(UINT nID);
    afx_msg void OnUpdatePopupMenuArtistTitle(CCmdUI* pCmdUI);
    afx_msg void OnUpdatePopupMenuDockToActiveWindow(CCmdUI* pCmdUI);
    afx_msg void OnUpdatePopupMenuElapsedTime(CCmdUI* pCmdUI);
    afx_msg void OnUpdatePopupMenuEqualizer(CCmdUI* pCmdUI);
    afx_msg void OnUpdatePopupMenuFileInfo(CCmdUI* pCmdUI);
    afx_msg void OnUpdatePopupMenuFilename(CCmdUI* pCmdUI);
    afx_msg void OnUpdatePopupMenuPlaylistEditor(CCmdUI* pCmdUI);
    afx_msg void OnUpdatePopupMenuRemainingTime(CCmdUI* pCmdUI);
    afx_msg void OnUpdatePopupMenuSeek(CCmdUI* pCmdUI);
    afx_msg void OnUpdatePopupMenuVideo(CCmdUI* pCmdUI);
    afx_msg void OnUpdatePopupMenuConsole(CCmdUI* pCmdUI);
    afx_msg void OnUpdatePopupMenuSettingsAfterEndOfThePlaybackExitStp(CCmdUI* pCmdUI);
    afx_msg void OnUpdatePopupMenuSettingsAfterEndOfThePlaybackTurnOffComputer(CCmdUI* pCmdUI);
    afx_msg void OnUpdatePopupMenuSettingsRandomlySelectFirstTrack(CCmdUI* pCmdUI);
    afx_msg void OnUpdatePopupMenuSettingsRepeat(CCmdUI* pCmdUI);
    afx_msg void OnUpdatePopupMenuSettingsRepeatTrack(CCmdUI* pCmdUI);
    afx_msg void OnUpdatePopupMenuSettingsShuffleMode(CCmdUI* pCmdUI);
    afx_msg void OnUpdatePopupMenuSettingsStopAfterEachAlbum(CCmdUI* pCmdUI);
    afx_msg void OnUpdatePopupMenuShowTotalTrackTime(CCmdUI* pCmdUI);
    afx_msg void OnUpdatePopupMenuTitle(CCmdUI* pCmdUI);
    LRESULT OnPostPlayerOver(WPARAM wParam, LPARAM lParam);
    static LRESULT CALLBACK HookCallWndProc(int nCode, WPARAM wParam, LPARAM lParam);

    void BuildAlbumMenu(CMenu* m);
    int BuildMenuLevel(CMenu* m, int l, int start, int finish, CString cur);
    void BuildPrevNextMenu(CMenu* m);
    void ClickAction(int action);
    void DrawFragment(DRAWITEMSTRUCT* psI, int frg);
    bool ExtractLevel(CString s, int l, CString* r);
    CString ExtractLevelPath(CString s, int l);
    int GetMaxLevel(UINT start, UINT finish);
    void GoNext();
    void GetPosForMenu(LPPOINT point);
    int GetStartLevel(UINT start, UINT finish);
    void HideTooltip();
    LRESULT IconNotify(WPARAM wParam, LPARAM lParam);
    LRESULT QueryEndSession(WPARAM wParam, LPARAM lParam);
    LRESULT MixControlChange(WPARAM wParam, LPARAM lParam);
    void LoadSkinOld(CString fname, SkinStruct* loadSkin);
    void LoadSkinNew(CString fname, SkinStruct* loadSkin);
    void LoadThePlaylist();
    void OpenRecent(int num);
    CString ReplaceAmpersand(CString s);
    void SaveRect(CString section, CString name, CRect rect);
    CRect GetRect(CString section, CString name);
    bool sup(CString n1, CString n2);
    void PreOpenOperation(bool add);
    void PostOpenOperation(UINT PLLPrev, BOOL randomFirst, bool add, bool forceStart, bool firstStart = false);
    void UpdMBTR();

    afx_msg void OnPopupMenuNext();
    afx_msg void OnPopupMenuOpenFile();
    afx_msg void OnPopupMenuAddFile();
    afx_msg void OnPopupMenuPause();
    afx_msg void OnPopupMenuPlay();
    afx_msg void OnPopupMenuPrevious();
    afx_msg void OnPopupMenuStop();
    virtual LRESULT OnHotKey(WPARAM wParam, LPARAM lParam);
    afx_msg void OnDrawItem(int nIDCtl, LPDRAWITEMSTRUCT lpDrawItemStruct);
    afx_msg void OnInitMenuPopup(CMenu* menu, UINT number, BOOL booleen);
    afx_msg void OnMeasureItem(int nIDCtl, LPMEASUREITEMSTRUCT lpMeasureItemStruct);
    afx_msg LRESULT OnMenuChar(UINT nChar, UINT nFlags, CMenu* pMenu);
    void PostNcDestroy();
    afx_msg void OnContextMenu(CWnd* pWnd, CPoint pos);
    afx_msg void OnConsoleClear();
    afx_msg void OnConsoleAot();
    afx_msg void OnConsoleCopy();
    afx_msg LRESULT OnAreYouMe(WPARAM, LPARAM);

    DECLARE_MESSAGE_MAP()
    DECLARE_DYNCREATE(CMainWnd)
};

#endif

extern CMainWnd s_mainWnd;
