// StandardOutputConfig.cpp : fichier d'implmentation
//

#include "StdAfx.h"
#include "resource.h"       // main symbols
#include "aerosubc.h"
#include "aeroglss.h"
#include "StandardOutputConfig.h"

#include "MainHelper.h"
#include <assert.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


#define DEFAULT_BLOCK_SIZE_INDEX 7
#define DEFAULT_BLOCK_COUNT_INDEX 6
#define WM_UPDATE_DISPLAY_TIMER 101

/*
 * dialog temporary config
 */
static config_t dialog_config;

/*
 * conversion tables for sliders/values
 */ 
static int size_table[] =
{
    65536, 49170, 32768, 24576, 16384, 8192, 6144, 4096, 2048, 1024
};
static int count_table[] =
{
    50, 40, 35, 30, 25, 20, 15, 10, 5, 2
};

/*
 * convert slider value to a block size 
 */
static int index_to_block_size(int value)
{
    if (value < 0)
        return size_table[0];
    if (value > 9)
        return size_table[9];

    return size_table[value];
}

/*
 * convert slider value to block count
 */
static int index_to_block_count(int value)
{
    if (value < 0)
        return count_table[0];
    if (value > 9)
        return count_table[9];

    return count_table[value];
}

/* 
 * convert block count to slider value
 */
static int block_count_to_index(int value)
{
    int i;

    for (i = 0; i < sizeof(count_table) / sizeof(count_table[0]); i++)
    {
        if (value == count_table[i])
            return i;
    }

    return DEFAULT_BLOCK_COUNT_INDEX;
}

/*
 * convert block size to slider value
 */
static int block_size_to_index(int value)
{
    int i;

    for (i = 0; i < sizeof(size_table) / sizeof(size_table[0]); i++)
    {
        if (value == size_table[i])
            return i;
    }

    return DEFAULT_BLOCK_SIZE_INDEX;
} 

CConfigOutputDialog configOutputDialog;

BOOL CALLBACK AfxDlgProc(HWND hWnd, UINT message, WPARAM, LPARAM);

BOOL CALLBACK config_proc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    if (!lParam)
        return FALSE;           // not WM_INITDIALOG yet

    configOutputDialog.Attach(hDlg);
    configOutputDialog.PreSubclassWindow();
    WNDPROC* pOldWndProc = configOutputDialog.GetSuperWndProcAddr();
    WNDPROC afxWndProc = AfxGetAfxWndProc();
    *pOldWndProc = (WNDPROC)SetWindowLong(hDlg, GWL_WNDPROC, (DWORD)afxWndProc);

    // Replace the dialog proc pointer with AfxDlgProc

    SetWindowLong(hDlg, DWL_DLGPROC, (DWORD)AfxDlgProc);

    // Now let MFC process this message

    return CallWindowProc(afxWndProc, hDlg, uMsg, wParam, NULL);
}

IMPLEMENT_DYNAMIC(CConfigOutputDialog, CPropertyPage)

CConfigOutputDialog::CConfigOutputDialog() : CPropertyPage()
{
    timer_slow = 0;
}

CConfigOutputDialog::~CConfigOutputDialog()
{
}

void CConfigOutputDialog::DoDataExchange(CDataExchange* pDX)
{
    CPropertyPage::DoDataExchange(pDX);
    CMenuXP::SetXPLookNFeel(this, CMenuXP::GetXPLookNFeel(GetParent()->GetParent()));
    DDX_Control(pDX, IDC_COMBO_DEVICE, m_comboDevice);
    DDX_Control(pDX, IDC_LABEL_TOTAL, m_labelTotal);
    DDX_Control(pDX, IDC_LABEL_BLOCKS, m_labelBlocks);
    DDX_Control(pDX, IDC_LABEL_WRITTEN, m_labelWritten);
    DDX_Control(pDX, IDC_LABEL_PLAYED, m_labelPlayed);
    DDX_Control(pDX, IDC_LABEL_START, m_labelStart);
    DDX_Control(pDX, IDC_LABEL_LATENCY, m_labelLatency);
    DDX_Control(pDX, IDC_LABEL_FORMAT, m_labelFormat);
    DDX_Control(pDX, IDC_LABEL_CONV, m_labelConv);
    DDX_Control(pDX, IDC_LABEL_SIZE, m_labelSize);
    DDX_Control(pDX, IDC_LABEL_COUNT, m_labelCount);
    DDX_Control(pDX, IDC_PROGRESS_USAGE, m_progressUsage);
    DDX_Control(pDX, IDC_SLIDER_COUNT, m_blockCount);
    DDX_Control(pDX, IDC_SLIDER_SIZE, m_blockSize);
    DDX_Check(pDX, IDC_CHECK_VOLUME, volume_enabled);
    DDX_Check(pDX, IDC_CHECK_BALANCE, reverse_balance);
    DDX_Check(pDX, IDC_CHECK_DOWNMIX, pcmconv_enabled);
}

BOOL CConfigOutputDialog::OnInitDialog()
{
    CPropertyPage::OnInitDialog();

    AeroAutoSubclass(m_hWnd, ASC_SUBCLASS_ALL_CONTROLS, 0L);
    SetProp(m_hWnd, _T("vista"), m_hWnd);

    m_HelperOut.translateDlgItemText(this, IDC_LOUTPUT_DEVICE);
    m_HelperOut.translateDlgItemText(this, IDC_LBUFFERING);
    m_HelperOut.translateDlgItemText(this, IDC_LBLOCK_SIZE);
    m_HelperOut.translateDlgItemText(this, IDC_LBLOCK_COUNT);
    m_HelperOut.translateDlgItemText(this, IDC_LPCM_CONVERSION);
    m_HelperOut.translateDlgItemText(this, IDC_LVOLUME_CONTROL);
    m_HelperOut.translateDlgItemText(this, IDC_CHECK_VOLUME);
    m_HelperOut.translateDlgItemText(this, IDC_CHECK_BALANCE);
    m_HelperOut.translateDlgItemText(this, IDC_CHECK_DOWNMIX);
    m_HelperOut.translateDlgItemText(this, IDC_LTOTAL_USAGE);
    m_HelperOut.translateDlgItemText(this, IDC_LTOTAL);
    m_HelperOut.translateDlgItemText(this, IDC_LBLOCKS);
    m_HelperOut.translateDlgItemText(this, IDC_LWRITTEN);
    m_HelperOut.translateDlgItemText(this, IDC_LPLAYED);
    m_HelperOut.translateDlgItemText(this, IDC_L_START);
    m_HelperOut.translateDlgItemText(this, IDC_LLATENCY);
    m_HelperOut.translateDlgItemText(this, IDC_LFORMAT);
    m_HelperOut.translateDlgItemText(this, IDC_LCONV);

    /*
     * initialise controls and set their respective ranges etc.
     */
    get_config(&dialog_config);

    /*
     * fill the combo with the wave devices
     */
    int i = (int)waveOutGetNumDevs();
    m_comboDevice.AddString(_T("Wave Mapper"));
    for (int j = 0; j < i; j++)
    {
        WAVEOUTCAPS woc;
        waveOutGetDevCaps(j, &woc, sizeof(WAVEOUTCAPS));
        m_comboDevice.AddString(woc.szPname);
    }
    m_comboDevice.SetCurSel(dialog_config.device + 1);

    volume_enabled = dialog_config.volume_enabled;
    reverse_balance = dialog_config.reverse_balance;
    pcmconv_enabled = dialog_config.pcmconv_enabled;

    /*
     * set up controls based on config
     */
    m_comboDevice.SetCurSel(dialog_config.device + 1);
    m_blockSize.SetPos(block_size_to_index(dialog_config.block_size));
    m_blockCount.SetPos(block_count_to_index(dialog_config.block_count));

    m_blockSize.SetRange(0, 9);
    m_blockSize.SetTic(1);
    m_blockSize.SetTicFreq(1);

    m_blockCount.SetRange(0, 9);
    m_blockCount.SetTic(1);
    m_blockCount.SetTicFreq(1);

    CString buf;

    buf.Format(_T("%d kB"), index_to_block_size(m_blockSize.GetPos()) >> 10);
    m_labelSize.SetWindowText(buf);
    buf.Format(_T("%d"), index_to_block_count(m_blockCount.GetPos()));
    m_labelCount.SetWindowText(buf);

    /*
     * begin stats update timer
     */
    timer_slow = 0;
    SetTimer(WM_UPDATE_DISPLAY_TIMER, 10, NULL);

    UpdateData(FALSE);

    return TRUE;
}

BOOL CConfigOutputDialog::OnEraseBkgnd(CDC* pDC)
{
    CDwmApiImpl dwmApi;
    if (dwmApi.Initialize() && dwmApi.IsDwmCompositionEnabled())
    {
        CRect rcClient;
        GetClientRect(&rcClient);

        VERIFY(pDC->PatBlt(0, 0, rcClient.Width(), rcClient.Height(), BLACKNESS));
        return TRUE;
    }
    return CPropertyPage::OnEraseBkgnd(pDC);
}

BEGIN_MESSAGE_MAP(CConfigOutputDialog, CPropertyPage)
  ON_WM_ERASEBKGND()
  ON_WM_TIMER()
  ON_WM_VSCROLL()
END_MESSAGE_MAP()

WNDPROC* CConfigOutputDialog::GetSuperWndProcAddr()
{
    return &m_wndproc;
}

void CConfigOutputDialog::OnOK()
{
    UpdateData(TRUE);

    /*
     * grab some various control states and save them back to our config struct
     */
    dialog_config.volume_enabled = volume_enabled;
    dialog_config.reverse_balance = reverse_balance;
    dialog_config.device = m_comboDevice.GetCurSel() - 1;
    dialog_config.pcmconv_enabled = pcmconv_enabled;
    dialog_config.block_size = index_to_block_size(m_blockSize.GetPos());
    dialog_config.block_count = index_to_block_count(m_blockCount.GetPos());

    /*
     * push the config struct back at the plugin. changes are updated when playback is next
     * started/restarted.
     */
    set_config(&dialog_config);
    config_save(&dialog_config);

    CPropertyPage::OnOK();
}

void CConfigOutputDialog::OnTimer(UINT nIDEvent)
{
    CString buffer;

    get_statistics(&stats);

    buffer.Format(_T("%d kB (%d x %d kB)"), stats.total_buffer_size / 1024, stats.block_count, stats.block_size / 1024);
    m_labelTotal.SetWindowText(buffer);
    buffer.Format(_T("%d/%d"), stats.block_usage, stats.block_count);
    m_labelBlocks.SetWindowText(buffer);

    if (stats.active)
    {
        buffer.Format(_T("%d"), stats.bytes_written);
        m_labelWritten.SetWindowText(buffer);
        buffer.Format(_T("%d"), stats.bytes_played);
        m_labelPlayed.SetWindowText(buffer);
        buffer.Format(_T("%d ms"), stats.start_position);
        m_labelStart.SetWindowText(buffer);
        buffer.Format(_T("%d ms"), stats.latency);
        m_labelLatency.SetWindowText(buffer);
        buffer.Format(_T("%d Hz, %dbit, %dch"), stats.format.samplerate, stats.format.bitspersamp, stats.format.numchannels);
        m_labelFormat.SetWindowText(buffer);
        m_labelConv.SetWindowText((stats.format.conv ? _T("Active") : _T("Inactive")));
        m_progressUsage.SetRange32(0, stats.total_buffer_size);
        m_progressUsage.SetPos(stats.total_buffer_usage);

        if (timer_slow)
        {
            KillTimer(WM_UPDATE_DISPLAY_TIMER);
            SetTimer(WM_UPDATE_DISPLAY_TIMER, 10, NULL);
            timer_slow = 0;
        }
    }
    else
    {
        m_labelWritten.SetWindowText(_T("0"));
        m_labelPlayed.SetWindowText(_T("0"));
        m_labelStart.SetWindowText(_T("0 ms"));
        m_labelLatency.SetWindowText(_T("(Inactive)"));
        m_labelFormat.SetWindowText(_T("(Inactive)"));
        m_labelConv.SetWindowText(_T("(Inactive)"));
        m_progressUsage.SetPos(0);

        if (!timer_slow)
        {
            KillTimer(WM_UPDATE_DISPLAY_TIMER);
            SetTimer(WM_UPDATE_DISPLAY_TIMER, 10, NULL);
            timer_slow = 1;
        }
    }
}

void CConfigOutputDialog::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
    CString buf;
    if (pScrollBar == reinterpret_cast<CScrollBar*>(&m_blockSize))
    {
        buf.Format(_T("%d kB"), index_to_block_size(m_blockSize.GetPos()) >> 10);
        m_labelSize.SetWindowText(buf);
    }
    else if (pScrollBar == reinterpret_cast<CScrollBar*>(&m_blockCount))
    {
        buf.Format(_T("%d"), index_to_block_count(m_blockCount.GetPos()));
        m_labelCount.SetWindowText(buf);
    }
}