// PlaylistDlg.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"       // main symbols
#include "aerosubc.h"
#include "wa_ipc.h"
#include "SysTrayPlay.h"
#include "PlaylistDlg.h"
#include "MainWnd.h"
#include "TagInfo.h"
#include "MP3InfoDlg.h"
#include "RenameDlg.h"
#include "MainHelper.h"
#include "PlaylistManager.h"
#include "MultiDllInput.h"
#include "WAEmuWnd.h"
#include "SortDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static int __stdcall BrowseCallbackProc(HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData)
{
    CString dir;
    dir = s_mainWnd.m_initDir;
    if (dir.Right(1) == "\\")
        dir = dir.Left(dir.GetLength() - 1);

    if (uMsg == BFFM_INITIALIZED)
        ::SendMessage(hwnd, BFFM_SETSELECTION, TRUE, (LPARAM)LPCTSTR(dir));
    return 0;
}

void CSrchEdit::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
    switch (nChar)
    {
        case VK_UP:
        case VK_DOWN:
        case VK_PRIOR:
        case VK_NEXT:
        case VK_HOME:
        case VK_END:
            s_playlistDlg.GetDlgItem(IDC_PLAYLIST_AT)->SetFocus();
            s_playlistDlg.GetDlgItem(IDC_PLAYLIST_AT)->PostMessage(WM_KEYDOWN, nChar, nRepCnt + 0x10000 * nFlags);
            return;
            break;
    }
    CEdit::OnKeyDown(nChar, nRepCnt, nFlags);
}

/////////////////////////////////////////////////////////////////////////////
// CPlaylistDlg dialog


CPlaylistDlg::CPlaylistDlg() : CDialog(CPlaylistDlg::IDD)
{
    m_canResize = FALSE;
    m_removeneThreadHan = NULL;
    m_removeDuplicateEntriesThreadHan = NULL;
    m_sortThreadHan = NULL;
    m_cachedPlaylistTags = new CSortedArray<CachedTag, CachedTag&>();
}

CPlaylistDlg::~CPlaylistDlg()
{
    DestroyAcceleratorTable(m_accel);
    m_menu.DestroyMenu();
    delete m_cachedPlaylistTags;
}

BEGIN_MESSAGE_MAP(CPlaylistDlg, CDialog)
    ON_WM_CLOSE()
    ON_COMMAND(PLAYLIST_STOP_ALL, OnStopAll)
    ON_COMMAND(ID_FILE_ADDFILE, OnFileAddFile)
    ON_COMMAND(ID_FILE_ADDFOLDER, OnFileAddFolder)
    ON_COMMAND(ID_FILE_EDITTAG, OnFileEditTag)
    ON_COMMAND(ID_FILE_REMOVE, OnFileRemove)
    ON_COMMAND(ID_PLAYLIST_OPEN, OnPlaylistOpen)
    ON_COMMAND(ID_PLAYLIST_SAVE, OnPlaylistSave)
    ON_COMMAND(ID_ARRANGE_SHUFFLE, OnArrangeShuffle)
    ON_COMMAND(ID_ARRANGE_SORT, OnArrangeSort)
    ON_COMMAND(ID_ARRANGE_SORTBYFILENAME, OnArrangeSortByFilename)
    ON_COMMAND(ID_ARRANGE_SORTBYTEMPLATE, OnArrangeSortByTemplate)
    ON_COMMAND(ID_TRACKS_SELECTION_ADDTOEXISTENTPLAYLIST, OnTracksSelectionAddToExistentPlaylist)
    ON_COMMAND(ID_TRACKS_SELECTION_CREATENEWPLAYLIST, OnTracksSelectionCreateNewPlaylist)
    ON_COMMAND(ID_TRACKS_ADDPLAYLIST, OnTracksAddPlaylist)
    ON_LBN_KILLFOCUS(IDC_PLAYLIST_AT, OnKillFocusPlaylistAt)
    ON_COMMAND(ID_ARRANGE_REMOVENE, OnArrangeRemovene)
    ON_COMMAND(ID_ARRANGE_REMOVEDUPLICATEENTRIES, OnArrangeRemoveDuplicateEntries)
    ON_EN_CHANGE(IDC_SEARCH, OnChangeSearch)
    ON_COMMAND(ID_TRACKS_REMOVEALL, OnTracksRemoveAll)
    ON_WM_SIZE()
    ON_WM_SHOWWINDOW()
    ON_COMMAND(ID_POPUP_SELECTALL, OnPopupSelectAll)
    ON_COMMAND(ID_TRACKS_REREADALLTAGS, OnTracksReReadAllTags)
    ON_COMMAND(ID_TRACKS_RENAMEFILES, OnTracksRenameFiles)
    ON_COMMAND(ID_TRACKS_FILEMANAGEMENT_DELETE, OnTracksFileManagementDelete)
    ON_COMMAND(ID_TRACKS_FILEMANAGEMENT_COPYTO, OnTracksFileManagementCopyTo)
    ON_COMMAND(ID_TRACKS_FILEMANAGEMENT_MOVETO, OnTracksFileManagementMoveTo)
    ON_COMMAND(ID_FILE_SAVEINEXTENDEDFORMAT, OnFileSaveInExtendedFormat)
    ON_COMMAND(ID_TRACKS_REFRESH, OnTrackRefresh)
    ON_WM_ENTERMENULOOP()
    ON_WM_EXITMENULOOP()
    ON_WM_DROPFILES()
    ON_WM_SYSCOMMAND()
    ON_COMMAND_RANGE(IDM_TRACKS,IDM_TRACKS+49999,OnTrackMenu)
    ON_WM_INITMENUPOPUP()
    ON_MENUXP_MESSAGES()
END_MESSAGE_MAP()

BEGIN_MESSAGE_MAP(CPLLB,CListBox)
    ON_WM_MOUSEMOVE()
    ON_WM_LBUTTONDOWN()
    ON_WM_RBUTTONDOWN()
    ON_WM_RBUTTONUP()
    ON_WM_LBUTTONDBLCLK()
    ON_WM_KEYDOWN()
    ON_WM_KEYUP()
    ON_WM_PAINT()
    ON_WM_ERASEBKGND( )
END_MESSAGE_MAP()

BEGIN_MESSAGE_MAP(CSrchEdit,CEdit)
    ON_WM_KEYDOWN()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPlaylistDlg message handlers

void CPlaylistDlg::DoDataExchange(CDataExchange* pDX)
{
    CMenuXP::SetXPLookNFeel(this, s_mainWnd.m_skin.xpLookNFeel);
    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_FILENAME, m_filename);
    DDX_Control(pDX, IDC_PLAYLIST_AT, m_playlistAt);
    DDX_Control(pDX, IDC_SEARCH, m_search);
}

void CPlaylistDlg::OnStopAll()
{
    StopRemoveDuplicateEntries();
    StopRemovene();
    StopSort();
    ::SendMessage(m_hWnd, WM_COMMAND, ID_TRACKS_REFRESH, 0);
}

void CPlaylistDlg::FillPlaylist()
{
    m_playlistAt.SetRedraw(FALSE);
    CString buf, buf2;
    GetDlgItem(IDC_SEARCH)->GetWindowText(buf);
    buf.MakeLower();
    while (m_playlistAt.GetCount() > 0)
        m_playlistAt.DeleteString(0);

    m_cachedPlaylistTags->RemoveAll();

    for (UINT i = 0; i < (UINT)s_playlistManager.GetSize(); i++)
    {
        m_cachedPlaylistTags->Add(s_playlistManager.Get(i));
        buf2 = m_cachedPlaylistTags->GetAt(i).GetPlaylist();
        buf2.MakeLower();
        if (buf2.Find(buf) != -1)
        {
            CString tmp;
            tmp.Format(_T("%04d"), i);
            m_playlistAt.AddString(tmp);
        }
    }
    ShowFileName();
    m_playlistAt.SetRedraw(TRUE);
    m_playlistAt.Invalidate();
}

void CPlaylistDlg::OnArrangeShuffle()
{
    if (m_playlistAt.GetCount() < 2)
        return;

    //Dsactivation du rafraichissement
    if (IsWindowVisible())
        m_playlistAt.SetRedraw(FALSE);

    int i, j, k;
    s_mainWnd.m_sortTemplate = "%7";
    for (i = 0; i < (UINT)s_playlistManager.GetSize() * 1.5 + 1; i++)
    {
        j = rand() % s_playlistManager.GetSize();
        k = rand() % s_playlistManager.GetSize();
        CachedTag tmp = s_playlistManager.Get(k);
        s_playlistManager.Add(s_playlistManager.Get(j), k);
        s_playlistManager.Add(tmp, j);
        if (s_playlistManager.GetCurrent() == j)
            s_playlistManager.SetCurrent(k);
        else if (s_playlistManager.GetCurrent() == k)
            s_playlistManager.SetCurrent(j);
    }

    ::SendMessage(m_hWnd, WM_COMMAND, ID_TRACKS_REFRESH, 0);
}

void CPlaylistDlg::RemoveneThread()
{
    //Dsactivation du rafraichissement
    if (IsWindowVisible())
        m_playlistAt.SetRedraw(FALSE);

    s_playlistManager.Removene();

    SendMessage(WM_COMMAND, ID_TRACKS_REFRESH, 0);
    m_removeneThreadHan = NULL;
}

bool CPlaylistDlg::StopRemovene()
{
    bool toret = (m_removeneThreadHan != NULL);
    if (toret)
    {
        ::TerminateThread(m_removeneThreadHan, 0);
        m_removeneThreadHan = NULL;

        ::SendMessage(m_hWnd, WM_COMMAND, ID_TRACKS_REFRESH, 0);
    }
    return toret;
}

void CPlaylistDlg::OnArrangeRemovene()
{
    CreateMemberThread<CPlaylistDlg>(&s_playlistDlg, &CPlaylistDlg::RemoveneThread, &m_removeneThreadHan);
}

void CPlaylistDlg::RemoveDuplicateEntriesThread()
{
    TRACE(_T("RemoveDuplicateEntriesThread"));

    //Dsactivation du rafraichissement
    if (IsWindowVisible())
        m_playlistAt.SetRedraw(FALSE);

    s_playlistManager.RemoveDuplicateEntries();

    ::SendMessage(s_playlistDlg.m_hWnd, WM_COMMAND, ID_TRACKS_REFRESH, 0);
    m_removeDuplicateEntriesThreadHan = NULL;
}

bool CPlaylistDlg::StopRemoveDuplicateEntries()
{
    bool toret = (m_removeDuplicateEntriesThreadHan != NULL);
    if (toret)
    {
        ::TerminateThread(m_removeDuplicateEntriesThreadHan, 0);
        m_removeDuplicateEntriesThreadHan = NULL;

        ::SendMessage(m_hWnd, WM_COMMAND, ID_TRACKS_REFRESH, 0);
    }
    return toret;
}

void CPlaylistDlg::OnArrangeRemoveDuplicateEntries()
{
    CreateMemberThread<CPlaylistDlg>(&s_playlistDlg, &CPlaylistDlg::RemoveDuplicateEntriesThread, &m_removeneThreadHan);
}

bool CPlaylistDlg::StopSort()
{
    bool toret = (m_sortThreadHan != NULL);
    if (toret)
    {
        ::TerminateThread(m_sortThreadHan, 0);
        m_sortThreadHan = NULL;

        ::SendMessage(m_hWnd, WM_COMMAND, ID_TRACKS_REFRESH, 0);
    }
    return toret;
}

void CPlaylistDlg::SortThread()
{
    //Dsactivation du rafraichissement
    if (IsWindowVisible())
        m_playlistAt.SetRedraw(FALSE);

    if (s_playlistManager.GetSize() == 0)
    {
        m_sortThreadHan = NULL;
        return;
    }
    s_playlistManager.QuickSort(s_mainWnd.m_sortTemplate);

    ::SendMessage(s_playlistDlg.m_hWnd, WM_COMMAND, ID_TRACKS_REFRESH, 0);
    m_sortThreadHan = NULL;
}

void CPlaylistDlg::OnArrangeSort()
{
    OnArrangeSortEngine();
}
void CPlaylistDlg::OnArrangeSortEngine(CString sortTemplate)
{
    s_mainWnd.m_sortTemplate = sortTemplate;
    CreateMemberThread<CPlaylistDlg>(this, &CPlaylistDlg::SortThread, &m_sortThreadHan);
}
void CPlaylistDlg::OnArrangeSortByFilename()
{
    OnArrangeSortEngine(_T("%8\\%9\\%7"));
}
void CPlaylistDlg::OnArrangeSortByTemplate()
{
    CSortDlg sortDlg;
    sortDlg.DoModal();
}

void CPlaylistDlg::OnChangeSearch()
{
    ::SendMessage(m_hWnd, WM_COMMAND, ID_TRACKS_REFRESH, 0);
}

BOOL CPlaylistDlg::DestroyWindow()
{
    Save();
    return CDialog::DestroyWindow();
}

void CPlaylistDlg::OnDrawItem(int nIDCtl, LPDRAWITEMSTRUCT lpDrawItemStruct)
{
    if (!CMenuXP::OnDrawItem(lpDrawItemStruct, m_hWnd) || !s_mainWnd.m_skin.xpLookNFeel)
        CDialog::OnDrawItem(nIDCtl, lpDrawItemStruct);
}                                                                              

void CPlaylistDlg::OnDropFiles(HDROP hDropInfo)
{
    UINT PLLPrev = (UINT)s_playlistManager.GetSize();

    TCHAR szFileName[_MAX_PATH];
    CString buf;
    UINT nFileCount = ::DragQueryFile(hDropInfo, 0xFFFFFFFF, NULL, 0);
    for (UINT i = 0; i < nFileCount; i++)
    {
        ::DragQueryFile(hDropInfo, i, szFileName, _MAX_PATH);
        buf = szFileName;
        if (::GetFileAttributes(szFileName) & FILE_ATTRIBUTE_DIRECTORY)
        {
            if (buf[buf.GetLength() - 1] != '\\')
                buf += "\\";
            s_mainWnd.FindMP3Files(buf);
        }
        else
        {
            if ((buf.Right(4)).CompareNoCase(_T(".M3U")) == 0 || (buf.Right(4)).CompareNoCase(_T(".PLS")) == 0)
                s_mainWnd.OpenPlaylist(buf, false, false);
            else
            {
                s_playlistManager.Add(buf);
                ::SendMessage(m_hWnd, WM_COMMAND, ID_TRACKS_REFRESH, 0);
            }
        }
    }

    s_playlistManager.QuickSort(s_mainWnd.m_sortTemplate);

    if (s_playlistManager.GetSize() > 0)
    {
        if (PLLPrev == 0)
        {
            if (stdIn.GetPlayerMode() != PLAYER_MODE_CLOSE)
                ::SendMessage(WAEmuWnd.m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
            if (s_mainWnd.m_shuffleMode)
            {
                s_mainWnd.m_firstInMyRandom = s_mainWnd.MyRandom(true, true);
                s_playlistManager.SetCurrent(s_mainWnd.m_firstInMyRandom);
            }
            else
            {
                s_playlistManager.SetCurrent(0);
                s_mainWnd.m_firstInMyRandom = 0xFFFFFFFF;
            }
            if (s_playlistManager.GetCurrent() < s_playlistManager.GetSize() - 1)
                s_playlistManager.incCurrent();
            else
            {
                s_playlistManager.SetCurrent(0);
            }
            if (s_mainWnd.m_autoPlay)
            {
                enqueueFileWithMetaStructW eFile;
                eFile.filename = (TCHAR *)s_playlistManager.GetPlaylist(s_playlistManager.GetCurrent()).GetString();
                ::SendMessage(WAEmuWnd.m_hWnd, IPC_PLAYFILEW, 0, (WPARAM)&eFile);
            }
            else if (stdIn.GetPlayerMode() != PLAYER_MODE_CLOSE)
                ::SendMessage(WAEmuWnd.m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
        }
        s_playlistManager.ReadTagAT(s_playlistManager.GetCurrent());
    }
    ::SendMessage(m_hWnd, WM_COMMAND, ID_TRACKS_REFRESH, 0);

    ::DragFinish(hDropInfo);
}

void CPlaylistDlg::OnEnterMenuLoop(BOOL bIsTrackPopupMenu)
{
    if (!bIsTrackPopupMenu)
    {
        HMENU tmp = GetMenu()->GetSubMenu(3)->m_hMenu;
        while (::GetMenuItemCount(tmp) > 0)
        {
            ::DeleteMenu(tmp, 0, MF_BYPOSITION);
        }
        try
        {
            s_mainWnd.BuildMenu(CMenu::FromHandle(tmp));
        }
        catch (...)
        {
            while (::GetMenuItemCount(tmp) > 0)
            {
                ::DeleteMenu(tmp, 0, MF_BYPOSITION);
            }
            ::AppendMenu(tmp, MF_STRING, 0, _T("Erreur"));
        }
    }
    CDialog::OnEnterMenuLoop(bIsTrackPopupMenu);
}

void CPlaylistDlg::OnExitMenuLoop(BOOL bIsTrackPopupMenu)
{
    if (!bIsTrackPopupMenu)
    {
        HMENU tmp = GetMenu()->GetSubMenu(3)->m_hMenu;
        while (::GetMenuItemCount(tmp) > 0)
        {
            ::DeleteMenu(tmp, 0, MF_BYPOSITION);
        }
    }
    CDialog::OnExitMenuLoop(bIsTrackPopupMenu);
}

void CPlaylistDlg::OnFileAddFile()
{
    UINT PLLPrev = s_playlistManager.GetSize();
    // open a file
    CFileDialog dialog(TRUE, _T("mp3"), _T(""), OFN_ALLOWMULTISELECT | OFN_FILEMUSTEXIST, stdIn.GetFileExtensionsFilter(), this);

    TCHAR buffer[_MAX_PATH*500 + 1];
    buffer[0] = '\0';
    dialog.m_ofn.lpstrFile = buffer;
    dialog.m_ofn.nMaxFile = _MAX_PATH * 500;
    dialog.m_ofn.lpstrInitialDir = LPCTSTR(s_mainWnd.m_initDir);
    int ret;
    ret = dialog.DoModal();
    if (ret == IDOK)
    {
        // a file was selected
        POSITION p = dialog.GetStartPosition();
        CString buf;
        while (p != NULL)
        {
            s_playlistManager.Add(dialog.GetNextPathName(p));
            s_mainWnd.m_initDir = buf.Left(buf.ReverseFind('\\') + 1);
        }
        ::SendMessage(m_hWnd, WM_COMMAND, ID_TRACKS_REFRESH, 0);
        if (PLLPrev == 0)
        {
            if (stdIn.GetPlayerMode() != PLAYER_MODE_CLOSE)
                ::SendMessage(WAEmuWnd.m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
            if (s_mainWnd.m_shuffleMode)
            {
                s_mainWnd.m_firstInMyRandom = s_mainWnd.MyRandom(true, true);
                s_playlistManager.SetCurrent(s_mainWnd.m_firstInMyRandom);
            }
            else
            {
                s_playlistManager.SetCurrent(0);
                s_mainWnd.m_firstInMyRandom = 0xFFFFFFFF;
            }
            if (s_playlistManager.GetCurrent() < s_playlistManager.GetSize() - 1)
                s_playlistManager.incCurrent();
            else
            {
                s_playlistManager.SetCurrent(0);
            }
            if (s_mainWnd.m_autoPlay)
            {
                enqueueFileWithMetaStructW eFile;
                eFile.filename = (TCHAR *)s_playlistManager.GetPlaylist(s_playlistManager.GetCurrent()).GetString();
                ::SendMessage(WAEmuWnd.m_hWnd, IPC_PLAYFILEW, 0, (WPARAM)&eFile);
            }
            else if (stdIn.GetPlayerMode() != PLAYER_MODE_CLOSE)
                ::SendMessage(WAEmuWnd.m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
        }
        ShowFileName();
    }
    m_playlistAt.SetFocus();
}

void CPlaylistDlg::OnFileAddFolder()
{
    LPMALLOC pMalloc;
    if (SHGetMalloc(&pMalloc) != NOERROR)
        return;

    BROWSEINFO bi;
    TCHAR buf[MAX_PATH];
    _tcscpy(buf, s_mainWnd.m_initDir.GetString());

    CString title;
    title = m_Helper.MyLoadString(IDS_OPENFOLDER);

    bi.hwndOwner = m_hWnd;
    bi.pszDisplayName = buf;
    bi.pidlRoot = NULL;
    bi.lpszTitle = LPCTSTR(title);
    bi.ulFlags = BIF_RETURNONLYFSDIRS;
    bi.lpfn = BrowseCallbackProc;

    LPITEMIDLIST pidl;
    if ((pidl = SHBrowseForFolder(&bi)) == NULL)
    {
        pMalloc->Free(pidl);
        pMalloc ->Release();
        return;
    }
    if (SHGetPathFromIDList(pidl, buf) == false)
    {
        pMalloc->Free(pidl);
        pMalloc ->Release();
        return;
    }
    pMalloc->Free(pidl);
    pMalloc->Release();

    //Dsactivation du rafraichissement
    if (IsWindowVisible())
        m_playlistAt.SetRedraw(FALSE);

    s_mainWnd.OpenFolder(buf, s_mainWnd.m_randomFirst, true);
}

extern const char* GenreList[];

void CPlaylistDlg::OnFileEditTag()
{
    int i = -1;
    CString buf;
    while ((i = m_playlistAt.GetFirstSelected(i + 1)) > -1)
    {
        int j = -1;
        CString tmp;
        m_playlistAt.GetText(i, tmp);
        j = _wtoi((LPCTSTR)tmp);
        if (!s_mainWnd.FileExists(s_playlistManager.GetPlaylist(j)))
            continue;
        CMP3InfoDlg TagEditor(s_playlistManager.GetPlaylist(j), TRUE);
        if (TagEditor.DoModal() == IDOK)
        {
            s_playlistManager.Add(TagEditor.m_filename, j);
            s_playlistManager.ReadTagAT(j, true);

            m_playlistAt.RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW);
        }
        else
            break;
    }
    SetFocus();
}

void CPlaylistDlg::OnFileRemove()
{
    Remove();
}

void CPlaylistDlg::OnFileSaveInExtendedFormat()
{
    if (s_mainWnd.m_extPlaylistFormat)
        s_mainWnd.m_extPlaylistFormat = 0;
    else
        s_mainWnd.m_extPlaylistFormat = 1;
    GetMenu()->CheckMenuItem(ID_FILE_SAVEINEXTENDEDFORMAT, MF_BYCOMMAND | (s_mainWnd.m_extPlaylistFormat ? MF_CHECKED : MF_UNCHECKED));
}

void CPlaylistDlg::OnKillFocusPlaylistAt()
{
    m_playlistAt.m_ctrlDown = false;
    m_playlistAt.m_shiftDown = false;
    m_playlistAt.m_prev = -1;
}

void CPlaylistDlg::ReloadMenu()
{
    if(m_menu.m_hMenu !=NULL)
    {
        SetMenu(NULL);
        m_menu.DestroyMenu();
    }
    m_menu.LoadMenu(IDR_PLMENU);
    m_Helper.translateMenu(&m_menu, IDR_PLMENU);
    SetMenu(&m_menu);
}
BOOL CPlaylistDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    ReloadMenu();
    AeroAutoSubclass(m_hWnd, ASC_SUBCLASS_ALL_CONTROLS, 0L);

    m_Helper.translateWindowText(this, IDD_PLAYLIST);

    SetIcon(s_mainWnd.m_stpIcon, TRUE);
    SetIcon(s_mainWnd.m_stpIconSm, FALSE);

    m_accel = LoadAccelerators(theApp.m_hInstance, MAKEINTRESOURCE(IDR_PLACCEL));

    CRect workAreaRect, windowRect;

    workAreaRect.left = 0;
    workAreaRect.right = GetSystemMetrics(SM_CXVIRTUALSCREEN);
    workAreaRect.top = 0;
    workAreaRect.bottom = GetSystemMetrics(SM_CYVIRTUALSCREEN);

    GetWindowRect(&windowRect);
    if (s_mainWnd.m_playlistRect.right == -1000)
        s_mainWnd.m_playlistRect.right = windowRect.Width();
    if (s_mainWnd.m_playlistRect.bottom == -1000)
        s_mainWnd.m_playlistRect.bottom = windowRect.Height();
    if (s_mainWnd.m_playlistRect.left == -1000)
        s_mainWnd.m_playlistRect.left = (workAreaRect.right - s_mainWnd.m_playlistRect.right) / 2;
    if (s_mainWnd.m_playlistRect.top == -1000)
        s_mainWnd.m_playlistRect.top = (workAreaRect.bottom - s_mainWnd.m_playlistRect.bottom) / 2;
    if (s_mainWnd.m_playlistRect.left < 0)
        s_mainWnd.m_playlistRect.left = 0;
    if (s_mainWnd.m_playlistRect.top < 0)
        s_mainWnd.m_playlistRect.top = 0;
    if (s_mainWnd.m_playlistRect.left + s_mainWnd.m_playlistRect.right > workAreaRect.right)
        s_mainWnd.m_playlistRect.left = workAreaRect.right - s_mainWnd.m_playlistRect.right;
    if (s_mainWnd.m_playlistRect.top + s_mainWnd.m_playlistRect.bottom > workAreaRect.bottom)
        s_mainWnd.m_playlistRect.top = workAreaRect.bottom - s_mainWnd.m_playlistRect.bottom;
    SetWindowPos(NULL, s_mainWnd.m_playlistRect.left, s_mainWnd.m_playlistRect.top, s_mainWnd.m_playlistRect.right, s_mainWnd.m_playlistRect.bottom, SWP_NOZORDER | SWP_NOACTIVATE);

    ResizeWindow();

    CMenu* pSysMenu = GetSystemMenu(FALSE);
    if (pSysMenu != NULL)
    {
        pSysMenu->AppendMenu(MF_SEPARATOR);
        pSysMenu->AppendMenu(MF_STRING, 9999, m_Helper.MyLoadString(IDS_ALWAYSONTOP));
    }
    if (s_mainWnd.m_playlistOnTop)
    {
        pSysMenu->CheckMenuItem(9999, MF_BYCOMMAND | MF_CHECKED);
        SetWindowPos(&wndTopMost, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE | SWP_NOACTIVATE);
    }
    else
        ModifyStyleEx(0, WS_EX_APPWINDOW);

    FillPlaylist();
    if (s_playlistManager.GetSize() > 0)
    {
        int j = -1;
        for (int i = 0; i < m_playlistAt.GetCount(); i++)
        {
            CString tmp;
            m_playlistAt.GetText(i, tmp);
            if (s_playlistManager.GetCurrent() == _wtoi((LPCTSTR)tmp))
            {
                j = i;
                break;
            }
        }
        if (j != -1)
        {
            m_playlistAt.SetSel(j, TRUE);
            m_playlistAt.SetCaretIndex(j, TRUE);
            m_playlistAt.SetTopIndex(j);
        }
    }
    ShowFileName();

    GetMenu()->CheckMenuItem(ID_FILE_SAVEINEXTENDEDFORMAT, MF_BYCOMMAND | (s_mainWnd.m_extPlaylistFormat ? MF_CHECKED : MF_UNCHECKED));

    return TRUE;
}

void CPlaylistDlg::ResizeWindow()
{
    CRect rect, rect2, rect3;
    GetClientRect(&rect);

    GetDlgItem(IDC_FRAME)->GetWindowRect(&rect2);
    ScreenToClient(&rect2);
    rect2.right = rect.Width() - rect2.left;
    rect2.bottom = rect.Height() - rect2.top;
    GetDlgItem(IDC_FRAME)->SetWindowPos(NULL, 0, 0, rect2.Width(), rect2.Height(), SWP_NOMOVE | SWP_NOZORDER | SWP_NOACTIVATE);

    GetDlgItem(IDC_SEARCH)->GetWindowRect(&rect2);
    ScreenToClient(&rect2);
    rect2.right = rect.Width() - rect2.left;
    GetDlgItem(IDC_SEARCH)->SetWindowPos(NULL, 0, 0, rect2.Width(), rect2.Height(), SWP_NOMOVE | SWP_NOZORDER | SWP_NOACTIVATE);

    GetDlgItem(IDC_PLAYLIST_AT)->GetWindowRect(&rect2);
    ScreenToClient(&rect2);
    rect2.right = rect.Width() - rect2.left;
    rect2.bottom = rect.Height() - rect2.top;
    GetDlgItem(IDC_PLAYLIST_AT)->SetWindowPos(NULL, 0, 0, rect2.Width(), rect2.Height(), SWP_NOMOVE | SWP_NOZORDER | SWP_NOACTIVATE);

    GetDlgItem(IDC_SEARCH)->GetWindowRect(&rect3);
    ScreenToClient(&rect3);
    rect2.top = rect.bottom - rect3.bottom;
    rect2.bottom = rect2.top + rect3.Height();
    GetDlgItem(IDC_FILENAME)->SetWindowPos(NULL, rect2.left, rect2.top, rect2.Width(), rect2.Height(), SWP_NOZORDER | SWP_NOACTIVATE);
}

void CPlaylistDlg::OnInitMenuPopup(CMenu* pMenu, UINT wParam, BOOL isActive)
{
    for (UINT i = 0; i < pMenu->GetMenuItemCount(); i++)
    {
        UINT menuID = pMenu->GetMenuItemID(i);
        CString menuTitle;
        //Si on est pas sur un sous menu
        if (menuID != 0 && menuID != 0xFFFFFFFF)
        {
            pMenu->GetMenuString(i, menuTitle, MF_BYPOSITION);
            if (menuTitle.Find(_T("|")) != -1)
            {
                CString sNumber = menuTitle.Left(menuTitle.Find(_T("|")));
                int number = 0;
                for (int j = 0; j < sNumber.GetLength(); j++)
                    number = number * 10 + sNumber.GetAt(j) - '0';
                CString tmp;
                tmp.Format(_T("%s\t%s"), s_playlistManager.GetTag(number, true), s_playlistManager.GetDuration(number, true));
                pMenu->ModifyMenu(menuID, MF_BYCOMMAND | pMenu->GetMenuState(menuID, MF_BYCOMMAND), menuID, tmp);
            }
        }
    }
    CDialog::OnInitMenuPopup(pMenu, wParam, isActive);
    CMenuXP::SetXPLookNFeel(this, pMenu->m_hMenu, s_mainWnd.m_skin.xpLookNFeel);
}

void CPlaylistDlg::OnMeasureItem(int, LPMEASUREITEMSTRUCT lpMeasureItemStruct)
{
    CMenuXP::OnMeasureItem(lpMeasureItemStruct);
}   

LRESULT CPlaylistDlg::OnMenuChar(UINT nChar, UINT nFlags, CMenu* pMenu)
{
    if (CMenuXP::IsOwnerDrawn(pMenu->m_hMenu))
        return CMenuXP::OnMenuChar(pMenu->m_hMenu, nChar, nFlags);                                                                         
    return CDialog::OnMenuChar(nChar, nFlags, pMenu);
}

void CPlaylistDlg::OnOK()
{
    if (GetFocus()->GetDlgCtrlID() == IDC_PLAYLIST_AT || GetFocus()->GetDlgCtrlID() == IDC_SEARCH)
    {
        if (m_playlistAt.GetFirstSelected() != -1)
        {
            CString tmp;
            m_playlistAt.GetText(m_playlistAt.GetFirstSelected(), tmp);
            PlayTheFile(_wtoi((LPCTSTR)tmp));
            m_playlistAt.RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW);
        }
    }
}

void CPlaylistDlg::OnPlaylistOpen()
{
    // open a file
    CFileDialog dialog(TRUE, _T("m3u"), _T(""), OFN_FILEMUSTEXIST, m_Helper.MyLoadString(IDS_OPENPLAYLIST), this);
    dialog.m_ofn.lpstrInitialDir = LPCTSTR(s_mainWnd.m_initDir);
    int ret;
    ret = dialog.DoModal();
    if (ret == IDOK)
    {
        //Dsactivation du rafraichissement
        if (IsWindowVisible())
            m_playlistAt.SetRedraw(FALSE);

        s_mainWnd.OpenPlaylist(dialog.GetPathName());
        s_mainWnd.m_initDir = dialog.GetPathName().Left(dialog.GetPathName().ReverseFind('\\') + 1);
    }
    m_playlistAt.SetFocus();
    ::SendMessage(m_hWnd, WM_COMMAND, ID_TRACKS_REFRESH, 0);
}

void CPlaylistDlg::OnPlaylistSave()
{
    PlaylistSave();
}


void CPlaylistDlg::OnPopupSelectAll()
{
    m_playlistAt.SetCaretIndex(0);
    m_playlistAt.SetSel(-1, TRUE);
    ShowFileName();
}

void CPlaylistDlg::OnSize(UINT nType, int cx, int cy)
{
    CDialog::OnSize(nType, cx, cy);
    if (m_canResize)
    {
        ResizeWindow();
    }
}

void CPlaylistDlg::OnShowWindow(BOOL bShow, UINT nStatus)
{
    CDialog::OnShowWindow(bShow, nStatus);

    m_canResize = bShow;

    if (bShow)
    {
        ::SendMessage(m_hWnd, WM_COMMAND, ID_TRACKS_REFRESH, 0);
    }
}

void CPlaylistDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
    if (nID == 9999)
    {
        if (s_mainWnd.m_playlistOnTop)
        {
            GetSystemMenu(FALSE)->CheckMenuItem(9999, MF_BYCOMMAND | MF_UNCHECKED);

            ShowWindow(SW_HIDE);
            ModifyStyleEx(0, WS_EX_APPWINDOW);
            ShowWindow(SW_SHOW);

            SetWindowPos(&wndNoTopMost, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE | SWP_NOACTIVATE);

            s_mainWnd.m_playlistOnTop = FALSE;
        }
        else
        {
            GetSystemMenu(FALSE)->CheckMenuItem(9999, MF_BYCOMMAND | MF_CHECKED);

            ShowWindow(SW_HIDE);
            ModifyStyleEx(WS_EX_APPWINDOW, 0);
            ShowWindow(SW_SHOW);

            SetWindowPos(&wndTopMost, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE | SWP_NOACTIVATE);

            s_mainWnd.m_playlistOnTop = TRUE;
        }
    }
    else
        CDialog::OnSysCommand(nID, lParam);
}

void CPlaylistDlg::OnTrackMenu(UINT nID)
{
    m_playlistAt.SetSel(-1, FALSE);
    int j = -1;
    for (int i = 0; i < m_playlistAt.GetCount(); i++)
    {
        CString tmp;
        m_playlistAt.GetText(i, tmp);
        if (nID - IDM_TRACKS == _wtoi((LPCTSTR)tmp))
        {
            j = i;
            break;
        }
    }
    if (j != -1)
    {
        m_playlistAt.SetSel(j, TRUE);
        m_playlistAt.SetCaretIndex(j, TRUE);
        m_playlistAt.SetTopIndex(j);
    }
    ShowFileName();
}

void CPlaylistDlg::OnTracksAddPlaylist()
{
    // open a file
    CFileDialog dialog(TRUE, _T("m3u"), _T(""), OFN_FILEMUSTEXIST, m_Helper.MyLoadString(IDS_OPENPLAYLIST), this);
    dialog.m_ofn.lpstrInitialDir = LPCTSTR(s_mainWnd.m_initDir);
    int ret;
    ret = dialog.DoModal();
    if (ret == IDOK)
    {
        //Dsactivation du rafraichissement
        if (IsWindowVisible())
            m_playlistAt.SetRedraw(FALSE);

        s_mainWnd.OpenPlaylist(dialog.GetPathName(), false, true);
        s_mainWnd.m_initDir = dialog.GetPathName().Left(dialog.GetPathName().ReverseFind('\\') + 1);
        ShowFileName();
    }
    m_playlistAt.SetFocus();
    ::SendMessage(m_hWnd, WM_COMMAND, ID_TRACKS_REFRESH, 0);
}

void CPlaylistDlg::OnTracksFileManagementDelete()
{
    if (MessageBox(m_Helper.MyLoadString(IDS_DELETEFILE), _T("STP"), MB_YESNO | MB_ICONEXCLAMATION | MB_DEFBUTTON2) == IDYES)
        Remove(false, true);
}

void CPlaylistDlg::OnTracksFileManagementCopyTo()
{
    LPMALLOC pMalloc;
    if (SHGetMalloc(&pMalloc) != NOERROR)
        return;

    BROWSEINFO bi;
    TCHAR buf[MAX_PATH];
    _tcscpy(buf, s_mainWnd.m_initDir.GetString());

    CString title;
    title = m_Helper.MyLoadString(IDS_TARGETFOLDER);

    bi.hwndOwner = m_hWnd;
    bi.pszDisplayName = buf;
    bi.pidlRoot = NULL;
    LPITEMIDLIST pidl;
    if ((pidl = SHBrowseForFolder(&bi)) == NULL)
    {
        pMalloc->Free(pidl);
        pMalloc ->Release();
        return;
    }
    if (SHGetPathFromIDList(pidl, buf) == false)
    {
        pMalloc->Free(pidl);
        pMalloc ->Release();
        return;
    }
    pMalloc->Free(pidl);
    pMalloc->Release();

    CString theDir = buf;
    if (theDir[theDir.GetLength() - 1] != '\\')
        theDir += "\\";

    s_mainWnd.m_initDir = theDir;

    CDialog dlg;
    dlg.Create(IDD_PROGRESS, this);
    CRect rect1, rect2;
    GetWindowRect(&rect1);
    dlg.GetWindowRect(&rect2);
    dlg.SetWindowPos(NULL, rect1.left + (rect1.right - rect1.left - (rect2.right - rect2.left)) / 2, rect1.top + (rect1.bottom - rect1.top - (rect2.bottom - rect2.top)) / 2, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
    dlg.ShowWindow(SW_SHOW);
    dlg.UpdateWindow();

    ((CProgressCtrl *)dlg.GetDlgItem(IDC_PROGRESS))->SetRange(0, m_playlistAt.GetSelCount());
    ((CProgressCtrl *)dlg.GetDlgItem(IDC_PROGRESS))->SetStep(1);

    CString srcfile, targetfile;
    int i = -1;
    while ((i = m_playlistAt.GetFirstSelected(i + 1)) > -1)
    {
        CString tmp;
        m_playlistAt.GetText(i, tmp);
        srcfile = s_playlistManager.GetPlaylist(_wtoi((LPCTSTR)tmp));
        targetfile = theDir + srcfile.Mid(srcfile.ReverseFind('\\') + 1);
        dlg.SetDlgItemText(IDC_FNAME, srcfile.Mid(srcfile.ReverseFind('\\') + 1));
        if (s_mainWnd.FileExists(targetfile))
        {
            if (dlg.MessageBox(m_Helper.MyLoadString(IDS_OVERWRITE) + _T("\n") + targetfile + _T("?"), _T("STP"), MB_YESNO | MB_ICONEXCLAMATION | MB_DEFBUTTON2) == IDYES)
            {
                ::SetFileAttributes(targetfile, ::GetFileAttributes(targetfile) & !FILE_ATTRIBUTE_READONLY);
                ::DeleteFile(targetfile);
                CopyFile(srcfile, targetfile, TRUE);
            }
        }
        else
            CopyFile(srcfile, targetfile, TRUE);
        ((CProgressCtrl *)dlg.GetDlgItem(IDC_PROGRESS))->StepIt();
    }
}

void CPlaylistDlg::OnTracksFileManagementMoveTo()
{
    LPMALLOC pMalloc;
    if (SHGetMalloc(&pMalloc) != NOERROR)
        return;

    BROWSEINFO bi;
    TCHAR buf[MAX_PATH];
    _tcscpy(buf, s_mainWnd.m_initDir.GetString());

    CString title;
    title = m_Helper.MyLoadString(IDS_TARGETFOLDER);

    bi.hwndOwner = m_hWnd;
    bi.pszDisplayName = buf;
    bi.pidlRoot = NULL;
    bi.lpszTitle = LPCTSTR(title);
    bi.ulFlags = BIF_RETURNONLYFSDIRS;
    bi.lpfn = BrowseCallbackProc;

    LPITEMIDLIST pidl;
    if ((pidl = SHBrowseForFolder(&bi)) == NULL)
    {
        pMalloc->Free(pidl);
        pMalloc ->Release();
        return;
    }
    if (SHGetPathFromIDList(pidl, buf) == false)
    {
        pMalloc->Free(pidl);
        pMalloc ->Release();
        return;
    }
    pMalloc->Free(pidl);
    pMalloc->Release();

    CString theDir = buf;
    if (theDir[theDir.GetLength() - 1] != '\\')
        theDir += "\\";

    s_mainWnd.m_initDir = theDir;

    CDialog dlg;
    dlg.Create(IDD_PROGRESS, this);
    CRect rect1, rect2;
    GetWindowRect(&rect1);
    dlg.GetWindowRect(&rect2);
    dlg.SetWindowPos(NULL, rect1.left + (rect1.right - rect1.left - (rect2.right - rect2.left)) / 2, rect1.top + (rect1.bottom - rect1.top - (rect2.bottom - rect2.top)) / 2, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
    dlg.ShowWindow(SW_SHOW);
    dlg.UpdateWindow();

    ((CProgressCtrl *)dlg.GetDlgItem(IDC_PROGRESS))->SetRange(0, m_playlistAt.GetSelCount());
    ((CProgressCtrl *)dlg.GetDlgItem(IDC_PROGRESS))->SetStep(1);

    CString srcfile, targetfile;
    int i = -1, tdi = 0, curpos = 0;
    while ((i = m_playlistAt.GetFirstSelected(i + 1)) > -1)
    {
        CString tmp;
        m_playlistAt.GetText(i, tmp);
        srcfile = s_playlistManager.GetPlaylist(_wtoi((LPCTSTR)tmp));
        targetfile = theDir + srcfile.Mid(srcfile.ReverseFind('\\') + 1);
        dlg.SetDlgItemText(IDC_FNAME, srcfile.Mid(srcfile.ReverseFind('\\') + 1));
        if (s_mainWnd.FileExists(targetfile))
        {
            if (dlg.MessageBox(m_Helper.MyLoadString(IDS_OVERWRITE) + _T("\n") + targetfile + _T("?"), _T("STP"), MB_YESNO | MB_ICONEXCLAMATION | MB_DEFBUTTON2) == IDYES)
            {
                ::SetFileAttributes(targetfile, ::GetFileAttributes(targetfile) & !FILE_ATTRIBUTE_READONLY);
                ::DeleteFile(targetfile);
                if (_wtoi((LPCTSTR)tmp) == s_playlistManager.GetCurrent())
                {
                    curpos = stdIn.GetOutputTime() / 1000;
                    if (::SendMessage(WAEmuWnd.m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
                        ::SendMessage(WAEmuWnd.m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
                }

                if (MoveFile(srcfile, targetfile))
                    s_playlistManager.Add(targetfile, i);

                if (_wtoi((LPCTSTR)tmp) == s_playlistManager.GetCurrent())
                {
                    enqueueFileWithMetaStructW eFile;
                    eFile.filename = (TCHAR *)s_playlistManager.GetPlaylist(s_playlistManager.GetCurrent()).GetString();
                    ::SendMessage(WAEmuWnd.m_hWnd, IPC_PLAYFILEW, 0, (WPARAM)&eFile);
                    if (tdi != 0)
                        stdIn.SetOutputTime(curpos * 1000);
                    if (stdIn.IsPaused())
                        ::SendMessage(WAEmuWnd.m_hWnd, WM_COMMAND, WINAMP_BUTTON3, 0);
                }
            }
        }
        else
        {
            if (_wtoi((LPCTSTR)tmp) == s_playlistManager.GetCurrent())
            {
                curpos = stdIn.GetOutputTime() / 1000;
                if (::SendMessage(WAEmuWnd.m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
                    ::SendMessage(WAEmuWnd.m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
            }

            if (MoveFile(srcfile, targetfile))
                s_playlistManager.Add(targetfile, i);

            if (_wtoi((LPCTSTR)tmp) == s_playlistManager.GetCurrent())
            {
                enqueueFileWithMetaStructW eFile;
                eFile.filename = (TCHAR *)s_playlistManager.GetPlaylist(s_playlistManager.GetCurrent()).GetString();
                ::SendMessage(WAEmuWnd.m_hWnd, IPC_PLAYFILEW, 0, (WPARAM)&eFile);
                if (tdi != 0)
                    stdIn.SetOutputTime(curpos * 1000);
                if (stdIn.IsPaused())
                    ::SendMessage(WAEmuWnd.m_hWnd, WM_COMMAND, WINAMP_BUTTON3, 0);
            }
        }
        ((CProgressCtrl *)dlg.GetDlgItem(IDC_PROGRESS))->StepIt();
    }
    ShowFileName();
}

void CPlaylistDlg::OnTracksRemoveAll()
{
    Remove(true);
}

void CPlaylistDlg::OnTracksRenameFiles()
{
    UINT i = 0xFFFFFFFF;
    int tdi = 0, curpos = 0, j;
    if (m_playlistAt.GetFirstSelected() == -1)
        return;

    CRenameDlg dlg;
    dlg.m_template = s_mainWnd.m_rNTemplate;

    if (dlg.DoModal() == IDOK)
    {
        s_mainWnd.m_rNTemplate = dlg.m_template;

        CFile mpfile;

        CString track, artist, title, album, fname, albtrack;

        char buf[31];
        while ((i = m_playlistAt.GetFirstSelected(i + 1)) > -1)
        {
            CString tmp;
            m_playlistAt.GetText(i, tmp);
            if (mpfile.Open(s_playlistManager.GetPlaylist(_wtoi((LPCTSTR)tmp)), CFile::modeRead | CFile::shareDenyNone))
            {
                track.Format(_T("%02d"), i + 1);
                int b, e;
                s_mainWnd.GetAlbumRange(&b, &e, true, i);
                albtrack.Format(_T("%02d"), i - b + 1);

                if (mpfile.GetLength() < 129)
                {
                    mpfile.Close();
                    title = _T("Track") + albtrack;
                    artist = _T("Unknown artist");
                    album = _T("Unknown album");
                }
                else
                {
                    mpfile.Seek(-128, CFile::end);
                    mpfile.Read(buf, 3);buf[3] = '\0';
                    if (strcmp(buf, "TAG"))
                    {
                        mpfile.Close();
                        title = _T("Track") + albtrack;
                        artist = _T("Unknown artist");
                        album = _T("Unknown album");
                    }
                    else
                    {
                        if (mpfile.Read(buf, 30))
                        {
                            buf[30] = '\0';
                            title = buf;
                            title.TrimRight();
                            if (title == "")
                                title = _T("Track") + albtrack;
                        }
                        if (mpfile.Read(buf, 30))
                        {
                            buf[30] = '\0';
                            artist = buf;
                            artist.TrimRight();
                            if (artist == "")
                                artist = _T("Unknown artist");
                        }
                        if (mpfile.Read(buf, 30))
                        {
                            buf[30] = '\0';
                            album = buf;
                            album.TrimRight();
                            if (album == "")
                                album = _T("Unknown album");
                        }
                        mpfile.Close();
                    }
                }

                fname = s_mainWnd.m_rNTemplate;
                fname.Replace(_T("%1"), track);
                fname.Replace(_T("%2"), artist);
                fname.Replace(_T("%3"), title);
                fname.Replace(_T("%4"), album);
                fname.Replace(_T("%5"), albtrack);

                j = s_playlistManager.GetPlaylist(i).ReverseFind('.');
                if (j >= 0)
                    fname += s_playlistManager.GetPlaylist(i).Mid(j);

                fname.Replace('\\', '_');
                fname.Replace('/', '_');
                fname.Replace(':', '_');
                fname.Replace('*', '_');
                fname.Replace('?', '_');
                fname.Replace('\"', '_');
                fname.Replace('<', '_');
                fname.Replace('>', '_');
                fname.Replace('|', '_');

                fname = s_playlistManager.GetPlaylist(i).Left(s_playlistManager.GetPlaylist(i).ReverseFind('\\') + 1) + fname;

                if (i == s_playlistManager.GetCurrent())
                {
                    curpos = stdIn.GetOutputTime() / 1000;

                    if (::SendMessage(WAEmuWnd.m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
                        ::SendMessage(WAEmuWnd.m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
                }

                if (::MoveFile(s_playlistManager.GetPlaylist(i), fname))
                    s_playlistManager.Add(fname, i);

                if (i == s_playlistManager.GetCurrent())
                {
                    enqueueFileWithMetaStructW eFile;
                    eFile.filename = (TCHAR *)s_playlistManager.GetPlaylist(s_playlistManager.GetCurrent()).GetString();
                    ::SendMessage(WAEmuWnd.m_hWnd, IPC_PLAYFILEW, 0, (WPARAM)&eFile);
                    if (tdi != 0)
                        stdIn.SetOutputTime(1000 * curpos);
                    if (stdIn.IsPaused())
                        ::SendMessage(WAEmuWnd.m_hWnd, WM_COMMAND, WINAMP_BUTTON3, 0);
                }
            }
        }
        ShowFileName();
    }
}

void CPlaylistDlg::OnTracksReReadAllTags()
{
    s_playlistManager.StopReadAll();
    int i;
    for (i = 0; i < s_playlistManager.GetSize(); i++)
    {
        if (i != s_playlistManager.GetCurrent())
            s_playlistManager.Add(s_playlistManager.Get(i), i);
    }
    s_playlistManager.ReadAllTags();
}

void CPlaylistDlg::OnTracksSelectionAddToExistentPlaylist()
{
    CFileDialog dialog(TRUE, _T("m3u"), _T(""), OFN_FILEMUSTEXIST, m_Helper.MyLoadString(IDS_OPENPLAYLIST), this);
    dialog.m_ofn.lpstrInitialDir = (TCHAR *)s_mainWnd.m_initDir.GetString();
    int ret;
    ret = dialog.DoModal();
    if (ret == IDOK)
    {
        s_mainWnd.m_initDir = dialog.GetPathName().Left(dialog.GetPathName().ReverseFind('\\') + 1);

        CStringArray CurPL;
        CString fname = dialog.GetPathName();
        CStdioFile plfile;

        if (!plfile.Open(fname, CFile::modeRead | CFile::shareDenyNone))
        {
            MessageBox(m_Helper.MyLoadString(IDS_CANTOPEN), _T("STP"), MB_ICONERROR);
            return;
        }

        //Dsactivation du rafraichissement
        if (IsWindowVisible())
            m_playlistAt.SetRedraw(FALSE);

        CString buf;
        CString CurPath = fname.Left(fname.ReverseFind('\\') + 1);
        CString CurDisk = fname.Left(fname.Find(':') + 1);
        while (plfile.ReadString(buf))
        {
            buf.TrimLeft();buf.TrimRight();
            if (buf.Left(4) == "#EXT" || buf == "" || buf.Left(5) == "#TAG=")
                continue;
            if (buf[1] == ':' || buf.Find(_T("://")) > 0)
                CurPL.Add(buf);
            else if (buf[0] == '\\')
                CurPL.Add(CurDisk + buf);
            else
                CurPL.Add(CurPath + buf);
        }
        plfile.Close();
        if (!plfile.Open(dialog.GetPathName(), CFile::modeCreate | CFile::modeWrite))
        {
            MessageBox(m_Helper.MyLoadString(IDS_CANTOPEN), _T("STP"), MB_ICONERROR);
            return;
        }

        UINT i;
        for (i = 0; i < (UINT)m_playlistAt.GetCount(); i++)
        {
            if (m_playlistAt.GetSel(i))
            {
                CString tmp;
                m_playlistAt.GetText(i, tmp);
                CurPL.Add(s_playlistManager.GetPlaylist(_wtoi((LPCTSTR)tmp)));
            }
        }

        CurPath = CurPath.Left(CurPath.GetLength() - 1);
        UINT ln = CurPL.GetUpperBound() + 1;
        for (i = 0; i < ln; i++)
        {
            buf = CurPL[i];
            if (CurPath.CompareNoCase(buf.Left(CurPath.GetLength())) == 0)
                buf = buf.Mid(CurPath.GetLength() + 1);
            if (CurPath.Left(2).CompareNoCase(buf.Left(2)) == 0)
                buf = buf.Mid(2);
            plfile.WriteString(buf + _T("\n"));
        }

        plfile.Close();
    }
    ::SendMessage(m_hWnd, WM_COMMAND, ID_TRACKS_REFRESH, 0);
}

void CPlaylistDlg::OnTracksSelectionCreateNewPlaylist()
{
    PlaylistSave(false);
}


void CPlaylistDlg::PlaylistSave(bool saveall)
{
    if (s_playlistManager.GetSize() == 0)
        return;
    if (!saveall && m_playlistAt.GetFirstSelected() == -1)
        return;
    CFileDialog dialog(FALSE, _T("m3u"), _T(""), OFN_OVERWRITEPROMPT, m_Helper.MyLoadString(IDS_OPENPLAYLIST), this);
    dialog.m_ofn.lpstrInitialDir = LPCTSTR(s_mainWnd.m_initDir);
    int ret;
    ret = dialog.DoModal();
    if (ret == IDOK)
    {
        // a file was selected
        s_mainWnd.m_initDir = dialog.GetPathName().Left(dialog.GetPathName().ReverseFind('\\') + 1);
        CStdioFile plfile;
        if (plfile.Open(dialog.GetPathName(), CFile::modeCreate | CFile::modeWrite))
        {
            CString PLPath;
            TCHAR PLpath[MAX_PATH];
            TCHAR PLdrive[MAX_PATH];
            TCHAR PLdir[MAX_PATH];
            TCHAR PLfname[MAX_PATH];
            TCHAR PLext[MAX_PATH];
            _tcscpy(PLpath, (TCHAR *)dialog.GetPathName().GetString());
            _tsplitpath(PLpath, PLdrive, PLdir, PLfname, PLext);

            PLPath.Format(_T("%s%s"), PLdrive, PLdir);

            TCHAR path[MAX_PATH];
            TCHAR drive[MAX_PATH];
            TCHAR dir[MAX_PATH];
            TCHAR fname[MAX_PATH];
            TCHAR ext[MAX_PATH];

            if (s_mainWnd.m_extPlaylistFormat)
            {
                plfile.WriteString(_T("#EXTM3U\n"));
            }
            UINT i;
            CString buf;
            for (i = 0; i < (UINT)m_playlistAt.GetCount(); i++)
            {
                if (m_playlistAt.GetSel(i) || saveall)
                {
                    CString tmp;
                    m_playlistAt.GetText(i, tmp);
                    buf = s_playlistManager.GetPlaylist(_wtoi((LPCTSTR)tmp));

                    _tcscpy(path, (TCHAR *)buf.GetString());
                    _tsplitpath(path, drive, dir, fname, ext);

                    if (_tcscmp(drive, PLdrive) == 0)
                    {
                        if (_tcscmp(dir, PLdir) == 0)
                        {
                            buf.Format(_T("%s%s"), fname, ext);
                        }
                        else
                        {
                            if (_tcsstr(dir, PLdir) != NULL)
                            {
                                buf.Format(_T("%s%s%s"), dir + _tcslen(PLdir), fname, ext);
                            }
                            else
                            {
                                CString points = _T("");
                                TCHAR tmpPLdir[MAX_PATH];
                                _tcscpy(tmpPLdir, PLdir);
                                tmpPLdir[_tcslen(PLdir) - 1] = 0;
                                TCHAR* pEnd;
                                do
                                {
                                    points += _T("..\\");
                                    pEnd = _tcsrchr(tmpPLdir, '\\');
                                    if (pEnd != NULL)
                                    {
                                        pEnd[0] = 0;
                                        if (_tcsstr(dir, tmpPLdir) != NULL)
                                        {
                                            break;
                                        }
                                    }
                                }
                                while (pEnd != NULL);
                                buf.Format(_T("%s%s%s%s"), points, dir + _tcslen(tmpPLdir) + 1, fname, ext);
                            }
                        }
                    }
                    else
                    {
                        buf.Format(_T("%s%s%s%s"), drive, dir, fname, ext);
                    }

                    plfile.WriteString(buf + _T("\n"));
                    if (s_mainWnd.m_extPlaylistFormat && s_playlistManager.GetRead(_wtoi((LPCTSTR)tmp)))
                    {
                        CString buf;
                        buf.Format(_T("#EXTINF:%d,%s\n"), s_playlistManager.GetDurationI(_wtoi((LPCTSTR)tmp)), s_playlistManager.GetTag(_wtoi((LPCTSTR)tmp)));
                        plfile.WriteString(buf);
                    }
                }
            }
            plfile.Close();
            s_mainWnd.AddRecent(dialog.GetPathName());
        }
        else
            MessageBox(m_Helper.MyLoadString(IDS_CANTOPEN), _T("STP"), MB_ICONERROR);
    }    
    m_playlistAt.SetFocus();
}

void CPlaylistDlg::PlayTheFile(int fnum)
{
    if (stdIn.GetPlayerMode() != PLAYER_MODE_CLOSE)
        ::SendMessage(WAEmuWnd.m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);

    s_playlistManager.SetCurrent(fnum);
    enqueueFileWithMetaStructW eFile;
    eFile.filename = (TCHAR *)s_playlistManager.GetPlaylist(s_playlistManager.GetCurrent()).GetString();
    ::SendMessage(WAEmuWnd.m_hWnd, IPC_PLAYFILEW, 0, (WPARAM)&eFile);
}

BOOL CPlaylistDlg::PreTranslateMessage(MSG* pMsg)
{
    int q = TranslateAccelerator(m_hWnd, m_accel, pMsg);
    if (q > 0)
        return q;
    else
        return CDialog::PreTranslateMessage(pMsg);
}

void CPlaylistDlg::Remove(bool RAll, bool FromDisk)
{
    int q;
    if (RAll)
    {
        ::SendMessage(WAEmuWnd.m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
        s_playlistManager.RemoveAll();
        ::SendMessage(m_hWnd, WM_COMMAND, ID_TRACKS_REFRESH, 0);
    }
    if ((q = m_playlistAt.GetFirstSelected()) == -1 && !RAll)
        return;
    int i = m_playlistAt.GetCount() - 1;
    bool CurDeleted = false;

    int nCount = m_playlistAt.GetSelCount();
    CArray<int, int> arrayListBoxSel;

    arrayListBoxSel.SetSize(nCount);
    m_playlistAt.GetSelItems(nCount, arrayListBoxSel.GetData()); 

    while (i >= 0)
    {
        if (m_playlistAt.GetSel(i) || RAll)
        {
            //Rcupration de l'id de la chaine de caractres...
            CString id;
            int n = m_playlistAt.GetTextLen(i);
            m_playlistAt.GetText(i, id.GetBuffer(n));
            id.ReleaseBuffer();

            int ii = _wtoi(id.GetBuffer(n));
            id.ReleaseBuffer();

            m_playlistAt.DeleteString(i);
            if (ii < s_playlistManager.GetCurrent())
                s_playlistManager.decCurrent();
            else if (ii == s_playlistManager.GetCurrent() && !CurDeleted)
            {
                CurDeleted = true;
                if (stdIn.GetPlayerMode() != PLAYER_MODE_CLOSE)
                    ::SendMessage(WAEmuWnd.m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
            }
            if (FromDisk)
            {
                ::SetFileAttributes(s_playlistManager.GetPlaylist(ii), ::GetFileAttributes(s_playlistManager.GetPlaylist(ii)) & !FILE_ATTRIBUTE_READONLY);
                ::DeleteFile(s_playlistManager.GetPlaylist(ii));
            }
            s_playlistManager.RemoveAt(ii);
            m_cachedPlaylistTags->RemoveAt(ii);
        }
        i--;
    }
    if (CurDeleted)
    {
        s_playlistManager.SetCurrent(-1);
    }
    if (m_playlistAt.SetSel(q, TRUE) == LB_ERR)
        m_playlistAt.SetSel(q - 1, TRUE);

    ::SendMessage(m_hWnd, WM_COMMAND, ID_TRACKS_REFRESH, 0);
}

void CPlaylistDlg::Save()
{
    CRect rect;
    GetWindowRect(&rect);
    s_mainWnd.m_playlistRect = rect;
}

void CPlaylistDlg::ShowFileName()
{
    if (IsWindowVisible())
    {
        ShowTimes();

        int i = m_playlistAt.GetFirstSelected();
        if (i > -1)
        {
            CString temp;
            m_playlistAt.GetText(i, temp);
            i = _wtoi((LPCTSTR)temp);

            m_filename.SetWindowText(m_cachedPlaylistTags->GetAt(i).GetPlaylist());
            m_filename.SetSel(m_cachedPlaylistTags->GetAt(i).GetPlaylist().GetLength());
        }
        else
            m_filename.SetWindowText(_T(""));
/*          GetMenu()->EnableMenuItem(ID_PLAYLIST_SAVE, CachedPlaylistTags->GetSize() == 0 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
          GetMenu()->EnableMenuItem(ID_FILE_REMOVE, i == -1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
          GetMenu()->EnableMenuItem(ID_TRACKS_REMOVEALL, CachedPlaylistTags->GetSize() < 1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
          GetMenu()->EnableMenuItem(ID_FILE_EDITTAG, i == -1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
          GetMenu()->EnableMenuItem(ID_TRACKS_REREADALLTAGS, CachedPlaylistTags->GetSize() < 1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
          GetMenu()->EnableMenuItem(ID_TRACKS_SELECTION_CREATENEWPLAYLIST, i == -1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
          GetMenu()->EnableMenuItem(ID_TRACKS_SELECTION_ADDTOEXISTENTPLAYLIST, i == -1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
          GetMenu()->EnableMenuItem(ID_ARRANGE_SORT, CachedPlaylistTags->GetSize() == 0 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
          GetMenu()->EnableMenuItem(ID_ARRANGE_SORTBYFILENAME, CachedPlaylistTags->GetSize() == 0 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
          GetMenu()->EnableMenuItem(ID_ARRANGE_SHUFFLE, CachedPlaylistTags->GetSize() == 0 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
          GetMenu()->EnableMenuItem(ID_TRACKS_RENAMEFILES, i == -1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
          GetMenu()->EnableMenuItem(ID_TRACKS_FILEMANAGEMENT_DELETE, i == -1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
          GetMenu()->EnableMenuItem(ID_TRACKS_FILEMANAGEMENT_COPYTO, i == -1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
          GetMenu()->EnableMenuItem(ID_TRACKS_FILEMANAGEMENT_MOVETO, i == -1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
*/
    }
}

void CPlaylistDlg::ShowTimes()
{
    if (IsWindowVisible())
    {
        UINT selected = 0, TotalTime = 0;

        CString buf;
        for (UINT q = 0; q < (UINT)m_playlistAt.GetCount(); q++)
        {
            CString tmp;
            m_playlistAt.GetText(q, tmp);
            TotalTime += m_cachedPlaylistTags->GetAt(_wtoi((LPCTSTR)tmp)).GetDurationI();
            if (m_playlistAt.GetSel(q))
            {
                CString tmp;
                m_playlistAt.GetText(q, tmp);
                selected += m_cachedPlaylistTags->GetAt(_wtoi((LPCTSTR)tmp)).GetDurationI();
            }
        }

        if (TotalTime >= 3600)
            buf.Format(_T("%02d:%02d:%02d / %02d:%02d:%02d"), selected / 3600, (selected % 3600) / 60, selected % 60, TotalTime / 3600, (TotalTime % 3600) / 60, TotalTime % 60);
        else
            buf.Format(_T("%02d:%02d / %02d:%02d"), selected / 60, selected % 60, TotalTime / 60, TotalTime % 60);
        if (IsWindowVisible())
            SetWindowText(m_Helper.MyLoadString(IDS_PLAYLIST) + _T(" ") + buf);
    }
}

void CPlaylistDlg::OnTrackRefresh()
{
    CAutoLock lock(&m_critSec);
    if (IsWindowVisible())
    {
        //Dsactivation du rafraichissement
        m_playlistAt.SetRedraw(FALSE);
        CStringArray playlist;
        for (int i = 0; i < m_playlistAt.GetCount(); i++)
        {
            if (m_playlistAt.GetSel(i))
            {
                CString tmp;
                m_playlistAt.GetText(i, tmp);
                playlist.Add(tmp);
            }
        }
        FillPlaylist();
        for (int j = 0; j < playlist.GetSize(); j++)
        {
            for (int i = 0; i < m_playlistAt.GetCount(); i++)
            {
                CString tmp;
                m_playlistAt.GetText(i, tmp);
                if (playlist[j] == tmp)
                {
                    m_playlistAt.SetSel(j);
                    break;
                }
            }
        }
        m_playlistAt.SetSel(-1, FALSE);
        ShowFileName();
        //Activation du rafraichissement
        m_playlistAt.SetRedraw(TRUE);
        m_playlistAt.Invalidate();
    }
}
CPLLB::CPLLB() : CListBox()
{
    m_shiftDown = false;
    m_ctrlDown = false;
    m_prev = -1;
}
CPLLB::~CPLLB()
{
}

void CPLLB::DrawItem(LPDRAWITEMSTRUCT lpDIS)
{
    if ((int)lpDIS->itemID < 0)
        return; 

    CDC* pDC = CDC::FromHandle(lpDIS->hDC);

    COLORREF crText;
    CString sText;

    if (lpDIS->itemAction & (ODA_SELECT | ODA_DRAWENTIRE))
    {
        if (lpDIS->itemState & ODS_SELECTED)
        {
            CBrush brush(::GetSysColor(COLOR_HIGHLIGHT));
            pDC->FillRect(&lpDIS->rcItem, &brush);
        }
        else
        {
            CBrush brush(::GetSysColor(COLOR_WINDOW));
            pDC->FillRect(&lpDIS->rcItem, &brush);
        }
    }       

    /*if ((lpDIS->itemAction & ODA_FOCUS) && (lpDIS->itemState & ODS_FOCUS))
       pDC->DrawFocusRect(&lpDIS->rcItem); 
    if ((lpDIS->itemAction & ODA_FOCUS) &&     !(lpDIS->itemState & ODS_FOCUS))
       pDC->DrawFocusRect(&lpDIS->rcItem);*/

    int nBkMode = pDC->SetBkMode(TRANSPARENT);

    CString temp;
    temp.Format(_T("%d"), lpDIS->itemID + 1);
    GetText(lpDIS->itemID, temp);

    if (lpDIS->itemState & ODS_SELECTED)
        crText = pDC->SetTextColor(::GetSysColor(COLOR_WINDOW));
    else if (lpDIS->itemState & ODS_DISABLED)
        crText = pDC->SetTextColor(::GetSysColor(COLOR_GRAYTEXT));
    else
    {
        crText = pDC->SetTextColor(_wtoi((LPCTSTR)temp) == (UINT)s_playlistManager.GetCurrent() ? RGB(0, 128, 128) : ::GetSysColor(COLOR_WINDOWTEXT));
    }

    CRect rect;
    UINT nFormat;

    if (_wtoi((LPCTSTR)temp) < s_playlistDlg.m_cachedPlaylistTags->GetSize())
        sText = s_playlistDlg.m_cachedPlaylistTags->GetAt(_wtoi((LPCTSTR)temp)).GetTag();
//    if (sText.GetLength() <= 9)
//        sText = s_playlistDlg.CachedPlaylistTags->GetAt(lpDIS->itemID).GetFilenameExt();

    long digits, q = s_playlistDlg.m_cachedPlaylistTags->GetSize(), chw = LOWORD(GetDialogBaseUnits());
    for (digits = 1; digits < 4; digits++)
    {
        if ((q = (int)(q / 10)) == 0)
            break;
    }

    rect = lpDIS->rcItem;
    rect.right = rect.left + digits * chw;

    nFormat = DT_RIGHT | DT_SINGLELINE | DT_VCENTER;

    pDC->DrawText(temp, -1, &rect, nFormat | DT_NOPREFIX);

    rect = lpDIS->rcItem;
    rect.left = rect.left + digits * chw + chw;
    rect.right = rect.right;// - (long)(chw * 4.5);

    nFormat = DT_LEFT | DT_SINGLELINE | DT_VCENTER;

    pDC->DrawText(sText, -1, &rect, nFormat | DT_NOPREFIX);
    nFormat = DT_RIGHT | DT_SINGLELINE | DT_VCENTER;
    temp.Format(_T("%s  "), s_playlistDlg.m_cachedPlaylistTags->GetAt(_wtoi((LPCTSTR)temp)).GetDuration());

    pDC->DrawText(temp, -1, &rect, nFormat | DT_NOPREFIX);

    pDC->SetTextColor(crText); 
    pDC->SetBkMode(nBkMode);
}

int CPLLB::GetFirstSelected(int from)
{
    if (s_playlistDlg.IsWindowVisible())
    {
        int i = 0;
        int q = GetSelCount();
        if (q == LB_ERR)
        {
            return GetCurSel();
        }
        CArray<int, int> arrayListBoxSel;
        arrayListBoxSel.SetSize(q);
        GetSelItems(q, arrayListBoxSel.GetData());
        while (i < arrayListBoxSel.GetSize())
        {
            if (arrayListBoxSel[i] >= from)
            {
                return arrayListBoxSel[i];
            }
            i++;
        }
    }
    return -1;
}

void CPLLB::MeasureItem(LPMEASUREITEMSTRUCT lpMIS)
{
    // Get current font metrics
    TEXTMETRIC metrics;
    HDC dc = ::GetDC(s_playlistDlg.m_hWnd);
    GetTextMetrics(dc, &metrics);
    ::ReleaseDC(m_hWnd, dc);

    // Set the height
    lpMIS->itemHeight = metrics.tmHeight + metrics.tmExternalLeading;
}



void CPLLB::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
    switch (nChar)
    {
        case VK_UP:
        case VK_DOWN:
        case VK_PRIOR:
        case VK_NEXT:
        case VK_HOME:
        case VK_END:
            {
                CListBox::OnKeyDown(nChar, nRepCnt, nFlags);
                if (m_shiftDown)
                {
                    int now = GetCaretIndex();
                    if (m_prev == -1)
                        m_prev = now;
                    if (m_prev < now)
                    {
                        SetSel(-1, FALSE);
                        SelItemRange(TRUE, m_prev, now);
                    }
                    else
                    {
                        SetSel(-1, FALSE);
                        SelItemRange(TRUE, now, m_prev);
                    }
                }
                else
                {
                    SetSel(-1, FALSE);
                    SetSel(GetCaretIndex(), TRUE);
                }
                s_playlistDlg.ShowFileName();
                break;
            }
        case VK_ADD:
            SetCaretIndex(0);
            SetSel(-1, TRUE);
            s_playlistDlg.ShowFileName();
            break;
        case VK_MULTIPLY:
            {
                int e = GetTopIndex();
                int w = GetCaretIndex();
                int q = GetCount();
                for (int i = 0; i < q; i++)
                    SetSel(i, !GetSel(i));
                s_playlistDlg.ShowFileName();
                SetCaretIndex(w);
                SetTopIndex(e);
                Invalidate();
                break;
            }
        case VK_DELETE:
            s_playlistDlg.Remove();
            break;
        case VK_SHIFT:
            m_prev = GetCaretIndex();
            m_shiftDown = true;
            break;
        case VK_CONTROL:
            m_ctrlDown = true;
            break;
        case 'A':
        case 'B':
        case 'C':
        case 'D':
        case 'E':
        case 'F':
        case 'G':
        case 'H':
        case 'I':
        case 'J':
        case 'K':
        case 'L':
        case 'M':
        case 'N':
        case 'O':
        case 'P':
        case 'Q':
        case 'R':
        case 'S':
        case 'T':
        case 'U':
        case 'V':
        case 'W':
        case 'X':
        case 'Y':
        case 'Z':
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
        case '8':
        case '9':
        case '0':
            s_playlistDlg.GetDlgItem(IDC_SEARCH)->SetFocus();
            s_playlistDlg.GetDlgItem(IDC_SEARCH)->PostMessage(WM_KEYDOWN, nChar, nRepCnt + 0x10000 * nFlags);
            return;
            break;
    }
}

void CPLLB::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags)
{
    if (nChar == VK_SHIFT)
    {
        m_shiftDown = false;
        m_prev = -1;
    }
    if (nChar == VK_CONTROL)
        m_ctrlDown = false;
    //CListBox::OnKeyUp(nChar,nRepCnt,nFlags);
}

void CPLLB::OnLButtonDblClk(UINT nFlags, CPoint point)
{
    BOOL q;
    int item;
    item = ItemFromPoint(point, q);
    CString temp;
    GetText(item, temp);
    item = _wtoi((LPCTSTR)temp);
    if (!q)
    {
        s_playlistDlg.PlayTheFile(item);
        RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW);
    }
    CListBox::OnLButtonDblClk(nFlags, point);
}

void CPLLB::OnLButtonDown(UINT nFlags, CPoint point)
{
    BOOL q;
    m_posfrom = ItemFromPoint(point, q);
    if (q)
        m_posfrom = -1;
    if (!(nFlags & MK_CONTROL) && !(nFlags & MK_SHIFT))
        SetSel(-1, FALSE);
    int last = GetCaretIndex();
    if (nFlags & MK_SHIFT)
    {
        if (last != m_posfrom)
        {
            SetSel(-1, FALSE);
            if (m_posfrom < last)
                SelItemRange(TRUE, m_posfrom, last);
            else
                SelItemRange(TRUE, last, m_posfrom);
        }
    }
    else if (nFlags & MK_CONTROL)
    {
        SetCaretIndex(m_posfrom);
        SetSel(m_posfrom, !GetSel(m_posfrom));
    }
    else if (m_posfrom > -1)
    {
        SetCaretIndex(m_posfrom);
        SetSel(m_posfrom, TRUE);
    }
    s_playlistDlg.ShowFileName();
}

void CPLLB::OnMouseMove(UINT nFlags, CPoint point)
{
    if (nFlags & MK_LBUTTON)
    {
        CString buf;
        BOOL q;
        CPoint point2;
        CRect rect;
        GetClientRect(&rect);
        int nTargetIndex = ItemFromPoint(point, q);
        for (; ;)
        {
            if (!q)
            {
                if (m_posfrom >= 0 && nTargetIndex >= 0)
                {
                    if (point.y - rect.top > 0 && point.y - rect.top < 10 && nTargetIndex > 0)
                        nTargetIndex--;
                    if (rect.bottom - point.y > 0 && rect.bottom - point.y < 10 && nTargetIndex < GetCount() - 1)
                        nTargetIndex++;
                    if (m_posfrom > nTargetIndex)
                    {
                        int offset = m_posfrom - nTargetIndex;
                        if (!GetSel(0))
                            for (int i = 1; i < GetCount(); i++)
                            {
                                if (GetSel(i))
                                {
                                    offset = (i - offset < 0 ? i : offset);
                                    CachedTag nTag = s_playlistManager.Get(i);
                                    s_playlistManager.RemoveAt(i);
                                    s_playlistManager.InsertAt(nTag, i - offset);
                                    nTag = s_playlistDlg.m_cachedPlaylistTags->GetAt(i);
                                    s_playlistDlg.m_cachedPlaylistTags->RemoveAt(i);
                                    s_playlistDlg.m_cachedPlaylistTags->InsertAt(i - offset, nTag);
                                    SetSel(i, FALSE);
                                    SetSel(i - offset, TRUE);

                                    if (s_playlistManager.GetCurrent() == i)
                                        s_playlistManager.SetCurrent(i - offset);
                                    else if (s_playlistManager.GetCurrent() < i && s_playlistManager.GetCurrent() >= (i - offset))
                                        s_playlistManager.incCurrent();
                                }
                            }
                    }
                    if (m_posfrom < nTargetIndex)
                    {
                        int offset = nTargetIndex - m_posfrom;
                        if (!GetSel(GetCount() - 1))
                            for (int i = (GetCount() - 2); i >= 0; i--)
                            {
                                if (GetSel(i))
                                {
                                    offset = (i + offset < GetCount() ? offset : GetCount() - 1 - i);
                                    CachedTag nTag = s_playlistManager.Get(i);
                                    s_playlistManager.RemoveAt(i);
                                    s_playlistManager.InsertAt(nTag, i + offset);
                                    nTag = s_playlistDlg.m_cachedPlaylistTags->GetAt(i);
                                    s_playlistDlg.m_cachedPlaylistTags->RemoveAt(i);
                                    s_playlistDlg.m_cachedPlaylistTags->InsertAt(i + offset, nTag);
                                    SetSel(i, FALSE);
                                    SetSel(i + offset, TRUE);

                                    if (s_playlistManager.GetCurrent() == i)
                                        s_playlistManager.SetCurrent(i + offset);
                                    else if (s_playlistManager.GetCurrent() > i && s_playlistManager.GetCurrent() <= (i + offset))
                                        s_playlistManager.decCurrent();
                                }
                            }
                    }
                    Invalidate();
                    m_posfrom = nTargetIndex;
                }
            }
            ::GetCursorPos(&point2);
            ScreenToClient(&point2);
            nTargetIndex = ItemFromPoint(point, q);
            if (point2.y - rect.top > 0 && point2.y - rect.top < 3 || rect.bottom - point2.y > 0 && rect.bottom - point2.y < 3)
                RedrawWindow();
            else
                break;
        }
    }
    CListBox::OnMouseMove(nFlags, point);
}

void CPLLB::OnPaint()
{
    CListBox::OnPaint();
    s_playlistDlg.ShowTimes();
}

void CPLLB::OnRButtonDown(UINT nFlags, CPoint point)
{
    BOOL q;
    m_posfrom = ItemFromPoint(point, q);
    if (q)
        m_posfrom = -1;
    if (!(nFlags & MK_CONTROL) && !(nFlags & MK_SHIFT))
        SetSel(-1, FALSE);
    int last = GetCaretIndex();
    if (nFlags & MK_SHIFT)
    {
        if (last != m_posfrom)
        {
            SetSel(-1, FALSE);
            if (m_posfrom < last)
                SelItemRange(TRUE, m_posfrom, last);
            else
                SelItemRange(TRUE, last, m_posfrom);
        }
    }
    else if (nFlags & MK_CONTROL)
    {
        SetCaretIndex(m_posfrom);
        SetSel(m_posfrom, !GetSel(m_posfrom));
    }
    else if (m_posfrom > -1)
    {
        SetCaretIndex(m_posfrom);
        SetSel(m_posfrom, TRUE);
    }
    s_playlistDlg.ShowFileName();
}

void CPLLB::OnRButtonUp(UINT nFlags, CPoint point)
{
    CMenu* pPopup,* menu = new CMenu();
    menu->LoadMenu(IDR_PLPOPUP);
    pPopup = menu->GetSubMenu(0);
    m_Helper.translateMenu(pPopup, IDR_PLPOPUP);
    int i = s_playlistDlg.m_playlistAt.GetFirstSelected();
    if (i != -1)
    {
        CString tmp;
        GetText(i, tmp);
        i = _wtoi((LPCTSTR)tmp);
    }
    pPopup->EnableMenuItem(ID_FILE_REMOVE, i == -1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
    pPopup->EnableMenuItem(ID_FILE_EDITTAG, i == -1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
    pPopup->EnableMenuItem(ID_TRACKS_SELECTION_CREATENEWPLAYLIST, i == -1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
    pPopup->EnableMenuItem(ID_TRACKS_SELECTION_ADDTOEXISTENTPLAYLIST, i == -1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);

    ClientToScreen(&point);
    pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_LEFTBUTTON | TPM_RIGHTBUTTON, point.x, point.y, &s_playlistDlg);
    menu->DestroyMenu();
    delete menu;
}

