#include "stdafx.h"
#include "resource.h"       // main symbols
#include "PlaylistManager.h"
#include "MainHelper.h"
#include "MainWnd.h"
#include "PlaylistDlg.h"
#include "MultiDllInput.h"
#include "WAEmuWnd.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

BOOL operator ==(CachedTag& first, CachedTag& second)
{
    CString strFirst = first.GetSortPath();
    CString strSecond = second.GetSortPath();

    return (strFirst.CompareNoCase(strSecond) == 0);
}

BOOL operator >(CachedTag& first, CachedTag& second)
{
    CString strFirst = first.GetSortPath();
    CString strSecond = second.GetSortPath();

    CString firstPath = _T("");
    if (strFirst.ReverseFind('\\') != -1)
        firstPath = strFirst.Left(strFirst.ReverseFind('\\'));
    CString secondPath = _T("");
    if (strSecond.ReverseFind('\\') != -1)
        secondPath = strSecond.Left(strSecond.ReverseFind('\\'));
    CString firstFilename = strFirst.Right(strFirst.GetLength() - strFirst.ReverseFind('\\') - 1);
    CString secondFilename = strSecond.Right(strSecond.GetLength() - strSecond.ReverseFind('\\') - 1);

    if (firstPath.CompareNoCase(secondPath) != 0)
    {
        if (firstPath.CompareNoCase(secondPath) > 0)
        {
            if (firstPath.Find(secondPath) == 0)
                return FALSE;
            else
                return TRUE;
        }
        else
        {
            if (secondPath.Find(firstPath) == 0)
                return TRUE;
            else
                return FALSE;
        }
    }
    return (firstFilename.CompareNoCase(secondFilename) > 0);
}

BOOL operator <(CachedTag& first, CachedTag& second)
{
    CString strFirst = first.GetSortPath();
    CString strSecond = second.GetSortPath();

    CString firstPath = _T("");
    if (strFirst.ReverseFind('\\') != -1)
        firstPath = strFirst.Left(strFirst.ReverseFind('\\'));
    CString secondPath = _T("");
    if (strSecond.ReverseFind('\\') != -1)
        secondPath = strSecond.Left(strSecond.ReverseFind('\\'));
    CString firstFilename = strFirst.Right(strFirst.GetLength() - strFirst.ReverseFind('\\') - 1);
    CString secondFilename = strSecond.Right(strSecond.GetLength() - strSecond.ReverseFind('\\') - 1);

    if (firstPath.CompareNoCase(secondPath) != 0)
    {
        if (firstPath.CompareNoCase(secondPath) < 0)
        {
            if (secondPath.Find(firstPath) == 0)
                return FALSE;
            else
                return TRUE;
        }
        else
        {
            if (firstPath.Find(secondPath) == 0)
                return TRUE;
            else
                return FALSE;
        }
    }
    return (firstFilename.CompareNoCase(secondFilename) < 0);
}


CachedTag::CachedTag()
{
    m_sortPath = "";
    m_duration = 0;
    m_read = false;
    m_artist = "";
    m_album = "";
    m_comment = "";
    m_composer = "";
    m_playlist = "";
    m_title = "";
    m_year = "";
    TCHAR path[MAX_PATH];
    _tcscpy(path, (TCHAR *)m_playlist.GetString());
    _tsplitpath(path, m_drive, m_dir, m_fname, m_ext);
}

CachedTag::CachedTag(const CachedTag& copie)
{
    m_sortPath = copie.m_sortPath;
    m_read = copie.m_read;
    m_duration = copie.m_duration;
    m_artist = copie.m_artist;
    m_album = copie.m_album;
    m_comment = copie.m_comment;
    m_composer = copie.m_composer;
    m_playlist = copie.m_playlist;
    m_title = copie.m_title;
    m_year = copie.m_year;
    TCHAR path[MAX_PATH];
    _tcscpy(path, (TCHAR *)m_playlist.GetString());
    _tsplitpath(path, m_drive, m_dir, m_fname, m_ext);
}

CachedTag::CachedTag(CString nPlaylist)
{
    m_sortPath = "";
    m_read = false;
    m_duration = 0;
    m_artist = "";
    m_album = "";
    m_comment = "";
    m_composer = "";
    m_playlist = nPlaylist;
    m_title = "";
    m_year = "";
    TCHAR path[MAX_PATH];
    _tcscpy(path, (TCHAR *)m_playlist.GetString());
    _tsplitpath(path, m_drive, m_dir, m_fname, m_ext);
}

CachedTag::~CachedTag()
{
}

void CachedTag::copy(CachedTag copie)
{
    m_sortPath = copie.m_sortPath;
    m_read = copie.m_read;
    m_duration = copie.m_duration;
    m_artist = copie.m_artist;
    m_album = copie.m_album;
    m_comment = copie.m_comment;
    m_composer = copie.m_composer;
    m_playlist = copie.m_playlist;
    m_title = copie.m_title;
    m_year = copie.m_year;
    TCHAR path[MAX_PATH];
    _tcscpy(path, (TCHAR *)m_playlist.GetString());
    _tsplitpath(path, m_drive, m_dir, m_fname, m_ext);
}

CString CachedTag::GetAlbum()
{
    if (!m_read)
        ReadTag();
    return m_album;
}

CString CachedTag::GetArtist(bool fastDisplay)
{
    if (fastDisplay && !m_read)
        return _T("");
    if (!m_read)
        ReadTag();
    return m_artist;
}

CString CachedTag::GetComment()
{
    if (!m_read)
        ReadTag();
    return m_comment;
}

CString CachedTag::GetComposer()
{
    if (!m_read)
        ReadTag();
    return m_composer;
}

CString CachedTag::GetFilenameExt()
{
    CString filename = m_playlist;
    TCHAR path[MAX_PATH];
    _tcscpy(path, (TCHAR *)m_playlist.GetString());
    _tsplitpath(path, m_drive, m_dir, m_fname, m_ext);
    filename.Format(_T("%s%s"), m_fname, m_ext);
    if (_tcsstr(path, _T("://")) != NULL)
        filename.Format(_T("%s"), path);
    return filename;
}

CString CachedTag::GetDuration(bool fastDisplay)
{
    if (fastDisplay && !m_read && m_duration == 0)
        return _T("");
    if (!fastDisplay && !m_read)
        ReadTag();
    CString dur = _T("mm:ss");
    if (m_duration != 0)
        dur.Format(_T(" %02d:%02d"), (int)(m_duration / 60), m_duration % 60);
    else
        dur = "";
    return dur;
}

int CachedTag::GetDurationI()
{
    return m_duration;
}

CString CachedTag::GetPlaylist()
{
    return m_playlist;
}

bool CachedTag::GetRead()
{
    return m_read;
}

CString CachedTag::GetTag(int ShowingMode, bool fastDisplay)
{
    CString CurrentSong = _T("");

    if (ShowingMode == 2)
        CurrentSong = GetFilenameExt();
    else if (ShowingMode == 1)
    {
        if (GetArtist(fastDisplay).GetLength() == 0)
            CurrentSong = GetTitle(fastDisplay);
        else
        {
            CurrentSong.Format(_T("%s - %s"), GetArtist(fastDisplay), GetTitle(fastDisplay));
        }
    }
    else if (ShowingMode == 0)
    {
        TCHAR info[MAX_PATH];
        int length = 0;
        stdIn.GetFileInfo((TCHAR *)GetPlaylist().GetString(), info, &length);
        CurrentSong.Format(_T("%s"), info);
        CurrentSong = GetTitle(fastDisplay);
    }
    return CurrentSong;
}

CString CachedTag::GetTitle(bool fastDisplay)
{
    if (fastDisplay && !m_read && m_title == "")
        return GetFilename();
    if (!m_read && m_title == "")
        ReadTag();
    return m_title;
}

CString CachedTag::GetYear()
{
    if (!m_read)
        ReadTag();
    return m_year;
}

CString CachedTag::GetDrive()
{
    CString tmp;
    tmp.Format(_T("%s"), m_drive);
    return tmp;
}

CString CachedTag::GetDir()
{
    CString tmp = _T("");
    if (m_playlist.Find(_T("://")) == -1)
        tmp.Format(_T("%s"), m_dir);
    return tmp;
}

CString CachedTag::GetFilename()
{
    CString tmp;
    if (_tcsstr(m_playlist, _T("://")) != NULL)
        tmp.Format(_T("%s"), m_playlist);
    else
        tmp.Format(_T("%s"), m_fname);
    return tmp;
}

CString CachedTag::GetExt()
{
    CString tmp;
    tmp.Format(_T("%s"), m_ext);
    return tmp;
}

void CachedTag::ReadTag(bool forceRead)
{
    if (m_read && !forceRead)
        return;
    TCHAR filename[MAX_PATH];
    TCHAR meta[10];
    TCHAR ret[MAX_PATH];

    wcscpy(filename, (LPCTSTR)GetPlaylist());

    extendedFileInfoStructW eFIStruct;

    eFIStruct.filename = filename;
    eFIStruct.metadata = meta;
    eFIStruct.ret = ret;
    eFIStruct.ret[0] = 0;
    eFIStruct.retlen = 0;

    stdIn.GetFileInfo(filename, (TCHAR *)ret, (int*)&eFIStruct.retlen);
    SetTitle(eFIStruct.ret);
    SetDuration(eFIStruct.retlen / 1000);

    eFIStruct.retlen = MAX_PATH;

    wcscpy(meta, _T("length"));
    wmemset(eFIStruct.ret, 0, MAX_PATH);
    if (stdIn.winampGetExtendedFileInfoW(eFIStruct) && wcslen(eFIStruct.ret) > 0)
    {
        if (_wtoi(eFIStruct.ret) != 0)
        {
            SetDuration(_wtoi(eFIStruct.ret) / 1000);
        }
    }
    else
    {
        SetDuration(-1);
    }

    eFIStruct.retlen = MAX_PATH;
    wcscpy(meta, _T("title"));
    wmemset(eFIStruct.ret, 0, MAX_PATH);
    if (stdIn.winampGetExtendedFileInfoW(eFIStruct) && wcslen(eFIStruct.ret) > 0)
    {
        SetTitle(eFIStruct.ret);
    }
    else
    {
        SetTitle(GetFilename());
    }
    eFIStruct.retlen = MAX_PATH;
    wcscpy(meta, _T("artist"));
    wmemset(eFIStruct.ret, 0, MAX_PATH);
    if (stdIn.winampGetExtendedFileInfoW(eFIStruct) && wcslen(eFIStruct.ret) > 0)
    {
        SetArtist(eFIStruct.ret);
    }
    else
    {
        SetArtist(_T(""));
    }
    eFIStruct.retlen = MAX_PATH;
    wcscpy(meta, _T("comment"));
    wmemset(eFIStruct.ret, 0, MAX_PATH);
    if (stdIn.winampGetExtendedFileInfoW(eFIStruct) && wcslen(eFIStruct.ret) >= 0)
    {
        SetComment(eFIStruct.ret);
    }
    else
    {
        SetComment(_T(""));
    }
    eFIStruct.retlen = MAX_PATH;
    wcscpy(meta, _T("album"));
    wmemset(eFIStruct.ret, 0, MAX_PATH);
    if (stdIn.winampGetExtendedFileInfoW(eFIStruct) && wcslen(eFIStruct.ret) >= 0)
    {
        SetAlbum(eFIStruct.ret);
    }
    else
    {
        SetAlbum(_T(""));
    }
    eFIStruct.retlen = MAX_PATH;
    wcscpy(meta, _T("year"));
    wmemset(eFIStruct.ret, 0, MAX_PATH);
    if (stdIn.winampGetExtendedFileInfoW(eFIStruct) && wcslen(eFIStruct.ret) >= 0)
    {
        SetYear(eFIStruct.ret);
    }
    else
    {
        SetYear(_T(""));
    }
    m_read = true;
}
void CachedTag::SetAlbum(CString nAlbum)
{
    m_album = nAlbum;
}

void CachedTag::SetArtist(CString nArtist)
{
    m_artist = nArtist;
}

void CachedTag::SetComment(CString nComment)
{
    m_comment = nComment;
}

void CachedTag::SetComposer(CString nComposer)
{
    m_composer = nComposer;
}

void CachedTag::SetDuration(int nDuration)
{
    m_duration = nDuration;
}

void CachedTag::SetPlaylist(CString nplaylist)
{
    m_playlist = nplaylist;
    TCHAR path[MAX_PATH];
    _tcscpy(path, (TCHAR *)m_playlist.GetString());
    _tsplitpath(path, m_drive, m_dir, m_fname, m_ext);
}

void CachedTag::SetTitle(CString nTitle)
{
    m_title = nTitle;
}

void CachedTag::SetYear(CString nYear)
{
    m_year = nYear;
}

void CachedTag::SetRead(bool nRead)
{
    m_read = nRead;
}

void CachedTag::SetSortPath(CString nSortPath)
{
    m_sortPath = nSortPath;
    m_sortPath.Replace(_T("%7"), GetFilenameExt());
    m_sortPath.Replace(_T("%8"), GetDrive());
    m_sortPath.Replace(_T("%9"), GetDir());
    if (m_sortPath.Find(_T("%")) != -1)
    {
        if (GetArtist() != "")
            m_sortPath.Replace(_T("%1"), GetArtist());
        else
            m_sortPath.Replace(_T("%1"), _T(""));
        if (GetTitle() != "")
            m_sortPath.Replace(_T("%2"), GetTitle());
        else
            m_sortPath.Replace(_T("%2"), _T(""));
        if (GetAlbum() != "")
            m_sortPath.Replace(_T("%3"), GetAlbum());
        else
            m_sortPath.Replace(_T("%3"), _T(""));
        if (GetYear() != "")
            m_sortPath.Replace(_T("%4"), GetYear());
        else
            m_sortPath.Replace(_T("%4"), _T(""));
        if (GetComment() != "")
            m_sortPath.Replace(_T("%5"), GetComment());
        else
            m_sortPath.Replace(_T("%5"), _T(""));
        if (GetDuration() != "")
            m_sortPath.Replace(_T("%6"), GetDuration());
        else
            m_sortPath.Replace(_T("%6"), _T(""));
    }
    m_sortPath.Replace(_T("\\\\"), _T("\\"));
}

CString CachedTag::GetSortPath()
{
    return m_sortPath;
}

CPlaylistManager::CPlaylistManager()
{
    m_cachedTags = new CSortedArray<CachedTag, CachedTag&>();
    m_sortedCachedTags = new CSortedArray<CachedTag, CachedTag&>();
    m_current = -1;
}

CPlaylistManager::~CPlaylistManager()
{
    m_cachedTags->RemoveAll();
    delete m_cachedTags;
    delete m_sortedCachedTags;
}

int CPlaylistManager::GetCurrent()
{
    return m_current;
}

void CPlaylistManager::SetCurrent(int cur)
{
    m_current = cur;
    if (cur == -1)
    {
        CachedTag tmp(_T(""));
        m_currentTag = tmp;
    }
    else
        m_currentTag = Get(m_current);
}

int CPlaylistManager::decCurrent()
{
    SetCurrent(m_current - 1);
    return m_current;
}
int CPlaylistManager::incCurrent()
{
    SetCurrent(m_current + 1);
    return m_current;
}

CachedTag CPlaylistManager::GetCurrentTag()
{
    return m_currentTag;
}

void CPlaylistManager::Add(CString tag, int index)
{
    CAutoLock lock(&m_critSec);
    if (m_cachedTags->GetSize() == MAXFILE)
        return;
    try
    {
        CachedTag tmp(tag);
        if (index == -1)
            InsertAt(tmp, m_cachedTags->GetSize());
        else
        {
            m_cachedTags->GetAt(index).copy(tmp);
        }
    }
    catch (...)
    {
    }
}

void CPlaylistManager::Add(CachedTag tag, int index)
{
    CAutoLock lock(&m_critSec);
    if (m_cachedTags->GetSize() == MAXFILE)
        return;
    try
    {
        if (index == -1)
            InsertAt(tag, m_cachedTags->GetSize());
        else
        {
            m_cachedTags->GetAt(index).copy(tag);
        }
    }
    catch (...)
    {
    }
}

void CPlaylistManager::InsertAt(CachedTag tag, int index)
{
    CAutoLock lock(&m_critSec);
    if (m_cachedTags->GetSize() == MAXFILE)
        return;
    try
    {
        CachedTag tmp(tag);
        m_cachedTags->InsertAt(index, tmp);
    }
    catch (...)
    {
    }
}

CachedTag CPlaylistManager::Get(int index)
{
    try
    {
        CachedTag tmp;
        if (index >= 0 && index < m_cachedTags->GetSize())
        {
            return m_cachedTags->GetAt(index);
        }
        else
            return tmp;
    }
    catch (...)
    {
        CachedTag tmp;
        tmp.SetAlbum(_T(""));
        tmp.SetArtist(_T(""));
        tmp.SetComment(_T(""));
        tmp.SetComposer(_T(""));
        tmp.SetDuration(0);
        tmp.SetPlaylist(_T(""));
        tmp.SetTitle(_T(""));
        tmp.SetYear(_T(""));
        return tmp;
    }
}

int CPlaylistManager::GetIndex(CString filename)
{
    int posIndex;
    //TRACE(_T("GetIndex %s %d\n"), filename, GetSize());
    if (GetSize() == 0)
        return -1;
    for (posIndex = 0; posIndex < GetSize() && Get(posIndex).GetPlaylist().Compare(filename); posIndex++)
        ;
    if (posIndex == GetSize())
        return -1;
    return posIndex;
}

CString CPlaylistManager::GetAlbum(int index)
{
    try
    {
        if (index >= 0 && index < m_cachedTags->GetSize())
        {
            return m_cachedTags->GetAt(index).GetAlbum();
        }
        else
            return _T("");
    }
    catch (...)
    {
        return _T("");
    }
}

CString CPlaylistManager::GetArtist(int index, bool fastDisplay)
{
    try
    {
        if (index >= 0 && index < m_cachedTags->GetSize())
        {
            return m_cachedTags->GetAt(index).GetArtist(fastDisplay);
        }
        else
            return _T("");
    }
    catch (...)
    {
        return _T("");
    }
}

void CPlaylistManager::SetArtist(int index, CString artist)
{
    CAutoLock lock(&m_critSec);
    try
    {
        if (index >= 0 && index < m_cachedTags->GetSize())
        {
            m_cachedTags->GetAt(index).SetArtist(artist);
        }
    }
    catch (...)
    {
    }
}

CString CPlaylistManager::GetComment(int index)
{
    try
    {
        if (index >= 0 && index < m_cachedTags->GetSize())
        {
            return m_cachedTags->GetAt(index).GetComment();
        }
        else
            return _T("");
    }
    catch (...)
    {
        return _T("");
    }
}

void CPlaylistManager::SetComment(int index, CString comment)
{
    CAutoLock lock(&m_critSec);
    try
    {
        if (index >= 0 && index < m_cachedTags->GetSize())
        {
            m_cachedTags->GetAt(index).SetComment(comment);
        }
    }
    catch (...)
    {
    }
}

CString CPlaylistManager::GetComposer(int index)
{
    try
    {
        if (index >= 0 && index < m_cachedTags->GetSize())
        {
            return m_cachedTags->GetAt(index).GetComposer();
        }
        else
            return _T("");
    }
    catch (...)
    {
        return _T("");
    }
}

CString CPlaylistManager::GetDuration(int index, bool fastDisplay)
{
    try
    {
        if (index >= 0 && index < m_cachedTags->GetSize())
        {
            return m_cachedTags->GetAt(index).GetDuration(fastDisplay);
        }
        else
            return _T("");
    }
    catch (...)
    {
        return _T("");
    }
}

void CPlaylistManager::SetDuration(int index, int nDuration)
{
    CAutoLock lock(&m_critSec);
    try
    {
        if (index >= 0 && index < m_cachedTags->GetSize())
        {
            m_cachedTags->GetAt(index).SetDuration(nDuration);
        }
    }
    catch (...)
    {
    }
}
int CPlaylistManager::GetDurationI(int index)
{
    try
    {
        if (index >= 0 && index < m_cachedTags->GetSize())
        {
            return m_cachedTags->GetAt(index).GetDurationI();
        }
        else
            return 0;
    }
    catch (...)
    {
        return 0;
    }
}

CString CPlaylistManager::GetFilenameExt(int index)
{
    try
    {
        if (index >= 0 && index < m_cachedTags->GetSize())
        {
            return m_cachedTags->GetAt(index).GetFilenameExt();
        }
        else
            return _T("");
    }
    catch (...)
    {
        return _T("");
    }
}

CString CPlaylistManager::GetPlaylist(int index)
{
    try
    {
        if (index >= 0 && index < m_cachedTags->GetSize())
        {
            return m_cachedTags->GetAt(index).GetPlaylist();
        }
        else
            return _T("");
    }
    catch (...)
    {
        return _T("");
    }
}

bool CPlaylistManager::GetRead(int index)
{
    try
    {
        if (index >= 0 && index < m_cachedTags->GetSize())
        {
            return m_cachedTags->GetAt(index).GetRead();
        }
        return false;
    }
    catch (...)
    {
        return false;
    }
}
int CPlaylistManager::GetShowingMode()
{
    return m_showingMode;
}

CString CPlaylistManager::GetTag(int index, bool fastDisplay)
{
    try
    {
        if (index >= 0 && index < m_cachedTags->GetSize())
        {
            return m_cachedTags->GetAt(index).GetTag(m_showingMode, fastDisplay);
        }
        else
            return _T("");
    }
    catch (...)
    {
        return _T("");
    }
}

CString CPlaylistManager::GetTitle(int index, bool fastDisplay)
{
    try
    {
        if (index >= 0 && index < m_cachedTags->GetSize())
        {
            return m_cachedTags->GetAt(index).GetTitle(fastDisplay);
        }
        else
            return _T("");
    }
    catch (...)
    {
        return _T("");
    }
}

void CPlaylistManager::SetTitle(int index, CString title)
{
    CAutoLock lock(&m_critSec);
    try
    {
        if (index >= 0 && index < m_cachedTags->GetSize())
        {
            m_cachedTags->GetAt(index).SetTitle(title);
        }
    }
    catch (...)
    {
    }
}

CString CPlaylistManager::GetYear(int index)
{
    try
    {
        if (index >= 0 && index < m_cachedTags->GetSize())
        {
            return m_cachedTags->GetAt(index).GetYear();
        }
        else
            return _T("");
    }
    catch (...)
    {
        return _T("");
    }
}

CString CPlaylistManager::GetDrive(int index)
{
    try
    {
        if (index >= 0 && index < m_cachedTags->GetSize())
        {
            return m_cachedTags->GetAt(index).GetDrive();
        }
        else
            return _T("");
    }
    catch (...)
    {
        return _T("");
    }
}

CString CPlaylistManager::GetDir(int index)
{
    try
    {
        if (index >= 0 && index < m_cachedTags->GetSize())
        {
            return m_cachedTags->GetAt(index).GetDir();
        }
        else
            return _T("");
    }
    catch (...)
    {
        return _T("");
    }
}

CString CPlaylistManager::GetFilename(int index)
{
    try
    {
        if (index >= 0 && index < m_cachedTags->GetSize())
        {
            return m_cachedTags->GetAt(index).GetFilename();
        }
        else
            return _T("");
    }
    catch (...)
    {
        return _T("");
    }
}

CString CPlaylistManager::GetExt(int index)
{
    try
    {
        if (index >= 0 && index < m_cachedTags->GetSize())
        {
            return m_cachedTags->GetAt(index).GetExt();
        }
        else
            return _T("");
    }
    catch (...)
    {
        return _T("");
    }
}

CString CPlaylistManager::GetSortPath(int index)
{
    try
    {
        if (index >= 0 && index < m_cachedTags->GetSize())
        {
            return m_cachedTags->GetAt(index).GetSortPath();
        }
        else
            return _T("");
    }
    catch (...)
    {
        return _T("");
    }
}

int CPlaylistManager::GetSize()
{
    return m_cachedTags->GetSize();
}

void CPlaylistManager::SetShowingMode(int nShowingMode)
{
    m_showingMode = nShowingMode;
}

bool CPlaylistManager::StopReadAll()
{
    bool toret = (m_hTagsThread != NULL);
    if (toret)
    {
        ::TerminateThread(m_hTagsThread, 0);
        TRACE(_T("Fin force ReadAllTagsThread"));
        s_playlistManager.m_critSec.Unlock();
        m_hTagsThread = NULL;
    }
    return toret;
}

bool CPlaylistManager::StopPerformAutomation()
{
    bool toret = (m_hAutomationThread != NULL);
    if (toret)
    {
        ::TerminateThread(m_hAutomationThread, 0);
        m_hAutomationThread = NULL;
    }
    return toret;
}

void CPlaylistManager::ReadAllTagsThread()
{
    CAutoLock lock(&m_critSec);
    TRACE(_T("Dbut ReadAllTagsThread"));
    try
    {
        for (UINT i = 0; i < (UINT)GetSize(); i++)
        {
            ReadTagAT(i, true);
        }
        if (s_playlistDlg.IsWindowVisible())
        {
            s_playlistDlg.ShowTimes();
            s_playlistDlg.m_playlistAt.RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW);
        }
    }
    catch (...)
    {
    }
    m_hTagsThread = NULL;
    TRACE(_T("Fin ReadAllTagsThread"));
}

void CPlaylistManager::ReadAllTags()
{
    CreateMemberThread(this, &CPlaylistManager::ReadAllTagsThread, &m_hTagsThread);
}

void CPlaylistManager::PerformAutomationThread()
{
    TRACE(_T("PerformAutomationThread"));
    if (s_mainWnd.m_autRemove || s_mainWnd.m_autRemovedup || s_mainWnd.m_autRescan || s_mainWnd.m_autSort)
        s_playlistManager.StopReadAll();

    if (s_playlistDlg.IsWindowVisible())
    {
        s_playlistDlg.StopRemoveDuplicateEntries();
        s_playlistDlg.StopRemovene();
        s_playlistDlg.StopSort();
    }

    if (s_mainWnd.m_autRemove)
    {
        TRACE(_T("Remove not existant files"));
        s_playlistDlg.OnArrangeRemovene();
    }

    if (s_mainWnd.m_autSort)
    {
        TRACE(_T("Sort playlist"));
        s_playlistDlg.OnArrangeSortEngine(s_mainWnd.m_sortTemplate);
    }

    if (s_mainWnd.m_autRemovedup)
    {
        TRACE(_T("Remove duplicate files"));
        s_playlistDlg.OnArrangeRemoveDuplicateEntries();
    }


    if (s_mainWnd.m_autRescan)
    {
        int PLLPrev = s_playlistManager.GetSize();
        for (int i = 0; i < s_mainWnd.m_autDirs.GetSize(); i++)
        {
            TRACE(_T("Rescan folders %s"), s_mainWnd.m_autDirs[i]);
            s_mainWnd.FindMP3Files(s_mainWnd.m_autDirs[i], true);
        }

        s_playlistManager.QuickSort(s_mainWnd.m_sortTemplate);

        if ((s_mainWnd.m_randomFirst || s_mainWnd.m_shuffleMode) && PLLPrev == 0)
        {
            s_mainWnd.m_firstInMyRandom = s_mainWnd.MyRandom(true, true);
            s_playlistManager.SetCurrent(s_mainWnd.m_firstInMyRandom);
        }
        else if (PLLPrev == 0)
        {
            s_playlistManager.SetCurrent(0);
            s_mainWnd.m_firstInMyRandom = 0xFFFFFFFF;
        }

        if (s_playlistDlg.IsWindowVisible())
        {
            if (s_playlistManager.GetSize() != PLLPrev)
            {
                ::SendMessage(s_playlistDlg.m_hWnd, WM_COMMAND, ID_TRACKS_REFRESH, 0);
            }
            s_playlistDlg.m_playlistAt.Invalidate();
        }

        if (PLLPrev == 0)
        {
            if (stdIn.GetPlayerMode() != PLAYER_MODE_CLOSE)
                ::SendMessage(WAEmuWnd.m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);

            if (s_playlistManager.GetCurrent() < s_playlistManager.GetSize() - 1)
                s_playlistManager.incCurrent();
            else
            {
                s_playlistManager.SetCurrent(0);
            }
            Sleep(77);
            s_playlistManager.ReadTagAT(s_playlistManager.GetCurrent());
        }
    }
    m_hAutomationThread = NULL;
    TRACE(_T("End PerformAutomationThread"));
}

void CPlaylistManager::PerformAutomation()
{
    TRACE(_T("PerformAutomation"));
    if (s_playlistDlg.m_removeDuplicateEntriesThreadHan != NULL || s_playlistDlg.m_removeneThreadHan != NULL || s_playlistDlg.m_sortThreadHan != NULL)
        return;

    CreateMemberThread(this, &CPlaylistManager::PerformAutomationThread, &m_hAutomationThread);
}
void CPlaylistManager::ReadTagAT(int index, bool forceRead)
{
    if (index >= 0 && index < m_cachedTags->GetSize())
    {
        m_cachedTags->GetAt(index).ReadTag(forceRead);
    }
}
void CPlaylistManager::RemoveAt(int index)
{
    CAutoLock lock(&m_critSec);
    m_cachedTags->RemoveAt(index);
}

void CPlaylistManager::RemoveAll()
{
    CAutoLock lock(&m_critSec);
    StopReadAll();
    StopPerformAutomation();
    m_cachedTags->RemoveAll();
}

void CPlaylistManager::QuickSort(CString sortTemplate)
{
    CAutoLock lock(&m_critSec);
    TRACE(_T("Dbut QuickSort"));
    CString playlist = _T("");
    if (s_playlistManager.GetCurrent() >= 0 && s_playlistManager.GetCurrent() < m_cachedTags->GetSize())
        playlist = GetPlaylist(s_playlistManager.GetCurrent());

    if (sortTemplate == _T(""))
    {
        sortTemplate = _T("%7");
    }

    CSortedArray<CachedTag, CachedTag&>* tmp;

    for (int i = 0; i< m_cachedTags->GetSize(); i++)
    {
        CachedTag cachedTag(m_cachedTags->GetAt(i));
        cachedTag.SetSortPath(sortTemplate);
        m_sortedCachedTags->OrderedInsert(cachedTag);
    }

    tmp = m_cachedTags;
    m_cachedTags = m_sortedCachedTags;
    m_sortedCachedTags = tmp;
    m_sortedCachedTags->RemoveAll();

    if (s_playlistManager.GetCurrent() >= 0 && s_playlistManager.GetCurrent() < m_cachedTags->GetSize())
        s_playlistManager.SetCurrent(GetIndex(playlist));
    TRACE(_T("Fin QuickSort"));
}

int CPlaylistManager::Compare(CachedTag& element1, CachedTag& element2)
{
    if (element1 > element2)
        return 1;
    if (element1 < element2)
        return -1;
    return 0;
}

void CPlaylistManager::RemoveDuplicateEntries()
{
    CAutoLock lock(&m_critSec);
    TRACE(_T("Dbut RemoveDuplicateEntries"));
    int i = 0, j;
    //Tri d'abord
    QuickSort(s_mainWnd.m_sortTemplate);

    while (i < GetSize() - 1 && GetSize() > 0)
    {
        j = i + 1;
        while (GetPlaylist(i) == GetPlaylist(j))
        {
            RemoveAt(j);

            if (j < m_current)
                m_current--;
            else if (j == m_current)
                m_current = i;
        }
        i++;
    }
    TRACE(_T("Fin RemoveDuplicateEntries"));
}

void CPlaylistManager::Removene()
{
    CAutoLock lock(&m_critSec);
    TRACE(_T("Dbut Removene"));
    CString playlist = _T("");
    if (s_playlistManager.GetCurrent() >= 0 && s_playlistManager.GetCurrent() < m_cachedTags->GetSize())
        playlist = GetPlaylist(s_playlistManager.GetCurrent());
    int i = 0;
    while (i < GetSize() && GetSize() > 0)
    {
        if (!s_mainWnd.FileExists(GetPlaylist(i)))
        {
            RemoveAt(i);
            if (i < m_current)
                m_current--;
        }
        else
            i++;
    }
    if (s_playlistManager.GetCurrent() >= 0 && s_playlistManager.GetCurrent() < m_cachedTags->GetSize())
        s_playlistManager.SetCurrent(GetIndex(playlist));
    TRACE(_T("Fin Removene"));
}
