#ifndef PLAYLISTMANAGER_H
#define PLAYLISTMANAGER_H

#define PAS 10
#define MAXFILE 10000

#include "QArray.h"

class CachedTag
{

public:
    CachedTag();
    ~CachedTag();
    CachedTag(CString nPlaylist);
    CachedTag(const CachedTag& copie);
    void copy(CachedTag copy);
    CString GetAlbum();
    CString GetArtist(bool fastDisplay = false);
    CString GetComment();
    CString GetComposer();
    CString GetDuration(bool fastDisplay = false);
    int GetDurationI();
    CString GetFilenameExt();
    CString GetPlaylist();
    bool GetRead();
    CString GetTag(int ShowingMode = 0, bool fastDisplay = false);
    CString GetTitle(bool fastDisplay = false);
    CString GetYear();
    CString GetDrive();
    CString GetDir();
    CString GetFilename();
    CString GetExt();
    CString GetSortPath();
    void ReadTag(bool forceRead = false);
    void SetAlbum(CString nAlbum);
    void SetArtist(CString nArtist);
    void SetComment(CString nComment);
    void SetComposer(CString nComposer);
    void SetDuration(int nDuration);
    void SetPlaylist(CString nPlaylist);
    void SetTitle(CString nTitle);
    void SetYear(CString nYear);
    void SetRead(bool nRead);
    void SetSortPath(CString nSortPath);

private:
    CString m_sortPath;
    bool m_read;
    int m_duration;
    CString m_artist;
    CString m_album;
    CString m_comment;
    CString m_composer;
    CString m_playlist;
    CString m_title;
    CString m_year;
    TCHAR m_drive[MAX_PATH];
    TCHAR m_dir[MAX_PATH];
    TCHAR m_fname[MAX_PATH];
    TCHAR m_ext[MAX_PATH];
};

class CPlaylistManager
{
public:
    CCritSec m_critSec;
    int GetCurrent();
    void SetCurrent(int cur);
    int decCurrent();
    int incCurrent();
    CachedTag GetCurrentTag();
    CPlaylistManager();
    ~CPlaylistManager();
    void OnStopAll();
    void Add(CString tag, int index = -1);
    void Add(CachedTag tag, int index = -1);
    CachedTag Get(int index);
    int GetIndex(CString filename);
    CString GetAlbum(int index);
    CString GetArtist(int index, bool fastDisplay = false);
    void SetArtist(int index, CString artist);
    CString GetComment(int index);
    void SetComment(int index, CString comment);
    CString GetComposer(int index);
    CString GetDuration(int index, bool fastDisplay = false);
    void SetDuration(int index, int nDuration);
    int GetDurationI(int index);
    CString GetFilenameExt(int index);
    CString GetPlaylist(int index);
    bool GetRead(int index);
    int GetShowingMode();
    int GetSize();
    CString GetTag(int index, bool fastDisplay = false);
    CString GetTitle(int index, bool fastDisplay = false);
    void SetTitle(int index, CString title);
    CString GetYear(int index);
    CString GetDrive(int index);
    CString GetDir(int index);
    CString GetFilename(int index);
    CString GetExt(int index);
    CString GetSortPath(int index);
    void InsertAt(CachedTag tag, int index);
    void PerformAutomation();
    void ReadAllTags();
    void ReadTagAT(int index, bool forceRead = false);
    void RemoveAll();
    void RemoveAt(int index);
    void SetShowingMode(int nShowingMode);
    bool StopPerformAutomation();
    bool StopReadAll();
    void QuickSort(CString sortTemplate);
    int Compare(CachedTag& element1, CachedTag& element2);
    void RemoveDuplicateEntries();
    void Removene();

private:
    CSortedArray<CachedTag, CachedTag&>* m_cachedTags;
    CSortedArray<CachedTag, CachedTag&>* m_sortedCachedTags;
    int m_showingMode;
    int m_current;
    CachedTag m_currentTag;
    HANDLE m_hTagsThread;
    HANDLE m_hAutomationThread;
    void ReadAllTagsThread();
    void PerformAutomationThread();
};
#endif

#define PLAYLIST_STOP_ALL           1024

extern CPlaylistManager s_playlistManager;