// SettingsDialog.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"       // main symbols
#include "aerosubc.h"
#include "aeroglss.h"
#include "SettingsDialog.h"
#include "MainWnd.h"
#include "MainHelper.h"
#include "Draw.h"
#include "CommonFunc.h"
#include "GdiplusWrapper.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#pragma comment(linker, "/SECTION:.rsrc,rw") 

extern CArray<prefsDlgRec*, prefsDlgRec*> listPrefs;

/////////////////////////////////////////////////////////////////////////////
// CSettingsDialog dialog


CSettingsDialog::CSettingsDialog() : CTreePropSheet()
{
    m_needToRestart = false;
    m_changeApparence = false;
    AddPage(&m_settingsDialogGeneral);
    AddPage(&m_settingsDialogHotkeys);
    AddPage(&m_settingsDialogW2File);
    AddPage(&m_settingsDialogMouseClicks);
    AddPage(&m_settingsDialogApparence);
    AddPage(&m_settingsDialogApparenceSkins);
    AddPage(&m_settingsDialogApparenceTooltip);
    AddPage(&m_settingsDialogPlugins);
    AddPage(&m_settingsDialogInput);
    AddPage(&m_settingsDialogOutput);
    AddPage(&m_settingsDialogGen);

    //Cration des pages supplmentaires
    CTestPropertyPage* myPage = NULL;
    for (int i = 0; i < listPrefs.GetSize(); i++)
    {
        prefsDlgRec* prefs = listPrefs[i];
        try
        {
            myPage = new CTestPropertyPage(prefs, prefs->dlgID);
            m_listPropertyPage.Add(myPage);
            AddPage(myPage);
            myPage->InstallHook();
        }
        catch (...)
        {
            listPrefs.RemoveAt(i);
        }
    }

    EnableStackedTabs(FALSE);

    SetTreeViewMode(TRUE, TRUE);
    SetTreeWidth(200);
}

CSettingsDialog::~CSettingsDialog()
{
    delete m_tooltip;
}

IMPLEMENT_MENUXP(CSettingsDialog, CTreePropSheet);

BEGIN_MESSAGE_MAP(CSettingsDialog, CTreePropSheet)
    ON_WM_CLOSE()
    ON_MENUXP_MESSAGES()
    ON_COMMAND(IDOK, OnOk)
    ON_COMMAND(ID_APPLY_NOW, OnApplyNow)
	ON_WM_ERASEBKGND()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSettingsDialog message handlers

BOOL CSettingsDialog::DestroyWindow()
{
    CPropertyPage* myPage;
    if (m_listPropertyPage.GetSize() != 0)
    {
        do
        {
            myPage = m_listPropertyPage[m_listPropertyPage.GetSize() - 1];
            delete myPage;
            myPage = NULL;
            m_listPropertyPage.SetSize(m_listPropertyPage.GetSize() - 1);
        }
        while (m_listPropertyPage.GetSize() != 0);
    }
    return CTreePropSheet::DestroyWindow();
}

void CSettingsDialog::DoDataExchange(CDataExchange* pDX)
{
    CTreePropSheet::DoDataExchange(pDX);
    CMenuXP::SetXPLookNFeel(this, s_mainWnd.m_skin.xpLookNFeel);
    DDX_Control(pDX, IDOK, m_btnOk);
    DDX_Control(pDX, IDCANCEL, m_btnCancel);
    DDX_Control(pDX, ID_APPLY_NOW, m_btnApplyNow);
}

IMPLEMENT_DYNAMIC(CSettingsDialog, CTreePropSheet)

#define BACKGROUND_PROC_STRING _T("BackgroundSubclass")

LRESULT CALLBACK NewWindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    WNDPROC pWndProc = (WNDPROC)GetProp(hWnd, BACKGROUND_PROC_STRING);
    HWND vista = (HWND)GetProp(hWnd, _T("vista"));
    CDwmApiImpl dwmApi;
    if (dwmApi.Initialize() && dwmApi.IsDwmCompositionEnabled())
    {
        switch(message)
        {
            case WM_PAINT:
            case WM_ERASEBKGND:
            {
                if(vista == NULL)
                {
                    LRESULT res = pWndProc(hWnd, message, wParam, lParam);

                    HWND hWndParent = ::GetParent(hWnd);
                    CRect parentClient, rectClient;
                    ::GetWindowRect(hWnd, rectClient);
                    ::GetClientRect(hWndParent, parentClient);
                    CWnd::FromHandle(hWndParent)->ScreenToClient(rectClient);

                    MARGINS marGlassInset =
                    {
                        rectClient.left, parentClient.right - rectClient.right, rectClient.top, parentClient.bottom - rectClient.bottom
                    };
                    SUCCEEDED(dwmApi.DwmExtendFrameIntoClientArea(hWndParent, &marGlassInset));
                    return res;
                }
            }
            case WM_NOTIFY:
            {
                PSHNOTIFY *pshn = (PSHNOTIFY*)lParam;
                if(lParam!=NULL && pshn->hdr.code == PSN_KILLACTIVE && vista == NULL)
                {
                    HWND hWndParent = ::GetParent(hWnd);
                    MARGINS marGlassInset =
                    {
                        -1, -1, -1, -1
                    };
                    SUCCEEDED(dwmApi.DwmExtendFrameIntoClientArea(hWndParent, &marGlassInset));
                }
                break;
            }
        }
    }
    return pWndProc(hWnd, message, wParam, lParam);
}

BOOL CSettingsDialog::OnInitDialog()
{
    CTreePropSheet::OnInitDialog();

    AeroAutoSubclass(m_hWnd, ASC_SUBCLASS_ALL_CONTROLS, 0L);

    m_loadSkin.playIcon = s_mainWnd.m_skin.playIcon;
    m_loadSkin.pauseIcon = s_mainWnd.m_skin.pauseIcon;
    m_loadSkin.stopIcon = s_mainWnd.m_skin.stopIcon;
    m_loadSkin.prevIcon = s_mainWnd.m_skin.prevIcon;
    m_loadSkin.nextIcon = s_mainWnd.m_skin.nextIcon;
    m_loadSkin.buttons = s_mainWnd.m_skin.buttons;
    m_loadSkin.buttonsDn = s_mainWnd.m_skin.buttonsDn;
    m_loadSkin.playIconName = s_mainWnd.m_skin.playIconName;
    m_loadSkin.pauseIconName = s_mainWnd.m_skin.pauseIconName;
    m_loadSkin.stopIconName = s_mainWnd.m_skin.stopIconName;
    m_loadSkin.prevIconName = s_mainWnd.m_skin.prevIconName;
    m_loadSkin.nextIconName = s_mainWnd.m_skin.nextIconName;
    m_loadSkin.buttonsName = s_mainWnd.m_skin.buttonsName;
    m_loadSkin.buttonsDnName = s_mainWnd.m_skin.buttonsDnName;
    m_loadSkin.startColor = s_mainWnd.m_skin.startColor;
    m_loadSkin.endColor = s_mainWnd.m_skin.endColor;
    m_loadSkin.border_color = s_mainWnd.m_skin.border_color;
    m_loadSkin.end_bk = s_mainWnd.m_skin.end_bk;
    m_loadSkin.mid_bk = s_mainWnd.m_skin.mid_bk;
    m_loadSkin.start_bk = s_mainWnd.m_skin.start_bk;
    m_loadSkin.theme = s_mainWnd.m_skin.theme;
    m_loadSkin.effect_bk = s_mainWnd.m_skin.effect_bk;
    m_loadSkin.granularity = s_mainWnd.m_skin.granularity;
    m_loadSkin.border = s_mainWnd.m_skin.border;
    m_loadSkin.custom_border = s_mainWnd.m_skin.custom_border;
    m_loadSkin.border_width = s_mainWnd.m_skin.border_width;
    m_loadSkin.border_height = s_mainWnd.m_skin.border_height;
    m_loadSkin.transparency = s_mainWnd.m_skin.transparency;
    m_loadSkin.xpLookNFeel = s_mainWnd.m_skin.xpLookNFeel;
    m_loadSkin.toolTipText = s_mainWnd.m_skin.toolTipText;
    m_loadSkin.toolTipText.Replace(_T("\\r"), _T("\r"));
    m_loadSkin.toolTipText.Replace(_T("\\n"), _T("\n"));
    m_loadSkin.toolTipText.Replace(_T("\\t"), _T("\t"));

    m_tooltip.Create(this);

    m_Helper.translateWindowText(this, IDD_SETTINGS);
    m_Helper.translateDlgItemText(this, IDOK);
    m_Helper.translateDlgItemText(this, IDCANCEL);

    SetIcon(s_mainWnd.m_stpIcon, TRUE);
    SetIcon(s_mainWnd.m_stpIconSm, FALSE);

    m_loadSkin.xpLookNFeel = s_mainWnd.m_skin.xpLookNFeel;

    TC_ITEM item;
    CString buf;

    item.mask = TCIF_TEXT;

    SetActivePage(0);
    buf = m_Helper.translateWindowTextString(&m_settingsDialogGeneral, IDD_SETTINGS_GENERAL);
    item.pszText = (LPTSTR)buf.GetString();
    GetTabControl()->SetItem(0, &item);

    SetActivePage(1);
    buf = m_Helper.translateWindowTextString(&m_settingsDialogHotkeys, IDD_SETTINGS_HOTKEYS);
    item.pszText = (LPTSTR)buf.GetString();
    GetTabControl()->SetItem(1, &item);

    SetActivePage(2);
    buf = m_Helper.translateWindowTextString(&m_settingsDialogW2File, IDD_SETTINGS_W2FILE);
    item.pszText = (LPTSTR)buf.GetString();
    GetTabControl()->SetItem(2, &item);

    SetActivePage(3);
    buf = m_Helper.translateWindowTextString(&m_settingsDialogMouseClicks, IDD_SETTINGS_MOUSE_CLICKS);
    item.pszText = (LPTSTR)buf.GetString();

    SetActivePage(4);
    buf = m_Helper.translateWindowTextString(&m_settingsDialogApparence, IDD_SETTINGS_APPARENCE);
    item.pszText = (LPTSTR)buf.GetString();
    GetTabControl()->SetItem(4, &item);

    SetActivePage(5);
    buf = m_Helper.translateWindowTextString(&m_settingsDialogApparenceSkins, IDD_SETTINGS_APPARENCE_SKINS);
    item.pszText = (LPTSTR)buf.GetString();
    GetTabControl()->SetItem(5, &item);

    SetActivePage(6);
    buf = m_Helper.translateWindowTextString(&m_settingsDialogApparenceTooltip, IDD_SETTINGS_APPARENCE_TOOLTIP);
    item.pszText = (LPTSTR)buf.GetString();
    GetTabControl()->SetItem(6, &item);

    SetActivePage(7);
    buf = m_Helper.translateWindowTextString(&m_settingsDialogPlugins, IDD_SETTINGS_PLUGINS);
    item.pszText = (LPTSTR)buf.GetString();
    GetTabControl()->SetItem(7, &item);

    SetActivePage(8);
    buf = m_Helper.translateWindowTextString(&m_settingsDialogInput, IDD_SETTINGS_INPUT);
    item.pszText = (LPTSTR)buf.GetString();
    GetTabControl()->SetItem(8, &item);

    SetActivePage(9);
    buf = m_Helper.translateWindowTextString(&m_settingsDialogOutput, IDD_SETTINGS_OUTPUT);
    item.pszText = (LPTSTR)buf.GetString();
    GetTabControl()->SetItem(9, &item);

    SetActivePage(10);
    buf = m_Helper.translateWindowTextString(&m_settingsDialogGen, IDD_SETTINGS_GEN);
    item.pszText = (LPTSTR)buf.GetString();
    GetTabControl()->SetItem(10, &item);

    for (int i = 0; i < listPrefs.GetSize(); i++)
    {
        prefsDlgRec* prefs = listPrefs[i];
        CString tmp2;
        if (prefs->where == 0)
        {
            buf = m_Helper.translateWindowTextString(&m_settingsDialogGen, IDD_SETTINGS_GEN);
            tmp2.Format(_T("%s::"), buf);
        }
        if (prefs->where == 2)
        {
            buf = m_Helper.translateWindowTextString(&m_settingsDialogApparence, IDD_SETTINGS_APPARENCE);
            tmp2.Format(_T("%s::"), buf);
        }
        TCHAR prefName[5000];
        Strcpy(prefName, 5000, prefs->name);
        buf.Format(_T("%s%s"), tmp2, prefName);
        item.pszText = (LPWSTR)buf.GetString();
        GetTabControl()->SetItem(11 + i, &item);
        SetActivePage(11 + i);
        HWND hWnd = GetActivePage()->m_hWnd;
        WNDPROC oldWndProc = (WNDPROC)SetWindowLongPtr(hWnd, GWLP_WNDPROC, (LONG_PTR)NewWindowProc);
        SetProp(hWnd, BACKGROUND_PROC_STRING, oldWndProc);
        if (!oldWndProc)
        {
            DWORD dwLastError = GetLastError();
            RemoveProp(hWnd, BACKGROUND_PROC_STRING);
            SetLastError(dwLastError);
            return FALSE;
        }
    }   

    RefillPageTree();

    SetActivePage(0);
    GetTabControl()->ShowWindow(SW_HIDE);

    UpdateData(FALSE);

    return TRUE;
}

void CSettingsDialog::SetActiveSettingPage(int index)
{
    /*
    ** Winamp Pro           20
    ** General Preferences  0
    ** File Types           1
    ** Playlist             23
    ** Titles               21
    ** Video                24
    ** Skins                40
    ** Classic Skins        22
    ** Plugins              30
    ** Input                31
    ** Output               32
    ** Visualisation        33
    ** DSP/Effect           34
    ** General Purpose      35
    */
    switch (index)
    {
        case 20:
        case 0:
        case 1:
        case 23:
        case 21:
        case 24:
        case 33:
        case 34:
            SetActivePage(0);
            break;
        case 40:
        case 22:
            SetActivePage(4);
            break;
        case 30:
        case 31:
            SetActivePage(8);
            break;
        case 32:
            SetActivePage(9);
            break;
        case 35:
            SetActivePage(10);
            break;
        default:
            if (index >= 60)
                for (int i = 0; i < m_listPropertyPage.GetSize(); i++)
                {
                    CTestPropertyPage* page = m_listPropertyPage[i];
                    if (page->PageDataBackup->dlgID == index)
                    {
                        SetActivePage(11 + i);
                        break;
                    }
                }
    }
}
void CSettingsDialog::SetActiveSettingPage(prefsDlgRec* iPrefs)
{
    for (int i = 0; i < m_listPropertyPage.GetSize(); i++)
    {
        CTestPropertyPage* page = m_listPropertyPage[i];
        if (page->PageDataBackup->proc == iPrefs->proc)
        {
            SetActiveSettingPage(11 + i);
            break;
        }
    }
}

void CSettingsDialog::OnOk()
{
    OnApplyNow();
    if (!m_needToRestart)
        PressButton(PSBTN_CANCEL);
}

void CSettingsDialog::OnApplyNow()
{
    PressButton(PSBTN_APPLYNOW);

    int res[3] = {IDOK, IDCANCEL, ID_APPLY_NOW};
    
    for(int i = 0; i < 3; i++)
    {
        CWnd *wnd = GetDlgItem(res[i]);
        wnd->ShowWindow(SW_SHOW);
        wnd->EnableWindow(TRUE);
    }

    UpdateData(TRUE);

    s_mainWnd.m_skin.playIcon = m_loadSkin.playIcon;
    s_mainWnd.m_skin.pauseIcon = m_loadSkin.pauseIcon;
    s_mainWnd.m_skin.stopIcon = m_loadSkin.stopIcon;
    s_mainWnd.m_skin.prevIcon = m_loadSkin.prevIcon;
    s_mainWnd.m_skin.nextIcon = m_loadSkin.nextIcon;
    s_mainWnd.m_skin.buttons = m_loadSkin.buttons;
    s_mainWnd.m_skin.buttonsDn = m_loadSkin.buttonsDn;
    s_mainWnd.m_skin.playIconName = m_loadSkin.playIconName;
    s_mainWnd.m_skin.pauseIconName = m_loadSkin.pauseIconName;
    s_mainWnd.m_skin.stopIconName = m_loadSkin.stopIconName;
    s_mainWnd.m_skin.prevIconName = m_loadSkin.prevIconName;
    s_mainWnd.m_skin.nextIconName = m_loadSkin.nextIconName;
    s_mainWnd.m_skin.buttonsName = m_loadSkin.buttonsName;
    s_mainWnd.m_skin.buttonsDnName = m_loadSkin.buttonsDnName;
    s_mainWnd.m_skin.startColor = m_loadSkin.startColor;
    s_mainWnd.m_skin.endColor = m_loadSkin.endColor;
    s_mainWnd.m_skin.border_color = m_loadSkin.border_color;
    s_mainWnd.m_skin.end_bk = m_loadSkin.end_bk;
    s_mainWnd.m_skin.mid_bk = m_loadSkin.mid_bk;
    s_mainWnd.m_skin.start_bk = m_loadSkin.start_bk;
    s_mainWnd.m_skin.theme = m_loadSkin.theme;
    s_mainWnd.m_skin.effect_bk = m_loadSkin.effect_bk;
    s_mainWnd.m_skin.granularity = m_loadSkin.granularity;
    s_mainWnd.m_skin.border = m_loadSkin.border;
    s_mainWnd.m_skin.custom_border = m_loadSkin.custom_border;
    s_mainWnd.m_skin.border_width = m_loadSkin.border_width;
    s_mainWnd.m_skin.border_height = m_loadSkin.border_height;
    s_mainWnd.m_skin.transparency = m_loadSkin.transparency;
    s_mainWnd.m_skin.xpLookNFeel = m_loadSkin.xpLookNFeel;
    m_loadSkin.toolTipText.Replace(_T("\r"), _T("\\r"));
    m_loadSkin.toolTipText.Replace(_T("\n"), _T("\\n"));
    m_loadSkin.toolTipText.Replace(_T("\t"), _T("\\t"));
    s_mainWnd.m_skin.toolTipText = m_loadSkin.toolTipText;

    if (m_needToRestart)
    {
        int res = MessageBox(m_Helper.MyLoadString(IDS_NEEDTORESTART), _T("STP"), MB_ICONINFORMATION | MB_YESNO);
        if(res != IDNO)
        {
            TCHAR fileName[MAX_PATH + 1];
            GetModuleFileName(AfxGetInstanceHandle(), fileName, MAX_PATH);
            ShellExecute(NULL, _T("open"), fileName, _T("-restart"), NULL, SW_SHOWNORMAL);
            ::SendMessage(s_mainWnd.m_hWnd, WM_COMMAND, ID_POPUPMENU_EXIT, 0);
        }
        else
            Invalidate();
    }
    else
        Invalidate();
}

void CSettingsDialog::DisplaySkin(CWnd* pSkinSampleWnd, int frg, int forceText)
{
    if (pSkinSampleWnd)
    {
        CDC* pDC = pSkinSampleWnd->GetDC();
        HDC hDC = CreateCompatibleDC(::GetWindowDC(pSkinSampleWnd->m_hWnd));
        CRect rect;
        pSkinSampleWnd->GetClientRect(&rect);

        int oWidth = 90;
        int oHeight = 90;
        SelectObject(hDC, m_loadSkin.buttons);

        int imgWidth = (oWidth* rect.Height() < oHeight* rect.Width()) ? oWidth* rect.Height() / oHeight : oWidth;
        int imgHeight = (oWidth* rect.Height() < oHeight* rect.Width()) ? oHeight* imgWidth / oWidth : oHeight;

        HDC hdc1 = CreateCompatibleDC(hDC);
        HBITMAP bm1 = CreateCompatibleBitmap(hDC, imgWidth, imgHeight);
        SelectObject(hdc1, bm1);
        if ((m_loadSkin.xpLookNFeel && forceText == -1) || forceText == 0)
        {
            DisplayImage(hDC, hdc1, rect);
        }
        else
        {
            DisplayText(hDC, hdc1, rect, -1, forceText);
        }
        if (frg != -1)
        {
            SelectObject(hDC, m_loadSkin.buttonsDn);
            HDC hdc2 = CreateCompatibleDC(hDC);
            HBITMAP bm2 = CreateCompatibleBitmap(hDC, imgWidth, imgHeight);
            SelectObject(hdc2, bm2);
            if ((m_loadSkin.xpLookNFeel && forceText == -1) || forceText == 0)
            {
                DisplayImage(hDC, hdc2, rect, frg);
            }
            else
            {
                DisplayText(hDC, hdc2, rect, frg, forceText);
            }
            StretchBlt(hdc1, rect.left, rect.top + frg * rect.Height() / 5 + 1, rect.Width(), rect.Height() / 5, hdc2, 0, frg * imgHeight / 5 + 1, imgWidth, imgHeight / 5 + 1, SRCCOPY);
            DeleteDC(hdc2);
            DeleteObject(bm2);
        }

        rect.right--;
        CDC::FromHandle(hdc1)->Draw3dRect(rect, RGB(0, 0, 0), RGB(0, 0, 0));

        if (gdiplusWrapper.IsInitialized())
        {
            Graphics* myGraphics = new Graphics(pDC->m_hDC);
            Bitmap* pBmp = new Bitmap(bm1, NULL);
            myGraphics->DrawImage(pBmp, 0, 0, imgWidth, imgHeight);
            delete pBmp;
            delete myGraphics;
        }
        else
            pDC->StretchBlt(rect.left, rect.top, rect.Width(), rect.Height(), CDC::FromHandle(hdc1), 0, 0, imgWidth, imgHeight, SRCCOPY);

        DeleteDC(hdc1);
        DeleteObject(bm1);
        SelectObject(hDC, m_loadSkin.buttons);

        DeleteDC(hDC);
    }
}
void CSettingsDialog::DisplayText(HDC hdc, HDC hdc5, CRect rect, int frg, int forceText)
{
    CString buf;
    CRect tmp;

    CFontDC font(hdc, DEFAULT_GUI_FONT);
    CFontDC font5(hdc5, DEFAULT_GUI_FONT);
    COLORREF cColor = SetBkColor(hdc, RGB(255, 0, 255));

    COLORREF crHighLight = ::GetSysColor(COLOR_HIGHLIGHT);
    CPenDC pen(hdc5, crHighLight);
    CBrushDC brush(hdc5, frg == -1 ? HLS_TRANSFORM(::GetSysColor(COLOR_3DFACE), +73, 0) : HLS_TRANSFORM(crHighLight, +70, -57));

    if (m_loadSkin.xpLookNFeel || forceText != 1)
    {
        CRect rc(rect);
        rc.right = rc.left + 16 + 8;
        CDC::FromHandle(hdc5)->FillSolidRect(rc, HLS_TRANSFORM(::GetSysColor(COLOR_3DFACE), +20, 0));
        rc.left = rc.right;
        rc.right = rect.right;
        CDC::FromHandle(hdc5)->FillSolidRect(rc, HLS_TRANSFORM(::GetSysColor(COLOR_3DFACE), +75, 0));
        if (frg != -1)
        {
            SetBkColor(hdc, cColor);
            CRect grad;
            grad = rect;
            if (m_showGradient && m_showGradientSel)
            {
                s_mainWnd.DrawGradient(CDC::FromHandle(hdc5), m_loadSkin.startColor, m_loadSkin.endColor, &grad);
                CBrushDC brush(hdc5, CLR_NONE);
                CDC::FromHandle(hdc5)->Rectangle(rect);
            }
            else
                CDC::FromHandle(hdc5)->Rectangle(rect);
        }
        else
        {
            SetBkColor(hdc, cColor);
            CRect grad;
            grad = rect;
            grad.right = grad.left + 30;
            if (m_showGradient)
            {
                s_mainWnd.DrawGradient(CDC::FromHandle(hdc5), m_loadSkin.startColor, m_loadSkin.endColor, &grad);
                CBrushDC brush(hdc5, CLR_NONE);
                CDC::FromHandle(hdc5)->Rectangle(rect);
                grad = rect;
                grad.left += 30;
                CDC::FromHandle(hdc5)->FillSolidRect(grad, m_loadSkin.endColor);
            }
        }
    }
    else
    {
        if (frg != -1)
        {
            CDC::FromHandle(hdc5)->SetTextColor(::GetSysColor(COLOR_HIGHLIGHTTEXT));
            SetBkColor(hdc5, cColor);
            CDC::FromHandle(hdc5)->FillSolidRect(0, 0, rect.Width(), rect.Height(), GetSysColor(COLOR_HIGHLIGHT));
        }
        else
        {
            CDC::FromHandle(hdc5)->SetTextColor(::GetSysColor(COLOR_WINDOWTEXT));
            SetBkColor(hdc5, cColor);
            CDC::FromHandle(hdc5)->FillSolidRect(0, 0, rect.Width(), rect.Height(), (forceText == 1) ? GetSysColor(COLOR_MENU) : GetSysColor(COLOR_MENU));
        }
    }

    SetBkMode(hdc5, TRANSPARENT);

    tmp = rect;
    if (frg != -1 && m_loadSkin.xpLookNFeel)
    {
        CPen pen(PS_SOLID, 1, ::GetSysColor(COLOR_HIGHLIGHT));

        tmp.left += 2;
        tmp.right -= 3;
        tmp.top = rect.top + frg * rect.Height() / 5 + 1;
        tmp.bottom = rect.top + (1 + frg) * rect.Height() / 5;

        CDC::FromHandle(hdc5)->Draw3dRect(tmp, ::GetSysColor(COLOR_HIGHLIGHT), ::GetSysColor(COLOR_HIGHLIGHT));
        tmp.left -= 2;
        tmp.right += 3;
    }
    tmp.left += 32;

    buf = m_Helper.MyLoadString(ID_POPUPMENU_PLAY);
    tmp.top = rect.top;
    tmp.bottom = rect.bottom - 4 * rect.Height() / 5;
    CDC::FromHandle(hdc5)->DrawText(buf, tmp, DT_LEFT | DT_SINGLELINE | DT_NOPREFIX | DT_VCENTER);
    if (m_loadSkin.xpLookNFeel || forceText == 1)
    {
        if (frg == 0)
            CDC::FromHandle(hdc5)->DrawState(CPoint(1 + ((frg != -1) ? 5 : 4), tmp.top + ((frg != -1) ? 6 : 4)), CSize(16, 16), m_loadSkin.playIcon, DSS_MONO, CBrush((frg != -1) ? HLS_TRANSFORM(::GetSysColor(COLOR_HIGHLIGHT), +50, -66) : HLS_TRANSFORM(::GetSysColor(COLOR_3DFACE), -27, 0)));
        CDC::FromHandle(hdc5)->DrawState(CPoint(1 + ((frg != -1) ? 3 : 4), tmp.top + ((frg != -1) ? 3 : 4)), CSize(16, 16), m_loadSkin.playIcon, DSS_NORMAL, (HBRUSH)NULL);
    }

    buf = m_Helper.MyLoadString(ID_POPUPMENU_PAUSE);
    tmp.top = rect.top + rect.Height() / 5;
    tmp.bottom = rect.bottom - 3 * rect.Height() / 5;
    CDC::FromHandle(hdc5)->DrawText(buf, tmp, DT_LEFT | DT_SINGLELINE | DT_NOPREFIX | DT_VCENTER);
    if (m_loadSkin.xpLookNFeel || forceText == 1)
    {
        if (frg == 1)
            CDC::FromHandle(hdc5)->DrawState(CPoint(1 + ((frg != -1) ? 5 : 4), tmp.top + ((frg != -1) ? 6 : 4)), CSize(16, 16), m_loadSkin.pauseIcon, DSS_MONO, CBrush((frg != -1) ? HLS_TRANSFORM(::GetSysColor(COLOR_HIGHLIGHT), +50, -66) : HLS_TRANSFORM(::GetSysColor(COLOR_3DFACE), -27, 0)));
        CDC::FromHandle(hdc5)->DrawState(CPoint(1 + ((frg != -1) ? 3 : 4), tmp.top + ((frg != -1) ? 3 : 4)), CSize(16, 16), m_loadSkin.pauseIcon, DSS_NORMAL, (HBRUSH)NULL);
    }
    buf = m_Helper.MyLoadString(ID_POPUPMENU_STOP);
    tmp.top = rect.top + 2 * rect.Height() / 5;
    tmp.bottom = rect.bottom - 2 * rect.Height() / 5;
    CDC::FromHandle(hdc5)->DrawText(buf, tmp, DT_LEFT | DT_SINGLELINE | DT_NOPREFIX | DT_VCENTER);
    if (m_loadSkin.xpLookNFeel || forceText == 1)
    {
        if (frg == 2)
            CDC::FromHandle(hdc5)->DrawState(CPoint(1 + ((frg != -1) ? 5 : 4), tmp.top + ((frg != -1) ? 6 : 4)), CSize(16, 16), m_loadSkin.stopIcon, DSS_MONO, CBrush((frg != -1) ? HLS_TRANSFORM(::GetSysColor(COLOR_HIGHLIGHT), +50, -66) : HLS_TRANSFORM(::GetSysColor(COLOR_3DFACE), -27, 0)));
        CDC::FromHandle(hdc5)->DrawState(CPoint(1 + ((frg != -1) ? 3 : 4), tmp.top + ((frg != -1) ? 3 : 4)), CSize(16, 16), m_loadSkin.stopIcon, DSS_NORMAL, (HBRUSH)NULL);
    }

    buf = m_Helper.MyLoadString(ID_POPUPMENU_PREVIOUS);
    tmp.top = rect.top + 3 * rect.Height() / 5;
    tmp.bottom = rect.bottom - rect.Height() / 5;
    CDC::FromHandle(hdc5)->DrawText(buf, tmp, DT_LEFT | DT_SINGLELINE | DT_NOPREFIX | DT_VCENTER);
    if (m_loadSkin.xpLookNFeel || forceText == 1)
    {
        if (frg == 3)
            CDC::FromHandle(hdc5)->DrawState(CPoint(1 + ((frg != -1) ? 5 : 4), tmp.top + ((frg != -1) ? 6 : 4)), CSize(16, 16), m_loadSkin.prevIcon, DSS_MONO, CBrush((frg != -1) ? HLS_TRANSFORM(::GetSysColor(COLOR_HIGHLIGHT), +50, -66) : HLS_TRANSFORM(::GetSysColor(COLOR_3DFACE), -27, 0)));
        CDC::FromHandle(hdc5)->DrawState(CPoint(1 + ((frg != -1) ? 3 : 4), tmp.top + ((frg != -1) ? 3 : 4)), CSize(16, 16), m_loadSkin.prevIcon, DSS_NORMAL, (HBRUSH)NULL);
    }

    buf = m_Helper.MyLoadString(ID_POPUPMENU_NEXT);
    tmp.top = rect.top + 4 * rect.Height() / 5;
    tmp.bottom = rect.bottom;
    CDC::FromHandle(hdc5)->DrawText(buf, tmp, DT_LEFT | DT_SINGLELINE | DT_NOPREFIX | DT_VCENTER);
    if (m_loadSkin.xpLookNFeel || forceText == 1)
    {
        if (frg == 4)
            CDC::FromHandle(hdc5)->DrawState(CPoint(1 + ((frg != -1) ? 5 : 4), tmp.top + ((frg != -1) ? 6 : 4)), CSize(16, 16), m_loadSkin.nextIcon, DSS_MONO, CBrush((frg != -1) ? HLS_TRANSFORM(::GetSysColor(COLOR_HIGHLIGHT), +50, -66) : HLS_TRANSFORM(::GetSysColor(COLOR_3DFACE), -27, 0)));
        CDC::FromHandle(hdc5)->DrawState(CPoint(1 + ((frg != -1) ? 3 : 4), tmp.top + ((frg != -1) ? 3 : 4)), CSize(16, 16), m_loadSkin.nextIcon, DSS_NORMAL, (HBRUSH)NULL);
    }

    ReleaseDC(CDC::FromHandle(hdc5));
}

void CSettingsDialog::DisplayImage(HDC hdc, HDC hdc5, CRect rect, int frg)
{
    int oWidth = 90;
    int oHeight = 90;
    int imgWidth = (oWidth* rect.Height() < oHeight* rect.Width()) ? oWidth* rect.Height() / oHeight : oWidth;
    int imgHeight = (oWidth* rect.Height() < oHeight* rect.Width()) ? oHeight* imgWidth / oWidth : oHeight;

    HDC hdc1 = CreateCompatibleDC(hdc);
    HDC hdc2 = CreateCompatibleDC(hdc);
    HDC hdc3 = CreateCompatibleDC(hdc);
    HDC hdc4 = CreateCompatibleDC(hdc);
    //Masque binaire
    HBITMAP bm1 = CreateBitmap(imgWidth, imgHeight, 1, 1, NULL);
    HBITMAP bm2 = CreateBitmap(imgWidth, imgHeight, 1, 1, NULL);
    HBITMAP bm3 = CreateCompatibleBitmap(hdc, imgWidth, imgHeight);
    HBITMAP bm4 = CreateCompatibleBitmap(hdc, imgWidth, imgHeight);
    SelectObject(hdc1, bm1);
    SelectObject(hdc2, bm2);
    SelectObject(hdc3, bm3);
    SelectObject(hdc4, bm4);

    COLORREF cColor = SetBkColor(hdc, RGB(255, 0, 255));
    COLORREF crHighLight = ::GetSysColor(COLOR_HIGHLIGHT);
    CPenDC pen(hdc5, crHighLight);
    CBrushDC brush(hdc5, frg == -1 ? HLS_TRANSFORM(::GetSysColor(COLOR_3DFACE), +73, 0) : HLS_TRANSFORM(crHighLight, +70, -57));

    //Image Original
    StretchBlt(hdc1, 0, 0, imgWidth, imgHeight, hdc, 0, 0, oWidth, oHeight, SRCCOPY);
    //Image invers
    StretchBlt(hdc2, 0, 0, imgWidth, imgHeight, hdc, 0, 0, oWidth, oHeight, NOTSRCCOPY);
    if (m_loadSkin.xpLookNFeel)
    {
        CRect rc(rect);
        rc.right = rc.left + 16 + 8;
        CDC::FromHandle(hdc5)->FillSolidRect(rc, HLS_TRANSFORM(::GetSysColor(COLOR_3DFACE), +20, 0));
        rc.left = rc.right;
        rc.right = rect.right;
        CDC::FromHandle(hdc5)->FillSolidRect(rc, HLS_TRANSFORM(::GetSysColor(COLOR_3DFACE), +75, 0));
        if (frg != -1)
        {
            SetBkColor(hdc, cColor);
            CRect grad;
            grad = rect;
            if (m_showGradient && m_showGradientSel)
            {
                s_mainWnd.DrawGradient(CDC::FromHandle(hdc5), m_loadSkin.startColor, m_loadSkin.endColor, &grad);
                CBrushDC brush(hdc5, CLR_NONE);
                CDC::FromHandle(hdc5)->Rectangle(rect);
            }
            else
                CDC::FromHandle(hdc5)->Rectangle(rect);
        }
        else
        {
            SetBkColor(hdc, cColor);
            CRect grad;
            grad = rect;
            grad.right = grad.left + 30;
            if (m_showGradient)
            {
                s_mainWnd.DrawGradient(CDC::FromHandle(hdc5), m_loadSkin.startColor, m_loadSkin.endColor, &grad);
                grad = rect;
                grad.left += 30;
                CDC::FromHandle(hdc5)->FillSolidRect(grad, m_loadSkin.endColor);
            }
        }
    }
    else
    {
        if (frg != -1)
        {
            SetBkColor(hdc1, cColor);
            CDC::FromHandle(hdc5)->FillSolidRect(0, 0, rect.Width(), imgHeight, GetSysColor(COLOR_HIGHLIGHT));
        }
        else
        {
            SetBkColor(hdc1, cColor);
            CDC::FromHandle(hdc5)->FillSolidRect(0, 0, rect.Width(), imgHeight, GetSysColor(COLOR_MENU));
        }
    }
    StretchBlt(hdc3, 0, 0, imgWidth, imgHeight, hdc5, (rect.Width() - imgWidth) / 2, 0, imgWidth, imgHeight, SRCCOPY);
    StretchBlt(hdc4, 0, 0, imgWidth, imgHeight, hdc, 0, 0, oWidth, oHeight, SRCCOPY);
    //Fond Orignal
    //Et entre Image invers et fond
    BitBlt(hdc3, 0, 0, imgWidth, imgHeight, hdc1, 0, 0, SRCAND);
    BitBlt(hdc4, 0, 0, imgWidth, imgHeight, hdc2, 0, 0, SRCAND);
    //Ou entre les 2 images obtenu
    BitBlt(hdc4, 0, 0, imgWidth, imgHeight, hdc3, 0, 0, SRCPAINT);
    BitBlt(hdc5, 0, 0, imgWidth, imgHeight, hdc4, 0, 0, SRCCOPY);

    CRect tmp = rect;
    if (frg != -1 && m_loadSkin.xpLookNFeel)
    {
        tmp.left += 2;
        tmp.right -= 3;
        tmp.top = rect.top + frg * rect.Height() / 5 + 1;
        tmp.bottom = rect.top + (1 + frg) * rect.Height() / 5;

        CDC::FromHandle(hdc5)->Draw3dRect(tmp, ::GetSysColor(COLOR_HIGHLIGHT), ::GetSysColor(COLOR_HIGHLIGHT));
        tmp.left -= 2;
        tmp.right += 3;
    }

    DeleteDC(hdc1);
    DeleteObject(bm1);
    DeleteDC(hdc2);
    DeleteObject(bm2);
    DeleteDC(hdc3);
    DeleteObject(bm3);
    DeleteDC(hdc4);
    DeleteObject(bm4);
}

BOOL CSettingsDialog::RelayToolTipEvent(MSG* pMsg)
{
    return m_tooltip.RelayEvent(pMsg);
}

BOOL CSettingsDialog::OnEraseBkgnd(CDC* pDC)
{
	CDwmApiImpl dwmApi;
	if(dwmApi.Initialize() && dwmApi.IsDwmCompositionEnabled())
	{
		CRect rect;
		GetClientRect(&rect);

		VERIFY(PatBlt(pDC->m_hDC, 0, 0, rect.Width(), rect.Height(), BLACKNESS));
		return TRUE;
	}
	return CPropertySheet::OnEraseBkgnd(pDC);
}
