// SettingsDialogGeneral.cpp : fichier d'implmentation
//

#include "StdAfx.h"
#include "resource.h"
#include "aerosubc.h"
#include "aeroglss.h"
#include "SysTrayPlay.h"
#include "SettingsDialog.h"
#include "MainHelper.h"
#include "MainWnd.h"
#include "MultiDllInput.h"
#include "cgfiltyp.h"
#include "Draw.h"
#include "GdiplusWrapper.h"

#include "SeekDialog.h"
#include "PlaylistDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CSeekDialog s_seekDialog;
extern CPlaylistDlg s_playlistDlg;
extern CSettingsDialog* s_settingsDialog;

CSkinSampleCtrl::CSkinSampleCtrl(int forceText)
{
    m_forceText = forceText;
}

CSkinSampleCtrl::~CSkinSampleCtrl()
{
}

void CSkinSampleCtrl::SetForceText(int forceText)
{
    m_forceText = forceText;
}

void CSkinSampleCtrl::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct)
{
    s_settingsDialog->DisplaySkin(this, -1, m_forceText);
}

BEGIN_MESSAGE_MAP(CSkinSampleCtrl, CStatic)
    ON_WM_MOUSEMOVE()
    ON_WM_MOUSELEAVE()
    ON_WM_CREATE()
END_MESSAGE_MAP()

void CSkinSampleCtrl::OnMouseMove(UINT nFlags, CPoint point)
{
    CRect rect;
    GetWindowRect(&rect);
    ScreenToClient(rect);

    int frg = -1;
    frg = 5 * (point.y - rect.top) / rect.Height();
    s_settingsDialog->DisplaySkin(this, frg, m_forceText);
    TRACKMOUSEEVENT track; //Declares structure
    track.cbSize = sizeof(track);
    track.dwFlags = TME_LEAVE; //Notify us when the mouse leaves
    track.hwndTrack = m_hWnd; //Assigns this window's hwnd
    TrackMouseEvent(&track); //Tracks the events like WM_MOUSELEAVE
}

void CSkinSampleCtrl::OnMouseLeave()
{
    s_settingsDialog->DisplaySkin(this, -1, m_forceText);
}

CGradientSampleCtrl::CGradientSampleCtrl()
{
    m_startColor = RGB(0xB4, 0xC4, 0xE4);
    m_endColor = RGB(0xFF, 0xFF, 0xFF);
}

CGradientSampleCtrl::~CGradientSampleCtrl()
{
}

void CGradientSampleCtrl::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct)
{
    CDC* pDC = GetDC();
    HDC hDC = GetWindowDC()->m_hDC;
    HDC hDC1 = CreateCompatibleDC(hDC);
    CRect rect;
    GetClientRect(&rect);

    HBITMAP bm1 = CreateCompatibleBitmap(hDC, rect.Width(), rect.Height());
    SelectObject(hDC1, bm1);

    s_mainWnd.DrawGradient(CDC::FromHandle(hDC1), s_settingsDialog->m_loadSkin.startColor, s_settingsDialog->m_loadSkin.endColor, &rect);

    CDC::FromHandle(hDC1)->Draw3dRect(rect, RGB(0, 0, 0), RGB(0, 0, 0));

    if (gdiplusWrapper.IsInitialized())
    {
        HDC hDC2 = CreateCompatibleDC(hDC);
        HBITMAP bm2 = CreateCompatibleBitmap(hDC, rect.Width(), rect.Height());
        SelectObject(hDC2, bm2);

        BitBlt(hDC2, 0, 0, rect.Width(), rect.Height(), hDC1, 0, 0, SRCPAINT);

        Graphics* myGraphics = new Graphics(pDC->m_hDC);
        Bitmap* pBmp = new Bitmap(bm2, NULL);
        myGraphics->DrawImage(pBmp, 0, 0, rect.Width(), rect.Height());
        delete pBmp;
        delete myGraphics;
        DeleteDC(hDC2);
        DeleteObject(bm2);
    }
    else
        pDC->StretchBlt(0, 0, rect.Width(), rect.Height(), CDC::FromHandle(hDC1), 0, 0, rect.Width(), rect.Height(), SRCCOPY);
    ::ReleaseDC(m_hWnd, hDC);
    DeleteObject(bm1);
    DeleteObject(hDC);
    DeleteObject(hDC1);
}

// Bote de dialogue CSettingGeneral

IMPLEMENT_MENUXP(CSettingsDialogGeneral, CPropertyPage);

IMPLEMENT_DYNAMIC(CSettingsDialogGeneral, CPropertyPage)

BOOL DeleteKeyAndSubKeys(HKEY    hkIn, LPCTSTR   pszSubKey)
{
    HKEY hk;
    TCHAR szTmp[MAX_PATH];
    long l;
    BOOL f;

    l = RegOpenKeyEx(hkIn, pszSubKey, 0, KEY_ALL_ACCESS, &hk);
    if (l != ERROR_SUCCESS)
        return FALSE;

    f = TRUE;
    while (f)
    {
        l = RegEnumKey(hk, 0, szTmp, MAX_PATH);
        if (l != ERROR_SUCCESS)
            break;
        f = DeleteKeyAndSubKeys(hk, szTmp);
    }

    RegCloseKey(hk);
    l = RegDeleteKey(hkIn, pszSubKey);

    return (l == ERROR_SUCCESS) ? TRUE : FALSE;
}

void CSettingsDialogGeneral::CreateContMenus()
{
    DeleteKeyAndSubKeys(HKEY_CLASSES_ROOT, _T("Directory\\Shell\\playstp"));
    DeleteKeyAndSubKeys(HKEY_CLASSES_ROOT, _T("Directory\\Shell\\addstp"));
    DeleteKeyAndSubKeys(HKEY_CLASSES_ROOT, _T("Drive\\Shell\\playstp"));
    DeleteKeyAndSubKeys(HKEY_CLASSES_ROOT, _T("Drive\\Shell\\addstp"));

    CString buf;

    TCHAR fname[MAX_PATH + 1];
    GetModuleFileName(theApp.m_hInstance, fname, MAX_PATH);

    CGCFileTypeAccess fileTypeAccess;

    if (((CButton *)GetDlgItem(IDC_CM_PLAY))->GetCheck() == 1)
    {
        // just pass file path in quotes on command line
        buf = fname;
        buf = _T("\"") + buf + _T("\"");
        buf += _T(" \"%1\"");
        fileTypeAccess.SetShellOpenCommand(buf);
        fileTypeAccess.SetShellOpenDescription(LPCTSTR(m_Helper.MyLoadString(IDS_PLAYWITHSTP)));
    }

    if (((CButton *)GetDlgItem(IDC_CM_ADD))->GetCheck() == 1)
    {
        buf = fname;
        buf = _T("\"") + buf + _T("\"");
        buf += _T(" -a");
        buf += _T(" \"%1\"");
        fileTypeAccess.SetShellAddCommand(buf);
        fileTypeAccess.SetShellAddDescription(LPCTSTR(m_Helper.MyLoadString(IDS_ADDTOPLAYLIST)));
    }

    fileTypeAccess.SetExtension(_T("Directory"));
    fileTypeAccess.RegSetDirectory();

    fileTypeAccess.SetExtension(_T("Drive"));
    fileTypeAccess.RegSetDirectory();
}

CSettingsDialogGeneral::CSettingsDialogGeneral() : CPropertyPage(CSettingsDialogGeneral::IDD), m_skinSample(-1)
{
}

CSettingsDialogGeneral::~CSettingsDialogGeneral()
{
}

void CSettingsDialogGeneral::DoDataExchange(CDataExchange* pDX)
{
    CPropertyPage::DoDataExchange(pDX);
    CMenuXP::SetXPLookNFeel(this, s_mainWnd.m_skin.xpLookNFeel);
    DDX_Text(pDX, IDC_TPC, m_tpc);
    DDX_Control(pDX, IDC_L_OPTIONS, m_l_options);
    DDX_Control(pDX, IDC_L_OPTIONS2, m_l_options2);
    DDX_Control(pDX, IDC_CHANGELANGUAGE, m_cbxLanguage);
    DDX_Control(pDX, IDC_SKIN, m_btnSkin);
    DDX_Control(pDX, IDC_SKINSAMPLE, m_skinSample);
}

int CSettingsDialogGeneral::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    return CPropertyPage::OnCreate(lpCreateStruct);
}

BOOL CSettingsDialogGeneral::OnInitDialog()
{
    CPropertyPage::OnInitDialog();

    AeroAutoSubclass(m_hWnd, ASC_SUBCLASS_ALL_CONTROLS, 0L);

    m_Helper.translateWindowText(this, IDD_SETTINGS_GENERAL);
    m_Helper.translateDlgItemText(this, IDC_LTRACKPERCOLUMN);
    m_Helper.translateDlgItemText(this, IDC_LPRESSHOLD);
    m_Helper.translateDlgItemText(this, IDC_LSETUP);
    m_Helper.translateDlgItemText(this, IDC_LFOLDERMENU);
    m_Helper.translateDlgItemText(this, IDC_CM_PLAY);
    m_Helper.translateDlgItemText(this, IDC_CM_ADD);
    m_Helper.translateDlgItemText(this, IDC_ADDBYDEF);
    m_Helper.translateDlgItemText(this, IDC_SKIN);
    m_Helper.translateDlgItemText(this, IDC_LADDITIONAL);
    m_Helper.translateDlgItemText(this, IDC_LLANGUAGE);

    m_tpc = s_mainWnd.m_tpc;
    ((CSpinButtonCtrl *)GetDlgItem(IDC_SPIN2))->SetRange(0, 100);

    m_l_options.AddString(m_Helper.MyLoadString(IDS_OPTION0));
    m_l_options.AddString(m_Helper.MyLoadString(IDS_OPTION1));
    m_l_options.AddString(m_Helper.MyLoadString(IDS_OPTION7));
    m_l_options.AddString(m_Helper.MyLoadString(IDS_OPTION2));
    m_l_options.AddString(m_Helper.MyLoadString(IDS_OPTION3));
    m_l_options.AddString(m_Helper.MyLoadString(IDS_OPTION4));
    m_l_options.AddString(m_Helper.MyLoadString(IDS_OPTION8));
    m_l_options.AddString(m_Helper.MyLoadString(IDS_OPTION6));
    m_l_options.AddString(m_Helper.MyLoadString(IDS_OPTION9));

    m_l_options.SetCheck(0, s_mainWnd.m_showGradientBar);
    m_l_options.SetCheck(1, s_mainWnd.m_graphicalButtons);
    m_l_options.SetCheck(2, s_mainWnd.m_skin.xpLookNFeel);
    m_l_options.SetCheck(3, s_mainWnd.m_SMTracks);
    m_l_options.SetCheck(4, s_mainWnd.m_SMAlbum);
    m_l_options.SetCheck(5, s_mainWnd.m_autoPlay);
    m_l_options.SetCheck(6, s_mainWnd.m_SMPrevNext);
    m_l_options.SetCheck(7, s_mainWnd.m_wAEmu);
    m_l_options.SetCheck(8, s_mainWnd.m_autoUpdate);

    m_skinSample.SetForceText(!s_mainWnd.m_graphicalButtons);

    CString extList = stdIn.GetFileExtensions();
    CString ext;
    int pos = 0,i = 0;
    m_defaultAdd = false;

    ext = extList.Tokenize(_T(";"), pos);
    while (ext != "")
    {
        if (extList.Find(ext, pos) != -1)
        {
            ext = extList.Tokenize(_T(";"), pos);
            continue;
        }
        m_l_options2.AddString(ext);

        if (ReadFileTypes(ext))
            m_l_options2.SetCheck(i, 1);
        else
            m_l_options2.SetCheck(i, 0);
        ext = extList.Tokenize(_T(";"), pos);
        i++;
    }
    if (m_defaultAdd)
        ((CButton *)GetDlgItem(IDC_ADDBYDEF))->SetCheck(1);

    CFileFind finder;
    TCHAR buffer[MAX_PATH + 1];
    CString directory = _T("");
    CString name = _T("");
    CString language = _T("");

    GetModuleFileName(theApp.m_hInstance, buffer, MAX_PATH);
    directory.Format(_T("%s"), buffer);
    directory = directory.Left(directory.ReverseFind('\\') + 1);
    name.Format(_T("%s"), buffer);
    name = name.Right(name.GetLength() - name.ReverseFind('\\') - 1);
    name = name.Left(name.ReverseFind('.'));

    m_cbxLanguage.AddString(_T("en"));
    BOOL bWorking = finder.FindFile(directory + name + _T("_*.ini"));
    while (bWorking)
    {
        bWorking = finder.FindNextFile();
        language = finder.GetFilePath();
        language = language.Right(language.GetLength() - language.ReverseFind('_') - 1);
        language = language.Left(language.ReverseFind('.'));
        if (m_cbxLanguage.FindStringExact(0, language) == CB_ERR)
        {
            m_cbxLanguage.AddString(language);
        }
    }
    m_cbxLanguage.SetCurSel(0);
    CString tmp;

    for (int i = 0; i < m_cbxLanguage.GetCount(); i++)
    {
        m_cbxLanguage.GetLBText(i, tmp);
        if (s_mainWnd.m_language == tmp)
        {
            m_cbxLanguage.SetCurSel(i);
            break;
        }
    }
    ReadContMenus();

    UpdateData(FALSE);

    return TRUE;
}

void CSettingsDialogGeneral::OnOK()
{
    UpdateData(TRUE);
    s_mainWnd.m_tpc = m_tpc;

    s_mainWnd.m_showGradientBar = m_l_options.GetCheck(0);
    s_mainWnd.m_graphicalButtons = m_l_options.GetCheck(1);
    if (s_mainWnd.m_graphicalButtons)
    {
        //Nettoyage manuel des textes pour viter la sur impression
        CMenuXP::ms_sCaptions.SetAt(ID_POPUPMENU_PLAY, (CString)_T(""));
        CMenuXP::ms_sCaptions.SetAt(ID_POPUPMENU_PAUSE, (CString)_T(""));
        CMenuXP::ms_sCaptions.SetAt(ID_POPUPMENU_STOP, (CString)_T(""));
        CMenuXP::ms_sCaptions.SetAt(ID_POPUPMENU_PREVIOUS, (CString)_T(""));
        CMenuXP::ms_sCaptions.SetAt(ID_POPUPMENU_NEXT, (CString)_T(""));
    }
    if (s_mainWnd.m_skin.xpLookNFeel != m_l_options.GetCheck(2))
    {
        CMenuXP::UninitializeHook();
        CMenuXP::InitializeHook();
        s_mainWnd.m_skin.xpLookNFeel = m_l_options.GetCheck(2);
        if(s_seekDialog.m_hWnd != NULL)
        {
            CMenuXP::SetXPLookNFeel(&s_seekDialog, s_mainWnd.m_skin.xpLookNFeel);
            CMenuXP::SetXPLookNFeel(&s_mainWnd, s_mainWnd.m_skin.xpLookNFeel);
            s_mainWnd.ReloadMenu();
            s_seekDialog.m_ipe.Invalidate();
            s_seekDialog.m_vol.Invalidate();
            s_seekDialog.m_mb.Invalidate();
            s_seekDialog.m_bar.Invalidate();
            s_seekDialog.Invalidate();
        }
        s_playlistDlg.ReloadMenu();
    }


    s_mainWnd.m_SMTracks = m_l_options.GetCheck(3);
    s_mainWnd.m_SMAlbum = m_l_options.GetCheck(4);
    s_mainWnd.m_autoPlay = m_l_options.GetCheck(5);
    s_mainWnd.m_SMPrevNext = m_l_options.GetCheck(6);
    if (s_mainWnd.m_wAEmu != m_l_options.GetCheck(7))
        s_settingsDialog->m_needToRestart = true;
    s_mainWnd.m_wAEmu = m_l_options.GetCheck(7);
    s_mainWnd.m_autoUpdate = m_l_options.GetCheck(8);
    CString tmp;

    m_cbxLanguage.GetLBText(m_cbxLanguage.GetCurSel(), tmp);
    if (tmp.Find(_T("en")) != -1)
    {
        theApp.WriteProfileString(_T("Preferences"), _T("Language"), _T(""));
        s_mainWnd.m_language = "";
    }
    else
    {
        theApp.WriteProfileString(_T("Preferences"), _T("Language"), tmp);
        s_mainWnd.m_language = tmp;
    }

    RegFileTypes();
    CreateContMenus();

    CPropertyPage::OnOK();
}

BOOL CSettingsDialogGeneral::OnEraseBkgnd(CDC* pDC)
{
    CDwmApiImpl dwmApi;
    if (dwmApi.Initialize() && dwmApi.IsDwmCompositionEnabled())
    {
        CRect rcClient;
        GetClientRect(&rcClient);

        VERIFY(pDC->PatBlt(0, 0, rcClient.Width(), rcClient.Height(), BLACKNESS));
        return TRUE;
    }
    return CPropertyPage::OnEraseBkgnd(pDC);
}

void CSettingsDialogGeneral::OnPopupLoad()
{
    // open a skin file
    CFileDialog dialog(TRUE, _T("skn"), _T(""), OFN_FILEMUSTEXIST, m_Helper.MyLoadString(IDS_OPENSKIN));
    TCHAR buf[MAX_PATH + 1];
    GetModuleFileName(theApp.m_hInstance, buf, MAX_PATH);
    for (int i = _tcslen(buf) - 1; i > 0; i--)
    {
        if (buf[i] == '\\')
        {
            buf[i + 1] = '\0';break;
        }
    }
    dialog.m_ofn.lpstrInitialDir = buf;
    int ret;
    ret = dialog.DoModal();
    if (ret == IDOK)
    {
        CString tmp = dialog.GetPathName();
        if (s_settingsDialog->m_settingsDialogApparenceSkins.m_listSkin.FindString(0, tmp) == -1)
        {
            s_settingsDialog->m_settingsDialogApparenceSkins.m_listSkin.AddString(tmp);
        }
        s_settingsDialog->m_currentSkinSel = s_settingsDialog->m_settingsDialogApparenceSkins.m_listSkin.FindString(0, tmp);
        s_settingsDialog->m_settingsDialogApparenceSkins.m_listSkin.SetCurSel(s_settingsDialog->m_currentSkinSel);
        s_mainWnd.LoadSkin(tmp, &s_settingsDialog->m_loadSkin);
        s_mainWnd.UseSkin(&s_settingsDialog->m_loadSkin);
        s_settingsDialog->m_loadSkin.toolTipText.Replace(_T("\\r"), _T("\r"));
        s_settingsDialog->m_loadSkin.toolTipText.Replace(_T("\\n"), _T("\n"));
        s_settingsDialog->m_loadSkin.toolTipText.Replace(_T("\\t"), _T("\t"));
        GetDlgItem(IDC_SKINSAMPLE)->InvalidateRect(NULL);
    }
}

void CSettingsDialogGeneral::OnPopupUseDefault()
{
    CString tmp = m_Helper.MyLoadString(IDS_DEFAULTSKIN);
    if (s_settingsDialog->m_settingsDialogApparenceSkins.m_listSkin.FindString(0, tmp) == -1)
    {
        s_settingsDialog->m_settingsDialogApparenceSkins.m_listSkin.AddString(tmp);
    }
    s_settingsDialog->m_currentSkinSel = s_settingsDialog->m_settingsDialogApparenceSkins.m_listSkin.FindString(0, tmp);
    s_settingsDialog->m_settingsDialogApparenceSkins.m_listSkin.SetCurSel(s_settingsDialog->m_currentSkinSel);
    s_mainWnd.LoadSkin(tmp, &s_settingsDialog->m_loadSkin);
    s_mainWnd.UseSkin(&s_settingsDialog->m_loadSkin);
    GetDlgItem(IDC_SKINSAMPLE)->InvalidateRect(NULL);
}

void CSettingsDialogGeneral::OnSkin()
{
    CMenu menu;
    CMenu* pPopup;
    if (menu.LoadMenu(IDR_SKINPOPUP))
    {
        pPopup = menu.GetSubMenu(0);
        m_Helper.translateMenu(pPopup, IDR_SKINPOPUP);
        CMenuXP::UpdateMenuBar(this);

        CRect rect;
        GetDlgItem(IDC_SKIN)->GetWindowRect(&rect);
        POINT point;
        point.x = (rect.left + rect.right) / 2;
        point.y = (rect.top + rect.bottom) / 2;
        pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_LEFTBUTTON | TPM_RIGHTBUTTON, point.x, point.y, this);
        GetDlgItem(IDC_SKINSAMPLE)->InvalidateRect(NULL);
    }
}

void CSettingsDialogGeneral::RegFileTypes()
{
    TCHAR fname[MAX_PATH + 1];
    GetModuleFileName(theApp.m_hInstance, fname, MAX_PATH);
    bool enqdef = (((CButton*)GetDlgItem(IDC_ADDBYDEF))->GetCheck() == 1);

    HKEY q;
    unsigned long z, mp;

    CString buf;
    for (int i = 0; i < m_l_options2.GetCount(); i++)
    {
        CString extension = _T("");
        m_l_options2.GetText(i, extension);
        CString dotExtension = _T(".");
        dotExtension.Append(extension.MakeLower());

        CString key = _T("STP");
        key.Append(extension.MakeUpper());
        // Efface de STPExtension
        DeleteKeyAndSubKeys(HKEY_CLASSES_ROOT, key);

        CString defaultIcon = key;
        defaultIcon.Append(_T("\\DefaultIcon"));
        CString shell = key;
        shell.Append(_T("\\Shell"));
        CString shellOpen = shell;
        shellOpen.Append(_T("\\Open"));
        CString shellOpenCommand = shellOpen;
        shellOpenCommand.Append(_T("\\Command"));
        CString shellAdd = shell;
        shellAdd.Append(_T("\\Add"));
        CString shellAddCommand = shellAdd;
        shellAddCommand.Append(_T("\\Command"));

        CGCFileTypeAccess fileTypeAccess;

        fileTypeAccess.SetExtension(extension);
        fileTypeAccess.SetDocumentDescription(LPCTSTR(m_Helper.MyLoadString(IDS_MP3AUDIO)));
        // use first icon in program
        buf = fname;
        buf += ",0";
        fileTypeAccess.SetDocumentDefaultIcon(buf);
        if (enqdef)
            fileTypeAccess.SetDocumentDefault(_T("addstp"));
        else
            fileTypeAccess.SetDocumentDefault(_T("openstp"));
        // just pass file path in quotes on command line
        buf = "\"";
        buf += fname;
        buf += "\"";
        buf += " \"%1\"";
        fileTypeAccess.SetShellOpenCommand(buf);
        fileTypeAccess.SetShellOpenDescription(LPCTSTR(m_Helper.MyLoadString(IDS_MENUOPEN)));
        fileTypeAccess.SetDocumentShellOpenCommand(buf);
        fileTypeAccess.SetDocumentShellOpenDescription(LPCTSTR(m_Helper.MyLoadString(IDS_MENUOPEN)));

        buf = "\"";
        buf += fname;
        buf += "\"";
        buf += " -a \"%1\"";
        fileTypeAccess.SetShellAddCommand(buf);
        fileTypeAccess.SetShellAddDescription(LPCTSTR(m_Helper.MyLoadString(IDS_ADDTOPLAYLIST)));
        fileTypeAccess.SetDocumentShellAddCommand(buf);
        fileTypeAccess.SetDocumentShellAddDescription(LPCTSTR(m_Helper.MyLoadString(IDS_ADDTOPLAYLIST)));
        fileTypeAccess.SetDocumentClassName(key);

        if (m_l_options2.GetCheck(i) == 1)
        {
            fileTypeAccess.RegSetAllExtensionInfo();
        }
        else
        {
            //Effacement paramtre par dfaut
            if (RegOpenKeyEx(HKEY_CLASSES_ROOT, dotExtension, 0, KEY_READ, &q) == ERROR_SUCCESS)
            {
                TCHAR fname[MAX_PATH + 1];
                z = REG_SZ;mp = MAX_PATH;
                if (RegQueryValueEx(q, _T(""), NULL, &z, (LPBYTE)fname, &mp) == ERROR_SUCCESS)
                {
                    if (key.CompareNoCase(fname) == 0)
                    {
                        fileTypeAccess.SetRegistryValue(HKEY_CLASSES_ROOT, dotExtension, _T(""), _T(""));
                    }
                }
                RegCloseKey(q);
            }

            //Effacement de addstp si = stp
            CString dotExtensionAdd = dotExtension + _T("\\shell\\addstp\\command");
            if (RegOpenKeyEx(HKEY_CLASSES_ROOT, dotExtensionAdd, 0, KEY_READ, &q) == ERROR_SUCCESS)
            {
                TCHAR fname[MAX_PATH + 1];
                z = REG_SZ;mp = MAX_PATH;
                if (RegQueryValueEx(q, _T(""), NULL, &z, (LPBYTE)fname, &mp) == ERROR_SUCCESS)
                {
                    CString addCommand;
                    fileTypeAccess.GetShellAddCommand(addCommand);
                    if (addCommand.CompareNoCase(fname) == 0)
                    {
                        dotExtensionAdd = dotExtension + _T("\\shell\\addstp");
                        DeleteKeyAndSubKeys(HKEY_CLASSES_ROOT, dotExtensionAdd);
                    }
                }
                RegCloseKey(q);
            }
            //Effacement de openstp si = stp
            CString dotExtensionOpen = dotExtension + _T("\\shell\\openstp\\command");
            if (RegOpenKeyEx(HKEY_CLASSES_ROOT, dotExtensionOpen, 0, KEY_READ, &q) == ERROR_SUCCESS)
            {
                TCHAR fname[MAX_PATH + 1];
                z = REG_SZ;mp = MAX_PATH;
                if (RegQueryValueEx(q, _T(""), NULL, &z, (LPBYTE)fname, &mp) == ERROR_SUCCESS)
                {
                    CString openCommand;
                    fileTypeAccess.GetShellOpenCommand(openCommand);
                    if (openCommand.CompareNoCase(fname) == 0)
                    {
                        dotExtensionOpen = dotExtension + _T("\\shell\\openstp");
                        DeleteKeyAndSubKeys(HKEY_CLASSES_ROOT, dotExtensionOpen);
                    }
                }
                RegCloseKey(q);
            }
        }
    }
}


void CSettingsDialogGeneral::ReadContMenus()
{
    HKEY q;    
    unsigned long z, mp;

    bool PrevPlay = true, PrevAdd = true;

    TCHAR fname[MAX_PATH + 1];
    GetModuleFileName(theApp.m_hInstance, fname, MAX_PATH); 
    CString mFile = _T("\"");
    mFile += fname;
    mFile += _T("\" \"%1\"");

    if (RegOpenKeyEx(HKEY_CLASSES_ROOT, _T("Directory\\Shell\\playstp\\Command"), 0, KEY_READ, &q) == ERROR_SUCCESS)
    {
        z = REG_SZ;mp = MAX_PATH;
        if (RegQueryValueEx(q, _T(""), NULL, &z, (LPBYTE)fname, &mp) == ERROR_SUCCESS)
        {
            if (mFile.CompareNoCase(fname) != 0)
                PrevPlay = false;
        }
        else
            PrevPlay = false;
        RegCloseKey(q);
    }
    else
        PrevPlay = false;

    GetModuleFileName(theApp.m_hInstance, fname, MAX_PATH); 
    mFile = "\"";
    mFile += fname;
    mFile += "\" \"%1\"";
    if (RegOpenKeyEx(HKEY_CLASSES_ROOT, _T("Drive\\Shell\\playstp\\Command"), 0, KEY_READ, &q) == ERROR_SUCCESS)
    {
        z = REG_SZ;mp = MAX_PATH;
        if (RegQueryValueEx(q, _T(""), NULL, &z, (LPBYTE)fname, &mp) == ERROR_SUCCESS)
        {
            if (mFile.CompareNoCase(fname) != 0)
                PrevPlay = false;
        }
        else
            PrevPlay = false;
        RegCloseKey(q);
    }
    else
        PrevPlay = false;

    GetModuleFileName(theApp.m_hInstance, fname, MAX_PATH); 
    mFile = "\"";
    mFile += fname;
    mFile += "\" -a \"%1\"";

    if (RegOpenKeyEx(HKEY_CLASSES_ROOT, _T("Directory\\Shell\\addstp\\Command"), 0, KEY_READ, &q) == ERROR_SUCCESS)
    {
        z = REG_SZ;mp = MAX_PATH;
        if (RegQueryValueEx(q, _T(""), NULL, &z, (LPBYTE)fname, &mp) == ERROR_SUCCESS)
        {
            if (mFile.CompareNoCase(fname) != 0)
                PrevAdd = false;
        }
        else
            PrevAdd = false;
        RegCloseKey(q);
    }
    else
        PrevAdd = false;

    GetModuleFileName(theApp.m_hInstance, fname, MAX_PATH); 
    mFile = "\"";
    mFile += fname;
    mFile += "\" -a \"%1\"";

    if (RegOpenKeyEx(HKEY_CLASSES_ROOT, _T("Drive\\Shell\\addstp\\Command"), 0, KEY_READ, &q) == ERROR_SUCCESS)
    {
        z = REG_SZ;mp = MAX_PATH;
        if (RegQueryValueEx(q, _T(""), NULL, &z, (LPBYTE)fname, &mp) == ERROR_SUCCESS)
        {
            if (mFile.CompareNoCase(fname) != 0)
                PrevAdd = false;
        }
        else
            PrevAdd = false;
        RegCloseKey(q);
    }
    else
        PrevAdd = false;

    if (PrevPlay)
        ((CButton *)GetDlgItem(IDC_CM_PLAY))->SetCheck(1);
    if (PrevAdd)
        ((CButton *)GetDlgItem(IDC_CM_ADD))->SetCheck(1);
}

bool CSettingsDialogGeneral::ReadFileTypes(CString ext)
{
    CString dotExtension = _T(".");
    dotExtension.Append(ext.MakeLower());

    CString key = _T("STP");
    key.Append(ext.MakeUpper());

    CString shell = key;
    shell.Append(_T("\\Shell"));
    CString shellOpen = shell;
    shellOpen.Append(_T("\\Open"));
    CString shellOpenCommand = shellOpen;
    shellOpenCommand.Append(_T("\\Command"));
    CString shellAdd = shell;
    shellAdd.Append(_T("\\Add"));
    CString shellAddCommand = shellAdd;
    shellAddCommand.Append(_T("\\Command"));

    TCHAR fname[MAX_PATH + 1];
    GetModuleFileName(theApp.m_hInstance, fname, MAX_PATH); 
    CString mFile;
    mFile = _T("\"");
    mFile += fname;
    mFile += _T("\"");
    mFile += _T(" -a \"%1\"");

    HKEY q;
    unsigned long z;
    unsigned long mp; 

    int qq = 0;

    if (RegOpenKeyEx(HKEY_CLASSES_ROOT, key, 0, KEY_READ, &q) == ERROR_SUCCESS)
    {
        qq++;
    }

    CString dotExtensionAdd = dotExtension + _T("\\shell\\addstp\\command");
    if (RegOpenKeyEx(HKEY_CLASSES_ROOT, dotExtensionAdd, 0, KEY_READ, &q) == ERROR_SUCCESS)
    {
        TCHAR fname[MAX_PATH + 1];
        z = REG_SZ;mp = MAX_PATH;
        if (RegQueryValueEx(q, _T(""), NULL, &z, (LPBYTE)fname, &mp) == ERROR_SUCCESS)
        {
            if (mFile.CompareNoCase(fname) == 0)
            {
                qq++;
            }
        }
        RegCloseKey(q);
    }

    mFile = _T("\"");
    mFile += fname;
    mFile += _T("\"");
    mFile += _T(" -a \"%1\"");

    CString dotExtensionOpen = dotExtension + _T("\\shell\\openstp\\command");
    if (RegOpenKeyEx(HKEY_CLASSES_ROOT, dotExtensionAdd, 0, KEY_READ, &q) == ERROR_SUCCESS)
    {
        TCHAR fname[MAX_PATH + 1];
        z = REG_SZ;mp = MAX_PATH;
        if (RegQueryValueEx(q, _T(""), NULL, &z, (LPBYTE)fname, &mp) == ERROR_SUCCESS)
        {
            if (mFile.CompareNoCase(fname) == 0)
            {
                qq++;
            }
        }
        RegCloseKey(q);
    }

    mFile = _T("addstp");
    CString shellExt = key + _T("\\shell");
    if (RegOpenKeyEx(HKEY_CLASSES_ROOT, shellExt, 0, KEY_READ, &q) == ERROR_SUCCESS)
    {
        TCHAR fname[MAX_PATH + 1];
        z = REG_SZ;mp = MAX_PATH;
        if (RegQueryValueEx(q, _T(""), NULL, &z, (LPBYTE)fname, &mp) == ERROR_SUCCESS)
        {
            if (mFile.CompareNoCase(fname) == 0)
            {
                m_defaultAdd = true;
            }
        }
        RegCloseKey(q);
    }

    return qq != 0;
}

BEGIN_MESSAGE_MAP(CSettingsDialogGeneral, CPropertyPage)
    ON_BN_CLICKED(IDC_SKIN, OnSkin)
    ON_COMMAND(ID_POPUP_LOAD, OnPopupLoad)
    ON_COMMAND(ID_POPUP_USEDEFAULT, OnPopupUseDefault)
    ON_WM_CREATE()
    ON_WM_MOUSEMOVE()
    ON_LBN_SELCHANGE(IDC_L_OPTIONS, OnLbnSelchangeLOptions)
    ON_WM_ERASEBKGND()
END_MESSAGE_MAP()

void CSettingsDialogGeneral::OnLbnSelchangeLOptions()
{
    UpdateData(TRUE);
    s_settingsDialog->m_loadSkin.xpLookNFeel = m_l_options.GetCheck(2);
    m_skinSample.SetForceText(!m_l_options.GetCheck(1));
    GetDlgItem(IDC_SKINSAMPLE)->Invalidate();
}
