// SettingsDialogOutput.cpp : fichier d'implmentation
//

#include "stdafx.h"
#include "resource.h"       // main symbols
#include "aerosubc.h"
#include "aeroglss.h"
#include "SysTrayPlay.h"
#include "SettingsDialog.h"
#include "MainHelper.h"
#include "MainWnd.h"
#include "MultiDllInput.h"
#include "MultiDllOutput.h"
#include "WAEmuwnd.h"
#include "CommonFunc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Bote de dialogue CSettingsDialogOutput

IMPLEMENT_MENUXP(CSettingsDialogOutput, CPropertyPage);

IMPLEMENT_DYNAMIC(CSettingsDialogOutput, CPropertyPage)

CSettingsDialogOutput::CSettingsDialogOutput() : CPropertyPage(CSettingsDialogOutput::IDD)
{
    m_current = -1;
}

CSettingsDialogOutput::~CSettingsDialogOutput()
{
}

void CSettingsDialogOutput::DoDataExchange(CDataExchange* pDX)
{
    CPropertyPage::DoDataExchange(pDX);
    CMenuXP::SetXPLookNFeel(this, s_mainWnd.m_skin.xpLookNFeel);
    DDX_Control(pDX, IDC_ABOUT_OUTPUT, m_btnAbout);
    DDX_Control(pDX, IDC_CONFIG_OUTPUT, m_btnConfiguration);
    DDX_Control(pDX, IDC_LOAD_OUTPUT, m_btnLoad);
    DDX_Control(pDX, IDC_UNLOAD_OUTPUT, m_btnUnload);
    DDX_Control(pDX, IDC_LIST_OUTPUT, m_listOutput);
    DDX_Control(pDX, IDC_OUTPUTDOWN, m_btnDown);
    DDX_Control(pDX, IDC_OUTPUTUP, m_btnUp);
}

BOOL CSettingsDialogOutput::OnInitDialog()
{
    CPropertyPage::OnInitDialog();

    AeroAutoSubclass(m_hWnd, ASC_SUBCLASS_ALL_CONTROLS, 0L);

    m_Helper.translateDlgItemText(this, IDC_CONFIG_OUTPUT);
    m_Helper.translateDlgItemText(this, IDC_ABOUT_OUTPUT);
    m_Helper.translateDlgItemText(this, IDC_LOAD_OUTPUT);
    m_Helper.translateDlgItemText(this, IDC_UNLOAD_OUTPUT);
    m_btnDown.SetIcon((HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_ICON_DOWN), IMAGE_ICON, 32, 32, LR_DEFAULTCOLOR));
    m_btnUp.SetIcon((HICON)::LoadImage(theApp.m_hInstance, MAKEINTRESOURCE(IDI_ICON_UP), IMAGE_ICON, 32, 32, LR_DEFAULTCOLOR));

    return TRUE;
}

BOOL CSettingsDialogOutput::OnEraseBkgnd(CDC* pDC)
{
    CDwmApiImpl dwmApi;
    if (dwmApi.Initialize() && dwmApi.IsDwmCompositionEnabled())
    {
        CRect rcClient;
        GetClientRect(&rcClient);

        VERIFY(pDC->PatBlt(0, 0, rcClient.Width(), rcClient.Height(), BLACKNESS));
        return TRUE;
    }
    return CPropertyPage::OnEraseBkgnd(pDC);
}

BOOL CSettingsDialogOutput::OnSetActive()
{
    while (m_listOutput.GetCount() != 0)
    {
        m_listOutput.DeleteString(0);
    }
    for (int i = 0; i < stdOut.GetSize(); i++)
    {
        CDllOutput* output = stdOut.getDllOutput(i);
        CString desc = _T("");
        TCHAR descTCHAR[500];
        Strcpy(descTCHAR, 500, output->GetDescription());
        desc.Format(_T("%s (%s)"), descTCHAR, output->GetFileName());
        m_listOutput.AddString(desc);
        if (stdOut.GetOut_Module() == output->GetOut_Module())
        {
            m_listOutput.SetCurSel(i);
            m_current = i;
        }
    }
    return TRUE;
}

void CSettingsDialogOutput::OnOK()
{
    UpdateData(TRUE);
    CString dllOutKey = _T("");
    for (int i = 0; i < stdOut.GetSize(); i++)
    {
        CDllOutput* output = stdOut.getDllOutput(i);
        dllOutKey.Format(_T("dllOutName%d"), i);
        AfxGetApp()->WriteProfileString(_T("Preferences"), dllOutKey, output->GetFilePath());
    }
    dllOutKey.Format(_T("dllOutName%d"), stdOut.GetSize());
    AfxGetApp()->WriteProfileString(_T("Preferences"), dllOutKey, _T("NONE"));
    CPropertyPage::OnOK();
}
BEGIN_MESSAGE_MAP(CSettingsDialogOutput, CPropertyPage)
  ON_BN_CLICKED(IDC_CONFIG_OUTPUT, OnBnClickedConfig)
  ON_BN_CLICKED(IDC_ABOUT_OUTPUT, OnBnClickedAbout)
  ON_BN_CLICKED(IDC_LOAD_OUTPUT, OnBnClickedLoad)
  ON_BN_CLICKED(IDC_UNLOAD_OUTPUT, OnBnClickedUnload)
  ON_BN_CLICKED(IDC_OUTPUTDOWN, OnBnClickedDown)
  ON_BN_CLICKED(IDC_OUTPUTUP, OnBnClickedUp)
  ON_LBN_SELCHANGE(IDC_LIST_OUTPUT,OnListChange)
  ON_WM_ERASEBKGND()
  ON_MENUXP_MESSAGES()
END_MESSAGE_MAP()


// Gestionnaires de messages CSettingsDialogOutput
void CSettingsDialogOutput::OnBnClickedConfig()
{
    for (int i = stdOut.GetSize() - 1; i >= 0; i--)
    {
        if (m_listOutput.GetSel(i) != 0)
        {
            CDllOutput* output = stdOut.getDllOutput(i);
            output->Config(GetParent()->m_hWnd);
        }
    }
}

void CSettingsDialogOutput::OnBnClickedAbout()
{
    for (int i = stdOut.GetSize() - 1; i >= 0; i--)
    {
        if (m_listOutput.GetSel(i) != 0)
        {
            CDllOutput* output = stdOut.getDllOutput(i);
            output->About(GetParent()->m_hWnd);
        }
    }
}

void CSettingsDialogOutput::OnBnClickedLoad()
{
    CString filter = _T("");
    CFileDialog dialog(TRUE, _T("dll"), _T(""), OFN_FILEMUSTEXIST, m_Helper.MyLoadString(IDS_OPENOUTPUT));
    TCHAR buffer[_MAX_PATH*500 + 1];
    buffer[0] = '\0';
    dialog.m_ofn.lpstrFile = buffer;
    dialog.m_ofn.nMaxFile = _MAX_PATH * 500;
    if (dialog.DoModal() == IDOK)
    {
        int oldSize = stdOut.GetSize();
        stdOut.Load(dialog.GetPathName(), WAEmuWnd.m_hWnd);
        if (oldSize != stdOut.GetSize())
        {
            CDllOutput* output = stdOut.getDllOutput(oldSize);
            CString desc = _T("");
            TCHAR descTCHAR[500];
            Strcpy(descTCHAR, 500, output->GetDescription());
            desc.Format(_T("%s (%s)"), descTCHAR, output->GetFileName());
            m_listOutput.AddString(desc);
        }
    }
}

void CSettingsDialogOutput::OnBnClickedUnload()
{
    if (-1 < m_listOutput.GetCurSel() && m_listOutput.GetCurSel() < stdOut.GetSize())
    {
        int index = m_listOutput.GetCurSel();
        CDllOutput* output = stdOut.getDllOutput(index);
        if (output->GetFilePath() != "STANDARD")
        {
            if (output->GetOut_Module() == stdOut.GetOut_Module() && stdIn.GetPlayerMode() != PLAYER_MODE_CLOSE)
                ::SendMessage(WAEmuWnd.m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
            stdOut.UnLoad(index);
            m_listOutput.DeleteString(index);
            m_current = -1;
            m_listOutput.SetCurSel(m_current);
            stdIn.SetOutMod(stdOut.GetOut_Module());
        }
    }
}

void CSettingsDialogOutput::OnListChange()
{
    m_current = m_listOutput.GetCurSel();
}

void CSettingsDialogOutput::OnBnClickedDown()
{
    UpdateData(TRUE);
    if (m_current >= 0 && m_current < stdOut.GetSize() - 1)
    {
        stdOut.UpDllOutput(m_current + 1);
        CDllOutput* output = stdOut.getDllOutput(m_current);
        CString desc = _T("");
        TCHAR descTCHAR[500];
        Strcpy(descTCHAR, 500, output->GetDescription());
        desc.Format(_T("%s (%s)"), descTCHAR, output->GetFileName());
        m_listOutput.DeleteString(m_current + 1);
        m_listOutput.InsertString(m_current, desc);
        m_current++;
        m_listOutput.SetCurSel(m_current);
    }
}

void CSettingsDialogOutput::OnBnClickedUp()
{
    UpdateData(TRUE);
    if (m_current > 0 && m_current <= stdOut.GetSize() - 1)
    {
        stdOut.UpDllOutput(m_current);
        CDllOutput* output = stdOut.getDllOutput(m_current - 1);
        CString desc = _T("");
        TCHAR descTCHAR[500];
        Strcpy(descTCHAR, 500, output->GetDescription());
        desc.Format(_T("%s (%s)"), descTCHAR, output->GetFileName());
        m_listOutput.DeleteString(m_current);
        m_listOutput.InsertString(m_current - 1, desc);
        m_current--;
        m_listOutput.SetCurSel(m_current);
    }
}