// SettingsDialogPlugins.cpp : fichier d'implmentation
//

#include "StdAfx.h"
#include "resource.h"
#include "aerosubc.h"
#include "aeroglss.h"
#include "SettingsDialog.h"
#include "MainHelper.h"
#include "MainWnd.h"
#include "MultiDllGen.h"
#include "MultiDllInput.h"
#include "MultiDllOutput.h"
#include "WAEmuwnd.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Bote de dialogue CSettingPlugins

IMPLEMENT_MENUXP(CSettingsDialogPlugins, CPropertyPage);

IMPLEMENT_DYNAMIC(CSettingsDialogPlugins, CPropertyPage)

CSettingsDialogPlugins::CSettingsDialogPlugins() : CPropertyPage(CSettingsDialogPlugins::IDD)
{
    m_activateWindowMessenger = 0;
    m_scrollMode = 0;
    m_speed = 0;
    m_icon = 0;
}

CSettingsDialogPlugins::~CSettingsDialogPlugins()
{
}


void CSettingsDialogPlugins::DoDataExchange(CDataExchange* pDX)
{
    CPropertyPage::DoDataExchange(pDX);
    CMenuXP::SetXPLookNFeel(this, s_mainWnd.m_skin.xpLookNFeel);
    DDX_Check(pDX, IDC_WMACTIVATE, m_activateWindowMessenger);
    DDX_Text(pDX, IDC_WMSPEED, m_speed);
    DDX_Text(pDX, IDC_WMTITLE, m_title);
    DDX_Control(pDX, IDC_WMICON, m_wmIcon);
    DDX_Control(pDX, IDC_PLUGINDIRECTORYBROWSE, m_btnBrowse);
    DDX_Control(pDX, IDC_PLUGINDIRECTORYLOAD, m_btnLoad);
    DDX_Control(pDX, IDC_PLUGINDIRECTORYLOADALL, m_btnLoadAll);
    DDX_CBIndex(pDX, IDC_WMICON, m_icon);
    DDX_Control(pDX, IDC_LISTPLUGINS, m_listPlugins);
    DDX_Radio(pDX, IDC_WMSCROLL, m_scrollMode);
}

BOOL CSettingsDialogPlugins::OnInitDialog()
{
    CPropertyPage::OnInitDialog();

    AeroAutoSubclass(m_hWnd, ASC_SUBCLASS_ALL_CONTROLS, 0L);

    m_activateWindowMessenger = s_mainWnd.m_activateWindowsMessenger;
    m_title = s_mainWnd.m_wmTitle;
    m_icon = s_mainWnd.m_wmIcon;
    m_speed = s_mainWnd.m_wmSpeed;
    m_scrollMode = s_mainWnd.m_scrollMode;

    m_Helper.translateWindowText(this, IDD_SETTINGS_PLUGINS);
    m_Helper.translateDlgItemText(this, IDC_LPLUGINDIRECTORY);
    m_Helper.translateDlgItemText(this, IDC_PLUGINDIRECTORYBROWSE);
    m_Helper.translateDlgItemText(this, IDC_PLUGINDIRECTORYLOAD);
    m_Helper.translateDlgItemText(this, IDC_PLUGINDIRECTORYLOADALL);
    m_Helper.translateDlgItemText(this, IDC_LWINDOWSMESSENGER);
    m_Helper.translateDlgItemText(this, IDC_WMACTIVATE);
    m_Helper.translateDlgItemText(this, IDC_LWMTITLE);
    m_Helper.translateDlgItemText(this, IDC_LWMICON);
    m_Helper.translateDlgItemText(this, IDC_LWMSPEED);
    m_Helper.translateDlgItemText(this, IDC_WMSCROLL);
    m_Helper.translateDlgItemText(this, IDC_WMSCROLL2);
    m_Helper.translateDlgItemText(this, IDC_WMSCROLL3);

    CheckDlgButton(IDC_WMSCROLL, m_scrollMode == 0);
    CheckDlgButton(IDC_WMSCROLL2, m_scrollMode == 1);
    CheckDlgButton(IDC_WMSCROLL3, m_scrollMode == 2);

    m_wmIcon.AddString(m_Helper.MyLoadString(IDS_WMMUSIC));
    m_wmIcon.AddString(m_Helper.MyLoadString(IDS_WMGAME));
    m_wmIcon.AddString(m_Helper.MyLoadString(IDS_WMOFFICE));
    m_wmIcon.SetCurSel(m_icon);

    ((CSpinButtonCtrl *)GetDlgItem(IDC_SPIN2))->SetRange(0, 10);

    UpdateData(FALSE);

    OnBnClickedWmactivate();

    return TRUE;
}

BOOL CSettingsDialogPlugins::OnEraseBkgnd(CDC* pDC)
{
    CDwmApiImpl dwmApi;
    if (dwmApi.Initialize() && dwmApi.IsDwmCompositionEnabled())
    {
        CRect rcClient;
        GetClientRect(&rcClient);

        VERIFY(pDC->PatBlt(0, 0, rcClient.Width(), rcClient.Height(), BLACKNESS));
        return TRUE;
    }
    return CPropertyPage::OnEraseBkgnd(pDC);
}

void CSettingsDialogPlugins::OnOK()
{
    UpdateData(TRUE);

    s_mainWnd.m_activateWindowsMessenger = m_activateWindowMessenger;
    s_mainWnd.m_wmTitle = m_title;
    s_mainWnd.m_wmIcon = m_icon;
    s_mainWnd.m_wmSpeed = m_speed;
    s_mainWnd.m_scrollMode = m_scrollMode;
    s_mainWnd.KillTimer(WM_DELAY);

    if (!m_activateWindowMessenger)
    {
        s_mainWnd.SendMSNMessage(false, _T("Office"), _T(""));
    }
    else
    {
        s_mainWnd.SetTimer(WM_DELAY, m_speed * 200, NULL);
    }

    CPropertyPage::OnOK();
}

BEGIN_MESSAGE_MAP(CSettingsDialogPlugins, CPropertyPage)
    ON_BN_CLICKED(IDC_WMACTIVATE, OnBnClickedWmactivate)
    ON_BN_CLICKED(IDC_PLUGINDIRECTORYBROWSE, OnBnClickedPlugindirectorybrowse)
    ON_BN_CLICKED(IDC_PLUGINDIRECTORYLOADALL, OnBnClickedPlugindirectoryloadall)
    ON_BN_CLICKED(IDC_PLUGINDIRECTORYLOAD, OnBnClickedPlugindirectoryload)
    ON_WM_ERASEBKGND()
END_MESSAGE_MAP()

void CSettingsDialogPlugins::OnBnClickedWmactivate()
{
    UpdateData(TRUE);

    GetDlgItem(IDC_WMTITLE)->EnableWindow(m_activateWindowMessenger);
    GetDlgItem(IDC_WMICON)->EnableWindow(m_activateWindowMessenger);
    GetDlgItem(IDC_WMSPEED)->EnableWindow(m_activateWindowMessenger);
    GetDlgItem(IDC_WMSCROLL)->EnableWindow(m_activateWindowMessenger);
    GetDlgItem(IDC_WMSCROLL2)->EnableWindow(m_activateWindowMessenger);
    GetDlgItem(IDC_WMSCROLL3)->EnableWindow(m_activateWindowMessenger);
}

void CSettingsDialogPlugins::OnBnClickedPlugindirectorybrowse()
{
    UpdateData(TRUE);
    LPMALLOC pMalloc;
    if (SHGetMalloc(&pMalloc) != NOERROR)
        return;

    BROWSEINFO bi;
    TCHAR buf[MAX_PATH];
    _tcscpy(buf, s_mainWnd.m_initDir.GetString());

    CString title;
    title = m_Helper.MyLoadString(IDS_OPENFOLDER);

    bi.hwndOwner = m_hWnd;
    bi.pszDisplayName = buf;
    bi.pidlRoot = NULL;
    bi.lpszTitle = LPCTSTR(title);
    bi.ulFlags = BIF_RETURNONLYFSDIRS;
    bi.lpfn = NULL;

    LPITEMIDLIST pidl;
    if ((pidl = SHBrowseForFolder(&bi)) == NULL)
    {
        pMalloc->Free(pidl);
        pMalloc ->Release();
        return;
    }
    if (SHGetPathFromIDList(pidl, buf) == false)
    {
        pMalloc->Free(pidl);
        pMalloc ->Release();
        return;
    }
    pMalloc->Free(pidl);
    pMalloc->Release();

    CString theDir;
    theDir.Format(_T("%s\\"), buf);
    GetDlgItem(IDC_PLUGINDIRECTORY)->SetWindowText(theDir.GetString());
    LoadPluginsName(theDir);

    GetDlgItem(IDC_PLUGINDIRECTORYLOADALL)->EnableWindow(m_listPlugins.GetCount() != 0);
    GetDlgItem(IDC_PLUGINDIRECTORYLOAD)->EnableWindow(m_listPlugins.GetCount() != 0);
}

void CSettingsDialogPlugins::LoadPluginsName(CString theDir)
{
    while (m_listPlugins.GetCount() != 0)
    {
        m_listPlugins.DeleteString(0);
    }

    CFileFind finder;
    CString filePath;
    BOOL bWorking;
    bWorking = finder.FindFile(theDir + _T("gen_*.dll"));
    while (bWorking)
    {
        bWorking = finder.FindNextFile();
        filePath = finder.GetFileName();
        bool alreadyLoad = false;
        for (int i = 0; i < stdIn.GetSize(); i++)
        {
            CDllInput* input = stdIn.getDllInput(i);
            if (input->GetFileName() == filePath)
            {
                alreadyLoad = true;
                break;
            }
        }
        if (alreadyLoad)
        {
            continue;
        }
        m_listPlugins.AddString(filePath);
    }
    bWorking = finder.FindFile(theDir + _T("in_*.dll"));
    while (bWorking)
    {
        bWorking = finder.FindNextFile();
        filePath = finder.GetFileName();
        bool alreadyLoad = false;
        for (int i = 0; i < stdIn.GetSize(); i++)
        {
            CDllInput* input = stdIn.getDllInput(i);
            if (input->GetFileName() == filePath)
            {
                alreadyLoad = true;
                break;
            }
        }
        if (alreadyLoad)
        {
            continue;
        }
        m_listPlugins.AddString(filePath);
    }
    bWorking = finder.FindFile(theDir + _T("out_*.dll"));
    while (bWorking)
    {
        bWorking = finder.FindNextFile();
        filePath = finder.GetFileName();
        bool alreadyLoad = false;
        for (int i = 0; i < stdIn.GetSize(); i++)
        {
            CDllInput* input = stdIn.getDllInput(i);
            if (input->GetFileName() == filePath)
            {
                alreadyLoad = true;
                break;
            }
        }
        if (alreadyLoad)
        {
            continue;
        }
        m_listPlugins.AddString(filePath);
    }
}
BOOL CSettingsDialogPlugins::OnSetActive()
{
    CString theDir;
    GetDlgItemText(IDC_PLUGINDIRECTORY, theDir);
    if (theDir != "")
    {
        LoadPluginsName(theDir);
    }

    GetDlgItem(IDC_PLUGINDIRECTORYLOADALL)->EnableWindow(m_listPlugins.GetCount() != 0);
    GetDlgItem(IDC_PLUGINDIRECTORYLOAD)->EnableWindow(m_listPlugins.GetCount() != 0);

    return TRUE;
}

void CSettingsDialogPlugins::LoadPlugin(CString plugin)
{
    for (int i = 0; i < m_listPlugins.GetCount(); i++)
    {
        CString pluginName;
        m_listPlugins.GetText(i, pluginName);
        if (plugin == pluginName)
        {
            m_listPlugins.DeleteString(i);
            break;
        }
    }
    CString theDir;
    GetDlgItemText(IDC_PLUGINDIRECTORY, theDir);
    if (plugin.MakeLower().Find(_T("gen_")) == 0)
    {
        stdGen.Load(theDir + plugin, WAEmuWnd.m_hWnd);
    }
    if (plugin.MakeLower().Find(_T("in_")) == 0)
    {
        stdIn.Load(theDir + plugin, WAEmuWnd.m_hWnd);
    }
    if (plugin.MakeLower().Find(_T("out_")) == 0)
    {
        stdOut.Load(theDir + plugin, WAEmuWnd.m_hWnd);
    }
}

void CSettingsDialogPlugins::OnBnClickedPlugindirectoryloadall()
{
    while (m_listPlugins.GetCount() != 0)
    {
        CString pluginName;
        m_listPlugins.GetText(0, pluginName);
        LoadPlugin(pluginName);
    }
}

void CSettingsDialogPlugins::OnBnClickedPlugindirectoryload()
{
    UpdateData(TRUE);

    int nCount = m_listPlugins.GetSelCount();
    CArray<int, int> arrayListBoxSel;

    arrayListBoxSel.SetSize(nCount);
    m_listPlugins.GetSelItems(nCount, arrayListBoxSel.GetData()); 

    for (int i = 0; i < arrayListBoxSel.GetCount(); i++)
    {
        int index = arrayListBoxSel.GetAt(i);

        CString pluginName;

        m_listPlugins.GetText(index - i, pluginName);

        LoadPlugin(pluginName);
    }

    GetDlgItem(IDC_PLUGINDIRECTORYLOADALL)->EnableWindow(m_listPlugins.GetCount() != 0);
    GetDlgItem(IDC_PLUGINDIRECTORYLOAD)->EnableWindow(m_listPlugins.GetCount() != 0);
}
