// SysTrayPlay.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "resource.h"       // main symbols
#include "SysTrayPlay.h"
#include "MainWnd.h"
#include "WAEmuWnd.h"
#include "PlaylistDlg.h"
#include "MultiDllOutput.h"
#include "MultiDllInput.h"
#include "MultiDllGen.h"
#include "MemLeakDetect.h"
#include "SeekDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

HINSTANCE g_hInst;
CSysTrayPlayApp theApp;
extern CSeekDialog s_seekDialog;

// Detect Memory Leaks

#ifdef _DEBUGA
CMemLeakDetect memLeakDetect;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSysTrayPlayApp construction

CSysTrayPlayApp::CSysTrayPlayApp() : m_GdiplusToken(NULL), m_GdiPlusStatus(NotImplemented)
{
    TCHAR profname[MAX_PATH];

    g_hInst = m_hInstance;
    GetModuleFileName(m_hInstance, profname, MAX_PATH);
    for (int i = _tcslen(profname) - 1; i > 0; i--)
    {
        if (profname[i] == '.')
        {
            profname[i + 1] = 'i';
            profname[i + 2] = 'n';
            profname[i + 3] = 'i';
            profname[i + 4] = '\0';
            break;
        }
    }

    free((void*)m_pszProfileName);
    m_pszProfileName = _tcsdup(profname);
}

/////////////////////////////////////////////////////////////////////////////
// CSysTrayPlayApp initialization

int CSysTrayPlayApp::ExitInstance()
{
    stdGen.UnloadAll();
    extern void EquQuit();
    EquQuit();
    stdIn.UnloadAll();
    stdOut.UnloadAll();

    CMenuXP::UninitializeHook();
    if (s_playlistDlg.m_hWnd != NULL)
        s_playlistDlg.DestroyWindow();
    if (s_seekDialog.m_hWnd != NULL)
         s_seekDialog.DestroyWindow();
    if (WAEmuWnd.m_hWnd != NULL)
        WAEmuWnd.DestroyWindow();
    if (s_mainWnd.m_hWnd != NULL)
        s_mainWnd.DestroyWindow();

    // Release the wndclass name winamp
    if (bClassRegisteredWinamp)
        ::UnregisterClass(_T("Winamp v1.x"), AfxGetInstanceHandle());

    // Release mutex winamp
    if (hMutexWinamp != NULL)
        ReleaseMutex(hMutexWinamp);

    if (Ok == m_GdiPlusStatus)
        GdiplusShutdown(m_GdiplusToken);

    return CWinApp::ExitInstance();
}

BOOL CALLBACK searcher(HWND hWnd, LPARAM lParam);

BOOL CSysTrayPlayApp::InitInstance()
{
    CMenuXP::InitializeHook();

    GdiplusStartupInput gdiplusStartupInput;
    m_GdiPlusStatus = GdiplusStartup(&m_GdiplusToken, &gdiplusStartupInput, NULL);

    hMutexWinamp = CreateMutex(NULL, FALSE, _T("Winamp v1.x"));

    struct CmdLineStruct
    {
        CString fname;
        bool addFlag;
        bool exitFlag;
        bool restartFlag;
    } CmdLine;

    DWORD result;

    CmdLine.fname = _T("");
    CmdLine.addFlag = false;
    CmdLine.exitFlag = false;
    CmdLine.restartFlag = false;
    CString tmp, tmp1;  
    for (int i = 1; i < __argc; i++)
    {
        LPCTSTR pszParam = __targv[i];
        tmp1.Format(_T(" %s"), pszParam);
        tmp += tmp1;

        BOOL bFlag = FALSE;
        if (pszParam[0] == '-')
        {
            // remove flag specifier
            bFlag = TRUE;
            pszParam++;
        }
        if (bFlag)
        {
            if (_tcscmp(pszParam, _T("?")) == 0)
            {
                AfxMessageBox(_T("STP command line :\n -a : Add a file/directory/playlist\n -x : Exit STP"), MB_OK);
                return FALSE;
            }
            else if (_tcscmp(pszParam, _T("a")) == 0)
                CmdLine.addFlag = true;
            else if (_tcscmp(pszParam, _T("x")) == 0)
                CmdLine.exitFlag = true;
            else if (_tcscmp(pszParam, _T("restart")) == 0)
                CmdLine.restartFlag = true;
        }
        else
        {
            if (CmdLine.fname == _T(""))
                CmdLine.fname = pszParam;
        }
    }

    if (GetLastError() == ERROR_ALREADY_EXISTS)
    {
        HWND hPrevWnd;
        EnumWindows(searcher, (LPARAM)&hPrevWnd);

        if ( hPrevWnd != NULL )
        {
            if (CmdLine.fname != "" || CmdLine.exitFlag)
            {
                if (CmdLine.exitFlag)
                {
                    ::SendMessageTimeout(hPrevWnd, WM_COMMAND, ID_POPUPMENU_EXIT, 0, SMTO_BLOCK, 500, &result);
                    return FALSE;
                }
                else
                {
                    MsgCmd cmd;
                    COPYDATASTRUCT cds;
                    cds.dwData = 0;
                    cds.cbData = sizeof(cmd);
                    cds.lpData = &cmd;
                    if (CmdLine.fname != "")
                    {
                        if (CmdLine.addFlag)
                            cmd.command = 1;
                        else
                            cmd.command = 0;
                        _tcscpy(cmd.fname, (TCHAR *)CmdLine.fname.GetString());
                    }
                    else
                        cmd.command = 2;
                    ::SendMessageTimeout(hPrevWnd, WM_COPYDATA, (WPARAM)NULL, (LPARAM)&cds, SMTO_BLOCK, 500, &result);
                    return FALSE;
                }
            }
            if (CmdLine.restartFlag)
            {
                while (IsWindow(hPrevWnd))
                    Sleep(100);
            }
        }
    }
    if (CmdLine.exitFlag) 
        return FALSE;

    WNDCLASS wndcls;
    memset(&wndcls, 0, sizeof(WNDCLASS));

    wndcls.lpfnWndProc = ::DefWindowProc;
    wndcls.hInstance = AfxGetInstanceHandle();
    wndcls.hIcon = LoadIcon(IDI_STP);
    wndcls.hCursor = LoadCursor(IDC_ARROW);
    wndcls.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
    wndcls.lpszMenuName = NULL;

    wndcls.lpszClassName = _T("STPAppClass");

    if (!AfxRegisterClass(&wndcls))
        return FALSE;
    bClassRegistered = TRUE;

    s_mainWnd.Create(_T("LISTBOX"), _T("STP"));

    m_pMainWnd = &s_mainWnd;

    memset(&wndcls, 0, sizeof(WNDCLASS));

    wndcls.lpfnWndProc = ::DefWindowProc;
    wndcls.hInstance = AfxGetInstanceHandle();
    wndcls.lpszMenuName = NULL;


    // Specify our own class name for using FindWindow later
    wndcls.lpszClassName = _T("Winamp v1.x");
    if (!AfxRegisterClass(&wndcls))
        return FALSE;

    bClassRegisteredWinamp = TRUE;

    WAEmuWnd.Create(wndcls.lpszClassName, _T(" - Winamp"));
    s_seekDialog.Create(CSeekDialog::IDD);
    s_playlistDlg.Create(CPlaylistDlg::IDD);

    s_mainWnd.Start2();
    if (CmdLine.fname != "")
    {
        MsgCmd cmd;
        COPYDATASTRUCT cds;
        cds.dwData = 0;
        cds.cbData = sizeof(cmd);
        cds.lpData = &cmd;
        if (CmdLine.fname != "")
        {
            if (CmdLine.addFlag)
                cmd.command = 1;
            else
                cmd.command = 0;
            _tcscpy(cmd.fname, (TCHAR *)CmdLine.fname.GetString());
        }
        else
            cmd.command = 2;
        ::SendMessageTimeout(s_mainWnd.m_hWnd, WM_COPYDATA, (WPARAM)NULL, (LPARAM)&cds, SMTO_BLOCK, 500, &result);
    }
    return TRUE;
}

BOOL CALLBACK searcher(HWND hWnd, LPARAM lParam)
{
    DWORD result;
    LRESULT res = ::SendMessageTimeout(hWnd, STP_WND, 0, 0, SMTO_BLOCK, 5, &result);
    if(!res)
       return TRUE;

    if(result == STP_WND)
    {
        HWND * target = (HWND *)lParam;
        *target = hWnd;
        return FALSE;
    }
    return TRUE;
}