#include "stdafx.h"
#include "Volume.h"
#include "MainWnd.h"
#include <Mmsystem.h>

#pragma comment(lib,"Winmm.lib")

#define BAD_DWORD     (DWORD)0

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CVolume::CVolume() : m_bAvailable(false), m_uMixerID(0L), m_dwMixerHandle(0L), m_dwMinimalVolume(BAD_DWORD), m_dwMaximalVolume(BAD_DWORD)
{
}

void CVolume::Init()
{
    if (mixerGetNumDevs() < 1)
        return;
    MMRESULT mmResult;
    mmResult = mixerOpen((LPHMIXER)&m_dwMixerHandle, 0, (DWORD)s_mainWnd.m_hWnd, 0L, CALLBACK_WINDOW);
    if (mmResult != MMSYSERR_NOERROR)
        return;
    MIXERLINE mixerLine;
    mixerLine.cbStruct = sizeof(MIXERLINE);
    mixerLine.dwComponentType = MIXERLINE_COMPONENTTYPE_SRC_WAVEOUT;
    mmResult = mixerGetLineInfo((HMIXEROBJ)m_dwMixerHandle, &mixerLine, MIXER_OBJECTF_HMIXER | MIXER_GETLINEINFOF_COMPONENTTYPE);
    if (mmResult != MMSYSERR_NOERROR)
        return;

    MIXERCONTROL control;
    control.cbStruct = sizeof(MIXERCONTROL);

    MIXERLINECONTROLS lineControls;
    lineControls.cbStruct = sizeof(MIXERLINECONTROLS);

    lineControls.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;
    lineControls.dwLineID = mixerLine.dwLineID;
    lineControls.cControls = 1;
    lineControls.cbmxctrl = sizeof(MIXERCONTROL);
    lineControls.pamxctrl = &control;
    mmResult = mixerGetLineControls((HMIXEROBJ)m_dwMixerHandle, &lineControls, MIXER_OBJECTF_HMIXER | MIXER_GETLINECONTROLSF_ONEBYTYPE);
    if (mmResult == MMSYSERR_NOERROR)
    {
        if (!(control.fdwControl & MIXERCONTROL_CONTROLF_DISABLED))
            m_bAvailable = true;
    }

    m_nChannelCount = mixerLine.cChannels;
    m_dwVolumeControlID = control.dwControlID;
    m_dwMinimalVolume = control.Bounds.dwMinimum;
    m_dwMaximalVolume = control.Bounds.dwMaximum;
    m_dwVolumeStep = control.Metrics.cSteps;
}

CVolume::~CVolume()
{
    if (m_dwMixerHandle)
        mixerClose((HMIXER)m_dwMixerHandle);
}

DWORD CVolume::GetCurrentVolume()
{
    if (!m_bAvailable)
        Init();
    if (!m_bAvailable)
        return BAD_DWORD;

    MIXERCONTROLDETAILS_UNSIGNED* aDetails = (MIXERCONTROLDETAILS_UNSIGNED*)malloc(m_nChannelCount*sizeof(MIXERCONTROLDETAILS_UNSIGNED));
    if (!aDetails)
        return BAD_DWORD;
    MIXERCONTROLDETAILS controlDetails;
    memset(&controlDetails, 0, sizeof(MIXERCONTROLDETAILS));
    controlDetails.cbStruct = sizeof(MIXERCONTROLDETAILS);
    controlDetails.dwControlID = m_dwVolumeControlID;
    controlDetails.cChannels = m_nChannelCount;
    controlDetails.cMultipleItems = 0;
    controlDetails.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
    controlDetails.paDetails = &aDetails[0];
    mixerGetControlDetails((HMIXEROBJ)m_dwMixerHandle, &controlDetails, MIXER_GETCONTROLDETAILSF_VALUE);
    DWORD max = m_dwMinimalVolume;
    for (int i = 0; i < m_nChannelCount; i++)
    {
        if (aDetails[i].dwValue > max)
            max = aDetails[i].dwValue;
    }
    delete aDetails;
    max = 255 * (max - m_dwMinimalVolume) / (m_dwMaximalVolume - m_dwMinimalVolume);
    return max;
}
void CVolume::SetCurrentVolume(DWORD dwValue)
{
    if (!m_bAvailable)
        return;
    dwValue = (m_dwMaximalVolume - m_dwMinimalVolume) * dwValue / 255 + m_dwMinimalVolume;
    if (dwValue == 0)
        dwValue = (m_dwMaximalVolume - m_dwMinimalVolume) / 1000;
    MIXERCONTROLDETAILS_UNSIGNED* aDetails = (MIXERCONTROLDETAILS_UNSIGNED*)malloc(m_nChannelCount*sizeof(MIXERCONTROLDETAILS_UNSIGNED));
    if (!aDetails)
        return;
    for (int i = 0; i < m_nChannelCount; i++)
    {
        aDetails[i].dwValue = dwValue;
    }
    MIXERCONTROLDETAILS controlDetails;

    memset(&controlDetails, 0, sizeof(MIXERCONTROLDETAILS));
    controlDetails.cbStruct = sizeof(MIXERCONTROLDETAILS);
    controlDetails.dwControlID = m_dwVolumeControlID;
    controlDetails.cChannels = m_nChannelCount;
    controlDetails.cMultipleItems = 0;
    controlDetails.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
    controlDetails.paDetails = &aDetails[0];
    mixerSetControlDetails((HMIXEROBJ)m_dwMixerHandle, &controlDetails, MIXER_GETCONTROLDETAILSF_VALUE);

    delete aDetails;
}

