/*****************************************************
** Module : CWinampWnd                              **
** Purpose : Winamp-style window                    **
**                                                  **
** Copyright James Spibey, 2000 (spib@bigfoot.com)  **
** Feel free to use and distribute. May not be sold ** 
** for profit.                                      **
*****************************************************/

#include "stdafx.h"
#include "WinampWnd.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CWinampWnd

CWinampWnd::CWinampWnd(CWnd* pParent /*=NULL*/)
{
    m_bFocus = FALSE;
    m_bDragging = FALSE;
    m_bResizing = FALSE;
    m_bDocked = TRUE;
    m_bAllowDock = FALSE;
    m_bAllowResize = FALSE;
    m_nHeightBeforeShadeMode = -1;
    m_nSnapOffset = 10;
    m_nDockedSide = DOCKED_BOTTOM_LL;
}

CWinampWnd::~CWinampWnd()
{
}

BEGIN_MESSAGE_MAP(CWinampWnd, CDialog)
    ON_WM_CREATE()
    ON_WM_SHOWWINDOW()
    ON_WM_PAINT()
    ON_WM_KILLFOCUS()
    ON_WM_SETFOCUS()
    ON_WM_LBUTTONDOWN()
    ON_WM_LBUTTONUP()
    ON_WM_MOUSEMOVE()
    ON_WM_SIZE()
    ON_WM_ERASEBKGND()
    ON_WM_TIMER()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWinampWnd message handlers

BOOL CWinampWnd::Create(CWnd* pParent)
{
    CMenu menu;
    EnableDocking();
    EnableResize();

    CString classname = AfxRegisterWndClass(CS_DBLCLKS);
    return CreateEx(WS_EX_LEFT | WS_EX_LTRREADING | WS_EX_RIGHTSCROLLBAR | WS_EX_ACCEPTFILES | WS_EX_CONTROLPARENT, classname, _T("Winamp Window"), WS_CLIPSIBLINGS | WS_POPUP, 0, 0, 275, 116, pParent->GetSafeHwnd(), menu);
}

int CWinampWnd::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (CDialog::OnCreate(lpCreateStruct) == -1)
        return -1;

    LoadSkin();

    CRect rect;
    CWnd* child = GetWindow(GW_CHILD);

    if (child != NULL)
    {
        child->SetWindowPos(NULL, rect.left, rect.bottom, rect.Width(), rect.Height(), SWP_SHOWWINDOW);
    }

    return TRUE;
}
void CWinampWnd::OnShowWindow(BOOL bShow, UINT nStatus)
{
    CRect rect;
    if (bShow)
    {
        CWnd* child = GetWindow(GW_CHILD);

        if (child != NULL)
        {
            child->GetWindowRect(&rect);
            child->SetWindowPos(NULL, 10, 20, rect.Width(), rect.Height(), SWP_NOACTIVATE);
            rect.InflateRect(0, 0, 18, 34);
            SetWindowPos(NULL, rect.left, rect.top, rect.Width(), rect.Height(), NULL);
        }
    }
}

void CWinampWnd::OnPaint()
{
    DrawInterface();
}

void CWinampWnd::DrawInterface(BOOL bPaint)
{
    CRect rc, rcClient, rcWnd;
    GetClientRect(&rcClient);
    GetWindowRect(&rcWnd);
    rc = rcClient;

    CDC* pDC = GetDC();
    CBrush pBrush(RGB(0, 0, 0));
    int bottom = rc.bottom;
    int top = rc.top;
    int left = rc.left;
    int right = rc.right;
    int height = bottom - top;
    int width = right - left;

    int FillsNeeded = rcWnd.Width() - (m_map.OffSet(TITLEBAR_LEFT_CORNER).Width() + m_map.OffSet(TITLEBAR_RIGHT_CORNER).Width());
    FillsNeeded = FillsNeeded / m_map.OffSet(TITLEBAR_FILL).Width();

    if (m_bFocus)
    {
        m_Interface.Draw(FALSE, this, 0, 0, m_map.OffSet(TITLEBAR_LEFT_CORNER));
        for (int i = 0; i <= FillsNeeded; i++)
            m_Interface.Draw(FALSE, this, m_map.OffSet(TITLEBAR_LEFT_CORNER).Width() + (m_map.OffSet(TITLEBAR_FILL).Width() * i), 0, m_map.OffSet(TITLEBAR_FILL));

        CString title;
        GetWindowText(title);

        title.MakeUpper();

        int nbLetter = title.GetLength();

        int sizeLetter = 0;

        for (int i = 0; i < nbLetter; i++)
        {
            int pos = title.GetAt(i) - 'A';
            if (pos < 0 || pos > 25)
                pos = 0;
            sizeLetter += m_map.OffSet(LETTER_A + pos).Width();
        }

        FillsNeeded = 1 + sizeLetter / m_map.OffSet(TITLEBAR_MIDDLE).Width();

        int startPos = (rcWnd.Width() - FillsNeeded* m_map.OffSet(TITLEBAR_MIDDLE).Width()) / 2;

        m_Interface.Draw(FALSE, this, startPos - m_map.OffSet(TITLEBAR_MIDDLE_LEFT).Width(), 0, m_map.OffSet(TITLEBAR_MIDDLE_LEFT));

        int endPos = 0;
        for (endPos = startPos; endPos < startPos + sizeLetter; endPos = endPos + m_map.OffSet(TITLEBAR_MIDDLE).Width())
        {
            m_Interface.Draw(FALSE, this, endPos, 0, m_map.OffSet(TITLEBAR_MIDDLE));
        }

        m_Interface.Draw(FALSE, this, endPos, 0, m_map.OffSet(TITLEBAR_MIDDLE_RIGHT));

        m_Interface.Draw(FALSE, this, (rcWnd.Width() - m_map.OffSet(TITLEBAR_RIGHT_CORNER).Width()), 0, m_map.OffSet(TITLEBAR_RIGHT_CORNER));

        startPos = (rcWnd.Width() - sizeLetter) / 2;
        for (int i = 0; i < nbLetter; i++)
        {
            int pos = title.GetAt(i) - 'A';
            if (pos < 0 || pos > 25)
                startPos += m_map.OffSet(LETTER_A).Width();
            else
            {
                m_Interface.Draw(FALSE, this, startPos, 4, m_map.OffSet(LETTER_A + pos));
                startPos += m_map.OffSet(LETTER_A + pos).Width();
            }
        }
    }
    else
    {
        m_Interface.Draw(FALSE, this, 0, 0, m_map.OffSet(INACTIVE_TITLEBAR_LEFT_CORNER));
        for (int i = 0; i <= FillsNeeded; i++)
            m_Interface.Draw(FALSE, this, m_map.OffSet(TITLEBAR_LEFT_CORNER).Width() + (m_map.OffSet(TITLEBAR_FILL).Width() * i), 0, m_map.OffSet(INACTIVE_TITLEBAR_FILL));

        CString title;
        GetWindowText(title);

        title.MakeUpper();

        int nbLetter = title.GetLength();

        int sizeLetter = 0;

        for (int i = 0; i < nbLetter; i++)
        {
            int pos = title.GetAt(i) - 'A';
            if (pos < 0 || pos > 25)
                pos = 0;
            sizeLetter += m_map.OffSet(LETTER_A + pos).Width();
        }

        FillsNeeded = 1 + sizeLetter / m_map.OffSet(TITLEBAR_MIDDLE).Width();

        int startPos = (rcWnd.Width() - FillsNeeded* m_map.OffSet(TITLEBAR_MIDDLE).Width()) / 2;

        m_Interface.Draw(FALSE, this, startPos - m_map.OffSet(TITLEBAR_MIDDLE_LEFT).Width(), 0, m_map.OffSet(INACTIVE_TITLEBAR_MIDDLE_LEFT));

        int endPos = 0;
        for (endPos = startPos; endPos < startPos + sizeLetter; endPos = endPos + m_map.OffSet(TITLEBAR_MIDDLE).Width())
        {
            m_Interface.Draw(FALSE, this, endPos, 0, m_map.OffSet(INACTIVE_TITLEBAR_MIDDLE));
        }

        m_Interface.Draw(FALSE, this, endPos, 0, m_map.OffSet(INACTIVE_TITLEBAR_MIDDLE_RIGHT));

        m_Interface.Draw(FALSE, this, (rcWnd.Width() - m_map.OffSet(TITLEBAR_RIGHT_CORNER).Width()), 0, m_map.OffSet(INACTIVE_TITLEBAR_RIGHT_CORNER));

        startPos = (rcWnd.Width() - sizeLetter) / 2;
        for (int i = 0; i < nbLetter; i++)
        {
            int pos = title.GetAt(i) - 'A';
            if (pos < 0 || pos > 25)
                startPos += m_map.OffSet(INACTIVE_LETTER_A).Width();
            else
            {
                m_Interface.Draw(FALSE, this, startPos, 4, m_map.OffSet(INACTIVE_LETTER_A + pos));
                startPos += m_map.OffSet(INACTIVE_LETTER_A + pos).Width();
            }
        }
    }

    FillsNeeded = (height - (m_map.OffSet(TITLEBAR_LEFT_CORNER).Height() + m_map.OffSet(BOTTOM_LEFT).Height())) / m_map.OffSet(LEFT_EDGE).Height();
    int y = 0;
    for (int i = 0; i <= FillsNeeded; i++)
    {
        y = m_map.OffSet(TITLEBAR_LEFT_CORNER).Height() + (m_map.OffSet(LEFT_EDGE).Height() * i);
        m_Interface.Draw(bPaint, this, 0, y, m_map.OffSet(LEFT_EDGE));
        m_Interface.Draw(bPaint, this, width - m_map.OffSet(RIGHT_EDGE).Width(), y, m_map.OffSet(RIGHT_EDGE));
    }

    m_Interface.Draw(bPaint, this, 0, height - m_map.OffSet(BOTTOM_LEFT).Height(), m_map.OffSet(BOTTOM_LEFT));

    FillsNeeded = (rcWnd.Width() - (m_map.OffSet(BOTTOM_LEFT).Width() + m_map.OffSet(BOTTOM_RIGHT).Width())) / m_map.OffSet(BOTTOM_FILL).Width();
    for (int i = 0; i <= FillsNeeded; i++)
        m_Interface.Draw(FALSE, this, m_map.OffSet(BOTTOM_LEFT).Width() + (m_map.OffSet(BOTTOM_FILL).Width() * i), height - m_map.OffSet(BOTTOM_RIGHT).Height(), m_map.OffSet(BOTTOM_FILL));

    m_Interface.Draw(bPaint, this, width - m_map.OffSet(BOTTOM_RIGHT).Width(), height - m_map.OffSet(BOTTOM_RIGHT).Height(), m_map.OffSet(BOTTOM_RIGHT));

    ReleaseDC(pDC);
    /*  CWnd *child = GetWindow(GW_CHILD);
        if(child!=NULL)
        {
            child->Invalidate();
        }
        */
}

void CWinampWnd::OnKillFocus(CWnd* pNewWnd)
{
    CDialog::OnKillFocus(pNewWnd);

    m_bFocus = FALSE;
    //  DrawInterface();    
}

void CWinampWnd::OnSetFocus(CWnd* pOldWnd)
{
    CDialog::OnSetFocus(pOldWnd);
    m_bFocus = TRUE;
    DrawInterface();    

    CWnd* child = GetWindow(GW_CHILD);

    if (child != NULL)
    {
        CRect rect;
        ::GetWindowRect(m_hWnd, &rect);
        //      child->SetWindowPos(NULL, rect.left + 9, rect.top + 17, rect.Width() - 18, rect.Height() - 34, SWP_NOZORDER);
        child->SetFocus();
        child->Invalidate();
    }
}

void CWinampWnd::OnLButtonDown(UINT nFlags, CPoint point)
{
    CRect rcClient;
    GetClientRect(&rcClient);

    CRect rcResize;
    CRect rcDrag;
    CRect rcClose;

    rcResize = rcClient;

    rcResize.left = rcResize.right - 20;
    rcResize.top = rcResize.bottom - 20;

    rcDrag = rcClient;
    rcDrag.bottom = rcDrag.top + 20;

    rcClose = rcClient;
    rcClose.right = rcClose.right - 3;
    rcClose.left = rcClose.right - 7;
    rcClose.top = rcClose.top + 3;
    rcClose.bottom = rcClose.top + 7;

    if (rcClose.PtInRect(point))
        m_Interface.Draw(FALSE, this, rcClose.left, rcClose.top, m_map.OffSet(CLOSE_BUTTON));
    else if (rcResize.PtInRect(point))
    {
        SetCapture();
        m_bResizing = TRUE;
        m_ptResizeFrom = point;
    }
    else if (rcDrag.PtInRect(point))
    {
        SetCapture();
        m_bDragging = TRUE;
        m_ptDragFrom = point;
    }

    CDialog::OnLButtonDown(nFlags, point);
}

void CWinampWnd::OnLButtonUp(UINT nFlags, CPoint point)
{
    CRect rcClient;
    GetClientRect(&rcClient);
    CRect rcWnd;
    GetWindowRect(&rcWnd);

    CRect rcClose;

    rcClose = rcClient;
    rcClose.right = rcClose.right - 3;
    rcClose.left = rcClose.right - 7;
    rcClose.top = rcClose.top + 3;
    rcClose.bottom = rcClose.top + 7;

    if (rcClose.PtInRect(point))
    {
        //Prefs.m_bShowPEWnd = FALSE;
        ShowWindow(SW_HIDE);
    }
    else if (m_bResizing)
    {
        ReleaseCapture();
        m_bResizing = FALSE;
    }
    else if (m_bDragging)
    {
        ReleaseCapture();
        m_bDragging = FALSE;
    }

    CDialog::OnLButtonUp(nFlags, point);
}

void CWinampWnd::OnMouseMove(UINT nFlags, CPoint point)
{
    CRect rcWnd;
    GetWindowRect(&rcWnd);

    ChooseCursor(point);

    //Fixed Resizing Code
    if (m_bResizing && (nFlags & MK_LBUTTON) && m_bAllowResize)
        Resize(point);

    //Docking
    if (m_bDragging && (nFlags & MK_LBUTTON))
        Dock(point);

    //Invalidate();

    CDialog::OnMouseMove(nFlags, point);
}

void CWinampWnd::Resize(CPoint point)
{
    CRect rcWnd, rcNew;
    GetWindowRect(&rcWnd);
    rcNew = rcWnd;

    BOOL bResize = FALSE;

    //Resize in X direction outwards
    if (point.x >= (m_ptResizeFrom.x + RESIZE_WIDTH))
    {
        rcNew.right += RESIZE_WIDTH;
        if (rcNew.Width() >= MIN_WIDTH)
        {
            bResize = TRUE;
            m_ptResizeFrom.x += RESIZE_WIDTH;
        }
        else
            rcNew.right = rcWnd.right;
    }
    //Resize in X direction inwards
    else if (point.x <= (m_ptResizeFrom.x - RESIZE_WIDTH))
    {
        rcNew.right -= RESIZE_WIDTH;
        if (rcNew.Width() >= MIN_WIDTH)
        {
            bResize = TRUE;
            m_ptResizeFrom.x -= RESIZE_WIDTH;
        }
        else
            rcNew.right = rcWnd.right;
    }
    //Resize in Y direction downwards
    if (point.y >= (m_ptResizeFrom.y + RESIZE_HEIGHT))
    {
        rcNew.bottom += RESIZE_HEIGHT;
        if (rcNew.Height() >= MIN_HEIGHT)
        {
            bResize = TRUE;
            m_ptResizeFrom.y += RESIZE_HEIGHT;
        }
        else
            rcNew.bottom = rcWnd.bottom;
    }
    //Resize in Y direction upwards
    else if (point.y <= (m_ptResizeFrom.y - RESIZE_HEIGHT))
    {
        rcNew.bottom -= RESIZE_HEIGHT;
        if (rcNew.Height() >= MIN_HEIGHT)
        {
            bResize = TRUE;
            m_ptResizeFrom.y -= RESIZE_HEIGHT;
        }
        else
            rcNew.bottom = rcWnd.bottom;
    }

    if (bResize)
    {
        SetWindowPos(NULL, rcNew.left, rcNew.top, rcNew.Width(), rcNew.Height(), SWP_SHOWWINDOW);
        CWnd* child = GetWindow(GW_CHILD);

        if (child != NULL)
        {
            child->SetWindowPos(&wndTop, 10, 20, rcNew.Width() - 18, rcNew.Height() - 34, SWP_SHOWWINDOW | SWP_NOZORDER);
        }
    }
}

void CWinampWnd::Dock(CPoint point)
{
    int xdiff = 0;
    int ydiff = 0;
    CRect rcWnd, rcParent;
    CWnd* pParent = GetParent();
    GetWindowRect(&rcWnd);
    pParent->GetWindowRect(&rcParent);

    if (point.x != 0 && point.y != 0)
    {
        xdiff = point.x - m_ptDragFrom.x;
        ydiff = point.y - m_ptDragFrom.y;
    }

    CRect rcNew(rcWnd.left + xdiff, rcWnd.top + ydiff, rcWnd.right + xdiff, rcWnd.bottom + ydiff);

    // Just Dragging
    if (!m_bAllowDock)
    {
        SetWindowPos(NULL, rcNew.left, rcNew.top, rcWnd.Width(), rcWnd.Height(), SWP_SHOWWINDOW);
        m_bDocked = FALSE;
        return;
    }

    /*********************************
    ** Dock to Right Side of Winamp **
    *********************************/
    if ((rcNew.left <= rcParent.right + m_nSnapOffset) && (rcNew.left >= rcParent.right - m_nSnapOffset))
    {
        if (!m_bDocked)
            SetWindowPos(NULL, rcParent.right, rcWnd.top + ydiff, rcWnd.Width(), rcWnd.Height(), SWP_SHOWWINDOW);
        else
        {
            // Dock top to top
            if ((rcNew.top <= rcParent.top + m_nSnapOffset) && (rcNew.top >= rcParent.top - m_nSnapOffset))
            {
                m_nDockedSide = DOCKED_RIGHT_TT;
                SetWindowPos(NULL, rcWnd.left, rcParent.top, rcWnd.Width(), rcWnd.Height(), SWP_SHOWWINDOW);
            }
            // Dock top to bottom
            else if ((rcNew.top <= rcParent.bottom + m_nSnapOffset) && (rcNew.top >= rcParent.bottom - m_nSnapOffset))
            {
                m_nDockedSide = DOCKED_RIGHT_TB;
                SetWindowPos(NULL, rcWnd.left, rcParent.bottom, rcWnd.Width(), rcWnd.Height(), SWP_SHOWWINDOW);
            }
                // Dock bottom to top
            else if ((rcNew.bottom <= rcParent.top + m_nSnapOffset) && (rcNew.bottom >= rcParent.top - m_nSnapOffset))
            {
                m_nDockedSide = DOCKED_RIGHT_BT;
                SetWindowPos(NULL, rcWnd.left, rcParent.top - rcWnd.Height(), rcWnd.Width(), rcWnd.Height(), SWP_SHOWWINDOW);
            }
            else
            {
                m_nDockedSide = 0;
                SetWindowPos(NULL, rcWnd.left, rcNew.top, rcWnd.Width(), rcWnd.Height(), SWP_SHOWWINDOW);
            }
        }
        m_bDocked = TRUE;
    }
    /********************************
    ** Dock to Left Side of Winamp **
    ********************************/
    else if ((rcNew.right <= rcParent.left + m_nSnapOffset) && (rcNew.right >= rcParent.left - m_nSnapOffset))
    {
        if (!m_bDocked)
            SetWindowPos(NULL, rcParent.left - rcWnd.Width(), rcNew.top, rcWnd.Width(), rcWnd.Height(), SWP_SHOWWINDOW);
        else
        {
            // Dock top to top
            if ((rcNew.top <= rcParent.top + m_nSnapOffset) && (rcNew.top >= rcParent.top - m_nSnapOffset))
            {
                m_nDockedSide = DOCKED_LEFT_TT;
                SetWindowPos(NULL, rcParent.left - rcWnd.Width(), rcParent.top, rcWnd.Width(), rcWnd.Height(), SWP_SHOWWINDOW);
            }
            // Dock top to bottom
            else if ((rcNew.top <= rcParent.bottom + m_nSnapOffset) && (rcNew.top >= rcParent.bottom - m_nSnapOffset))
            {
                m_nDockedSide = DOCKED_LEFT_TB; 
                SetWindowPos(NULL, rcParent.left - rcWnd.Width(), rcParent.bottom, rcWnd.Width(), rcWnd.Height(), SWP_SHOWWINDOW);
            }
                // Dock bottom to top
            else if ((rcNew.bottom <= rcParent.top + m_nSnapOffset) && (rcNew.bottom >= rcParent.top - m_nSnapOffset))
            {
                m_nDockedSide = DOCKED_LEFT_BT;
                SetWindowPos(NULL, rcParent.left - rcWnd.Width(), rcParent.top - rcWnd.Height(), rcWnd.Width(), rcWnd.Height(), SWP_SHOWWINDOW);
            }
            else
            {
                m_nDockedSide = 0;
                SetWindowPos(NULL, rcParent.left - rcWnd.Width(), rcNew.top, rcWnd.Width(), rcWnd.Height(), SWP_SHOWWINDOW);
            }
        }
        m_bDocked = TRUE;
    }
        /*******************************
        ** Dock to Top Side of Winamp **
        *******************************/
    else if ((rcNew.bottom <= rcParent.top + m_nSnapOffset) && (rcNew.bottom >= rcParent.top - m_nSnapOffset))
    {
        if (!m_bDocked)
            SetWindowPos(NULL, rcNew.left, rcParent.top - rcNew.Height(), rcWnd.Width(), rcWnd.Height(), SWP_SHOWWINDOW);
        else
        {
            // Dock left to left
            if ((rcNew.left <= rcParent.left + m_nSnapOffset) && (rcNew.left >= rcParent.left - m_nSnapOffset))
            {
                m_nDockedSide = DOCKED_TOP_LL;
                SetWindowPos(NULL, rcParent.left, rcParent.top - rcNew.Height(), rcWnd.Width(), rcWnd.Height(), SWP_SHOWWINDOW);
            }
            // Dock left to right
            else if ((rcNew.left <= rcParent.right + m_nSnapOffset) && (rcNew.left >= rcParent.right - m_nSnapOffset))
            {
                m_nDockedSide = DOCKED_TOP_LR;
                SetWindowPos(NULL, rcParent.right, rcParent.top - rcNew.Height(), rcWnd.Width(), rcWnd.Height(), SWP_SHOWWINDOW);
            }
                // Dock right to left
            else if ((rcNew.right <= rcParent.left + m_nSnapOffset) && (rcNew.right >= rcParent.left - m_nSnapOffset))
            {
                m_nDockedSide = DOCKED_TOP_RL;
                SetWindowPos(NULL, rcParent.left - rcNew.Width(), rcParent.top - rcNew.Height(), rcWnd.Width(), rcWnd.Height(), SWP_SHOWWINDOW);
            }
            else
            {
                m_nDockedSide = 0;
                SetWindowPos(NULL, rcNew.left, rcParent.top - rcNew.Height(), rcWnd.Width(), rcWnd.Height(), SWP_SHOWWINDOW);
            }
        }
        m_bDocked = TRUE;
    }
        /**********************************
        ** Dock to Bottom Side of Winamp **
        **********************************/
    else if ((rcNew.top <= rcParent.bottom + m_nSnapOffset) && (rcNew.top >= rcParent.bottom - m_nSnapOffset))
    {
        if (!m_bDocked)
            SetWindowPos(NULL, rcNew.left, rcParent.bottom, rcWnd.Width(), rcWnd.Height(), SWP_SHOWWINDOW);
        else
        {
            // Dock left to left
            if ((rcNew.left <= rcParent.left + m_nSnapOffset) && (rcNew.left >= rcParent.left - m_nSnapOffset))
            {
                m_nDockedSide = DOCKED_BOTTOM_LL;
                SetWindowPos(NULL, rcParent.left, rcParent.bottom, rcWnd.Width(), rcWnd.Height(), SWP_SHOWWINDOW);
            }
            // Dock left to right
            else if ((rcNew.left <= rcParent.right + m_nSnapOffset) && (rcNew.left >= rcParent.right - m_nSnapOffset))
            {
                m_nDockedSide = DOCKED_BOTTOM_LR;
                SetWindowPos(NULL, rcParent.right, rcParent.bottom, rcWnd.Width(), rcWnd.Height(), SWP_SHOWWINDOW);
            }
                // Dock right to left
            else if ((rcNew.right <= rcParent.left + m_nSnapOffset) && (rcNew.right >= rcParent.left - m_nSnapOffset))
            {
                m_nDockedSide = DOCKED_BOTTOM_RL;
                SetWindowPos(NULL, rcParent.left - rcNew.Width(), rcParent.bottom, rcWnd.Width(), rcWnd.Height(), SWP_SHOWWINDOW);
            }
            else
            {
                m_nDockedSide = 0;  
                SetWindowPos(NULL, rcNew.left, rcParent.bottom, rcWnd.Width(), rcWnd.Height(), SWP_SHOWWINDOW);
            }
        }
        m_bDocked = TRUE;
    }
        /**********************
        ** Carry on dragging **
        **********************/
    else
    {
        SetWindowPos(NULL, rcNew.left, rcNew.top, rcWnd.Width(), rcWnd.Height(), SWP_SHOWWINDOW);
        m_bDocked = FALSE;
    }
}

void CWinampWnd::ChooseCursor(CPoint point)
{
    HCURSOR hCursor;
    CRect rcClient;
    GetClientRect(&rcClient);

    CRect rcResize;
    CRect rcDrag;
    CRect rcClose;
    CRect rcMinMax;

    rcResize = rcClient;

    rcResize.left = rcResize.right - 20;
    rcResize.top = rcResize.bottom - 20;

    rcDrag = rcClient;
    rcDrag.bottom = rcDrag.top + 20;

    rcClose = rcClient;
    rcClose.right = rcClose.right - 3;
    rcClose.left = rcClose.right - 7;
    rcClose.top = rcClose.top + 3;
    rcClose.bottom = rcClose.top + 7;

    rcMinMax = rcClient;
    rcMinMax.right = rcMinMax.right - 13;
    rcMinMax.left = rcMinMax.right - 7;
    rcMinMax.top = rcMinMax.top + 3;
    rcMinMax.bottom = rcMinMax.top + 7;

    if (rcResize.PtInRect(point) && m_bAllowResize)
        hCursor = AfxGetApp()->LoadCursor(IDC_RESIZE);
    else if (rcClose.PtInRect(point))
        hCursor = AfxGetApp()->LoadCursor(IDC_CLOSE);
    else if (rcMinMax.PtInRect(point))
        hCursor = AfxGetApp()->LoadCursor(IDC_DEFAULT_ARROW);
    else if (rcDrag.PtInRect(point))
        hCursor = AfxGetApp()->LoadCursor(IDC_DRAG);
    else
        hCursor = AfxGetApp()->LoadCursor(IDC_DEFAULT_ARROW);

    SetCursor(hCursor);
}

void CWinampWnd::OnSize(UINT nType, int cx, int cy)
{
    CDialog::OnSize(nType, cx, cy);

    DrawInterface();

    CWnd* child = GetWindow(GW_CHILD);

    if (child != NULL)
    {
        child->SetWindowPos(NULL, 10, 20, cx - 18, cy - 34, SWP_NOZORDER);
    }
}

BOOL CWinampWnd::LoadSkin()
{
    //To load a bitmap from disk
    if (!m_Interface.LoadBitmap(_T("Skin.bmp")))
        m_Interface.LoadBitmap(IDB_SKIN);

    Invalidate();
    return TRUE;
}

BOOL CWinampWnd::OnEraseBkgnd(CDC* pDC)
{
    return FALSE;   
    //return CDialog::OnEraseBkgnd(pDC);
}


void CWinampWnd::TrackWindow(int x, int y)
{
    if (!m_bDocked)
        return;

    CRect rcWinamp, rcWnd;
    int nLeft, nTop;

    GetWindowRect(&rcWnd);
    GetParent()->GetWindowRect(&rcWinamp);

    switch (m_nDockedSide)
    {
        case DOCKED_TOP_LL:
            nTop = rcWinamp.top - rcWnd.Height();
            nLeft = rcWinamp.left;
            break;
        case DOCKED_TOP_LR:
            nTop = rcWinamp.top - rcWnd.Height();
            nLeft = rcWinamp.right;
            break;
        case DOCKED_TOP_RL:
            nTop = rcWinamp.top - rcWnd.Height();
            nLeft = rcWinamp.left - rcWnd.Width();
            break;
        case DOCKED_LEFT_TT:
            nTop = rcWinamp.top;
            nLeft = rcWinamp.left - rcWnd.Width();
            break;
        case DOCKED_LEFT_TB:
            nTop = rcWinamp.bottom;
            nLeft = rcWinamp.left - rcWnd.Width();
            break;
        case DOCKED_LEFT_BT:
            nTop = rcWinamp.top - rcWnd.Height();
            nLeft = rcWinamp.left - rcWnd.Width();
            break;
        case DOCKED_BOTTOM_LL:
            nTop = rcWinamp.bottom;
            nLeft = rcWinamp.left;
            break;
        case DOCKED_BOTTOM_RL:
            nTop = rcWinamp.bottom;
            nLeft = rcWinamp.left - rcWnd.Width();
            break;
        case DOCKED_BOTTOM_LR:
            nTop = rcWinamp.bottom;
            nLeft = rcWinamp.right;
            break;
        case DOCKED_RIGHT_TT:
            nTop = rcWinamp.top;
            nLeft = rcWinamp.right;
            break;
        case DOCKED_RIGHT_TB:
            nTop = rcWinamp.bottom;
            nLeft = rcWinamp.right;
            break;
        case DOCKED_RIGHT_BT:
            nTop = rcWinamp.top - rcWnd.Height();
            nLeft = rcWinamp.right;
            break;
        default:
            return;
    }

    SetWindowPos(NULL, nLeft, nTop, rcWnd.Width(), rcWnd.Height(), SWP_NOACTIVATE);
}