// ConfigDialog.cpp
//

#include "StdAfx.h"
#include "resource.h"       // main symbols
#include "aerosubc.h"
#include "aeroglss.h"

#include <wa_ipc.h>

#include "TagInfo.h"
#include "in_mpg123.h"

#include "HttpFile.h"
#include "ICYInfo.h"

#include "Param.h"
#include "ConfigDialog.h"

#include "MainHelper.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CIn_mpg123dParam prm;

static const int List_Bps[] =
{
    16, 24, 32, -32, -64
};

void reflect_setting();
void reflect_thread_setting();

CConfigDialog configDialog;

BOOL CALLBACK AfxDlgProc(HWND hWnd, UINT message, WPARAM, LPARAM);

BOOL CALLBACK in_config_proc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    if (!lParam)
        return FALSE;           // not WM_INITDIALOG yet

    configDialog.Attach(hDlg);
    configDialog.PreSubclassWindow();
    WNDPROC* pOldWndProc = configDialog.GetSuperWndProcAddr();
    WNDPROC afxWndProc = AfxGetAfxWndProc();
    *pOldWndProc = (WNDPROC)SetWindowLong(hDlg, GWL_WNDPROC, (DWORD)afxWndProc);

    // Replace the dialog proc pointer with AfxDlgProc

    SetWindowLong(hDlg, DWL_DLGPROC, (DWORD)AfxDlgProc);

    // Now let MFC process this message

    return CallWindowProc(afxWndProc, hDlg, uMsg, wParam, NULL);
}

IMPLEMENT_DYNAMIC(CConfigDialog, CPropertyPage)

CConfigDialog::CConfigDialog() : CPropertyPage()
{
    m_bEnable = false;
    m_bFullBuffering = false;
    m_bSupZero = false;
    m_bReverseLR = false;
    m_bReversePhase = false;
    m_bAltVolume = false;
    m_bDispAvg = false;
}

CConfigDialog::~CConfigDialog()
{
}

void CConfigDialog::DoDataExchange(CDataExchange* pDX)
{
    CPropertyPage::DoDataExchange(pDX);
    CMenuXP::SetXPLookNFeel(this, CMenuXP::GetXPLookNFeel(GetParent()->GetParent()));
    DDX_Control(pDX, IDC_COMBO_DECODER_PRIORITY, m_comboDecodePriority);
    DDX_Control(pDX, IDC_COMBO_OUTPUTBPS, m_comboOutputBps);
    DDX_Check(pDX, IDC_CH_ENABLE, m_bEnable);
    DDX_Check(pDX, IDC_CH_FULLBUFFERING, m_bFullBuffering);
    DDX_Check(pDX, IDC_CH_SUPZERO, m_bSupZero);
    DDX_Check(pDX, IDC_CH_REVERSE_LR, m_bReverseLR);
    DDX_Check(pDX, IDC_CH_REVERSE_PHASE, m_bReversePhase);
    DDX_Check(pDX, IDC_CH_VOLUME, m_bAltVolume);
    DDX_Check(pDX, IDC_CH_DISPAVG, m_bDispAvg);
}

BOOL CConfigDialog::OnInitDialog()
{
    CPropertyPage::OnInitDialog();

    AeroAutoSubclass(m_hWnd, ASC_SUBCLASS_ALL_CONTROLS, 0L);
    SetProp(m_hWnd, _T("vista"), m_hWnd);

    m_HelperIn.translateDlgItemText(this, IDC_CH_ENABLE);
    m_HelperIn.translateDlgItemText(this, IDC_CH_FULLBUFFERING);
    m_HelperIn.translateDlgItemText(this, IDC_CH_SUPZERO);
    m_HelperIn.translateDlgItemText(this, IDC_CH_REVERSE_LR);
    m_HelperIn.translateDlgItemText(this, IDC_CH_REVERSE_PHASE);
    m_HelperIn.translateDlgItemText(this, IDC_CH_VOLUME);
    m_HelperIn.translateDlgItemText(this, IDC_CH_DISPAVG);
    m_HelperIn.translateDlgItemText(this, IDC_THREADPRIORITY);
    m_HelperIn.translateDlgItemText(this, IDC_OUTPUTFORMAT);

    m_bEnable = prm.m_bEnable;
    m_bFullBuffering = prm.m_bFullBuffering;
    m_bSupZero = prm.m_bSupZero;
    m_bReverseLR = prm.m_bReverseLR;
    m_bReversePhase = prm.m_bReversePhase;
    m_bAltVolume = prm.m_bAltVolume;
    m_bDispAvg = prm.m_bDispAvg;

    CString buf = m_HelperIn.MyLoadString(IDS_LNORMAL);
    m_comboDecodePriority.AddString(buf);
    buf = m_HelperIn.MyLoadString(IDS_LABOVENORMAL);
    m_comboDecodePriority.AddString(buf);
    buf = m_HelperIn.MyLoadString(IDS_LHIGHEST);
    m_comboDecodePriority.AddString(buf);

    m_comboDecodePriority.SetCurSel(prm.m_nDecodeThreadPriority);

    buf = m_HelperIn.MyLoadString(IDS_L16BITSINT);
    m_comboOutputBps.AddString(buf);
    buf = m_HelperIn.MyLoadString(IDS_L24BITSINT);
    m_comboOutputBps.AddString(buf);
    buf = m_HelperIn.MyLoadString(IDS_L32BITSINT);
    m_comboOutputBps.AddString(buf);
    buf = m_HelperIn.MyLoadString(IDS_L32BITSFLOAT);
    m_comboOutputBps.AddString(buf);
    buf = m_HelperIn.MyLoadString(IDS_L64BITSFLOAT);
    m_comboOutputBps.AddString(buf);

    m_comboOutputBps.SetCurSel(prm.m_nOutputBps);
    for (int idx = 0; idx < 5; idx++)
    {
        if (List_Bps[idx] == prm.m_nOutputBps)
        {
            m_comboOutputBps.SetCurSel(idx);
            break;
        }
    }

    UpdateData(FALSE);

    OnBnClickedChEnable();

    return TRUE;
}

BOOL CConfigDialog::OnEraseBkgnd(CDC* pDC)
{
    CDwmApiImpl dwmApi;
    if (dwmApi.Initialize() && dwmApi.IsDwmCompositionEnabled())
    {
        CRect rcClient;
        GetClientRect(&rcClient);

        VERIFY(pDC->PatBlt(0, 0, rcClient.Width(), rcClient.Height(), BLACKNESS));
        return TRUE;
    }
    return CPropertyPage::OnEraseBkgnd(pDC);
}

BOOL CConfigDialog::OnKillActive()
{
    UpdateData(TRUE);

    prm.m_bEnable = m_bEnable == TRUE;
    return CPropertyPage::OnKillActive();
}

BEGIN_MESSAGE_MAP(CConfigDialog, CPropertyPage)
  ON_WM_ERASEBKGND()
  ON_CBN_SELCHANGE(IDC_COMBO_DECODER_PRIORITY, OnBnClickedChEnable)
  ON_CBN_SELCHANGE(IDC_COMBO_OUTPUTBPS, OnBnClickedChEnable)
  ON_BN_CLICKED(IDC_CH_ENABLE, OnBnClickedChEnable)
  ON_BN_CLICKED(IDC_CH_FULLBUFFERING, OnBnClickedChEnable)
  ON_BN_CLICKED(IDC_CH_SUPZERO, OnBnClickedChEnable)
  ON_BN_CLICKED(IDC_CH_REVERSE_LR, OnBnClickedChEnable)
  ON_BN_CLICKED(IDC_CH_REVERSE_PHASE, OnBnClickedChEnable)
  ON_BN_CLICKED(IDC_CH_VOLUME, OnBnClickedChEnable)
  ON_BN_CLICKED(IDC_CH_DISPAVG, OnBnClickedChEnable)
END_MESSAGE_MAP()

WNDPROC* CConfigDialog::GetSuperWndProcAddr()
{
    return &m_wndproc;
}

void CConfigDialog::OnBnClickedChEnable()
{
    UpdateData(TRUE);
    SetModified();
    GetDlgItem(IDC_COMBO_DECODER_PRIORITY)->EnableWindow(m_bEnable);
    GetDlgItem(IDC_COMBO_OUTPUTBPS)->EnableWindow(m_bEnable);
    GetDlgItem(IDC_CH_FULLBUFFERING)->EnableWindow(m_bEnable);
    GetDlgItem(IDC_CH_SUPZERO)->EnableWindow(m_bEnable);
    GetDlgItem(IDC_CH_REVERSE_LR)->EnableWindow(m_bEnable);
    GetDlgItem(IDC_CH_REVERSE_PHASE)->EnableWindow(m_bEnable);
    GetDlgItem(IDC_CH_VOLUME)->EnableWindow(m_bEnable);
    GetDlgItem(IDC_CH_DISPAVG)->EnableWindow(m_bEnable);
}

void CConfigDialog::OnOK()
{
    UpdateData(TRUE);

    prm.m_bEnable = m_bEnable == TRUE;
    prm.m_bFullBuffering = m_bFullBuffering == TRUE;
    prm.m_bSupZero = m_bSupZero == TRUE;
    prm.m_bReverseLR = m_bReverseLR == TRUE;
    prm.m_bReversePhase = m_bReversePhase == TRUE;
    prm.m_bAltVolume = m_bAltVolume == TRUE;
    prm.m_bDispAvg = m_bDispAvg == TRUE;
    prm.m_bPostProc = prm.m_bReverseLR || prm.m_bReversePhase || prm.m_bAltVolume;

    bool bFlag = false;

    if (prm.m_nDecodeThreadPriority != m_comboDecodePriority.GetCurSel())
    {
        prm.m_nDecodeThreadPriority = m_comboDecodePriority.GetCurSel();
        bFlag = true;
    }

    if (prm.m_nOutputBps != List_Bps[m_comboOutputBps.GetCurSel()])
    {
        prm.m_nOutputBps = List_Bps[m_comboOutputBps.GetCurSel()];
        bFlag = true;
    }

    if (bFlag)
    {
        ::reflect_thread_setting();
    }
    ::reflect_setting();

    CPropertyPage::OnOK();
}