// FormattingDialog.cpp
//

#include "StdAfx.h"
#include "resource.h"
#include "aerosubc.h"
#include "aeroglss.h"

#include <wa_ipc.h>

#include "TagInfo.h"
#include "in_mpg123.h"

#include "HttpFile.h"
#include "ICYInfo.h"

#include "Param.h"
#include "FormattingDialog.h"

#include "MainHelper.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

void reflect_setting();
void reflect_thread_setting();

CFormattingDialog formattingDialog;

BOOL CALLBACK AfxDlgProc(HWND hWnd, UINT message, WPARAM, LPARAM);

BOOL CALLBACK in_formatting_proc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    if (!lParam)
        return FALSE;           // not WM_INITDIALOG yet

    formattingDialog.Attach(hDlg);
    formattingDialog.PreSubclassWindow();
    WNDPROC* pOldWndProc = formattingDialog.GetSuperWndProcAddr();
    WNDPROC afxWndProc = AfxGetAfxWndProc();
    *pOldWndProc = (WNDPROC)SetWindowLong(hDlg, GWL_WNDPROC, (DWORD)afxWndProc);

    // Replace the dialog proc pointer with AfxDlgProc

    SetWindowLong(hDlg, DWL_DLGPROC, (DWORD)AfxDlgProc);

    // Now let MFC process this message

    return CallWindowProc(afxWndProc, hDlg, uMsg, wParam, NULL);
}

IMPLEMENT_DYNAMIC(CFormattingDialog, CPropertyPage)

CFormattingDialog::CFormattingDialog() : CPropertyPage()
{
    m_strID3Format = _T("");
}

CFormattingDialog::~CFormattingDialog()
{
}

void CFormattingDialog::DoDataExchange(CDataExchange* pDX)
{
    CPropertyPage::DoDataExchange(pDX);
    CMenuXP::SetXPLookNFeel(this, CMenuXP::GetXPLookNFeel(GetParent()->GetParent()));
    DDX_Control(pDX, IDC_COMBO_TAG_PRIORITY, m_comboTagPriority);
    DDX_Text(pDX, IDC_ED_TITLE, m_strID3Format);
}

BOOL CFormattingDialog::OnInitDialog()
{
    CPropertyPage::OnInitDialog();

    AeroAutoSubclass(m_hWnd, ASC_SUBCLASS_ALL_CONTROLS, 0L);
    SetProp(m_hWnd, _T("vista"), m_hWnd);

    m_HelperIn.translateDlgItemText(this, IDC_LTAG_PRIORITY);
    m_HelperIn.translateDlgItemText(this, IDC_LFORMATTING_FORMAT);
    m_HelperIn.translateDlgItemText(this, IDC_LFORMATTING_TITLE);
    m_HelperIn.translateDlgItemText(this, IDC_LFORMATTING_ARTIST);
    m_HelperIn.translateDlgItemText(this, IDC_LFORMATTING_ALBUM);
    m_HelperIn.translateDlgItemText(this, IDC_LFORMATTING_YEAR);
    m_HelperIn.translateDlgItemText(this, IDC_LFORMATTING_COMMENT);
    m_HelperIn.translateDlgItemText(this, IDC_LFORMATTING_GENRE);
    m_HelperIn.translateDlgItemText(this, IDC_LFORMATTING_ID3TRACK);
    m_HelperIn.translateDlgItemText(this, IDC_LFORMATTING_ID3COMPOSER);
    m_HelperIn.translateDlgItemText(this, IDC_LFORMATTING_ID3ARTIST);
    m_HelperIn.translateDlgItemText(this, IDC_LFORMATTING_COPYRIGHT);
    m_HelperIn.translateDlgItemText(this, IDC_LFORMATTING_ID3ENCODER);
    m_HelperIn.translateDlgItemText(this, IDC_LFORMATTING_FILENAME);
    m_HelperIn.translateDlgItemText(this, IDC_LFORMATTING_FILEPATH);
    m_HelperIn.translateDlgItemText(this, IDC_LFORMATTING_FILEEXTENSION);
    m_HelperIn.translateDlgItemText(this, IDC_LFORMATTING_DEFAULT);

    m_strID3Format = prm.m_strID3Format;

    CString buf = m_HelperIn.MyLoadString(IDS_LID3v2APEID3v1);
    m_comboTagPriority.AddString(buf);
    buf = m_HelperIn.MyLoadString(IDS_LID3v2ID3v1APE);
    m_comboTagPriority.AddString(buf);
    buf = m_HelperIn.MyLoadString(IDS_LAPEID3v2ID3v1);
    m_comboTagPriority.AddString(buf);
    buf = m_HelperIn.MyLoadString(IDS_LAPEID3v1ID3v2);
    m_comboTagPriority.AddString(buf);
    buf = m_HelperIn.MyLoadString(IDS_LID3v1ID3v2APE);
    m_comboTagPriority.AddString(buf);
    buf = m_HelperIn.MyLoadString(IDS_LID3v1APEID3v2);
    m_comboTagPriority.AddString(buf);

    m_comboTagPriority.SetCurSel(prm.m_nTagPriority);

    UpdateData(FALSE);

    return TRUE;
}

BOOL CFormattingDialog::OnEraseBkgnd(CDC* pDC)
{
    CDwmApiImpl dwmApi;
    if (dwmApi.Initialize() && dwmApi.IsDwmCompositionEnabled())
    {
        CRect rcClient;
        GetClientRect(&rcClient);

        VERIFY(pDC->PatBlt(0, 0, rcClient.Width(), rcClient.Height(), BLACKNESS));
        return TRUE;
    }
    return CPropertyPage::OnEraseBkgnd(pDC);
}

BOOL CFormattingDialog::OnSetActive()
{
    m_comboTagPriority.EnableWindow(prm.m_bEnable);
    GetDlgItem(IDC_ED_TITLE)->EnableWindow(prm.m_bEnable);
    return CPropertyPage::OnSetActive();
}

BEGIN_MESSAGE_MAP(CFormattingDialog, CPropertyPage)
  ON_WM_ERASEBKGND()
END_MESSAGE_MAP()

WNDPROC* CFormattingDialog::GetSuperWndProcAddr()
{
    return &m_wndproc;
}

void CFormattingDialog::OnOK()
{
    UpdateData(TRUE);

    prm.m_strID3Format = m_strID3Format;

    if (prm.m_nTagPriority != m_comboTagPriority.GetCurSel())
    {
        prm.m_nTagPriority = m_comboTagPriority.GetCurSel();
        ::reflect_thread_setting();
    }
    ::reflect_setting();

    CPropertyPage::OnOK();
}