#include "StdAfx.h"
#include <string.h>
#include "MultiDllInput.h"
#include "CommonFunc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

void CMultiDllInput::UnloadAll()
{
    CString dllInKey = _T("");
    dllInKey.Format(_T("dllInName%d"), listDllInput.GetSize());
    AfxGetApp()->WriteProfileString(_T("Preferences"), dllInKey, _T("NONE"));
    for(int i = 0;listDllInput.GetSize()>0;i++)
    {
        currentDllInput = listDllInput[0];
        dllInKey.Format(_T("dllInName%d"), i);
        AfxGetApp()->WriteProfileString(_T("Preferences"), dllInKey, currentDllInput->GetFilePath());
        currentDllInput->Quit();
        delete currentDllInput;
        currentDllInput = NULL;
        listDllInput.RemoveAt(0);
    }
}

void CMultiDllInput::LoadAll(HWND hWnd)
{
    CString dllInName = _T("");
    CString dllInKey = _T("");
    bool alreadyLoad = false;
    int i = 0;
    do
    {
        dllInKey.Format(_T("dllInName%d"), i);
        dllInName = AfxGetApp()->GetProfileString(_T("Preferences"), dllInKey, _T(""));
        if (dllInName == "NONE")
            dllInName = "";
        if (((dllInName == "" || dllInName == "STANDARD") && !alreadyLoad) || ((dllInName != "" && dllInName != "STANDARD")))
        {
            if (dllInName == "STANDARD" || dllInName == "")
            {
                currentDllInput = new CDllInput(_T(""), hWnd);
                alreadyLoad = true;
            }
            else
                currentDllInput = new CDllInput(dllInName, hWnd);
            if (dllInName != currentDllInput->GetFilePath() && dllInName != "STANDARD" && dllInName != "")
            {
                delete currentDllInput;
                currentDllInput = NULL;
            }
            else
            {
                currentDllInput->Init();
                listDllInput.Add(currentDllInput);
            }
        }
        i++;
    }
    while (dllInName.GetLength() != 0);
    currentDllInput = listDllInput[0];
}

void CMultiDllInput::GetFileInfo(TCHAR* file, TCHAR* title, int* length_in_ms) // if file == NULL, current playing is used
{
    try
    {
        *length_in_ms = 0;
        title[0] = 0;

        if (currentDllInput != NULL && file == NULL)
        {
            currentDllInput->GetFileInfo(file, title, length_in_ms);
            return;
        }
        CDllInput* dllInfo;
        int res = -1;
        TCHAR fn[5000];
        _tcscpy(fn, file);
        TCHAR* cFn = _tcsrev(_tcslwr(_tcsdup(fn)));

        for (int i = 0; i < listDllInput.GetSize(); i++)
        {
            dllInfo = listDllInput[i];
            if (_tcsstr(fn, _T("://")) != NULL)
            {
                if (dllInfo->IsOurFile(fn) != 0)
                {
                    dllInfo->GetFileInfo(fn, title, length_in_ms);
                    res = 0;
                }
                if (res != -1)
                    break;
            }
            else
            {
                WCHAR extension[5000];

                Strcpy(extension, 5000, dllInfo->GetFileExtensions(), 5000);
                WCHAR* pExtToken = _wcsrev(_wcslwr(_wcsdup(extension)));
                WCHAR* extToken = wcstok(pExtToken, _T(";"));
                while (extToken != NULL)
                {
                    if (wcsstr(cFn, extToken) == cFn)
                    {
                        dllInfo->GetFileInfo(fn, title, length_in_ms);
                        res = 0;
                        break;
                    }
                    extToken = wcstok(NULL, _T(";"));
                }
                delete pExtToken;
                if (res != -1)
                    break;
            }
        }
        delete cFn;
    }
    catch (...)
    {
        *length_in_ms = 0;
        title[0] = 0;
    }
}

int CMultiDllInput::InfoBox(TCHAR* file, HWND hwndParent)
{
    int res = -1;
    TCHAR fn[5000];
    _tcscpy(fn, file);
    TCHAR* cFn = _tcsrev(_tcslwr(_tcsdup(fn)));

    for (int i = 0; i < listDllInput.GetSize(); i++)
    {
        CDllInput* curDllInput = listDllInput[i];
        if (wcsstr(fn, _T("://")) != NULL)
        {
            if (curDllInput->IsOurFile(fn) != 0)
            {
                TCHAR descTCHAR[500];
                Strcpy(descTCHAR, 500, curDllInput->GetDescription());
                TRACE(_T("%s"), descTCHAR);
                res = curDllInput->InfoBox(file, hwndParent);
                break;
            }
        }
        else
        {
            WCHAR extension[5000];

            Strcpy(extension, 5000, curDllInput->GetFileExtensions(), 5000);
            WCHAR* pExtToken = _wcsrev(_wcslwr(_wcsdup(extension)));
            WCHAR* extToken = wcstok(pExtToken, _T(";"));
            while (extToken != NULL)
            {
                if (wcsstr(cFn, extToken) == cFn)
                {
                    TCHAR descTCHAR[500];
                    Strcpy(descTCHAR, 500, curDllInput->GetDescription());
                    TRACE(_T("%s"), descTCHAR);
                    res = curDllInput->InfoBox(file, hwndParent);
                    break;
                }
                extToken = wcstok(NULL, _T(";"));
            }
            delete pExtToken;
            if (res != -1)
                break;
        }
    }
    delete cFn;
    return res;
}

int CMultiDllInput::IsOurFile(TCHAR* fn) // called before extension checks, to allow detection of mms://, etc
{
    int res = 0;
    for (int i = 0; i < listDllInput.GetSize(); i++)
    {
        CDllInput* curDllInput = listDllInput[i];
        res = curDllInput->IsOurFile(fn);
        if (res)
            break;
    }
    return res;
}

// playback stuff
int CMultiDllInput::Play(TCHAR* fn1)    // return zero on success, -1 on file-not-found, some other value on other (stopping winamp) error
{
    int res = -1;
    TCHAR fn[5000];
    _tcscpy(fn, fn1);
    TCHAR* cFn = _tcsrev(_tcslwr(_tcsdup(fn)));

    if (currentDllInput->GetPlayerMode() != PLAYER_MODE_CLOSE)
        currentDllInput->Stop();
    for (int i = 0; i < listDllInput.GetSize(); i++)
    {
        currentDllInput = listDllInput[i];
        if (wcsstr(fn, _T("://")) != NULL)
        {
            if (currentDllInput->IsOurFile(fn) != 0)
            {
                TCHAR descTCHAR[500];
                Strcpy(descTCHAR, 500, currentDllInput->GetDescription());
                TRACE(_T("%s"), descTCHAR);
                res = currentDllInput->Play(fn) + 2;
            }
            if (res != -1)
                break;
        }
        else
        {
            WCHAR extension[5000];

            Strcpy(extension, 5000, currentDllInput->GetFileExtensions(), strlen(currentDllInput->GetFileExtensions()));
            WCHAR* pExtToken = _wcsrev(_wcslwr(_wcsdup(extension)));
            WCHAR* extToken = wcstok(pExtToken, _T(";"));
            while (extToken != NULL)
            {
                if (wcsstr(cFn, extToken) == cFn)
                {
                    TCHAR descTCHAR[500];
                    Strcpy(descTCHAR, 500, currentDllInput->GetDescription());
                    TRACE(_T("%s"), descTCHAR);
                    res = currentDllInput->Play(fn);
                    //                TRACE(_T("res %d"), res);
                    break;
                }
                extToken = wcstok(NULL, _T(";"));
            }
            delete pExtToken;
            if (res != -1)
                break;
        }
    }
    if (res > 1)
        res -= 2;
    delete cFn;
    if (res != 0)
    {
        TRACE(_T("Lecture impossible %s"), fn);
    }
    return res;
}

int CMultiDllInput::winampGetExtendedFileInfo(extendedFileInfoStruct ExtendedFileInfo)
{
    int res = -1;
    char fn[5000];
    strcpy(fn, ExtendedFileInfo.filename);
    char* cFn = _strrev(_strlwr(_strdup(fn)));

    for (int i = 0; i < listDllInput.GetSize(); i++)
    {
        CDllInput* curDllInput = listDllInput[i];
        if (strstr(fn, "://") != NULL)
        {
            WCHAR filename[5000];

            Strcpy(filename, 5000, fn, 5000);
            if (curDllInput->IsOurFile(filename) != 0)
            {
                TCHAR descTCHAR[500];
                Strcpy(descTCHAR, 500, curDllInput->GetDescription());
                TRACE(_T("%s"), descTCHAR);
                res = curDllInput->winampGetExtendedFileInfo(ExtendedFileInfo);
                break;
            }
        }
        else
        {
            char* pExtToken = _strrev(_strlwr(_strdup(curDllInput->GetFileExtensions())));
            char* extToken = strtok(pExtToken, ";");
            while (extToken != NULL)
            {
                if (strstr(cFn, extToken) == cFn)
                {
                    TCHAR descTCHAR[500];
                    Strcpy(descTCHAR, 500, curDllInput->GetDescription());
                    TRACE(_T("%s"), descTCHAR);
                    res = curDllInput->winampGetExtendedFileInfo(ExtendedFileInfo);
                    break;
                }
                extToken = strtok(NULL, ";");
            }
            delete pExtToken;
            if (res != -1)
                break;
        }
    }
    delete cFn;
    return res;
}

int CMultiDllInput::winampGetExtendedFileInfoW(extendedFileInfoStructW ExtendedFileInfo)
{
    int res = -1;
    TCHAR fn[5000];
    _tcscpy(fn, ExtendedFileInfo.filename);
    wchar_t* cFn = _wcsrev(_wcslwr(_wcsdup(fn)));

    for (int i = 0; i < listDllInput.GetSize(); i++)
    {
        CDllInput* curDllInput = listDllInput[i];
        if (wcsstr(fn, _T("://")) != NULL)
        {
            if (curDllInput->IsOurFile(fn) != 0)
            {
                TCHAR descTCHAR[500];
                Strcpy(descTCHAR, 500, curDllInput->GetDescription());
                TRACE(_T("%s"), descTCHAR);
                res = curDllInput->winampGetExtendedFileInfoW(ExtendedFileInfo);
                break;
            }
        }
        else
        {
            WCHAR extension[5000];

            Strcpy(extension, 5000, curDllInput->GetFileExtensions(), 5000);
            WCHAR* pExtToken = _wcsrev(_wcslwr(_wcsdup(extension)));
            WCHAR* extToken = wcstok(pExtToken, _T(";"));
            while (extToken != NULL)
            {
                if (wcsstr(cFn, extToken) == cFn)
                {
                    //                  TCHAR descTCHAR[500];
                    //                  Strcpy(descTCHAR, 500, curDllInput->GetDescription());
                    //                  TRACE(_T("%s"), descTCHAR);
                    res = curDllInput->winampGetExtendedFileInfoW(ExtendedFileInfo);
                    break;
                }
                extToken = wcstok(NULL, _T(";"));
            }
            delete pExtToken;
            if (res != -1)
                break;
        }
    }
    delete cFn;
    return res;
}

void CMultiDllInput::Pause()     // pause stream
{
    currentDllInput->Pause();
}

void CMultiDllInput::UnPause()     // unpause stream
{
    currentDllInput->UnPause();
}

int CMultiDllInput::IsPaused()      // ispaused? return 1 if paused, 0 if not
{
    return currentDllInput->IsPaused();
}

void CMultiDllInput::Stop()        // stop (unload) stream
{
    currentDllInput->Stop();
}
// time stuff
int CMultiDllInput::GetLength()     // get length in ms
{
    int res = 0;
    try
    {
        res = currentDllInput->GetLength();
    }
    catch (...)
    {
    }
    return res;
}

int CMultiDllInput::GetOutputTime()   // returns current output time in ms. (usually returns outMod->GetOutputTime()
{
    int res = 0;
    try
    {
        res = currentDllInput->GetOutputTime();
    }
    catch (...)
    {
    }
    return res;
}

void CMultiDllInput::SetOutputTime(int time_in_ms) // seeks to point in stream (in ms). Usually you signal yoru thread to seek, which seeks and calls outMod->Flush()..
{
    try
    {
        currentDllInput->SetOutputTime(time_in_ms);
    }
    catch (...)
    {
    }
}

// volume stuff
void CMultiDllInput::SetVolume(int volume) // from 0 to 255.. usually just call outMod->SetVolume
{
    try
    {
        //      currentDllInput->SetVolume(volume);
        for (int i = 0; i < listDllInput.GetSize(); i++)
        {
            CDllInput* curDllInput = listDllInput[i];
            curDllInput->SetVolume(volume);
        }
    }
    catch (...)
    {
    }
}

void CMultiDllInput::SetPan(int pan) // from -127 to 127.. usually just call outMod->SetPan
{
    try
    {
        currentDllInput->SetPan(pan);
    }
    catch (...)
    {
    }
}

// info setting (filled in by winamp)
void CMultiDllInput::SetInfo(int bitrate, int srate, int stereo, int synched) // if -1, changes ignored? :)
{
    try
    {
        currentDllInput->SetInfo(bitrate, srate, stereo, synched);
    }
    catch (...)
    {
    }
}

int CMultiDllInput::GetVersion()        // module type (IN_VER)
{
    int res = 0;
    try
    {
        res = currentDllInput->GetIn_Module()->version;
    }
    catch (...)
    {
    }
    return res;
}
char* CMultiDllInput::GetDescription()  // description of module, with version string
{
    try
    {
        return currentDllInput->GetDescription();
    }
    catch (...)
    {
        return "";
    }
}

void CMultiDllInput::SethMainWindow(HWND hWnd)   // winamp's main window (filled in by winamp)
{
    for (int i = 0; i < listDllInput.GetSize(); i++)
    {
        CDllInput* curDllInput = listDllInput[i];
        curDllInput->SethMainWindow(hWnd);
    }
}

CString CMultiDllInput::GetFileExtensions()   // "mp3\0Layer 3 MPEG\0mp2\0Layer 2 MPEG\0mpg\0Layer 1 MPEG\0"
{
    CString res = _T(""),ext;
    for (int i = 0; i < listDllInput.GetSize(); i++)
    {
        CDllInput* curDllInput = listDllInput[i];
        ext = curDllInput->GetFileExtensions();
        if (ext != "")
            res = res + ext;
        if (res.Right(1) != _T(";"))
            res += _T(";");
    }
    res.MakeUpper();
    return res;
}

int CMultiDllInput::isSeekable()      // is this stream seekable?
{
    int res = 0;
    try
    {
        res = currentDllInput->GetIn_Module()->is_seekable;
    }
    catch (...)
    {
    }
    return res;
}

int CMultiDllInput::isUsesOutputPlug()      // does this plug-in use the output plug-ins? (musn't ever change, ever :)
{
    int res = 0;
    try
    {
        res = currentDllInput->GetIn_Module()->UsesOutputPlug;
    }
    catch (...)
    {
    }
    return res;
}

void CMultiDllInput::SetOutMod(Out_Module* out) // filled in by winamp, optionally used :)
{
    for (int i = 0; i < listDllInput.GetSize(); i++)
    {
        CDllInput* curDllInput = listDllInput[i];
        curDllInput->SetOutMod(out);
    }
}

In_Module* CMultiDllInput::GetIn_Module()
{
    return currentDllInput->GetIn_Module();
}

int CMultiDllInput::GetPlayerMode()
{
    int ret = PLAYER_MODE_CLOSE;
    if (currentDllInput != NULL)
        ret = currentDllInput->GetPlayerMode();
    return ret;
}

void CMultiDllInput::SetPlayerMode(int statut)
{
    currentDllInput->SetPlayerMode(statut);
}

CString CMultiDllInput::GetFileExtensionsFilter()
{
    char* extensions;
    CString ext = _T("");
    CString extDesc = _T("");
    int pos = 0;
    int startExt = 0;
    int startExtDesc = 0;
    int extSize = 0;
    int extDescSize = 0;
    CString allSupported = _T("All supported files|");
    CString res = _T("");

    for (int i = 0; i < listDllInput.GetSize(); i++)
    {
        CDllInput* curDllInput = listDllInput[i];
        extensions = curDllInput->GetIn_Module()->FileExtensions;
        ext = extensions;
        extSize = ext.GetLength();
        pos = 0;
        startExt = 0;
        startExtDesc = ext.GetLength() + 1;
        extDesc = extensions + startExtDesc;
        extDescSize = extDesc.GetLength();
        do
        {
            for (int j = 0; j < extSize + 1; j++)
            {
                if (*(extensions + pos) == ';' || *(extensions + pos) == '\0')
                {
                    ext = extensions;
                    ext = ext.Mid(startExt, pos - startExt);
                    startExt = pos + 1;
                    if (ext != "")
                    {
                        allSupported += _T("*.") + ext + _T(";");
                        if (extDesc == "")
                        {
                            if (res != "")
                                res += _T("*.") + ext + _T(";");
                        }
                        else
                        {
                            if (res.Right(1) == ";")
                                res = res.Left(res.GetLength() - 1);
                            res += _T("|") + extDesc + _T("|*.") + ext + _T(";");
                            extDesc = _T("");
                        }
                    }
                }
                pos++;
            }
            extensions += extSize + extDescSize + 2;
            ext = extensions;
            pos = 0;
            startExt = 0;
            extSize = ext.GetLength();
            startExtDesc = ext.GetLength() + 1;
            extDesc = extensions + startExtDesc;
            extDescSize = extDesc.GetLength();
            if (res.Right(1) == ";")
                res = res.Left(res.GetLength() - 1);
        }
        while (ext != "");
    }

    res = allSupported + res + _T("|");
    return res;
}

int CMultiDllInput::GetSize() // return the number of loaded input
{
    return listDllInput.GetSize();
}

CDllInput* CMultiDllInput::getDllInput(int pos) //Return the DllInput at pos
{
    return listDllInput[pos];
}

void CMultiDllInput::UnLoad(int pos) //Unload the DllInput at pos
{
    if (pos < listDllInput.GetSize())
    {
        CDllInput* input = listDllInput[pos];
        input->Quit();
        delete input;
        for (int i = pos; i < listDllInput.GetSize() - 1; i++)
            listDllInput[i] = listDllInput[i + 1];
        listDllInput.SetSize(listDllInput.GetSize() - 1);
        currentDllInput = listDllInput[0];
    }
}

void CMultiDllInput::Load(CString file, HWND hWnd) //Load the DllInput
{
    CDllInput* input = new CDllInput(file, hWnd);
    if (input->GetFilePath() != "STANDARD")
    {
        input->Init();
        listDllInput.SetSize(listDllInput.GetSize() + 1);
        listDllInput[listDllInput.GetSize() - 1] = input;
    }
    else
        delete input;
}

void CMultiDllInput::UpDllInput(int pos) //Up the DllInput in the list
{
    if (pos == 0)
        return;
    CDllInput* input = listDllInput[pos];
    listDllInput[pos] = listDllInput[pos - 1];
    listDllInput[pos - 1] = input;
}
