// In_mpg123dParam.cpp
//
//////////////////////////////////////////////////////////////////////

#include "StdAfx.h"

#include "CommonFunc.h"

#include <wa_ipc.h>
#include "TagInfo.h"
#include "in_mpg123.h"

#include "HttpFile.h"
#include "ICYInfo.h"

#include "MainHelper.h"
#include "Param.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif

CIn_mpg123dParam::CIn_mpg123dParam()
{
    // Initialize members

    m_nStartPage = 0;

    m_bEnable = true;
    m_nDecodeThreadPriority = 2 /* THREAD_PRIORITY_HIGHEST */;
    m_nOutputBps = 16;
    m_bSupZero = false;
    m_bDispAvg = false;
    m_bAltVolume = false;
    m_bReverseLR = false;
    m_bReversePhase = false;
    m_bFullBuffering = false;

    m_nVolume = 255;
    m_nPan = 0;

    m_bReplayGainEnable = false;
    m_nReplayGainMode = 0;
    m_nReplayGainTag = REPLAYGAIN_TAG_INIT;
    m_nReplayGainPreAmpWithRG = 0;
    m_nReplayGainPreAmpWithoutRG = 0;
    m_bReplayGainHardLimit = TRUE;

    m_bEnableStreaming = false;
    m_nReceiveThreadPriority = 2;
    m_nBufferLength = 256;
    m_bProxy = false;
    m_strProxy = _T("");
    m_bSaveStream = false;
    m_strSaveStream = _T("");
    m_bTitleStreaming = true;
    m_bTitleStreamingDecode = true;
    m_bTitleStreamingUDP = false;
    m_strTitleStreamingFormat = _T("%1 (%2)");

    m_nTagPriority = 0;
    m_strID3Format = _T("%1 - %2");
    m_strSIFFormat = _T("%IART - %INAM");
    GetProfile();
}

CIn_mpg123dParam::~CIn_mpg123dParam()
{
    WriteProfile();
}

void CIn_mpg123dParam::GetProfile()
{
    TCHAR buffer[MAX_PATH];

    m_HelperIn.config_get_filename(buffer, sizeof(buffer));

    static const WCHAR szAppName[] = _T("Shibatch mpg123");

    m_bEnable = GetPrivateProfileInt(szAppName, _T("Enable"), 1, buffer) != 0;
    m_nDecodeThreadPriority = GetPrivateProfileInt(szAppName, _T("DecodePriority"), 2, buffer);
    m_nOutputBps = GetPrivateProfileInt(szAppName, _T("OutputFormat"), 16, buffer);
    m_bFullBuffering = GetPrivateProfileInt(szAppName, _T("FullBuffering"), 0, buffer) != 0;
    m_bSupZero = GetPrivateProfileInt(szAppName, _T("SupZero"), 0, buffer) != 0;
    m_bReverseLR = GetPrivateProfileInt(szAppName, _T("ReverseLR"), 0, buffer) != 0;
    m_bReversePhase = GetPrivateProfileInt(szAppName, _T("ReversePhase"), 0, buffer) != 0;
    m_bAltVolume = GetPrivateProfileInt(szAppName, _T("Volume"), 0, buffer) != 0;
    m_bDispAvg = GetPrivateProfileInt(szAppName, _T("DispAvg"), 0, buffer) != 0;

    if (m_nOutputBps == 64)
        m_nOutputBps = -64;

    m_bPostProc = m_bReverseLR || m_bReversePhase || m_bAltVolume;

    m_nPan = GetPrivateProfileInt(szAppName, _T("XPan"), 0, buffer);

    m_bReplayGainEnable = GetPrivateProfileInt(szAppName, _T("ReplayGain_Enable"), 0, buffer) != 0;
    m_nReplayGainMode = GetPrivateProfileInt(szAppName, _T("ReplayGain_Mode"), 0, buffer);
    m_nReplayGainTag = GetPrivateProfileInt(szAppName, _T("ReplayGain_Tag"), REPLAYGAIN_TAG_INIT, buffer);
    m_nReplayGainPreAmpWithRG = GetPrivateProfileInt(szAppName, _T("ReplayGain_PreAmp_WithRG"), 0, buffer);
    m_nReplayGainPreAmpWithoutRG = GetPrivateProfileInt(szAppName, _T("ReplayGain_PreAmp_WithoutRG"), 0, buffer);
    m_bReplayGainHardLimit = GetPrivateProfileInt(szAppName, _T("ReplayGain_HardLimit"), 1, buffer);

    m_bEnableStreaming = GetPrivateProfileInt(szAppName, _T("EnableStreaming"), 0, buffer) != 0;
    m_nReceiveThreadPriority = GetPrivateProfileInt(szAppName, _T("ReceivePriority"), 2, buffer);
    m_nBufferLength = Max((int)GetPrivateProfileInt(szAppName, _T("StrBuf"), 256, buffer), MIN_STREAM_SIZE / 1024);
    m_bProxy = GetPrivateProfileInt(szAppName, _T("EnableProxy"), 0, buffer) != 0;
    m_bSaveStream = GetPrivateProfileInt(szAppName, _T("SaveStream"), 0, buffer) != 0;
    m_bTitleStreaming = GetPrivateProfileInt(szAppName, _T("TitleStreaming"), 1, buffer) != 0;
    m_bTitleStreamingUDP = GetPrivateProfileInt(szAppName, _T("UseUDP"), 0, buffer) != 0;

    TCHAR cbuf[256];

    GetPrivateProfileString(szAppName, _T("Proxy"), _T(""), cbuf, MAX_PATH, buffer);
    m_strProxy.Format(_T("%s"), cbuf);
    GetPrivateProfileString(szAppName, _T("SaveStreamFile"), _T(""), cbuf, MAX_PATH, buffer);
    m_strSaveStream.Format(_T("%s"), cbuf);
    GetPrivateProfileString(szAppName, _T("TitleStreamingFormat"), _T("%1 (%2)"), cbuf, MAX_PATH, buffer);
    m_strTitleStreamingFormat.Format(_T("%s"), cbuf);

    GetPrivateProfileString(szAppName, _T("FormatEx"), _T("[%artist% - ]%title%"), cbuf, MAX_PATH, buffer);
    m_strID3Format.Format(_T("%s"), cbuf);
    GetPrivateProfileString(szAppName, _T("SIFFormat"), _T("%IART - %INAM"), cbuf, MAX_PATH, buffer);
    m_strSIFFormat.Format(_T("%s"), cbuf);
    m_nTagPriority = GetPrivateProfileInt(szAppName, _T("TagPriority"), 0, buffer);
}

void CIn_mpg123dParam::WriteProfile()
{
    TCHAR buffer[MAX_PATH];

    m_HelperIn.config_get_filename(buffer, sizeof(buffer));

    WCHAR string[32];
    static const WCHAR szAppName[] = _T("Shibatch mpg123");

    _itow(m_bEnable, string, 10);
    WritePrivateProfileString(szAppName, _T("Enable"), string, buffer);

    _itow(m_nDecodeThreadPriority, string, 10);
    WritePrivateProfileString(szAppName, _T("DecodePriority"), string, buffer);

    _itow(m_nOutputBps, string, 10);
    WritePrivateProfileString(szAppName, _T("OutputFormat"), string, buffer);

    _itow(m_bFullBuffering, string, 10);
    WritePrivateProfileString(szAppName, _T("FullBuffering"), string, buffer);

    _itow(m_bSupZero, string, 10);
    WritePrivateProfileString(szAppName, _T("SupZero"), string, buffer);

    _itow(m_bReverseLR, string, 10);
    WritePrivateProfileString(szAppName, _T("ReverseLR"), string, buffer);

    _itow(m_bReversePhase, string, 10);
    WritePrivateProfileString(szAppName, _T("ReversePhase"), string, buffer);

    _itow(m_bAltVolume, string, 10);
    WritePrivateProfileString(szAppName, _T("Volume"), string, buffer);

    _itow(m_bDispAvg, string, 10);
    WritePrivateProfileString(szAppName, _T("DispAvg"), string, buffer);

    _itow(m_nPan, string, 10);
    WritePrivateProfileString(szAppName, _T("XPan"), string, buffer);

    _itow(m_bReplayGainEnable, string, 10);
    WritePrivateProfileString(szAppName, _T("ReplayGain_Enable"), string, buffer);

    _itow(m_nReplayGainMode, string, 10);
    WritePrivateProfileString(szAppName, _T("ReplayGain_Mode"), string, buffer);

    _itow(m_nReplayGainTag, string, 10);
    WritePrivateProfileString(szAppName, _T("ReplayGain_Tag"), string, buffer);

    _itow(m_nReplayGainPreAmpWithRG, string, 10);
    WritePrivateProfileString(szAppName, _T("ReplayGain_PreAmp_WithRG"), string, buffer);

    _itow(m_nReplayGainPreAmpWithoutRG, string, 10);
    WritePrivateProfileString(szAppName, _T("ReplayGain_PreAmp_WithoutRG"), string, buffer);

    _itow(m_bReplayGainHardLimit, string, 10);
    WritePrivateProfileString(szAppName, _T("ReplayGain_HardLimit"), string, buffer);

    _itow(m_bEnableStreaming, string, 10);
    WritePrivateProfileString(szAppName, _T("EnableStreaming"), string, buffer);

    _itow(m_nReceiveThreadPriority, string, 10);
    WritePrivateProfileString(szAppName, _T("ReceivePriority"), string, buffer);

    _itow(m_nBufferLength, string, 10);
    WritePrivateProfileString(szAppName, _T("StrBuf"), string, buffer);

    _itow(m_bProxy, string, 10);
    WritePrivateProfileString(szAppName, _T("EnableProxy"), string, buffer);

    _itow(m_bSaveStream, string, 10);
    WritePrivateProfileString(szAppName, _T("SaveStream"), string, buffer);

    _itow(m_bTitleStreaming, string, 10);
    WritePrivateProfileString(szAppName, _T("TitleStreaming"), string, buffer);

    _itow(m_bTitleStreamingDecode, string, 10);
    WritePrivateProfileString(szAppName, _T("DecodeText"), string, buffer);

    _itow(m_bTitleStreamingUDP, string, 10);
    WritePrivateProfileString(szAppName, _T("UseUDP"), string, buffer);

    WritePrivateProfileString(szAppName, _T("Proxy"), (LPCTSTR)m_strProxy, buffer);
    WritePrivateProfileString(szAppName, _T("SaveStreamFile"), (LPCTSTR)m_strSaveStream, buffer);
    WritePrivateProfileString(szAppName, _T("TitleStreamingFormat"), (LPCTSTR)m_strTitleStreamingFormat, buffer);

    WritePrivateProfileString(szAppName, _T("FormatEx"), (LPCTSTR)m_strID3Format, buffer);
    WritePrivateProfileString(szAppName, _T("SIFFormat"), (LPCTSTR)m_strSIFFormat, buffer);

    _itow(m_nTagPriority, string, 10);
    WritePrivateProfileString(szAppName, _T("TagPriority"), string, buffer);
}

WCHAR CIn_mpg123dParam::x2c(WCHAR* what)
{
    WCHAR digit;

    digit = (what[0] >= 'A' ? ((what[0] & 0xdf) - 'A') + 10 : (what[0] - '0'));
    digit *= 16;
    digit += (what[1] >= 'A' ? ((what[1] & 0xdf) - 'A') + 10 : (what[1] - '0'));

    return digit;
}

void CIn_mpg123dParam::URLDecode(WCHAR* src, WCHAR* dst)
{
    WCHAR c;
    WCHAR* p;
    bool bFlag = false;
    bool bRadioSpy = false;

    p = src;

    while (c = (WCHAR)*p++)
    {
        switch (c)
        {
            case 0xff:
                if ((*p == 0x66) && (*(p + 1) == 0x66) && (*(p + 2) == 0x66) && (*(p + 3) == 0x66) && (*(p + 4)) && (*(p + 5)))
                {
                    bRadioSpy = true;
                    p += 4; /* 66 66 66 66 '8' 'C' */
                    c = x2c(p);

                    if (((c >= 0x81) && (c <= 0x9F)) || ((c >= 0xE0) && (c <= 0xEF)))
                    {
                        if (bFlag)
                        {
                            bFlag = false;
                        }
                        else
                        {
                            bFlag = true;
                        }
                    }
                    else
                    {
                        bFlag = false;
                    }

                    p += 2;
                }
                else
                {
                    if (bFlag)
                    {
                        *(dst - 1) = NULL;
                    }
                    bFlag = false;
                    c = NULL;
                }
                break;
            case '%':
                bFlag = false;
                if ((*p) && (*(p + 1)))
                {
                    c = x2c(p);
                    p += 2;
                }
                else
                {
                    c = NULL;
                }
                break;
            default:
                if ((c < 0x40) || (c > 0xfc))
                {
                    if (bFlag)
                    {
                        dst--;
                    }
                }

                if (bRadioSpy)
                {
                    if ((c < 0x20) || (c >= 0x80))
                    {
                        *dst++ = NULL;
                    }
                }

                bFlag = false;
        }
        *dst++ = c;
    }
    *dst = NULL;
}

int CIn_mpg123dParam::GetPriorityValue(int n)
{
    int ret;

    switch (n)
    {
        case 0:
            ret = THREAD_PRIORITY_NORMAL;
            break;
        case 1:
            ret = THREAD_PRIORITY_ABOVE_NORMAL;
            break;
        default:
            ret = THREAD_PRIORITY_HIGHEST;
            break;
    }

    return ret;
}

