// ReplaygainDialog.cpp
//

#include "StdAfx.h"
#include "resource.h"       // main symbols
#include "aerosubc.h"
#include "aeroglss.h"

#include <wa_ipc.h>

#include "TagInfo.h"
#include "in_mpg123.h"

#include "HttpFile.h"
#include "ICYInfo.h"

#include "Param.h"
#include "ReplayGainDialog.h"

#include "MainHelper.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

void reflect_setting();
void reflect_thread_setting();

CReplayGainDialog replayGainDialog;

BOOL CALLBACK AfxDlgProc(HWND hWnd, UINT message, WPARAM, LPARAM);

BOOL CALLBACK in_replaygain_proc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    if (!lParam)
        return FALSE;           // not WM_INITDIALOG yet

    replayGainDialog.Attach(hDlg);
    replayGainDialog.PreSubclassWindow();
    WNDPROC* pOldWndProc = replayGainDialog.GetSuperWndProcAddr();
    WNDPROC afxWndProc = AfxGetAfxWndProc();
    *pOldWndProc = (WNDPROC)SetWindowLong(hDlg, GWL_WNDPROC, (DWORD)afxWndProc);

    // Replace the dialog proc pointer with AfxDlgProc

    SetWindowLong(hDlg, DWL_DLGPROC, (DWORD)AfxDlgProc);

    // Now let MFC process this message

    return CallWindowProc(afxWndProc, hDlg, uMsg, wParam, NULL);
}

IMPLEMENT_DYNAMIC(CReplayGainDialog, CPropertyPage)

CReplayGainDialog::CReplayGainDialog() : CPropertyPage()
{
    m_bReplayGainEnable = FALSE;
    m_bReplayGainTagID3v2 = FALSE;
    m_bReplayGainTagAPE = FALSE;
    m_bReplayGainTagLAME = FALSE;
    m_bReplayGainHardLimit = FALSE;
}

CReplayGainDialog::~CReplayGainDialog()
{
}

void CReplayGainDialog::DoDataExchange(CDataExchange* pDX)
{
    CPropertyPage::DoDataExchange(pDX);
    CMenuXP::SetXPLookNFeel(this, CMenuXP::GetXPLookNFeel(GetParent()->GetParent()));
    DDX_Check(pDX, IDC_CH_REPLAYGAIN_ENABLE, m_bReplayGainEnable);
    DDX_Control(pDX, IDC_COMBO_REPLAYGAIN_MODE, m_comboReplayGainMode);
    DDX_Check(pDX, IDC_CH_REPLAYGAIN_TAG_ID3V2, m_bReplayGainTagID3v2);
    DDX_Check(pDX, IDC_CH_REPLAYGAIN_TAG_APE, m_bReplayGainTagAPE);
    DDX_Check(pDX, IDC_CH_REPLAYGAIN_TAG_LAME, m_bReplayGainTagLAME);
    DDX_Control(pDX, IDC_TRACK_REPLAYGAIN_PREAMP_WITH_RG, m_trackReplayGainPreAmpWithRG);
    DDX_Control(pDX, IDC_ST_REPLAYGAIN_PREAMP_WITH_RG_DB, m_ReplayGainPreAmpWithRG_DB);
    DDX_Control(pDX, IDC_TRACK_REPLAYGAIN_PREAMP_WITHOUT_RG, m_trackReplayGainPreAmpWithoutRG);
    DDX_Control(pDX, IDC_ST_REPLAYGAIN_PREAMP_WITHOUT_RG_DB, m_ReplayGainPreAmpWithoutRG_DB);
    DDX_Check(pDX, IDC_CH_REPLAYGAIN_HARDLIMIT, m_bReplayGainHardLimit);
}

BEGIN_MESSAGE_MAP(CReplayGainDialog, CPropertyPage)
    ON_WM_ERASEBKGND()
    ON_BN_CLICKED(IDC_CH_REPLAYGAIN_ENABLE, OnBnClickedReplayGainEnable)
    ON_BN_CLICKED(IDC_CH_REPLAYGAIN_TAG_ID3V2, OnBnClickedReplayGainEnable)
    ON_BN_CLICKED(IDC_CH_REPLAYGAIN_TAG_APE, OnBnClickedReplayGainEnable)
    ON_BN_CLICKED(IDC_CH_REPLAYGAIN_TAG_LAME, OnBnClickedReplayGainEnable)
    ON_CBN_SELCHANGE(IDC_COMBO_REPLAYGAIN_MODE, OnBnClickedReplayGainEnable)
    ON_BN_CLICKED(IDC_CH_REPLAYGAIN_HARDLIMIT, OnBnClickedReplayGainEnable)
    ON_WM_HSCROLL()
END_MESSAGE_MAP()

void CReplayGainDialog::OnOK()
{
    UpdateData(TRUE);

    prm.m_bReplayGainEnable = m_bReplayGainEnable == TRUE;
    prm.m_nReplayGainTag = m_bReplayGainTagID3v2 ? REPLAYGAIN_TAG_ID3V2 : REPLAYGAIN_TAG_NONE;
    prm.m_nReplayGainTag |= m_bReplayGainTagAPE ? REPLAYGAIN_TAG_APE : REPLAYGAIN_TAG_NONE;
    prm.m_nReplayGainTag |= m_bReplayGainTagLAME ? REPLAYGAIN_TAG_LAME : REPLAYGAIN_TAG_NONE;
    prm.m_bReplayGainHardLimit = m_bReplayGainHardLimit;
    prm.m_nReplayGainPreAmpWithRG = m_trackReplayGainPreAmpWithRG.GetPos();
    prm.m_nReplayGainPreAmpWithoutRG = m_trackReplayGainPreAmpWithoutRG.GetPos();

    bool bFlag = false;

    if (prm.m_nReplayGainMode != m_comboReplayGainMode.GetCurSel())
    {
        prm.m_nReplayGainMode = m_comboReplayGainMode.GetCurSel();
        bFlag = true;
    }
    if (bFlag)
    {
        ::reflect_thread_setting();
    }
    ::reflect_setting();
}

BOOL CReplayGainDialog::OnInitDialog()
{
    CPropertyPage::OnInitDialog();

    AeroAutoSubclass(m_hWnd, ASC_SUBCLASS_ALL_CONTROLS, 0L);
    SetProp(m_hWnd, _T("vista"), m_hWnd);

    m_HelperIn.translateDlgItemText(this, IDC_CH_REPLAYGAIN_ENABLE);
    m_HelperIn.translateDlgItemText(this, IDC_CH_REPLAYGAIN_TAG_ID3V2);
    m_HelperIn.translateDlgItemText(this, IDC_CH_REPLAYGAIN_TAG_APE);
    m_HelperIn.translateDlgItemText(this, IDC_CH_REPLAYGAIN_TAG_LAME);
    m_HelperIn.translateDlgItemText(this, IDC_CH_REPLAYGAIN_HARDLIMIT);
    m_HelperIn.translateDlgItemText(this, IDC_LREPLAYGAIN_MODE);
    m_HelperIn.translateDlgItemText(this, IDC_LREADTAG);
    m_HelperIn.translateDlgItemText(this, IDC_LREPLAYGAIN_PREAMP);
    m_HelperIn.translateDlgItemText(this, IDC_LFILES_WITH_REPLAYGAIN_INFO);
    m_HelperIn.translateDlgItemText(this, IDC_LFILES_WITHOUT_REPLAYGAIN_INFO);

    m_bReplayGainEnable = prm.m_bReplayGainEnable;

    m_bReplayGainTagID3v2 = prm.m_nReplayGainTag & REPLAYGAIN_TAG_ID3V2;
    m_bReplayGainTagAPE = (prm.m_nReplayGainTag & REPLAYGAIN_TAG_APE) == REPLAYGAIN_TAG_APE;
    m_bReplayGainTagLAME = (prm.m_nReplayGainTag & REPLAYGAIN_TAG_LAME) == REPLAYGAIN_TAG_LAME;
    m_bReplayGainHardLimit = prm.m_bReplayGainHardLimit;

    m_trackReplayGainPreAmpWithRG.SetRange(-120, 120);
    m_trackReplayGainPreAmpWithRG.SetTic(25);
    m_trackReplayGainPreAmpWithRG.SetTicFreq(10);
    m_trackReplayGainPreAmpWithRG.SetPos(prm.m_nReplayGainPreAmpWithRG);
    ShowDB(prm.m_nReplayGainPreAmpWithRG, &m_ReplayGainPreAmpWithRG_DB);
    SendMessage(WM_HSCROLL, TB_ENDTRACK, (LPARAM)GetDlgItem(IDC_TRACK_REPLAYGAIN_PREAMP_WITH_RG));

    m_trackReplayGainPreAmpWithoutRG.SetRange(-120, 120);
    m_trackReplayGainPreAmpWithoutRG.SetTic(25);
    m_trackReplayGainPreAmpWithoutRG.SetTicFreq(10);
    m_trackReplayGainPreAmpWithoutRG.SetPos(prm.m_nReplayGainPreAmpWithoutRG);
    ShowDB(prm.m_nReplayGainPreAmpWithoutRG, &m_ReplayGainPreAmpWithoutRG_DB);
    SendMessage(WM_HSCROLL, TB_ENDTRACK, (LPARAM)GetDlgItem(IDC_TRACK_REPLAYGAIN_PREAMP_WITHOUT_RG));

    CString buf = m_HelperIn.MyLoadString(IDS_LTRACKGAIN);
    m_comboReplayGainMode.AddString(buf);
    buf = m_HelperIn.MyLoadString(IDS_LALBUMGAIN);
    m_comboReplayGainMode.AddString(buf);

    m_comboReplayGainMode.SetCurSel(prm.m_nReplayGainMode);

    UpdateData(FALSE);

    OnBnClickedReplayGainEnable();

    return TRUE;
}

BOOL CReplayGainDialog::OnEraseBkgnd(CDC* pDC)
{
    CDwmApiImpl dwmApi;
    if (dwmApi.Initialize() && dwmApi.IsDwmCompositionEnabled())
    {
        CRect rcClient;
        GetClientRect(&rcClient);

        VERIFY(pDC->PatBlt(0, 0, rcClient.Width(), rcClient.Height(), BLACKNESS));
        return TRUE;
    }
    return CPropertyPage::OnEraseBkgnd(pDC);
}

BOOL CReplayGainDialog::OnSetActive()
{
    OnBnClickedReplayGainEnable();
    return CPropertyPage::OnSetActive();
}

WNDPROC* CReplayGainDialog::GetSuperWndProcAddr()
{
    return &m_wndproc;
}

void CReplayGainDialog::OnBnClickedReplayGainEnable()
{
    UpdateData(TRUE);

    GetDlgItem(IDC_CH_REPLAYGAIN_ENABLE)->EnableWindow(prm.m_bEnable);
    GetDlgItem(IDC_COMBO_REPLAYGAIN_MODE)->EnableWindow(prm.m_bEnable && m_bReplayGainEnable);
    GetDlgItem(IDC_CH_REPLAYGAIN_TAG_ID3V2)->EnableWindow(prm.m_bEnable && m_bReplayGainEnable);
    GetDlgItem(IDC_CH_REPLAYGAIN_TAG_LAME)->EnableWindow(prm.m_bEnable && m_bReplayGainEnable);
    GetDlgItem(IDC_CH_REPLAYGAIN_TAG_APE)->EnableWindow(prm.m_bEnable && m_bReplayGainEnable);
    GetDlgItem(IDC_TRACK_REPLAYGAIN_PREAMP_WITH_RG)->EnableWindow(prm.m_bEnable && m_bReplayGainEnable);
    GetDlgItem(IDC_TRACK_REPLAYGAIN_PREAMP_WITHOUT_RG)->EnableWindow(prm.m_bEnable && m_bReplayGainEnable);
    GetDlgItem(IDC_CH_REPLAYGAIN_HARDLIMIT)->EnableWindow(prm.m_bEnable && m_bReplayGainEnable);
}

void CReplayGainDialog::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
    if (pScrollBar == reinterpret_cast<CScrollBar*>(&m_trackReplayGainPreAmpWithRG))
    {
        ShowDB(m_trackReplayGainPreAmpWithRG.GetPos(), &m_ReplayGainPreAmpWithRG_DB);
    }
    else if (pScrollBar == reinterpret_cast<CScrollBar*>(&m_trackReplayGainPreAmpWithoutRG))
    {
        ShowDB(m_trackReplayGainPreAmpWithoutRG.GetPos(), &m_ReplayGainPreAmpWithoutRG_DB);
    }
}
void CReplayGainDialog::ShowDB(int pos, CStatic* staticDB)
{
    TCHAR strBuff[16];
    TCHAR* str = strBuff;

    if (pos > 0)
    {
        *str++ = _T('+');
    }
    else if (pos < 0)
    {
        *str++ = _T('-');
    }
    else
    {
        *str++ = _T('+');
        *str++ = _T('/');
        *str++ = _T('-');
    }

    pos = abs(pos);

    _itow(pos / 10, str, 10);
    str += wcslen(str);
    *str++ = '.';
    _itow(pos - pos / 10 * 10, str, 10);
    str += wcslen(str);
    *str++ = ' ';
    *str++ = 'd';
    *str++ = 'B';
    *str = '\0';

    staticDB->SetWindowText(strBuff);
}

