#include "StdAfx.h"
#include "StandardInput.h"

#pragma warning(disable : 4005)

#include <afxsock.h>
#include <process.h>
#include <MMSystem.h>

#include <wa_ipc.h>

#include "mpg123.h"
#include "mpglib.h"

#include "CommonFunc.h"
#include "TagInfo.h"
#include "in_mpg123.h"
#include "AsyncSocket.h"
#include "HttpFile.h"
#include "ICYInfo.h"
#include "Param.h"
#include "paramlist.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

void __cdecl SAVSAInit(int maxlatency_in_ms, int srate);
void __cdecl SAVSADeInit();
void __cdecl SAAddPCMData(void* PCMData, int nch, int bps, int timestamp);
int __cdecl SAGetMode();
int __cdecl SAAdd(void* data, int timestamp, int csa);
void __cdecl VSAAddPCMData(void* PCMData, int nch, int bps, int timestamp);
int __cdecl VSAGetMode(int* specNch, int* waveNch);
int __cdecl VSAAdd(void* data, int timestamp);
void __cdecl VSASetInfo(int nch, int srate);
int __cdecl dsp_isactive();
int __cdecl dsp_dosamples(short int* samples, int numsamples, int bps, int nch, int srate);
void __cdecl setInfo(int bitrate, int srate, int stereo, int synched);

char inDescription[50] = "Module standard redvelopper par Mickal SEBIRE";
int infoBitrate;
int infoSamplerate;
int infoStereo; 
int infoSynched;

extern In_Module modVideo;
extern In_Module mod;
In_Module* standardIn;

extern void equMakeTable(float* lbc, float* rbc, paramlist*, float fs);
extern int equ_modifySamples(char* buf, int nsamples, int nch, int bps);
extern void equ_clearbuf(int, int);

float last_srate = 0;
int last_nch = 0, last_bps = 0;
float lbands[18] =
{
    1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0
};
float rbands[18] =
{
    1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0
};
paramlist paramroot;

void SAVSAInit(int maxlatency_in_ms, int srate)
{
    //TRACE(_T("SASAInit\n"));
}

void SAVSADeInit()
{
    //TRACE(_T("SAVSADeInit\n"));
}

void SAAddPCMData(void* PCMData, int nch, int bps, int timestamp)
{
    //TRACE(_T("SAAddPCMData\n"));
}

int SAGetMode()
{
    //TRACE(_T("SAGetMode\n"));
    return 0;
}

int SAAdd(void* data, int timestamp, int csa)
{
    //TRACE(_T("SAAdd\n"));
    return 0;
}

void VSAAddPCMData(void* PCMData, int nch, int bps, int timestamp)
{
    //TRACE(_T("VSAAddPCMDatA\n"));
}

int VSAGetMode(int* specNch, int* waveNch)
{
    return 0;
}

int VSAAdd(void* data, int timestamp)
{
    //TRACE(_T("VSAAdd\n"));
    return 0;
}

void VSASetInfo(int nch, int srate)
{
    //TRACE(_T("VSASetInfo\n"));
}

int dsp_isactive()
{
    //TRACE(_T("dsp_isactive\n"));
    extern int enable;
    return enable;
}

int dsp_dosamples(short int* samples, int numsamples, int bps, int nch, int srate)
{
    //TRACE(_T("dsp_dosamples numsamples %d bps %d nch %d srate %d\n"),numsamples, bps, nch, srate);

    if ((nch != 1 && nch != 2) || (bps != 8 && bps != 16 && bps != 24))
        return numsamples;

    if (last_srate != srate)
    {
        equMakeTable(lbands, rbands, &paramroot, (float)srate);
        last_srate = (float)srate;
        last_nch = nch;
        last_bps = bps;
        equ_clearbuf(bps, srate);
    }
    else if (last_nch != nch || last_bps != bps)
    {
        last_nch = nch;
        last_bps = bps;
        equ_clearbuf(bps, srate);
    }

    equ_modifySamples((char*)samples, numsamples, nch, bps);

    return numsamples;
}

void setInfo(int bitrate, int srate, int stereo, int synched)
{
    //TRACE(_T("SetInfo %d %d %d %d\n"), bitrate, srate, stereo, synched);
    infoBitrate = bitrate;
    infoSamplerate = srate;
    infoStereo = stereo; 
    infoSynched = synched;
}

CStandardInput::~CStandardInput()
{
}

CStandardInput::CStandardInput(CString dllPath, HWND hWnd) : InputInterface(dllPath)
{
    standardIn = &mod;
    //  standardIn = &modVideo;


    standardIn->version = mod.version;                      //   module type (IN_VER)
    standardIn->description = inDescription;                //   description of module, with version string
    standardIn->hMainWindow = hWnd;                         //   winamp's main window (filled in by winamp)
    standardIn->hDllInstance = AfxGetInstanceHandle();      //   DLL instance handle (Also filled in by winamp)
    //  about,                                                  //   about dialog
    standardIn->SAVSAInit = SAVSAInit;                      //   call once in Play(). maxlatency_in_ms should be the value returned from outMod->Open()
    standardIn->SAVSADeInit = SAVSADeInit;                  //   call in Stop()
    standardIn->SAAddPCMData = SAAddPCMData;                //  
    standardIn->SAGetMode = SAGetMode;                      //   gets csa (the current type (4=ws,2=osc,1=spec))
    standardIn->SAAdd = SAAdd;                              //   sets the spec data, filled in by winamp
    standardIn->VSAAddPCMData = VSAAddPCMData;              //   sets the vis data directly from PCM data
    standardIn->VSAGetMode = VSAGetMode;                    //   use to figure out what to give to VSAAdd
    standardIn->VSAAdd = VSAAdd;                            //   filled in by winamp, called by plug-in
    standardIn->VSASetInfo = VSASetInfo;                    //  
    standardIn->dsp_isactive = dsp_isactive;                //  
    standardIn->dsp_dosamples = dsp_dosamples;              //  
    standardIn->SetInfo = setInfo;                          //   if -1, changes ignored? :)
}

void CStandardInput::Config(HWND hwndParent) // configuration dialog
{
    standardIn->Config(hwndParent);
}

void CStandardInput::About(HWND hwndParent)  // about dialog
{
    standardIn->About(hwndParent);
}

void CStandardInput::GetFileInfo(TCHAR* file, TCHAR* title, int* length_in_ms) // if file == NULL, current playing is used
{
    standardIn->GetFileInfo(file, title, length_in_ms);
}

int CStandardInput::InfoBox(TCHAR* file, HWND hwndParent)
{
    int res = standardIn->InfoBox(file, hwndParent);
    return res;
}

int CStandardInput::IsOurFile(TCHAR* fn)  // called before extension checks, to allow detection of mms://, etc
{
    int res = standardIn->IsOurFile(fn);
    return res;
}

// playback stuff
int CStandardInput::Play(TCHAR* fn)   // return zero on success, -1 on file-not-found, some other value on other (stopping winamp) error
{
    int res = standardIn->Play(fn);
    return res;
}

void CStandardInput::Pause()      // pause stream
{
    standardIn->Pause();
}

void CStandardInput::UnPause()      // unpause stream
{
    standardIn->UnPause();
}

int CStandardInput::IsPaused()     // ispaused? return 1 if paused, 0 if not
{
    int res = standardIn->IsPaused();
    return res;
}

void CStandardInput::Stop()       // stop (unload) stream
{
    standardIn->Stop();
}
// time stuff
int CStandardInput::GetLength()      // get length in ms
{
    int res = standardIn->GetLength();
    return res;
}

int CStandardInput::GetOutputTime()    // returns current output time in ms. (usually returns outMod->GetOutputTime()
{
    int res = standardIn->GetOutputTime();
    return res;
}

void CStandardInput::SetOutputTime(int time_in_ms)  // seeks to point in stream (in ms). Usually you signal yoru thread to seek, which seeks and calls outMod->Flush()..
{
    standardIn->SetOutputTime(time_in_ms);
}

// volume stuff
void CStandardInput::SetVolume(int volume)  // from 0 to 255.. usually just call outMod->SetVolume
{
    standardIn->SetVolume(volume);
}

void CStandardInput::SetPan(int pan)  // from -127 to 127.. usually just call outMod->SetPan
{
    standardIn->SetPan(pan);
}

// info setting (filled in by winamp)
void CStandardInput::SetInfo(int bitrate, int srate, int stereo, int synched) // if -1, changes ignored? :)
{
    standardIn->SetInfo(bitrate, srate, stereo, synched);
}

int CStandardInput::GetVersion()       // module type (IN_VER)
{
    return standardIn->version;
}
char* CStandardInput::GetDescription()  // description of module, with version string
{
    return standardIn->description;
}

void CStandardInput::SethMainWindow(HWND hWnd)    // winamp's main window (filled in by winamp)
{
    standardIn->hMainWindow = hWnd;
}
void CStandardInput::SethDllInstance(HINSTANCE hDLL)    // DLL instance handle (Also filled in by winamp)
{
}

char* CStandardInput::GetFileExtensions()   // "mp3\0Layer 3 MPEG\0mp2\0Layer 2 MPEG\0mpg\0Layer 1 MPEG\0"
{
    return standardIn->FileExtensions;
}

int CStandardInput::isSeekable()     // is this stream seekable?
{
    return standardIn->is_seekable;
}

int CStandardInput::isUsesOutputPlug()     // does this plug-in use the output plug-ins? (musn't ever change, ever :)
{
    return standardIn->UsesOutputPlug;
}

void CStandardInput::SetOutMod(Out_Module* out) // filled in by winamp, optionally used :)
{
    standardIn->outMod = out;
}

In_Module* CStandardInput::GetIn_Module()
{
    return standardIn;
}

int CStandardInput::winampGetExtendedFileInfo(extendedFileInfoStruct ExtendedFileInfo)
{
    return m_Tag.GetExtendedFileInfo(&ExtendedFileInfo, prm.m_nTagPriority);
}

int CStandardInput::winampGetExtendedFileInfoW(extendedFileInfoStructW ExtendedFileInfo)
{
    return m_Tag.GetExtendedFileInfoW(&ExtendedFileInfo, prm.m_nTagPriority);
}
