// StreamingDialog.cpp
//

#include "StdAfx.h"
#include "resource.h"       // main symbols
#include "aerosubc.h"
#include "aeroglss.h"

#include "CommonFunc.h"

#include <wa_ipc.h>
#include "TagInfo.h"
#include "in_mpg123.h"

#include "HttpFile.h"
#include "ICYInfo.h"

#include "Param.h"
#include "StreamingDialog.h"

#include "MainHelper.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

void reflect_setting();
void reflect_thread_setting();

CStreamingDialog streamDialog;

BOOL CALLBACK AfxDlgProc(HWND hWnd, UINT message, WPARAM, LPARAM);

BOOL CALLBACK in_streaming_proc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    if (!lParam)
        return FALSE;           // not WM_INITDIALOG yet

    streamDialog.Attach(hDlg);
    streamDialog.PreSubclassWindow();
    WNDPROC* pOldWndProc = streamDialog.GetSuperWndProcAddr();
    WNDPROC afxWndProc = AfxGetAfxWndProc();
    *pOldWndProc = (WNDPROC)SetWindowLong(hDlg, GWL_WNDPROC, (DWORD)afxWndProc);

    // Replace the dialog proc pointer with AfxDlgProc

    SetWindowLong(hDlg, DWL_DLGPROC, (DWORD)AfxDlgProc);

    // Now let MFC process this message

    return CallWindowProc(afxWndProc, hDlg, uMsg, wParam, NULL);
}

IMPLEMENT_DYNAMIC(CStreamingDialog, CPropertyPage)

CStreamingDialog::CStreamingDialog() : CPropertyPage()
{
    m_bEnableStreaming = false;
    m_strStrBuf = _T("");
    m_strProxy = _T("");
    m_strSaveStream = _T("");
    m_strTitleStreamingFormat = _T("");
    m_bProxy = false;
    m_bSaveStream = false;
    m_bTitleStreaming = false;
    m_bTitleStreamingDecode = false;
    m_bTitleStreamingUDP = false;
}

CStreamingDialog::~CStreamingDialog()
{
}

void CStreamingDialog::DoDataExchange(CDataExchange* pDX)
{
    CPropertyPage::DoDataExchange(pDX);
    CMenuXP::SetXPLookNFeel(this, CMenuXP::GetXPLookNFeel(GetParent()->GetParent()));
    DDX_Check(pDX, IDC_CH_ENABLE_STREAMING, m_bEnableStreaming);
    DDX_Control(pDX, IDC_COMBO_PRIORITY, m_comboPriority);
    DDX_Text(pDX, IDC_ED_STRBUF, m_strStrBuf);
    DDX_Control(pDX, IDC_ED_TITLESTREAMING_FORMAT, m_editTitleStreamingFormat);
    DDX_Control(pDX, IDC_BUTTON_FILEDIALOG, m_btnBrowse);
    DDX_Control(pDX, IDC_ED_PROXY, m_editProxy);
    DDX_Control(pDX, IDC_ED_STRBUF, m_editStrBuf);
    DDX_Control(pDX, IDC_ED_SAVESTREAM, m_editSaveStream);
    DDX_Text(pDX, IDC_ED_PROXY, m_strProxy);
    DDX_Text(pDX, IDC_ED_SAVESTREAM, m_strSaveStream);
    DDX_Text(pDX, IDC_ED_TITLESTREAMING_FORMAT, m_strTitleStreamingFormat);
    DDX_Check(pDX, IDC_PROXY, m_bProxy);
    DDX_Check(pDX, IDC_SAVESTREAM, m_bSaveStream);
    DDX_Check(pDX, IDC_TITLESTREAMING, m_bTitleStreaming);
    DDX_Check(pDX, IDC_TITLESTREAMING_DECODE, m_bTitleStreamingDecode);
    DDX_Check(pDX, IDC_TITLESTREAMING_UDP, m_bTitleStreamingUDP);
}

BOOL CStreamingDialog::OnInitDialog()
{
    CPropertyPage::OnInitDialog();

    AeroAutoSubclass(m_hWnd, ASC_SUBCLASS_ALL_CONTROLS, 0L);
    SetProp(m_hWnd, _T("vista"), m_hWnd);

    m_HelperIn.translateDlgItemText(this, IDC_LSTREAMING_THRAD_PRIORITY);
    m_HelperIn.translateDlgItemText(this, IDC_LSTREAMING_BUFFER_SIZE);
    m_HelperIn.translateDlgItemText(this, IDC_LSTREAMING_HTTP_PROXY);
    m_HelperIn.translateDlgItemText(this, IDC_LSTREAMING_SAVE);
    m_HelperIn.translateDlgItemText(this, IDC_LSTREAMING_TITLE_STREAMING);
    m_HelperIn.translateDlgItemText(this, IDC_LSTREAMING_TITLE_FORMAT);
    m_HelperIn.translateDlgItemText(this, IDC_LSTREAMING_EXAMPLE);
    m_HelperIn.translateDlgItemText(this, IDC_CH_ENABLE_STREAMING);
    m_HelperIn.translateDlgItemText(this, IDC_PROXY);
    m_HelperIn.translateDlgItemText(this, IDC_SAVESTREAM);
    m_HelperIn.translateDlgItemText(this, IDC_TITLESTREAMING);
    m_HelperIn.translateDlgItemText(this, IDC_TITLESTREAMING_UDP);
    m_HelperIn.translateDlgItemText(this, IDC_TITLESTREAMING_DECODE);
    m_HelperIn.translateDlgItemText(this, IDC_LSTREAMING_KB);
    m_HelperIn.translateDlgItemText(this, IDC_BUTTON_FILEDIALOG);

    SetDlgItemText(IDC_ED_SAVESTREAM, prm.m_strSaveStream);
    SetDlgItemText(IDC_ED_PROXY, prm.m_strProxy);

    m_strStrBuf.Format(_T("%d"), prm.m_nBufferLength);
    m_strTitleStreamingFormat = prm.m_strTitleStreamingFormat;

    CString buf = m_HelperIn.MyLoadString(IDS_LNORMAL);
    m_comboPriority.AddString(buf);
    buf = m_HelperIn.MyLoadString(IDS_LABOVENORMAL);
    m_comboPriority.AddString(buf);
    buf = m_HelperIn.MyLoadString(IDS_LHIGHEST);
    m_comboPriority.AddString(buf);

    m_comboPriority.SetCurSel(prm.m_nReceiveThreadPriority);

    m_bEnableStreaming = prm.m_bEnableStreaming;
    m_strProxy = prm.m_strProxy;
    m_strSaveStream = prm.m_strSaveStream;
    m_strTitleStreamingFormat = prm.m_strTitleStreamingFormat;
    m_bTitleStreaming = prm.m_bTitleStreaming;
    m_bTitleStreamingUDP = prm.m_bTitleStreamingUDP;
    m_bTitleStreamingDecode = prm.m_bTitleStreamingDecode;
    m_bProxy = prm.m_bProxy;
    m_bSaveStream = prm.m_bSaveStream;

    UpdateData(FALSE);

    OnBnClickedChEnableStreaming();

    return TRUE;
}

BOOL CStreamingDialog::OnEraseBkgnd(CDC* pDC)
{
    CDwmApiImpl dwmApi;
    if (dwmApi.Initialize() && dwmApi.IsDwmCompositionEnabled())
    {
        CRect rcClient;
        GetClientRect(&rcClient);

        VERIFY(pDC->PatBlt(0, 0, rcClient.Width(), rcClient.Height(), BLACKNESS));
        return TRUE;
    }
    return CPropertyPage::OnEraseBkgnd(pDC);
}

BOOL CStreamingDialog::OnSetActive()
{
    OnBnClickedChEnableStreaming();
    return CPropertyPage::OnSetActive();
}

BEGIN_MESSAGE_MAP(CStreamingDialog, CPropertyPage)
  ON_WM_ERASEBKGND()
  ON_BN_CLICKED(IDC_CH_ENABLE_STREAMING, OnBnClickedChEnableStreaming)
  ON_BN_CLICKED(IDC_PROXY, OnBnClickedChEnableStreaming)
  ON_BN_CLICKED(IDC_SAVESTREAM, OnBnClickedChEnableStreaming)
  ON_BN_CLICKED(IDC_BUTTON_FILEDIALOG, OnBnClickedButtonFiledialog)
  ON_BN_CLICKED(IDC_TITLESTREAMING, OnBnClickedChEnableStreaming)
END_MESSAGE_MAP()

WNDPROC* CStreamingDialog::GetSuperWndProcAddr()
{
    return &m_wndproc;
}

void CStreamingDialog::OnBnClickedChEnableStreaming()
{
    UpdateData(TRUE);

    GetDlgItem(IDC_CH_ENABLE_STREAMING)->EnableWindow(prm.m_bEnable);
    GetDlgItem(IDC_COMBO_PRIORITY)->EnableWindow(prm.m_bEnable && m_bEnableStreaming);
    m_editStrBuf.EnableWindow(prm.m_bEnable && m_bEnableStreaming);
    GetDlgItem(IDC_PROXY)->EnableWindow(prm.m_bEnable && m_bEnableStreaming);
    m_editProxy.EnableWindow(prm.m_bEnable && m_bProxy && m_bEnableStreaming);
    GetDlgItem(IDC_SAVESTREAM)->EnableWindow(prm.m_bEnable && m_bEnableStreaming);
    m_editSaveStream.EnableWindow(prm.m_bEnable && m_bSaveStream && m_bEnableStreaming);
    m_btnBrowse.EnableWindow(prm.m_bEnable && m_bSaveStream && m_bEnableStreaming);
    GetDlgItem(IDC_TITLESTREAMING)->EnableWindow(prm.m_bEnable && m_bEnableStreaming);
    GetDlgItem(IDC_TITLESTREAMING_DECODE)->EnableWindow(prm.m_bEnable && m_bTitleStreaming && m_bEnableStreaming);
    GetDlgItem(IDC_TITLESTREAMING_UDP)->EnableWindow(prm.m_bEnable && m_bTitleStreaming && m_bEnableStreaming);
    GetDlgItem(IDC_ED_TITLESTREAMING_FORMAT)->EnableWindow(prm.m_bEnable && m_bTitleStreaming && m_bEnableStreaming);
}

void CStreamingDialog::OnBnClickedButtonFiledialog()
{
    CString str;

    str = m_HelperIn.MyLoadString(IDS_SAVESTREAMDLG);

    CFileDialog fileDialog(FALSE, _T("*.mp3"), _T("notitle.mp3"), OFN_OVERWRITEPROMPT | OFN_PATHMUSTEXIST, str, this, 0);
    if (fileDialog.DoModal() == IDOK)
    {
        SetDlgItemText(IDC_ED_SAVESTREAM, fileDialog.GetPathName());
    }
}

void CStreamingDialog::OnOK()
{
    UpdateData(TRUE);

    prm.m_bEnableStreaming = m_bEnableStreaming == TRUE;
    prm.m_strProxy = m_strProxy;
    prm.m_strSaveStream = m_strSaveStream;
    prm.m_strTitleStreamingFormat = m_strTitleStreamingFormat;
    prm.m_bTitleStreaming = m_bTitleStreaming == TRUE;
    prm.m_bTitleStreamingUDP = m_bTitleStreamingUDP == TRUE;
    prm.m_bTitleStreamingDecode = m_bTitleStreamingDecode == TRUE;
    prm.m_bProxy = m_bProxy == TRUE;
    prm.m_bSaveStream = m_bSaveStream == TRUE;

    prm.m_nBufferLength = Max(_tstoi(m_strStrBuf), MIN_STREAM_SIZE / 1024);
    prm.m_strTitleStreamingFormat = m_strTitleStreamingFormat;

    if (prm.m_nReceiveThreadPriority != m_comboPriority.GetCurSel())
    {
        prm.m_nReceiveThreadPriority = m_comboPriority.GetCurSel();
        ::reflect_thread_setting();
    }
    ::reflect_setting();

    CPropertyPage::OnOK();
}