#include "StdAfx.h"
#include <MMSystem.h>
//#include <mbctype.h>
//#include <mbstring.h>

#include <wa_ipc.h>

#include "CommonFunc.h"

#include "in_mpg123.h"

#include "TagInfo.h"
#include "VbrTag.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

bool getmp3info(bool Decode, const WCHAR* fn, mp3info* info, VBRTAGDATA* vbr_tag);

const char* GenreList[MAX_GENRE] =
{
    "Blues", "Classic Rock", "Country", "Dance", "Disco", "Funk", "Grunge", "Hip-Hop", "Jazz", "Metal", "New Age", "Oldies", "Other", "Pop", "R&B", "Rap", "Reggae", "Rock", "Techno", "Industrial", "Alternative", "Ska", "Death Metal", "Pranks", "Soundtrack", "Euro-Techno", "Ambient", "Trip-Hop", "Vocal", "Jazz+Funk", "Fusion", "Trance", "Classical", "Instrumental", "Acid", "House", "Game", "Sound Clip", "Gospel", "Noise", "AlternRock", "Bass", "Soul", "Punk", "Space", "Meditative", "Instrumental Pop", "Instrumental Rock", "Ethnic", "Gothic", "Darkwave", "Techno-Industrial", "Electronic", "Pop-Folk", "Eurodance", "Dream", "Southern Rock", "Comedy", "Cult", "Gangsta", "Top 40", "Christian Rap", "Pop/Funk", "Jungle", "Native American", "Cabaret", "New Wave", "Psychadelic", "Rave", "Showtunes", "Trailer", "Lo-Fi", "Tribal", "Acid Punk", "Acid Jazz", "Polka", "Retro", "Musical", "Rock & Roll", "Hard Rock", "Folk", "Folk/Rock", "National Folk", "Swing", "Fast-Fusion", "Bebob", "Latin", "Revival", "Celtic", "Bluegrass", "Avantgarde", "Gothic Rock", "Progressive Rock", "Psychedelic Rock", "Symphonic Rock", "Slow Rock", "Big Band", "Chorus", "Easy Listening", "Acoustic", "Humour", "Speech", "Chanson", "Opera", "Chamber Music", "Sonata", "Symphony", "Booty Bass", "Primus", "Porn Groove", "Satire", "Slow Jam", "Club", "Tango", "Samba", "Folklore", "Ballad", "Power Ballad", "Rhythmic Soul", "Freestyle", "Duet", "Punk Rock", "Drum Solo", "A capella", "Euro-House", "Dance Hall", "Goa", "Drum & Bass", "Club House", "Hardcore", "Terror", "Indie", "BritPop", "NegerPunk", "Polsk Punk", "Beat", "Christian Gangsta", "Heavy Metal", "Black Metal", "Crossover", "Contemporary C", "Christian Rock", "Merengue", "Salsa", "Thrash Metal", "Anime", "JPop", "SynthPop",
};

Tag::Tag()
{
    ::InitializeCriticalSection(&CriticalSection);

    FlushCache();
}

Tag::~Tag()
{
    ::DeleteCriticalSection(&CriticalSection);
}

void Tag::FlushCache()
{
    ::EnterCriticalSection(&CriticalSection);

    *Cache.FileName = '\0';
    *Cache.Title = '\0';
    *Cache.Artist = '\0';
    *Cache.Album = '\0';
    *Cache.Year = '\0';
    *Cache.Comment = '\0';
    *Cache.Genre = '\0';
    *Cache.Track = '\0';
    *Cache.AlbumArtist = '\0';
    *Cache.Composer = '\0';
    *Cache.OrgArtist = '\0';
    *Cache.Copyright = '\0';
    *Cache.Url = '\0';
    *Cache.Encoder = '\0';
    *Cache.Disc = '\0';
    Cache.Length = 0;
    GetTagTime = 0;

    ::LeaveCriticalSection(&CriticalSection);
}

bool Tag::Get(const WCHAR* FileName, const int TagPriority, const WCHAR* Format, WCHAR* Title)
{
    bool FindTag = *FileName && (wcscmp(FileName, Cache.FileName) == 0) && ((::timeGetTime() - GetTagTime) <= 2000);

    if (FindTag)
    {
        GetTagTime = ::timeGetTime();
    }
    else
    {
        FlushCache();
        wcscpy(Cache.FileName, FileName);

        FindTag = GetTitle(TagPriority, &Cache);

        GetTagTime = ::timeGetTime();
    }
    if (FindTag)
        DoFormat(&Cache, Format, Title, MAX_PATH);

    return FindTag;
}

bool Tag::GetReplayGainInfo(const WCHAR* FileName, const int TagPriority, const UINT ReplayGainTag, _ReplayGainInfo* Info)
{
    bool RetCode = false;

    switch (TagPriority)
    {
        case 0:
        case 1:
        case 4:
            RetCode = ((ReplayGainTag & REPLAYGAIN_TAG_ID3V2) ? GetTitleFromID3v2(NULL, Info, FileName) : false) || ((ReplayGainTag & REPLAYGAIN_TAG_APE) ? GetTitleFromAPE(NULL, Info, FileName) : false);
            break;
        case 2:
        case 3:
        case 5:
            RetCode = ((ReplayGainTag & REPLAYGAIN_TAG_APE) ? GetTitleFromAPE(NULL, Info, FileName) : false) || ((ReplayGainTag & REPLAYGAIN_TAG_ID3V2) ? GetTitleFromID3v2(NULL, Info, FileName) : false);
            break;
    }

    return RetCode;
}

int Tag::GetExtendedFileInfo(extendedFileInfoStruct* ExtendedFileInfo, const int TagPriority)
{
    ::EnterCriticalSection(&CriticalSection);

    WCHAR filename[MAX_PATHLEN];

    Strcpy(filename, MAX_PATHLEN, ExtendedFileInfo->filename, MAX_PATHLEN);

    bool FindTag = *ExtendedFileInfo->filename && (wcscmp(filename, Cache.FileName) == 0) && ((::timeGetTime() - GetTagTime) <= 2000);

    if (FindTag)
    {
        GetTagTime = ::timeGetTime();
    }
    else
    {
        mp3info info;
        VBRTAGDATA vbr_tag;

        if (wmemcmp(filename, _T("http://"), 7) && wmemcmp(filename, _T("https://"), 8) && getmp3info(false, filename, &info, &vbr_tag))
        {
            WCHAR BackFileName[MAX_PATHLEN];

            FlushCache();
            wcscpy(BackFileName, Cache.FileName);
            wcscpy(Cache.FileName, filename);
            Cache.Length = info.length;

            FindTag = GetTitle(TagPriority, &Cache);
            if (FindTag)
            {
                GetTagTime = ::timeGetTime();
            }
            else
            {
                wcscpy(Cache.FileName, BackFileName);
            }
        }
    }

    int RetCode = 0;

    if (FindTag)
    {
        WCHAR Buff[16];
        WCHAR* RetBuff;
        char RetRetBuff[MAX_PATHLEN];
        const char* MetaData = reinterpret_cast<const char*>(ExtendedFileInfo->metadata);

        if (_stricmp(MetaData, "length") == 0)
        {
            _itow(Cache.Length, Buff, 10);
            RetBuff = Buff;
            RetCode = 1;
        }
        else if (_stricmp(MetaData, "title") == 0)
        {
            RetBuff = Cache.Title;
            RetCode = 1;
        }
        else if (_stricmp(MetaData, "artist") == 0)
        {
            RetBuff = Cache.Artist;
            RetCode = 1;
        }
        else if (_stricmp(MetaData, "comment") == 0)
        {
            RetBuff = Cache.Comment;
            RetCode = 1;
        }
        else if (_stricmp(MetaData, "album") == 0)
        {
            RetBuff = Cache.Album;
            RetCode = 1;
        }
        else if (_stricmp(MetaData, "year") == 0)
        {
            RetBuff = Cache.Year;
            RetCode = 1;
        }
        else if (_stricmp(MetaData, "genre") == 0)
        {
            RetBuff = Cache.Genre;
            RetCode = 1;
        }
        else if (_stricmp(MetaData, "track") == 0)
        {
            RetBuff = Cache.Track;
            RetCode = 1;
        }
        else
        {
            RetCode = 0;
        }

        if (RetCode && ExtendedFileInfo->retlen)
        {
            Strcpy(RetRetBuff, MAX_PATHLEN, RetBuff);
            strncpy(ExtendedFileInfo->ret, RetRetBuff, ExtendedFileInfo->retlen - 1);
        }
    }
    ::LeaveCriticalSection(&CriticalSection);

    return RetCode;
}


int Tag::GetExtendedFileInfoW(extendedFileInfoStructW* ExtendedFileInfo, const int TagPriority)
{
    ::EnterCriticalSection(&CriticalSection);

    bool FindTag = *ExtendedFileInfo->filename && (wcscmp(ExtendedFileInfo->filename, Cache.FileName) == 0) && ((::timeGetTime() - GetTagTime) <= 2000);

    if (FindTag)
    {
        GetTagTime = ::timeGetTime();
    }
    else
    {
        mp3info info;
        VBRTAGDATA vbr_tag;

        if (wmemcmp(ExtendedFileInfo->filename, _T("http://"), 7) && wmemcmp(ExtendedFileInfo->filename, _T("https://"), 8) && getmp3info(false, ExtendedFileInfo->filename, &info, &vbr_tag))
        {
            WCHAR BackFileName[MAX_PATHLEN];

            FlushCache();
            wcscpy(BackFileName, Cache.FileName);
            wcscpy(Cache.FileName, ExtendedFileInfo->filename);
            Cache.Length = info.length;

            FindTag = GetTitle(TagPriority, &Cache);
            if (FindTag)
            {
                GetTagTime = ::timeGetTime();
            }
            else
            {
                wcscpy(Cache.FileName, BackFileName);
            }
        }
    }

    int RetCode = 0;

    if (FindTag)
    {
        WCHAR Buff[16];
        WCHAR* RetBuff;
        const wchar_t* MetaData = reinterpret_cast<const wchar_t*>(ExtendedFileInfo->metadata);

        if (_wcsicmp(MetaData, _T("length")) == 0)
        {
            _itow(Cache.Length, Buff, 10);
            RetBuff = Buff;
            RetCode = 1;
        }
        else if (_wcsicmp(MetaData, _T("title")) == 0)
        {
            RetBuff = Cache.Title;
            RetCode = 1;
        }
        else if (_wcsicmp(MetaData, _T("artist")) == 0)
        {
            RetBuff = Cache.Artist;
            RetCode = 1;
        }
        else if (_wcsicmp(MetaData, _T("comment")) == 0)
        {
            RetBuff = Cache.Comment;
            RetCode = 1;
        }
        else if (_wcsicmp(MetaData, _T("album")) == 0)
        {
            RetBuff = Cache.Album;
            RetCode = 1;
        }
        else if (_wcsicmp(MetaData, _T("year")) == 0)
        {
            RetBuff = Cache.Year;
            RetCode = 1;
        }
        else if (_wcsicmp(MetaData, _T("genre")) == 0)
        {
            RetBuff = Cache.Genre;
            RetCode = 1;
        }
        else if (_wcsicmp(MetaData, _T("track")) == 0)
        {
            RetBuff = Cache.Track;
            RetCode = 1;
        }
        else
        {
            RetCode = 0;
        }

        if (RetCode && ExtendedFileInfo->retlen)
        {
            Strncpy(ExtendedFileInfo->ret, RetBuff, ExtendedFileInfo->retlen - 1);
        }
    }
    ::LeaveCriticalSection(&CriticalSection);

    return RetCode;
}

bool Tag::GetTitle(const int TagPriority, _TagInfo* Info)
{
    bool RetCode = false;

    switch (TagPriority)
    {
        case 0:
            RetCode = GetTitleFromID3v2(Info) || GetTitleFromAPE(Info) || GetTitleFromID3v1(Info);
            break;
        case 1:
            RetCode = GetTitleFromID3v2(Info) || GetTitleFromID3v1(Info) || GetTitleFromAPE(Info);
            break;
        case 2:
            RetCode = GetTitleFromAPE(Info) || GetTitleFromID3v2(Info) || GetTitleFromID3v1(Info);
            break;
        case 3:
            RetCode = GetTitleFromAPE(Info) || GetTitleFromID3v1(Info) || GetTitleFromID3v2(Info);
            break;
        case 4:
            RetCode = GetTitleFromID3v1(Info) || GetTitleFromID3v2(Info) || GetTitleFromAPE(Info);
            break;
        case 5:
            RetCode = GetTitleFromID3v1(Info) || GetTitleFromAPE(Info) || GetTitleFromID3v2(Info);
            break;
    }

    return RetCode;
}

void Tag::StoreReplayGainInfo(const int ReplayGainFieldName, WCHAR* Buff, _ReplayGainInfo* Info)
{
    if ((ReplayGainFieldName == REPLAYGAIN_TRACK_GAIN) || (ReplayGainFieldName == REPLAYGAIN_ALBUM_GAIN))
    {
        WCHAR* dB = wcsstr(Buff, _T(" dB"));

        if (dB)
            *dB = '\0';
    }

    const double GainPeak = _wtof(Buff);

    switch (ReplayGainFieldName)
    {
        case REPLAYGAIN_TRACK_GAIN:
            Info->ValidTrackGain = true;
            Info->TrackGain = GainPeak;
            break;
        case REPLAYGAIN_TRACK_PEAK:
            Info->ValidTrackPeak = true;
            Info->TrackPeak = GainPeak;
            break;
        case REPLAYGAIN_ALBUM_GAIN:
            Info->ValidAlbumGain = true;
            Info->AlbumGain = GainPeak;
            break;
        case REPLAYGAIN_ALBUM_PEAK:
            Info->ValidAlbumPeak = true;
            Info->AlbumPeak = GainPeak;
            break;
    }
}

void Tag::DoFormat(const _TagInfo* Info, const WCHAR* Format, WCHAR* Title, const size_t TitleSize)
{
    WCHAR* MaxTitle = Title + TitleSize;
    WCHAR Str;
    const int MaxBlock = 8;
    int BlockIdx = -1;
    WCHAR* Block[MaxBlock];
    size_t MaxReplaceSize[MaxBlock];

    for (; (Str = *Format) != '\0'; Format++)
    {
        if (Str == '[')
        {
            if (++BlockIdx == MaxBlock)
                break;
            Block[BlockIdx] = Title;
            MaxReplaceSize[BlockIdx] = 0;
            continue;
        }
        else if (Str == ']')
        {
            if (BlockIdx == -1)
                break;
            if (MaxReplaceSize[BlockIdx] == 0)
                Title = Block[BlockIdx];
            BlockIdx--;
            continue;
        }
        else if (Str == '%')
        {
            const WCHAR* CommandPnt = Format + 1;

            if (*CommandPnt == '%')
            {
                Format++;
                if (Title < MaxTitle)
                    *Title++ = '%';
            }
            else if (*CommandPnt == '[')
            {
                Format++;
                if (Title < MaxTitle)
                    *Title++ = '[';
            }
            else if (*CommandPnt == ']')
            {
                Format++;
                if (Title < MaxTitle)
                    *Title++ = ']';
            }
            else
            {
                const int CommandLen = KanjiStrChr(CommandPnt, '%');

                if (CommandLen == -1)
                    break;

                WCHAR Command[MAX_FORMATTEXT];

                wmemcpy(Command, CommandPnt, CommandLen);
                *(Command + CommandLen) = '\0';

                Format += 1 + CommandLen;

                const size_t TitleRemainSize = MaxTitle - Title;
                if (_wcsicmp(Command, _T("title")) == 0)
                {
                    wcscpy(Title, Info->Title);
                }
                else if (_wcsicmp(Command, _T("artist")) == 0)
                {
                    wcscpy(Title, Info->Artist);
                }
                else if (_wcsicmp(Command, _T("album")) == 0)
                {
                    wcscpy(Title, Info->Album);
                }
                else if (_wcsicmp(Command, _T("year")) == 0)
                {
                    wcscpy(Title, Info->Year);
                }
                else if (_wcsicmp(Command, _T("comment")) == 0)
                {
                    wcscpy(Title, Info->Comment);
                }
                else if (_wcsicmp(Command, _T("genre")) == 0)
                {
                    wcscpy(Title, Info->Genre);
                }
                else if (_wcsicmp(Command, _T("filename")) == 0)
                {
                    if (Title < MaxTitle)
                    {
                        WCHAR Name[MAX_PATHLEN];

                        CutPathFileName(Info->FileName, NULL, 0, Name, sizeof Name);
                        *Title = '\0';
                        CutFileNameExt(Name, Title, TitleRemainSize);
                    }
                }
                else if (_wcsicmp(Command, _T("filepath")) == 0)
                {
                    if (Title < MaxTitle)
                    {
                        *Title = '\0';
                        CutPathFileName(Info->FileName, Title, TitleRemainSize);
                    }
                }
                else if (_wcsicmp(Command, _T("fileext")) == 0)
                {
                    if (Title < MaxTitle)
                    {
                        *Title = '\0';
                        CutFileNameExt(Info->FileName, NULL, 0, Title, TitleRemainSize);
                    }
                }
                else if (_wcsicmp(Command, _T("track")) == 0)
                {
                    if (Title < MaxTitle)
                    {
                        *Title = '\0';
                        if (*Info->Track)
                            FormatItow(_wtoi(Info->Track), Title, TitleRemainSize, 2, '0');
                    }
                }
                else if (_wcsicmp(Command, _T("composer")) == 0)
                {
                    wcscpy(Title, Info->Composer);
                }
                else if (_wcsicmp(Command, _T("origartist")) == 0)
                {
                    wcscpy(Title, Info->OrgArtist);
                }
                else if (_wcsicmp(Command, _T("copyright")) == 0)
                {
                    wcscpy(Title, Info->Copyright);
                }
                else if (_wcsicmp(Command, _T("encoder")) == 0)
                {
                    wcscpy(Title, Info->Encoder);
                }
            }
            const size_t ReplaceSize = wcslen(Title);

            Title += ReplaceSize;

            for (int Idx = 0; Idx <= BlockIdx; Idx++)
            {
                MaxReplaceSize[Idx] = Max(ReplaceSize, MaxReplaceSize[Idx]);
            }

            continue;
        }

        if (Title < MaxTitle)
            *Title++ = Str;
    }

    if (Title < MaxTitle)
    {
        *Title = '\0';
    }
    else
    {
        *(MaxTitle - 1) = '\0';
    }
}

