#ifndef TAGINFO_H
#define TAGINFO_H

#define MAX_PATHLEN     512
#define MAX_FORMATTEXT  512
#define MAX_MUSICTEXT   512

#define MAX_GENRE       148

class
Tag
{
public:
    struct
    _TagInfo
    {
        int Length;
        WCHAR FileName[MAX_PATHLEN];
        WCHAR Title[MAX_MUSICTEXT];
        WCHAR Artist[MAX_MUSICTEXT];
        WCHAR Album[MAX_MUSICTEXT];
        WCHAR Year[MAX_MUSICTEXT];
        WCHAR Comment[MAX_MUSICTEXT];
        WCHAR Genre[MAX_MUSICTEXT];
        WCHAR Track[MAX_MUSICTEXT];
        WCHAR AlbumArtist[MAX_MUSICTEXT];
        WCHAR Composer[MAX_MUSICTEXT];
        WCHAR OrgArtist[MAX_MUSICTEXT];
        WCHAR Copyright[MAX_MUSICTEXT];
        WCHAR Url[MAX_MUSICTEXT];
        WCHAR Encoder[MAX_MUSICTEXT];
        WCHAR Disc[MAX_MUSICTEXT];
    };
    struct
    _ReplayGainInfo
    {
        bool ValidTrackGain;
        bool ValidTrackPeak;
        bool ValidAlbumGain;
        bool ValidAlbumPeak;
        double TrackGain;
        double TrackPeak;
        double AlbumGain;
        double AlbumPeak;
    };
    enum
    {
        REPLAYGAIN_NONE,
        REPLAYGAIN_TRACK_GAIN,
        REPLAYGAIN_TRACK_PEAK,
        REPLAYGAIN_ALBUM_GAIN,
        REPLAYGAIN_ALBUM_PEAK,
    };

    Tag();
    ~Tag();

    void FlushCache();
    bool Get(const WCHAR* FileName, const int TagPriority, const WCHAR* Format, WCHAR* Title);
    bool GetReplayGainInfo(const WCHAR* FileName, const int TagPriority, const UINT ReplayGainTag, _ReplayGainInfo* Info);
    int GetExtendedFileInfo(extendedFileInfoStruct* ExtendedFileInfo, const int TagPriority);
    int GetExtendedFileInfoW(extendedFileInfoStructW* ExtendedFileInfo, const int TagPriority);
    bool GetTitle(const int TagPriority, _TagInfo* TagInfo);

private:
    CRITICAL_SECTION CriticalSection;
    DWORD GetTagTime;

public:
    _TagInfo Cache;
    bool GetTitleFromID3v2(_TagInfo* TagInfo, _ReplayGainInfo* ReplayGainInfo = NULL, const WCHAR* FileName = NULL);
    bool GetTitleFromAPE(_TagInfo* TagInfo, _ReplayGainInfo* ReplayGainInfo = NULL, const WCHAR* FileName = NULL);
    bool GetTitleFromID3v1(_TagInfo* TagInfo);
    void StoreReplayGainInfo(const int ReplayGainFieldName, WCHAR* Buff, _ReplayGainInfo* Info);
    void DoFormat(const _TagInfo* Info, const WCHAR* Format, WCHAR* Title, const size_t TitleSize);
};

#include "APETag_.h"
#include "ID3v1Info.h"

#endif

extern Tag m_Tag;