//------------------------------------------------------------------------------
// File: Video.cpp
//
// Desc: Video DirectShow filter
//
// Author : Ashok Jaiswal
//
// Data/Time : September 2004
//------------------------------------------------------------------------------

#include "StdAfx.h"

#include <initguid.h>
#include "VideoFilter.h"
#include <wa_ipc.h>
#include "Dvdmedia.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CVideoFilter::CVideoFilter(IUnknown* pUnk, HRESULT* phr, IVideoOutput* videoOutput) : CBaseRenderer(CLSID_Video, NAME("Video Filter"), pUnk, phr), m_videoOutput(videoOutput)
{
}

CVideoFilter::~CVideoFilter()
{
}

// make sure media type is what we want
//
HRESULT CVideoFilter::CheckMediaType(const CMediaType* pmt)
{
    CheckPointer(pmt, E_POINTER);

    if (pmt->majortype == MEDIATYPE_Video && (pmt->subtype == MEDIASUBTYPE_YV12 || pmt->subtype == MEDIASUBTYPE_IYUV || pmt->subtype == MEDIASUBTYPE_YUY2))
    {
        VIDEOINFOHEADER2* pVI = (VIDEOINFOHEADER2*)pmt->Format();
        CheckPointer(pVI, E_UNEXPECTED);

        long width = m_nWidth = pVI->dwPictAspectRatioX;
        long height = m_nHeight = pVI->dwPictAspectRatioY;

        m_videoOutput->showStatusMsg("Video from NxS Info Tool!");

        if (pmt->subtype == MEDIASUBTYPE_YV12)
            m_videoOutput->open(width, height, 0, 0, mmioFOURCC('Y', 'V', '1', '2'));
        if (pmt->subtype == MEDIASUBTYPE_IYUV)
            m_videoOutput->open(width, height, 0, 0, mmioFOURCC('I', 'Y', 'U', 'V'));
        if (pmt->subtype == MEDIASUBTYPE_YUY2)
            m_videoOutput->open(width, height, 0, 0, mmioFOURCC('Y', 'U', 'Y', '2'));
        return S_OK;
    }
    else
    {
        return E_INVALIDARG;
    }
}

HRESULT CVideoFilter::DoRenderSample(IMediaSample* pMediaSample)
{
    CheckPointer(pMediaSample, E_POINTER);

    BYTE* pBits;
    pMediaSample->GetPointer(&pBits);

    YV12_PLANES image_vbd;
    image_vbd.y.baseAddr = (unsigned char*)pBits;
    image_vbd.v.baseAddr = ((unsigned char*)pBits) + m_nWidth * m_nHeight;
    image_vbd.u.baseAddr = ((unsigned char*)pBits) + m_nWidth * m_nHeight * 5 / 4;
    image_vbd.y.rowBytes = m_nWidth;
    image_vbd.v.rowBytes = m_nWidth / 2;
    image_vbd.u.rowBytes = m_nWidth / 2;

    m_videoOutput->draw(&image_vbd);
    return S_OK;
}
