#pragma warning(disable : 4005)

#include "StdAfx.h"
#include <vfw.h>
#include <process.h>
#include <wchar.h>

#include "InterfaceIn.h"
#include <wa_ipc.h>
#include "CommonFunc.h"
#include "in_vid.h"

#include <dshow.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CPlayer player;

In_Module modVideo =
{
    IN_VER,                     //   module type (IN_VER)
    "Video ",   //   description of module, with version string 
    NULL,                       //   winamp's main window (filled in by winamp)
    NULL,                       //   DLL instance handle (Also filled in by winamp)
    FILE_EXT,                   // " ""mp3\0Layer 3 MPEG\0mp2\0Layer 2 MPEG\0mpg\0Layer 1 MPEG\0"""
    1,                          //   is this stream seekable? 
    1,                          //   does this plug-in use the output plug-ins? (musn't ever change, ever :)
    VideoInput::config,                     //   configuration dialog
    VideoInput::about,                      //   about dialog
    VideoInput::init,                       //   called at program init
    VideoInput::quit,                       //   called at program quit
    VideoInput::getfileinfo,                //   if file == NULL, current playing is used
    VideoInput::infobox,                    //  
    VideoInput::isourfile,                  //   called before extension checks, to allow detection of mms:
    VideoInput::play,                       //   return zero on success, -1 on file-not-found, some other value on other (stopping winamp) error
    VideoInput::pause,                      //   pause stream
    VideoInput::unpause,                    //   unpause stream
    VideoInput::ispaused,                   //   ispaused? return 1 if paused, 0 if not
    VideoInput::stop,                       //   stop (unload) stream
    VideoInput::getlength,                  //   get length in ms
    VideoInput::getoutputtime,              //   returns current output time in ms. (usually returns outMod->GetOutputTime()
    VideoInput::setoutputtime,              //   seeks to point in stream (in ms). Usually you signal yoru thread to seek, which seeks and calls outMod->Flush()..
    VideoInput::setvolume,                  //   from 0 to 255.. usually just call outMod->SetVolume
    VideoInput::setpan,                     //   from -127 to 127.. usually just call outMod->SetPan
    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, VideoInput::eq_set, NULL, NULL
};

CPlayer::CPlayer()
{
}

CPlayer::~CPlayer()
{
}

void CPlayer::Close()
{
    Uninit();
}

BOOL CPlayer::IsRunning()
{
    BOOL bRes = FALSE;

    if (m_MC != NULL)
    {
        FILTER_STATE eFs;
        if (SUCCEEDED(m_MC->GetState(500, (OAFilterState *)&eFs)) && (eFs == State_Running))
        {
            bRes = TRUE;
        }
    }

    return bRes;
}

BOOL CPlayer::IsFinished()
{
    BOOL bRet = TRUE;

    if (m_ME != NULL)
    {
        bRet = FALSE;
        long ec, p1, p2;
        for (; ;)
        {
            ec = p1 = p2 = 0;
            if (NOERROR != m_ME->GetEvent(&ec, &p1, &p2, 0))
            {
                break;
            }
            if (ec == EC_COMPLETE)
            {
                bRet = TRUE;
            }
            m_ME->FreeEventParams(ec, p1, p2);
        }
    }

    return bRet;
}

CString CPlayer::GetFilterName(IBaseFilter* pBF)
{
    ASSERT(pBF != NULL);

    FILTER_INFO fi;
    memset(&fi, 0, sizeof(fi));
    if (pBF != NULL)
    {
        VERIFY(SUCCEEDED(pBF->QueryFilterInfo(&fi)));
    }
    return fi.achName;
}

HRESULT CPlayer::AddFilterByClsid(IGraphBuilder* pGB, REFCLSID clsid, LPCTSTR lpszName, IBaseFilter** ppFilter)
{
    //Create Filter
    CComPtr<IBaseFilter> spFilter = NULL;
    HRESULT hr = spFilter.CoCreateInstance(clsid);
    if (SUCCEEDED(hr))
    {
        USES_CONVERSION;
        //add null render to graph builder
        if (lpszName == NULL)
            hr = pGB->AddFilter(spFilter, GetFilterName(spFilter));
        else
            hr = pGB->AddFilter(spFilter, lpszName);
        if (SUCCEEDED(hr) && (ppFilter != NULL))
        {
            *ppFilter = spFilter;
            (*ppFilter)->AddRef();
        }
    }

    return hr;
}

HRESULT CPlayer::CreateFilterByName(LPCTSTR lpszName, IBaseFilter** ppFilter)
{
    CString strCodecId(lpszName);

    HRESULT hr = S_OK;

    IBindCtx* pCtx;
    hr = ::CreateBindCtx(0, &pCtx);
    ULONG ulEaten = 0;
    IMoniker* pmk = NULL;
    USES_CONVERSION;
    hr = ::MkParseDisplayName(pCtx, lpszName, &ulEaten, &pmk);
    pCtx->Release();
    if (SUCCEEDED(hr))
    {
        hr = pmk->BindToObject(NULL, NULL, IID_IBaseFilter, (void**)ppFilter);
        pmk->Release();
    }

    return hr;
}

HRESULT CPlayer::AddFilterByName(IGraphBuilder* pGB, LPCTSTR lpszDispName, LPCTSTR lpszName, IBaseFilter** ppFilter)
{
    //Create Filter
    CComPtr<IBaseFilter> spFilter = NULL;
    HRESULT hr = CreateFilterByName(lpszDispName, &spFilter);
    if (SUCCEEDED(hr) && !(spFilter == NULL))
    {
        USES_CONVERSION;
        if (lpszName == NULL)
            hr = pGB->AddFilter(spFilter, GetFilterName(spFilter));
        else
            hr = pGB->AddFilter(spFilter, lpszName);
        if (SUCCEEDED(hr) && (ppFilter != NULL))
        {
            *ppFilter = spFilter;
            (*ppFilter)->AddRef();
        }
    }

    return hr;
}


HRESULT CPlayer::RenderAVI(LPCTSTR szFilePath, CComPtr<IGraphBuilder>& gb)
{
    LPCTSTR szExt = _tcsrchr(szFilePath, '.');
    if (!szExt)
    {
        return E_FAIL;
    }
    ++szExt;
    if (_tcsicmp(szExt, _T("avi")) != 0)
        return E_INVALIDARG;
    USES_CONVERSION;
    const GUID CLSID_AviSource =
    {
        0xCEA8DEFF, 0x0AF7, 0x4DB9, {0x9A,0x38,0xFB,0x3C,0x3A,0xEF,0xC0,0xDE}
    };
    CComPtr<IBaseFilter> spAviSource;
    HRESULT hr = AddFilterByClsid(gb, CLSID_AviSource, szFilePath, &spAviSource);
    if (spAviSource == NULL)
        return hr;
    CComQIPtr<IFileSourceFilter> FileSource(spAviSource);
    hr = FileSource->Load(szFilePath, NULL);
    if (FAILED(hr))
        return hr;
    CComPtr<IEnumPins> ep;
    hr = spAviSource->EnumPins(&ep);
    if (FAILED(hr))
        return hr;
    CComPtr<IPin> pin;
    DWORD dwCount;
    int i = 0;
    for (; ;)
    {
        dwCount = 0;
        hr = ep->Next(1, &pin, &dwCount);
        if (hr != NOERROR)
        {
            if (i)
                hr = NOERROR;
            break;
        }
        hr = gb->Render(pin);
        if (hr == NOERROR)
            ++i;
        pin = NULL;
    }
    return hr;
    return E_FAIL;
}

HRESULT CPlayer::RenderWMV(LPCTSTR szFilePath, CComPtr<IGraphBuilder>& gb)
{
    LPCTSTR szExt = _tcsrchr(szFilePath, '.');
    if (!szExt)
    {
        return E_FAIL;
    }
    ++szExt;
    if (_tcsicmp(szExt, _T("wma")) != 0 && _tcsicmp(szExt, _T("wmv")) != 0)
        return E_INVALIDARG;
    static const GUID CLSID_AsyncFileSource =
    {
        0xE436EBB5, 0x524F, 0x11CE, {0x9F,0x53,0x00,0x20,0xAF,0x0B,0xA7,0x70}
    };
    USES_CONVERSION;
    const GUID CLSID_WMVParser =
    {
        0x1932C124, 0x77DA, 0x4151, {0x99,0xAA,0x23,0x4F,0xEA,0x09,0xF4,0x63}
    };
    CComPtr<IBaseFilter> spFileSource;
    HRESULT hr = AddFilterByClsid(gb, CLSID_AsyncFileSource, szFilePath, &spFileSource);
    if (spFileSource == NULL)
        return hr;
    CComQIPtr<IFileSourceFilter> FileSource(spFileSource);
    hr = FileSource->Load(szFilePath, NULL);
    if (FAILED(hr))
        return hr;
    hr = AddFilterByClsid(gb, CLSID_WMVParser, _T("WMV Parser"), NULL);
    CComPtr<IEnumPins> ep;
    hr = spFileSource->EnumPins(&ep);
    if (FAILED(hr))
        return hr;
    CComPtr<IPin> pin;
    DWORD dwCount = 0;
    hr = ep->Next(1, &pin, &dwCount);
    if (FAILED(hr))
        return hr;
    hr = gb->Render(pin);
    return hr;
    return E_FAIL;
}

HRESULT CPlayer::RenderMediaFile(LPCTSTR szFilePath, CComPtr<IGraphBuilder>& gb)
{
    static const GUID CLSID_AsyncFileSource =
    {
        0xE436EBB5, 0x524F, 0x11CE, {0x9F,0x53,0x00,0x20,0xAF,0x0B,0xA7,0x70}
    };
    USES_CONVERSION;
    CComPtr<IBaseFilter> spFileSource;
    HRESULT hr = AddFilterByClsid(gb, CLSID_AsyncFileSource, szFilePath, &spFileSource);
    if (spFileSource == NULL)
        return hr;
    CComQIPtr<IFileSourceFilter> FileSource(spFileSource);
    hr = FileSource->Load(szFilePath, NULL);
    if (FAILED(hr))
        return hr;
    CComPtr<IEnumPins> ep;
    hr = spFileSource->EnumPins(&ep);
    if (FAILED(hr))
        return hr;
    CComPtr<IPin> pin;
    DWORD dwCount = 0;
    hr = ep->Next(1, &pin, &dwCount);
    if (FAILED(hr))
        return hr;
    hr = gb->Render(pin);
    return hr;
    return E_FAIL;
}

BOOL CPlayer::Open(LPCTSTR szUrl)
{
    Init();
    {
        TCHAR buf[512];
        GetModuleFileName(NULL, buf, 512);
        *_tcsrchr(buf, '\\') = 0;
        SetCurrentDirectory(buf);
    }

    USES_CONVERSION;

    HRESULT hr = E_FAIL;

    if (FAILED(hr))
    {
        hr = RenderWMV(szUrl, m_GB);
    }
    if (FAILED(hr))
    {
        hr = RenderAVI(szUrl, m_GB);
    }
    if (FAILED(hr))
    {
        hr = RenderMediaFile(szUrl, m_GB);
    }
    if (FAILED(hr))
    {
        //Render File,create play link
        hr = m_GB->RenderFile(szUrl, NULL);
    }

    if (FAILED(hr))
        return FALSE;

    // QueryInterface for DirectShow interfaces
    hr = m_GB->QueryInterface(IID_IMediaControl, (void**)&m_MC);
    if (FAILED(hr))
        return FALSE;
    hr = m_GB->QueryInterface(IID_IMediaEventEx, (void**)&m_ME);
    if (FAILED(hr))
        return FALSE;
    hr = m_GB->QueryInterface(IID_IMediaSeeking, (void**)&m_MS);
    if (FAILED(hr))
        return FALSE;
    hr = m_GB->QueryInterface(IID_IMediaPosition, (void**)&m_MP);
    if (FAILED(hr))
        return FALSE;

    Play();

    return TRUE;
}

void CPlayer::Pause()
{
    if (m_MC != NULL)
    {
        HRESULT hr = m_MC->Pause();
        if (FAILED(hr))
        {
            TRACE(_T("Failed to pause"));
        }
    }
}

void CPlayer::Play()
{
    if (m_MC != NULL)
    {
        HRESULT hr = m_MC->Run();   
        if (FAILED(hr))
        {
            TRACE(_T("Failed to pause"));
        }
    }
}

void CPlayer::Stop()
{
    if (m_MC != NULL)
    {
        LONGLONG pos = 0;
        HRESULT hr = m_MC->Stop();
        if (FAILED(hr))
        {
            TRACE(_T("Failed to stop"));
        }
        m_MS->SetPositions(&pos, AM_SEEKING_AbsolutePositioning, NULL, AM_SEEKING_NoPositioning);
        m_MC->Pause();
    }
}

LONGLONG CPlayer::GetPosition()
{
    LONGLONG llPos = 0;
    if (m_MS != NULL)
    {
        HRESULT hr = m_MS->GetCurrentPosition(&llPos);  
        llPos /= 10000; // llen en ms.
        if (FAILED(hr))
        {
            TRACE(_T("Failed to get play position"));
        }
    }
    return llPos;
}

void CPlayer::SetPosition(LONGLONG llPos)
{
    if (m_MS != NULL)
    {
        m_MS->SetTimeFormat(&TIME_FORMAT_MEDIA_TIME); //100 nanosecondes units => 10^-7 secondes.
        llPos *= 10000; // llen en 100ns.
        HRESULT hr = m_MS->SetPositions(&llPos, AM_SEEKING_AbsolutePositioning | AM_SEEKING_SeekToKeyFrame, NULL, AM_SEEKING_NoPositioning);
    }
}

LONGLONG CPlayer::GetLength()
{
    LONGLONG llLen = 0;
    if (m_MS != NULL)
    {
        m_MS->SetTimeFormat(&TIME_FORMAT_MEDIA_TIME); //100 nanosecondes units => 10^-7 secondes.
        HRESULT hr = m_MS->GetDuration(&llLen);
        if (FAILED(hr))
        {
            TRACE(_T("Failed to get movie duration"));
        }
        llLen /= 10000; // llen en ms.
    }
    return llLen;
}

HRESULT CPlayer::Init()
{
    Uninit();
    HRESULT hr = CoCreateInstance(CLSID_FilterGraph, NULL, CLSCTX_INPROC_SERVER, IID_IGraphBuilder, (void**)&m_GB);
    m_BV = new CVideoFilter(0, &hr, (IVideoOutput*)::SendMessage(modVideo.hMainWindow, WM_WA_IPC, 0, IPC_GET_IVIDEOOUTPUT)); 
    m_BA = new CAudioFilter(0, &hr);
    hr = m_GB->AddFilter((IBaseFilter *)m_BV, L"Video filter");
    hr = m_GB->AddFilter((IBaseFilter *)m_BA, L"Audio filter");

    return hr;
}

HRESULT CPlayer::Uninit()
{
    HRESULT hr = S_OK;

    //stop graph
    Stop();

    if (m_BV != NULL)
        m_GB->RemoveFilter(m_BV);
    if (m_BA != NULL)
        m_GB->RemoveFilter(m_BA);
    m_GB = NULL;
    m_MC = NULL;
    m_ME = NULL;
    m_MS = NULL;
    m_MP = NULL;
    m_BV = NULL;
    m_BA = NULL;

    return hr;
}

namespace VideoInput
{
    void __cdecl
    init()
    {
        CoInitialize(0);

        WNDCLASS wndcls;
        memset(&wndcls, 0, sizeof(WNDCLASS));

        wndcls.lpfnWndProc = ::DefWindowProc;
        wndcls.hInstance = modVideo.hDllInstance;
        wndcls.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
        wndcls.lpszMenuName = NULL;

        wndcls.lpszClassName = _T("player");

        AfxRegisterClass(&wndcls);
        player.CreateEx(WS_EX_TOPMOST | WS_EX_TOOLWINDOW, _T("player"), _T("player"), WS_POPUP, 0, ::GetSystemMetrics(SM_CYFULLSCREEN), 0, 0, NULL, NULL, NULL);
    }

    void __cdecl
    quit()
    {
        player.Close();
    }

    //

    void __cdecl
    config(HWND hwndParent)
    {
    }

    void __cdecl
    about(HWND hwndParent)
    {
    }

    int __cdecl
    isourfile(const in_char* fn)
    {
//        MessageBox(modVideo.hMainWindow, fn, "", MB_OK);
        return 1;
    }

    void __cdecl
    pause()
    {
        player.Pause();
    }

    void __cdecl
    unpause()
    {
        player.Play();
    }

    int __cdecl
    ispaused()
    {
        return player.IsRunning();
    }

    int __cdecl
    getoutputtime()
    {
        return (int)player.GetPosition();
    }

    void __cdecl
    setoutputtime(int time_in_ms)
    {
        player.SetPosition(time_in_ms);
    }

    void __cdecl
    setvolume(int volume)
    {
        modVideo.outMod->SetVolume(volume);
    }

    void __cdecl
    setpan(int pan)
    {
        modVideo.outMod->SetPan(pan);
    }

    int __cdecl
    infobox(const in_char* fn, HWND hwnd)
    {
        return 1;
    }

    int __cdecl
    getlength()
    {
        return (int)player.GetLength();
    }

    void __cdecl
    eq_set(int on, char data[10], int preamp)
    {
    }

    int __cdecl
    play(const in_char* fn)
    {
        return !player.Open(fn);
    }

    void __cdecl
    stop()
    {
        player.Stop();
    }

    void __cdecl
    getfileinfo(const in_char* filename, in_char* title, int* length_in_ms)
    {
    }
}
/*
extern "C"
{
    __declspec(dllexport) In_Module* __cdecl
    winampGetInModule2()
    {
        return &modVideo;
    }
}
*/
#ifdef VID

bool getmp3info(bool Decode, const WCHAR* fn, mp3info* info, VBRTAGDATA* vbr_tag)
{
    return true;
}

void UpdateWinampDisplay()
{
}

void reflect_thread_setting()
{
}

void reflect_setting()
{
}


void ReceiveUDPMsg(const WCHAR* src)
{
}

void UpdateUDPSocket(CString server, UINT nPort)
{
}

UINT ReadStream(char* src, UINT nCount)
{
    return 1;
}

void UpdateWinampDisplay();

Tag m_Tag;
CIn_mpg123dParam* prm;
Helper m_HelperIn;
bool m_paused;              // are we paused?

#endif